/* afni_note.c */

#include "afni_note.h"

/*===<date_and_time.c>========================================================

     Title: DATE AND TIME

   Purpose: Return string containing date and time

      Call: char *date_and_time(void)

    Return: 

   History: tross_datetime in thd_notes.c
*/
char *date_and_time(void)
{
   time_t tnow = time(NULL) ; int i ; char * qh , * ch ;

   ch = ctime(&tnow); 
   i  = strlen(ch); 
   qh = (char *)malloc(i+2); 
   strcpy(qh,ch); 
   qh[i-1]='\0';

   return qh ;
}

/*===<make_afni_note.c>====================================================

     Title: MAKE AFNI NOTE

   Purpose: Create note field for afni header

      Call: 

    Return: 0 = success

   History: 02_07_26 - lrf

*/
int make_afni_note(NOTE *note)
{
  char afni_note[MAX_NOTE_LEN] ;

  char *attr_str = "type = string-attribute" ;
  char *attr_int = "type = integer-attribute" ;

  char *number_base = "NOTE_NUMBER_" ;
  char *date_base   = "NOTE_DATE_"   ;
  char *count_base  = "NOTES_COUNT"  ;

  char number_name[MAXSTR], number_count[MAXSTR];
  char   date_name[MAXSTR],   date_count[MAXSTR];
  char  count_name[MAXSTR],  count_count[MAXSTR];

  char   count_str[MAXSTR],   number_str[MAXSTR],   date_str[MAXSTR];
  char count_field[MAXSTR], number_field[MAXSTR], date_field[MAXSTR];

  int  i;
  int  num_notes = 1;		/* Hardwired for now! */

  int  debug = 0;

  if(!note->notes_exist) return 0 ;

  /* Create count field */

  sprintf(count_str," %i",num_notes);
  sprintf(count_name,"name = %s",count_base);
  sprintf(count_count,"count = %i",strlen(count_str)-1);

  sprintf(count_field,"%s\n%s\n%s\n%s",
	  attr_int,count_name,count_count,count_str);

  if(debug) printf("\ncount_field:\n%s\n\n",count_field);

  /* Create number field */

  for(i=0; i<num_notes; i++){
    sprintf(number_name,"name = %s%03i",number_base,i+1);
  }

  sprintf(number_str,"'%s~",note->comment);
  sprintf(number_count,"count = %i",strlen(number_str)-1);

  sprintf(number_field,"%s\n%s\n%s\n%s",
	  attr_str,number_name,number_count,number_str);

  if(debug) printf("\nnumber_field:\n%s\n\n",number_field);

  /* Create date field */

  for(i=0; i<num_notes; i++){
    sprintf(date_name,"name = %s%03i",date_base,i+1);
  }

  sprintf(date_str,"'%s~",note->datetime);
  sprintf(date_count,"count = %i",strlen(date_str)-1);

  sprintf(date_field,"%s\n%s\n%s\n%s",
	  attr_str,date_name,date_count,date_str);

  if(debug) printf("\ndate_field:\n%s\n\n",date_field);

  sprintf(afni_note,"\n%s\n\n%s\n\n%s\n",
	  count_field,number_field,date_field);

  strcpy(note->afni_note,afni_note);

  if(debug) printf("\n final note::\n%s\n\n",note->afni_note);

  return(1);

}

/*===<set_scanner_type.c>====================================================

     Title: SET SCANNER TYPE

   Purpose: Set scanner type

      Call: NOTE *set_scanner_type() 

    Return: NOTE *note

   History: 02_07_25 - lrf

*/
NOTE *set_scanner_type(char *scanner_type) 
{
  NOTE *scanner_note = (NOTE *) malloc(sizeof(NOTE));
  int debug = 0;

  scanner_note->notes_exist = 1;
  strcpy( scanner_note->datetime, date_and_time() );
  strcpy( scanner_note->comment, scanner_type );
  make_afni_note(scanner_note);
  if(debug) printf("scanner_note = %s\n",scanner_note->afni_note);

  return(scanner_note);

}
