#include "mrilib.h"

void instructions()
{
	printf("\ndicom_to_dif [options] fname \n"
	       "Reads images from DICOM file 'fname' and writes them to raw\n"
	       "file(s) 'fname.slice.diff_dir.bval' etc.\n"
	       "\n"
	       "\tOptions:\n"
	       "\t\t-ndif  [0] #diffusion directions\n"
	       "\t\t-nslc  [0] #slices\n"
	       "\t\t-nbvl  [0] #b-values\n"
	       "\t\t-oname ['x'] output name\n\n");

	return;
}
int debug = 0;
int main( int argc , char *argv[] )
{
  MRI_IMARR *imar ;
  int kk,ii,kcount, nn ;
  char iname[1024] ;
  char oname[1024] ;

  int idif=0,ndif = 0;
  int islc=0,nslc = 0;
  int ibvl=0,nbvl = 0;
  int nopt = 0;
  int minarg = 1;

  /*printf("argc = %i\n",argc);*/
  if( argc < 2 || strcmp(argv[1],"-help") == 0 ){
    instructions();
    exit(0) ;
  }
           
  strcpy(oname,"x");

  for( kk=1 ; kk < argc ; kk++ ){
      if (!strncmp(argv[kk], "-oname", 6)) {
	if (++kk >= argc) { instructions(); exit(0); }
	sprintf(oname,argv[kk]);
	nopt++;nopt++;
      }
      if (!strncmp(argv[kk], "-ndif", 5)) {
	if (++kk >= argc) { instructions(); exit(0); }
	ndif = atoi(argv[kk]);
	nopt++;nopt++;
      }
      if (!strncmp(argv[kk], "-nslc", 5)) {
	if (++kk >= argc) { instructions(); exit(0); }
	nslc = atoi(argv[kk]);
	nopt++;nopt++;
      }
      if (!strncmp(argv[kk], "-nbvl", 4)) {
	if (++kk >= argc) { instructions(); exit(0); }
	nbvl = atoi(argv[kk]);
	nopt++;nopt++;
      }
      if (!strncmp(argv[kk], "-debug", 6)) {
	if (++kk >= argc) { instructions(); exit(0); }
	nbvl = atoi(argv[kk]);
	nopt++;nopt++;
      }
  }
  if(debug) printf("nopt = %i, ndif = %i, nslc = %i, nbvl = %i\n",nopt,ndif,nslc,nbvl);

  kcount = 0 ;
  for( kk=minarg+nopt ; kk < argc ; kk++ ){

    islc = kcount % nslc ;
    if(kcount>0 && islc==0) idif++ ;
    if(kcount>0 && (kcount%(ndif*nslc)==0) ) {idif = 0; ibvl++ ;}

    if(debug) printf("argv[%i] = %s, idif=%i, islc=%i, ibvl=%i\n",
		     kk,argv[kk],idif,islc,ibvl);

    imar = mri_read_dicom( argv[kk] ) ;
    if( imar == NULL ){
      fprintf(stderr,"++ Can't read from file %s\n",argv[kk]) ; continue ;
    }
    nn = IMARR_COUNT(imar) ;

    for( ii=0 ; ii < nn ; ii++ ){
      if(nn>1)
	sprintf(iname,"%s.%02d.d%02d.b%02d.%04d",oname,islc+1,idif,ibvl+1,ii+1) ;
      else
	sprintf(iname,"%s.%02d.d%02d.b%02d",oname,islc+1,idif,ibvl+1) ;
      mri_write( iname, IMARR_SUBIM(imar,ii) ) ;
    }
    DESTROY_IMARR(imar) ;

    kcount++ ;
  } /* kk */

  exit(0) ;
}
