/**<dif2afni.c>***************************************************************

      Title: DIFfusion TO AFNI

    Purpose: Convert diffusion images collected with ses13.psd
             into AFNI 3D file

     Author: Lawrence R. Frank
             (c) University of California, San Diego

    History: 00_11_18 cp stack.c (ECW) -> dif2afni.c
                      system call to to3d

      Notes: This program is basically a combination of
             ECW's stack.c with a system call to RWC's to3d.c
	     with some additional available input.

	     I've disabled stack options '-behead' and '-rm'

*/
#include "d2a.h"

#define MAXNDIF  3000	/* Maximum number of diffusion directions.
			   Defined in grses.c */
#define debug  0
#define debug2 0

typedef struct {
  char   filename[80];
  char   tpattern[80];
  char   fmat[100];
  char   orient[10];
  int    nsl; 
  int    ndif; 
  float  tr ;
} TO3Dstc;

char hfile[80], *pfile;
FILE *fin,*fout;

main(int argc,char *argv[])
{
        int need_help = 0;
	int idif, isl ;
	int i,j,ncount,sl1,nsl,dif1,ndif,nopt,ntype,done;
	int imsize,sl,dif,nx,ny,head,headsize,datasize,attach;
	int file_exists(),file_size(),get_old_header();
	long int ier,isize,gsize;
	short int *im,*ihead,*idata;
	char rmcmd[80],*inroot,*outroot;
	char fname[80],dname[80],hname[80],fnum[80],head_id[80];
	FILE *fd;
	int write_file(), read_file();
	void instructions();
	void send_help() ;
	char to3d_str[10000];	/* command line for program to3d */
	char buck3d_str[10000];	/* command line for program 3dbucket */
	float TR = 1.0;
	char gdxname[80],gdyname[80],gdzname[80];
	float gdx[MAXNDIF],gdy[MAXNDIF],gdz[MAXNDIF];
	TO3Dstc to3dstc;
	int open_files(char *, char *);
	int write_header(char *, char * ) ;
	char header[RDB_HEADER_SIZE_BYTES];

	/* System remove call */
	sprintf(rmcmd,"/bin/rm -f ");

	/* Initialize some stuff */

	sl1 = 1;
	nsl = 0;
	dif1 = 1;
	ndif = 0;
	inroot = argv[argc-1];
	outroot = inroot;
	head = 0;
	attach = 0;

	/* parse arguments */

	nopt = 0;
	if(argc==2) {
	  if (!strncmp(argv[1], "-help", 5)) { send_help( ); exit(0); }
	  else { instructions(); exit(0); }
	}

	for (i = 1; i < argc-1; i++) { /* ------- Options ------- */
	        if (!strncmp(argv[i], "-oname", 6)) {  /* Outfile root */
			if (++i >= argc) { instructions(); exit(0); }
			outroot = argv[i];
			nopt++; nopt++;
		}
		else if (!strncmp(argv[i], "-head", 7)) {  /* Write header from Pfile */
			if (++i >= argc) { instructions(); exit(0); }
			pfile = argv[i];
			head = 1;
			nopt++; nopt++;
		}
		else {
			printf("Option %s is not recognized\n",argv[i]);
			instructions();
			exit(0);
		}

	} /* end of options */

	if (argc-nopt < 2){
		instructions();
		exit(0);
	}

	/* Print out header from pfile */

	if(head) {
	  printf("Generating header from pfile %s\n",pfile);
	  sprintf(hfile,"%s.header",outroot); /* Header name */

	  if(!open_files(pfile,header)) exit (0);

	  if(!write_header(hfile,header)) {
	    printf("\aError writing file %s!\n",hfile);
	    exit (1);
	  } else {
	    printf("Wrote file  %s\n",hfile);
	  }
	}

	/* Get image name type and file size 
	   ntype: 1=root.xx.dxx 
	          2=root.xxx 
	          3=root.dxx */

	ntype = 0;
	sprintf(fname,"%s.%02d.d%02d",inroot,sl1,dif1);
	printf("fname = %s\n",fname);
	if (isize = file_size(fname)) ntype = 1;
	if (!ntype && ndif<2) {
		sprintf(fname,"%s.%03d",inroot,sl1);
		if (isize = file_size(fname)) ntype = 2;
		printf("fname = %s\n",fname);
	}
	if (!ntype) {
	        sprintf(fname,"%s.d%02d",inroot,dif1);
		if (isize = file_size(fname)) ntype = 3;
		nsl = 1;
		printf("fname = %s\n",fname);
	}
	if (!ntype){
	        printf("\aUnknown image type = %i! Exiting...\n",ntype);
		instructions();
		exit(0);
	}
	imsize = isize/sizeof(short int);

	/* Set nx, ny, and head based on file length */
	switch(imsize) {
		case 4096:
			nx = 64;          /* raw image formats */
			ny = 64;
			head = 0;
			break;
		case 9216:
			nx = 96;
			ny = 96;
			head = 0;
			break;
		case 16384:
			nx = 128;
			ny = 128;
			head = 0;
			break;
		case 65536:
			nx = 256;
			ny = 256;
			head = 0;
			break;
		case 72704:                /* GE 4.x formats */
			nx = 256;
			ny = 256;
			head = 14336;
			sprintf(head_id,"ge4x_im");
			break;
		case 269312:
			nx = 512;
			ny = 512;
			head = 14336;
			sprintf(head_id,"ge4x_im");
			break;
		case 19456:                /* Siemens formats */
			nx = 128;
			ny = 128;
			head = 6144;
			sprintf(head_id,"vis_im");
			break;
		case 68608:
			nx = 256;
			ny = 256;
			head = 6144;
			sprintf(head_id,"vis_im");
			break;
		case 265216:
			nx = 512;
			ny = 512;
			head = 6144;
			sprintf(head_id,"vis_im");
			break;
	}
	attach = head;   /* attach header if one exists */

	/* Get nsl */

	if (!nsl) {
		done = 0;
		i = sl1;
		while (!done) {
			strcpy(fname,inroot);
			if (ntype == 1) sprintf(fnum,".%02d.d%02d",i,dif1);
			else if (ntype == 2) sprintf(fnum,".%03d",i);
			strcat(fname,fnum);
			if (file_exists(fname)) i++;
			else done = 1;
		}
		nsl = --i - sl1 + 1;
	}
	printf("Found %i slices\n",nsl);
	
	/* Get ndif */

	if (ntype == 2) ndif = 1;
	if (!ndif) {
		done = 0;
		i = dif1;
		while (!done) {
			strcpy(fname,inroot);
			if (ntype == 1) sprintf(fnum,".%02d.d%02d",sl1,i);
			else if (ntype == 3) sprintf(fnum,".d%02d",i);
			strcat(fname,fnum);
			if (file_exists(fname)) i++;
			else done = 1;
		}
		ndif = --i - dif1 + 1;
	}
	printf("Found %i diffusion directions\n",ndif);
	
	/* Gradient direction files.
	   They are always ".01", per dell.c 
	   They are saved from dell.c as ascii
	   but we want to convert them to text 
	   and save them as AFNI *.1D files */

	sprintf(gdxname,"%s.01.gdx",inroot);
	sprintf(gdyname,"%s.01.gdy",inroot);
	sprintf(gdzname,"%s.01.gdz",inroot);
	gsize = ndif * sizeof(float);
	read_file(&ier,gdxname,&gsize,gdx);
	read_file(&ier,gdyname,&gsize,gdy);
	read_file(&ier,gdzname,&gsize,gdz);

	if(debug2) {
	  for (dif=dif1-1;dif<dif1+ndif-1;dif++) { /* note offset of 0 */
	    printf("Diff dir %i: (gdx,gdy,gdz) = (%f,%f,%f)\n",
		   dif,gdx[dif],gdy[dif],gdz[dif]);
	  }	  
	}

	/* Write out gradient files in AFNI *.1D format */

	/* X grad */
	/* sprintf(gdxname,"%s_gdx.1D",outroot); */
	sprintf(gdxname,"%s.GDX",outroot);
	printf("Writing xdiff gradient file to %s\n",gdxname);
	if ((fd = fopen(gdxname,"w")) == NULL) {
		printf("Problem opening %s\n",gdxname);
		instructions();
		exit(0);
	}
	for (dif=dif1-1;dif<dif1+ndif-1;dif++) {
	  fprintf(fd,"%g\n",gdx[dif]);
	}
	fclose(fd);

	/* Y grad */

	/* sprintf(gdyname,"%s_gdy.1D",outroot); */
	sprintf(gdyname,"%s.GDY",outroot);
	printf("Writing ydiff gradient file to %s\n",gdyname);
	if ((fd = fopen(gdyname,"w")) == NULL) {
		printf("Problem opening %s\n",gdyname);
		instructions();
		exit(0);
	}
	for (dif=dif1-1;dif<dif1+ndif-1;dif++) {
	  fprintf(fd,"%g\n",gdy[dif]);
	}
	fclose(fd);

	/* Z grad */

	/* sprintf(gdzname,"%s_gdz.1D",outroot); */
	sprintf(gdzname,"%s.GDZ",outroot);
	printf("Writing zdiff gradient file to %s\n",gdzname);
	if ((fd = fopen(gdzname,"w")) == NULL) {
		printf("Problem opening %s\n",gdzname);
		instructions();
		exit(0);
	}
	for (dif=dif1-1;dif<dif1+ndif-1;dif++) {
	  fprintf(fd,"%g\n",gdz[dif]);
	}
	fclose(fd);

	/* allocate arrays */

	headsize = head/sizeof(short int);
	datasize = imsize - headsize;
	im = (short int *)calloc(imsize,sizeof(short int));
	ihead = (short int *)calloc(headsize,sizeof(short int));
	idata = (short int *)calloc(ndif*nsl*imsize,sizeof(short int));

	/* Read in images */

	for (idif=dif1;idif<dif1+ndif;idif++) {
	  ncount = 0;
	  for (sl=sl1;sl<sl1+nsl;sl++) {
	    strcpy(fname,inroot);
	         if (ntype == 1) sprintf(fnum,".%02d.d%02d",sl,idif);
	    else if (ntype == 2) sprintf(fnum,".%03d",sl);
	    else if (ntype == 3) sprintf(fnum,".d%02d",idif);
	    strcat(fname,fnum);
	    read_file(&ier,fname,&isize,idata+ncount*imsize);
	    if (ier) {instructions(); exit(0); }
	    ncount++;
	  }	

	if (head) for (i=0;i<headsize;i++) ihead[i] = idata[i];

	/* Write data block */

	strcpy(dname,outroot);
	sprintf(fnum,".%02d.brik",idif);
	strcat(dname,fnum);
	isize = nsl * ndif * imsize * sizeof(short int);
	write_file(&ier,dname,&isize,idata);
	if (ier) {
	  instructions();
	  exit(0);
	}

	/* Get old header if called for */

	if (!head) attach = get_old_header(inroot,head_id,ihead,&headsize);

	/* Write info to header */

	strcpy(hname,outroot);
	sprintf(fnum,".%02d.head",idif);
	strcat(hname,fnum);
	if ((fd = fopen(hname,"w")) == NULL) {
		printf("Problem opening %s\n",hname);
		instructions();
		exit(0);
	}
	fprintf(fd,"#Standard UCSD/MRI header\n");
	if (sl1>1) fprintf(fd,"nz1:%d\n",sl1);
	if (dif1>1) fprintf(fd,"nt1:%d\n",dif1);
	if (head) fprintf(fd,"head:%d\n",head);
	if (attach) fprintf(fd,"attach:%d:%s\n",
			    headsize*sizeof(short int),head_id);
	fprintf(fd,"end\n");
	if (attach) fwrite(ihead,sizeof(short int),headsize,fd);
	fclose(fd);

	/* Construct to3dstc */

	sprintf (to3dstc.filename,"%s.%02d",outroot,idif);
	strcpy (to3dstc.orient, "RAI");
	to3dstc.nsl  = nsl;
	to3dstc.ndif = ndif;
	sprintf(to3dstc.fmat, "3D:0:0:%i:%i:%i:%s.%02d.brik",
		nx,ny,nsl,outroot,idif);
	if(debug) {
	  printf("to3dstc.filename = %s\n",to3dstc.filename);
	  printf("to3dstc.orient = %s\n",to3dstc.orient);
	  printf("to3dstc.fmat = %s\n",to3dstc.fmat);
	  printf("to3dstc.nsl = %i\n",to3dstc.nsl);
	  printf("to3dstc.ndif = %i\n",to3dstc.ndif);
	}

	/*----- Construct command line for program to3d -----*/
	
	sprintf(to3d_str,"to3d -prefix %s -orient %s -abuc %s",
		to3dstc.filename,
		to3dstc.orient,
		to3dstc.fmat);

	if(debug) printf("to3d_str = %s\n",to3d_str); 

	/*----- Invoke program to3d to write out AFNI dataset -----*/

	printf("Writing AFNI dataset ");
	printf("into %s\n", to3dstc.filename);
	system (to3d_str); 

	if(idif==dif1) {
	  sprintf(buck3d_str,"3dbucket -abuc -prefix %s %s+orig",
		  outroot,to3dstc.filename);
	} else {
	  sprintf(buck3d_str,"3dbucket -abuc -glueto %s+orig %s+orig",
		  outroot,to3dstc.filename);
	}
	if(debug) printf("++++++++++++++++++++++++ buck3d_str = %s\n",
			 buck3d_str);
	system (buck3d_str); 

	/* Remove .HEAD file */
	sprintf(fname,"%s%s+orig.HEAD\n",rmcmd,to3dstc.filename);
	system(fname);
	/* Remove .BRIK file */
	sprintf(fname,"%s%s+orig.BRIK\n",rmcmd,to3dstc.filename);
	system(fname);
	/* Remove stack header */
	strcpy(fname,rmcmd);
	strcat(fname,hname);
	system(fname);
	/* Remove stack data */
	strcpy(fname,rmcmd);
	strcat(fname,dname);
	system(fname);

	}

	free(idata);


	printf("...done!\n");
}

int file_exists(char fname[])
{
	int	i;
	FILE *fd;				/* file descriptor  */

	if ((fd = fopen(fname, "r")) != NULL) {
		fclose(fd);
		i = 1;
	}
	else i = 0;

	return i;
}

int file_size(char fname[])
{
	int	fp;				/* file descriptor  */
	int	fsize;				/* file size  */
	struct stat file_stat;			/* status structure */

	if ((fp = open(fname, O_RDONLY)) <= 0)	/* file must exist */
	  return 0; 

	fstat(fp, &file_stat);			/* get file size in bytes   */
	fsize = file_stat.st_size;

	close(fp);

	return fsize;
}

int get_old_header(char *fname,char *htype,short int *ar,int *n)
{
	int i,done;
	FILE *fd;
	char str[100];

	strcpy(str,fname);
	strcat(str,".head");
	if ((fd = fopen(str, "r")) == NULL) return 0;

	done = 0;
	do {            /* look for attach: field */
		if (feof(fd)) return 0;
		fgets(str,98,fd);
		if (!strncmp(str, "attach", 6)) {
			sscanf(str,"attach:%d:%s",n,htype);
			*n /= sizeof(short int);
			done = 1;
		}
		else if (!strncmp(str, "end", 3)) return 0;
	} while (!done);

	done = 0;
	do {                           /* go to beginning of header */
		if (feof(fd)) return 0;
		fgets(str,98,fd);
		if (!strncmp(str, "end", 3)) done = 1;
	} while (!done);
	
	i = fread(ar,sizeof(short int),*n,fd);
	fclose(fd);
	if (i<*n) *n = i;
	return 1;
}

/* ----------------------------------------------------------------
   Write file subroutine. A.Jesmanowicz, MCW 1989
	ier   :	return error :	1 - opening problem,
				2 - negative length.
	fname :	file name.
	size  :	size of the file.
	arr   :	file array.
   --------------------------------------------------------------*/
void write_iqm(long int *ier,char fname[],long int *size,char arr[])
{
	int	isize = *size;
	int	fp;				/* file descriptor  */

	{	int i=0;			/* cut junk from the name */
		while(fname[i] != 32 && fname[i]) i++;
		fname[i] = 0;
	}

	if(isize < 0)				/* size has to be real */
	{	*ier=2;				/* or ier = 2.	       */
		return;
	}

	if ((fp = open(fname, O_WRONLY|O_CREAT|O_EXCL,0644)) <= 0)
	{	*ier=1;				/* file must not exist */
		return;				/* or ier = 1.	   */
	}

	*ier=0;					/* no error : ier=0 */
	write(fp, arr, isize);			/* write whole file */
	close(fp);
	return;  
}

/* Report for write_iqm  */

int write_iqm_error( long int *ier , char *fname )
{
  char *errstr;

  if(*ier==1) errstr = "FILE EXISTS";
  if(*ier==2) errstr = "FILE SIZE < 0";

  if (*ier!=0) {
    printf("ERROR WRITING FILE %s! %s\n",fname,errstr);
  }

  return(*ier);

}

int write_file( long int *ier,
		char     fname[],
		long int *size,
		char     arr[] )
{
  write_iqm(ier,fname,size,arr);
  return(write_iqm_error(ier,fname));
}

void read_iqm(long int *ier,char fname[],long int *size,char arr[])
{
	int	isize = *size;
	int	fp;				/* file descriptor  */
	struct stat file_stat;			/* status structure */

	{	int i=0;			/* cut junk from the name */
		while(fname[i] != 32 && fname[i]) i++;
		fname[i] = 0;
	}

	if ((fp = open(fname, O_RDONLY)) <= 0)	/* file must exist */
	{	*ier=1;				/* or ier = 1.	   */
		return;
	}

	fstat(fp, &file_stat);			/* get file size in bytes   */

	if(file_stat.st_size > isize && isize)	/* file can not be too long */
	{	*ier=2;				/* or ier = 2.		    */
		return;
	}
	*size =  file_stat.st_size;		/* return file size */

	read(fp, arr, file_stat.st_size);	/* read whole file  */
	close(fp);
	*ier=0;					/* no error : ier=0 */
	return;
}

/* Report for read_iqm  */

int read_iqm_error( long int *ier , char *fname )
{
  char *errstr;

  if(*ier==1) errstr = "FILE DOES NOT EXIST";
  if(*ier==2) errstr = "FILE SIZE TOO LONG";

  if (*ier!=0) {
    printf("ERROR READING FILE %s! %s\n",fname,errstr);
  }

  return(*ier);

}

int read_file( long int *ier,
	       char     fname[],
	       long int *size,
	       char     arr[] )
{
  read_iqm(ier,fname,size,arr);
  return(read_iqm_error(ier,fname));
}


int open_files(char *fname, char *header)
{
  if (!(fin=fopen(fname,"r"))) {
    printf("Can't open rawfile %s!\n",fname);
    return(0);
  }

  /* read raw hdr */

  if (fread(header, sizeof(char), RDB_HEADER_SIZE_BYTES, fin) 
      != RDB_HEADER_SIZE_BYTES)
    printf("header too short!\n");

#ifdef Linux
   printf("Linux system detected - swapping bytes in header ...\n");
   SwapPfileHeader(header);
#endif

  return(1);
}

int write_header(char *fname, char *header )
{
  if (!(fout=fopen(fname,"w"))) {
    printf("Can't open rawfile %s!\n",fname);
    return(0);
  }

  if (fwrite(header, sizeof(char), RDB_HEADER_SIZE_BYTES, fout) 
      != RDB_HEADER_SIZE_BYTES)
    printf("header too short!\n");

  return(1);
}

void instructions()
{
  printf("\nCreates an AFNI 3D dataset from a 3D+diffusion data set.\n"
	 "Images are assumed to be in root.slice.diff format\n"
	 "\n"
	 "Usage: dif2afni [options] inroot\n"
	 "\n"
	 "I/O Options:\n\n"
	 "\t-head      Pfile           write header from Pfile\n"
	 "\t-oname     fname [inroot]  output file name\n\n"
	 "For more help, type 'dif2afni -help'\n\n"
	 );

  return;
}

void send_help( )
{
  printf("\nExample: \n\n"
	 "If you have reconstructed images 'im.(slices).(diff)'\n"
	 "from pfile P12345.7, you can create an afni dataset\n"
	 "called 'brain+orig' with header information retrieved\n"
	 "from the pfile by typing:\n\n"
	 "dif2afni -head P12345.7 -oname brain im\n\n");
}
