/* utils.c */

#include "ioutils.h"

/*-----------------------------------------------------------------
   Sleep a given # of milliseconds (uses the Unix select routine)
------------------------------------------------------------------*/

void cox_sleep( int msec )
{
   struct timeval tv ;
   if( msec <= 0 ) return ;
   tv.tv_sec  = msec/1000 ;
   tv.tv_usec = (msec%1000)*1000 ;
   select( 1 , NULL,NULL,NULL , &tv ) ;
   return ;
}

/*===<file_exists.c>==========================================================

     Title: FILE EXISTS

   Purpose: Test if a file exists 

      Call: int file_exists(char *fname)
*/
int file_exists( char *fname )
{
  FILE *outfile;
        
  if(fname==NULL) return(0) ;

  outfile = fopen (fname,"r");
  if (outfile == NULL)
    return (0);
  else 
    fclose (outfile);
  return (1);
}

/*===<file_size.c>============================================================

     Title: FILE SIZE determination

   Purpose: Determine size of file of arbitrary type

      Call: int file_size ( char *fname, int ftype )
*/
int file_size( char *fname, int ftype )
{
  FILE *imfile ;
  int  length ;
  
  imfile = fopen( fname , "r" ) ;
  if( imfile == NULL ){
    fprintf( stderr , "couldn't open image file %s\n" , fname ) ;
    return (-1) ;
  }

  fseek( imfile , 0L , SEEK_END ) ;  /* get the length of the file */
  length = ftell( imfile ) ;
  fclose( imfile ) ; 

  switch( ftype ){
      default: 
      case MRI_double :  length /= sizeof(double); break ;
      case MRI_float  :  length /= sizeof(float); break ;
      case MRI_short  :  length /= sizeof(short); break ;
      case MRI_byte   :  break ;
   }

  return (length) ;
}

/*===<file_lines.c>============================================================

     Title: FILE LINES determination

   Purpose: Determine the number of lines in a file

      Call: int file_lines ( char *fname )
*/
int file_lines( char *fname )
{
  FILE *fin ;
  int  nlines=0 ;
  int  len = 512 ;
  char *cbuf,nc ;

  if( (fin = fopen( fname , "r" )) == NULL ){
    printf("Couldn't open header file %s\n" , fname ) ;
    return (-1) ;
  }

  /*cbuf = (byte *)malloc((unsigned) len * sizeof(byte));*/
  cbuf = (char *)malloc((unsigned) len * sizeof(char));
  while ( fgets(cbuf,len,fin) != NULL ) {
    nlines ++ ;
  }

  fclose( fin ) ;

  return ( nlines ) ;
}

/* ----------------------------------------------------------------
   Write file subroutine. A.Jesmanowicz, MCW 1989
	ier   :	return error :	1 - opening problem,
				2 - negative length.
	fname :	file name.
	size  :	size of the file.
	arr   :	file array.
   --------------------------------------------------------------*/
void write_iqm( long int *ier,
		char     fname[],
		long int *size,
		char     arr[] )
{
	int	isize = *size;
	int	fp;				/* file descriptor  */

	{	int i=0;			/* cut junk from the name */
		while(fname[i] != 32 && fname[i]) i++;
		fname[i] = 0;
	}

	if(isize < 0)				/* size has to be real */
	{	*ier=2;				/* or ier = 2.	       */
		return;
	}

	if(ALLOW_OVERWRITE) {	/* ALLOW_OVERWRITE is a global */
	  if ((fp = open(fname, O_WRONLY|O_CREAT|O_TRUNC, 0644)) <= 0)
	    {	*ier=1;				/* file can exist */
	    printf("write_iqm = %i, fp=%i\n",ALLOW_OVERWRITE,fp);
	    return;				/* but open failure give ier = 1. */
	    }
	} else {
	  if ((fp = open(fname, O_WRONLY|O_CREAT|O_EXCL,0444)) <= 0)
	    {	*ier=1;				/* file must not exist */
	    return;				/* or ier = 1.	   */
	    }
	}

	*ier=0;					/* no error : ier=0 */
	write(fp, arr, isize);			/* write whole file */
	close(fp);
	return;  
}

/* Report for write_iqm  */

long int write_iqm_error( long int *ier , char *fname )
{
  char *errstr;

  if(*ier==1) errstr = "FILE EXISTS";
  if(*ier==2) errstr = "FILE SIZE < 0";

  if (*ier!=0) {
    printf("ERROR WRITING FILE %s! %s\n",fname,errstr);
  }

  return(*ier);

}

long int write_file( long int *ier,
		     char     fname[],
		     long int *size,
		     char     arr[] )
{
  write_iqm(ier,fname,size,arr);
  return(write_iqm_error(ier,fname));
}
