#include "print_header.h"
#include "d2a.h"

char *PSDNAME(char *header) 
{
   MRIMAGEDATATYPE   *hr_im; 
   POOL_HEADER       *ph = (POOL_HEADER *) header;
   hr_im = (MRIMAGEDATATYPE *) &(ph->rdb_hdr_image);
   return((char *)hr_im->psdname) ;
}

int GETNDIF(char *header) 
{
  POOL_HEADER    *ph = (POOL_HEADER *) header;
  RDB_HEADER_REC *hr = (RDB_HEADER_REC  *) &(ph->rdb_hdr_rec);
  int num_dif ;

  if(strncmp(PSDNAME(header),"3dsfd",5) == 0) {	/* 03-Oct-20.  Make room for 3dsfd */
    num_dif = (int) hr->rdb_hdr_user23 ;
  } else {
    num_dif = (int) hr->rdb_hdr_user21 ;
  }

  return(num_dif) ;
}

int GETNSLICES(char *header) 
{
  POOL_HEADER    *ph = (POOL_HEADER *) header;
  RDB_HEADER_REC *hr = (RDB_HEADER_REC  *) &(ph->rdb_hdr_rec);
  int num_slc ;

  if(strncmp(PSDNAME(header),"3dsfd",5) == 0) {	/* 03-Oct-20.  Make room for 3dsfd */
    num_slc = (int) hr->rdb_hdr_user17 ;
  } else {
    num_slc = (int)hr->rdb_hdr_nslices ;
  }

  return(num_slc);
}

int print_header(char *fname)
{
  FILE *fin ;
  float B0;			/* field, in Guass */

  if (!(fin=fopen(fname,"r"))) {
    printf("Can't open rawfile %s!\n",fname);
    return(0);
  }

  /* read raw hdr */

  if (fread(header, sizeof(char), RDB_HEADER_SIZE_BYTES, fin) 
      != RDB_HEADER_SIZE_BYTES) {
    printf("header too short!\n\a");
    exit(0) ;
  }

  PrintPfileHeaderSummary(header,dimstr) ;
  printf("Image extent = %s\n\n",dimstr);

  memcpy(&raw, header+RDB_HDR_OFF, RDB_HDR_SIZE);

  /* Print */

  nslc      = raw.rdb_hdr_nslices;    /* number of slices */
  rhnecho   = raw.rdb_hdr_nechoes;    /* number of echoes */
  rhfrsize  = raw.rdb_hdr_frame_size; /* a/d samples in file */
  nsamp     = rhfrsize;
  necho     = rhnecho;		      /* number of echoes */
  doconcat  = (int) raw.rdb_hdr_user11;     /* do concat acq */
  if(doconcat)  nsamp = 2*nsamp;
  nviews    = (int) raw.rdb_hdr_user4;      /* views */
  rhnframes = raw.rdb_hdr_nframes;    /* total number of views in slice */
  nframes   = (int) raw.rdb_hdr_user1;      /* number of temporal frames */
  rawimsize = raw.rdb_hdr_im_size;    /* picture size */
  nbpi      = nviews*nsamp*4;	      /* num bytes per image */
  imrot     = raw.rdb_hdr_rotation;   /* image rotate */
  imtran    = raw.rdb_hdr_transpose;  /* image transpose */
  xoff      = raw.rdb_hdr_xoff;       /* x offset */
  yoff      = raw.rdb_hdr_yoff;	      /* y offset */
  ngap      = (int) raw.rdb_hdr_user10;     /* concat gap */
  dts       = raw.rdb_hdr_user13;     /* sampling interval, usec */
  bmapflag  = (int) raw.rdb_hdr_user14;     /* set if b0 map data collected */
  mapdt     = raw.rdb_hdr_user15;     /* b0 map delta te, us  */
  gfov      = raw.rdb_hdr_user0;      /* gradient fov	*/
  N         = (int) raw.rdb_hdr_user3;      /* gradient design fov */
  gtype     = raw.rdb_hdr_user5;      /* gradient trajectory type  */
  gamp      = raw.rdb_hdr_user6;      /* gradient max, G/cm	*/
  gslew     = raw.rdb_hdr_user7;      /* gradient slew rate, mT/m/ms	*/
  bline     = raw.rdb_hdr_baseline_views; /* number of baselines */
  frcvr     = 1000*raw.rdb_hdr_user12; /* receiver center freqency, Hz  */

  B0 = (.1*raw.rdb_hdr_ps_mps_freq+raw.rdb_hdr_user19)*2*M_PI/Gamma; /* Gauss  */

  if(strncmp(PSDNAME(header),"3dsfd",5) == 0) {	/* 03-Oct-20.  Make room for 3dsfd */

    /* Diffusion user variables ruser20-ruser29 */

    nb0    = 1;
    nbvals = 1 ;
    bstep  = 0 ;
    bslope = 1 ;
    dome   = 0;
    /*nvert = */
    optm = 0.0 ;

    difmode = (int) raw.rdb_hdr_user21;	/* Diffusion mode */
    ndif    = (int) raw.rdb_hdr_user22;	/* Number of diffusion directions */
    difnex  = (int) raw.rdb_hdr_user23;	/* Number of averages */
    ntess   = (int) raw.rdb_hdr_user24;	/* Diffusion dome factor */
    polytope= (int) raw.rdb_hdr_user25;	/* Tesselation polytope type */
    Delta   = (float)raw.rdb_hdr_user26; /* Time between diffusion gradient (us) */

    pwgd    = (float)raw.rdb_hdr_user27; /* Diffusion gradient width (us) */
    pwdramp = (float)raw.rdb_hdr_user28; /* Diffusion gradient ramp width (us) */
    agdx1   = (float)raw.rdb_hdr_user29; /* Amp of x diffusion gradient (G/cm) */
    agdy1   = (float)raw.rdb_hdr_user30; /* Amp of y diffusion gradient (G/cm) */
    agdz1   = (float)raw.rdb_hdr_user31; /* Amp of z diffusion gradient (G/cm) */

  } else {

    /* Diffusion user variables ruser20-ruser29 */

    difmode = (int) raw.rdb_hdr_user20;	/* Diffusion mode */
    ndif    = (int) raw.rdb_hdr_user21;	/* Number of diffusion directions */
    nb0     = (int) raw.rdb_hdr_user22;	/* Number of b=0 acquisitions */
    nbvals  = (int) raw.rdb_hdr_user23;	/* Number of b-values */
    bstep   = (int) raw.rdb_hdr_user24;	/* B-value stepping */
    bslope  = (int) raw.rdb_hdr_user25;	/* Increase of log-bstepping slope over 1 */
    ntess   = (int) raw.rdb_hdr_user26;	/* Diffusion dome factor */
    nvert   = (int) raw.rdb_hdr_user27;	/* Number of vertices in tesselated polytope */
    polytope= (int) raw.rdb_hdr_user28;	/* Tesselation polytope type */
    dome    = (int) raw.rdb_hdr_user29;	/* Dome = half sphere */

    pwgd    = (float)raw.rdb_hdr_user30; /* Diffusion gradient width (us) */
    pwdramp = (float)raw.rdb_hdr_user31; /* Diffusion gradient ramp width (us) */
    Delta   = (float)raw.rdb_hdr_user32; /* Time between diffu gradient centers (us) */
    agdx1   = (float)raw.rdb_hdr_user33; /* Amp of x diffusion gradient (G/cm) */
    agdy1   = (float)raw.rdb_hdr_user34; /* Amp of y diffusion gradient (G/cm) */
    agdz1   = (float)raw.rdb_hdr_user35; /* Amp of z diffusion gradient (G/cm) */
    
    optm    = (float)raw.rdb_hdr_user36; /* Mixing time (ms) */

  }

  printf("Diffusion mode = %i\n"
	 "Number of diffusion directions = %i\n" 
	 "Number of b-values = %i\n"
	 "Tesselation level = %i\n"
	 "B-value stepping mode = %i\n"
	 "Diffusion gradient width = %lf (ms)\n" 
	 "Diffusion gradient ramp width = %lf (us)\n" 
	 "Time between diffusion gradients (us) = %lf\n" 
	 "(x,y,z) Gradient amplitudes (%lf,%lf,%lf) (G/cm)\n\n", 
	 difmode,ndif,nbvals,ntess,bstep,pwgd/1000,pwdramp,Delta,agdx1,agdy1,agdz1);

  return(1);
}
