/**<DIFF_aniso_meth.c>****************************************************

     Title: DIFFusion ANISOtropy METHods

   Purpose: Routine to do actual work in the computation
            of the diffusion.

      Call: static void DIFF_aniso_meth( int method, 
					 float fs[] , complex cs[],
					 void *udp  , int nbrick, 
					 float *val , complex *cval);

   History: I used RWC's STATS_tsfunc.c in 3dTstat.c as a template
            1-Sept-02 Masking now done above in MAKER.  Get rid of mask here.

**************************************************************************/
#include "diffuse.h"

void DIFF_aniso_meth( int     method , 
		      float   fs[] , 
		      complex zs[] , 
		      void    *udp , 
		      int     nbrick , /* NOT USED! */
		      float   *val ,
		      complex *cval )
{
   char *program_name = "DIFF_aniso_meth" ;
   static int ncall ;
   struct DIF_state uda, *dif ;

   int debug = 0;

   dif = &uda;
   dif = (struct DIF_state *) udp ;

   /** is this a "notification"? **/

   if( fs == NULL && zs == NULL ){

      if( dif->img->ndif > 0 ){  /* the "start notification" */

         ncall = 0 ;		/* keep track of number of calls */

      } else {  /* the "end notification" */

	/* nothing to do here */

      }
      return ;
   }

   /** OK, actually do some work **/

   if(debug) printf("DIFF_aniso_meth: beginning ...\n");

   /* here fs is just the measurements on a sphere */
   /* b0 image assumed to be in fs[0] */

   switch( method ){

      case METH_DTI_AVE:{		/* Average diffusion coeff */

	*val = aveD( fs , dif ); 

      }
      break ;

      case METH_DTI_FAI:{		/* Fractional anisotropy index */

	// *val = dti_fai( fs , dif ); 
	*val = calc_fai( fs, 3 ); 

      }
      break ;

      case METH_DTI_RAI:{		/* Relative anisotropy index */

	*val = dti_rai( fs , dif ); 

      }
      break ;

      case METH_DTI_LAT:{		/* Lattice anisotropy index */

	*val = lat( fs , dif ); 

      }
      break ;

      case METH_DTI_SDV:{		/* Spherical diffusion variance */

	*val = sdv( fs , dif );

      }
      break ;

      case METH_DTI_DTI:{		/* Full diffusion tensor calculation */
#define RFA_SCALE sqrt(3./2.)

	switch (dif->img->difmode) {

	case DIFF_SAMP_NONE:			/* no diffusion */

	  break;

	case DIFF_SAMP_XYZ0:			/* manual method */

	  break;

	case DIFF_SAMP_GE:              /* GE's diffusion directions */
	case DIFF_SAMP_TESS:		/* tesselated polytope method */

	  diffusion_tensor( fs , dif );
	  tensor_eig( fs, val, dif->dti->eig ); 

	  break;

	default:
	  printf("DIFF_aniso_mode: Invalid diffusion mode %i selected.\n",
		dif->img->difmode);
	  exit(0);
	}

      }

      break ;

      case METH_SHD_SHD:{		/* Spherical Harmonic Decomposition */

	DTI_prc_state   *prc = dif->dti->prc ;
	SHD_calc_state *calc = dif->shd->calc ;

	double meanb0 ;

	/* Check to make sure shd initialized.  If not, do so. */

	if( !dif->sta->shd_initialized ) init_shd( dif );

	if(debug) printf("%s:  METH_SHD:  nl = %i, nm = %i\n",
			 __FILE__,calc->nl,calc->nm);

	/* Compute dap = -log(d/d(b=0)), and put into calc->fv */
	meanb0 = dat2dap( calc->fv, fs, calc->nvert, calc->nb0 ) ;

	/* No, leave data the same, just shift to get rid of b=0 */
	/* meanb0 = dat2dat( calc->fv, fs, calc->nvert, calc->nb0 ) ; */

	SpharmTransform( calc->Alm, calc->fv, calc->Lmin, calc->Lmax, 
			 calc->nvert, calc->Ylm, calc->wt ) ; 

	int ncoeff = 0 ;
	for( int ll=calc->Lmin ; ll<=calc->Lmax ; ll++ ) {
	  for( int mm=-ll ; mm<=ll ; mm++ ) {
	    ZCPY(cval[ncoeff],calc->Alm[ll][mm]) ;
	    ncoeff++ ;
	  }
	}

	if(debug) PRINT_ZV(cval,calc->nlm,"cval");

      }
      break ;

      case METH_SWD_SWD:{		/* Spherical Wave Decomposition. 16-Jul-04 */

	DTI_prc_state   *prc = dif->dti->prc ;
	SHD_calc_state *calc = dif->shd->calc ;
	IMG_state       *img = dif->img ;
	Grad          *dgrad = img->dgrad ;
	double meanb0 ;

	/* Check to make sure shd initialized.  If not, do so. */

	if( !dif->sta->shd_initialized ) init_shd( dif );

	if(debug) printf("%s:  METH_SHD_SWD:  nl = %i, nm = %i\n",
			 __FILE__,calc->nl,calc->nm);

	/* Compute dap = -log(d/d(b=0)), and put into calc->fv */

	meanb0 = dat2dap( calc->fv, fs, calc->nvert, calc->nb0 ) ;

	for(int ibv=0; ibv<img->nbvals; ibv++) {
	  for(int iv=0; iv<img->ndif; iv++) {
	    calc->fm[ibv][iv] = fs[iv + ibv*img->ndif];
	  }
	}

	SphWaveTransform( calc->Almr, calc->fm, calc->Lmin, calc->Lmax, 
			  calc->nvert, calc->jrad, calc->nrad, calc->Wlmr, calc->wt ) ; 

	int ncoeff = 0 ;
	for( int rr=0 ; rr<calc->nrad ; rr++ ) {
	  for( int ll=calc->Lmin ; ll<=calc->Lmax ; ll++ ) {
	    for( int mm=-ll ; mm<=ll ; mm++ ) {
	      ZCPY(cval[ncoeff],calc->Almr[rr][ll][mm]) ;
	      ncoeff++ ;
	    }
	  }
	}

	if(debug) PRINT_ZV(cval,calc->nlm,"cval");

      }
      break ;

      case METH_SHD_FIB:{	/* SHD fiber channels */

	DTI_prc_state   *prc = dif->dti->prc ;
	SHD_calc_state *calc = dif->shd->calc ;

	/* Check to make sure shd initialized. If not, do so. */

	if( !dif->sta->shd_initialized ) init_shd( dif );

	if(debug) PRINT_ZV(zs,calc->nlm,"DIFF_aniso_meth: zs");

	/* Move input complex input into dif->shd->calc->Alm[L][M] */

	zvtoAlm( calc->Alm, zs, calc->Lmax );

	/* Compute energy */

	alm2egy( calc->egyL, calc->Alm, calc->Lmax );  
	
	/* Copy to output array val */

	vmove( val, calc->egyL, calc->nl );

      }

      break ;

      case METH_SHD_EOS:{	/* SHD even/odd */

	DTI_prc_state   *prc = dif->dti->prc ;
	SHD_calc_state *calc = dif->shd->calc ;

	int    k ;
	float  esum=0.0,osum=0.0 ;

	/* Check to make sure shd initialized.  If not, do so. */

	if( !dif->sta->shd_initialized ) init_shd( dif );

	if(debug) PRINT_ZV(zs,calc->nlm,"DIFF_aniso_meth: zs");

	/* Move input complex input into dif->shd->calc->Alm[L][M] */

	zvtoAlm( calc->Alm, zs, calc->Lmax );

	/* Compute energy */

	alm2egy( calc->egyL, calc->Alm, calc->Lmax );

	esum = osum = 0.0;
	for( k=1; k<calc->nl ; k++ ) { /* start at L=1 */
	  if(EVEN(k)) esum += calc->egyL[k];
	  if( ODD(k)) osum += calc->egyL[k];
	}
	val[0] = esum;
	val[1] = osum;
	  
	if(debug) printf("DIFF_aniso_meth: METH_SHD_EOS val[0] = %f,val[1] = %f\n",
			 val[0],val[1]);

      }

      break ;

      /* TEMPORARILY CHANGED TO */

      case METH_SHD_FMI:{	/* SHD fractional multifiber index */

	DTI_prc_state   *prc = dif->dti->prc ;
	SHD_calc_state *calc = dif->shd->calc ;

	int    k ;
	float  esum=0.0,osum=0.0;
	float  isum,sum2,sum4;
	float  val2,val4,vale;

	/* Check to make sure shd initialized. If not, do so. */

	if( !dif->sta->shd_initialized ) init_shd( dif );

	if(debug) PRINT_ZV(zs,calc->nlm,"DIFF_aniso_meth: zs");

	/* Move input complex input into dif->shd->calc->Alm[L][M] */

	zvtoAlm( calc->Alm, zs, calc->Lmax );

	/* Compute energy */

	alm2egy( calc->egyL, calc->Alm, calc->Lmax );  

	/* Loop over L */

	for( k=0; k<calc->nl ; k++ ) {
	  if(k==0)         isum  = calc->egyL[k];
	  if(k==2)         sum2  = calc->egyL[k];
	  if(k==4)         sum4  = calc->egyL[k];
	  if(EVEN(k)&&k>0) esum += calc->egyL[k];
	  if( ODD(k))      osum += calc->egyL[k];
	}

	vale = sqrt(esum/isum) ;
	val2 = sqrt(sum2/isum) ;
	val4 = sqrt(sum4/isum) ;
	val[0] = val2+val4;

	if(debug) printf("DIFF_aniso_meth: METH_SHD_FMI val[0] = %f\n",val[0]);

      }

      break ;

      case METH_SHD_GRA:{	/* Generalized Relative Anisotropy index (SHD way) */

	SHD_calc_state *calc = dif->shd->calc ;

	/* Check to make sure shd initialized.  If not, do so. */

	if( !dif->sta->shd_initialized ) init_shd( dif );

	/* Move complex input into dif->shd->calc->Alm[L][M] */

	zvtoAlm( calc->Alm, zs, calc->Lmax );
	if(debug) {
	  printf("DIFF_aniso_meth (GRA): Alm:\n");
	  printAlm(calc->Alm,calc->Lmax);
	}

	/* GRA */

	val[0] = alm2gra( calc->Alm, calc->Lmax ) ;
	if(debug) printf("DIFF_aniso_meth: METH_SHD_GRA val[0] = %f\n",val[0]);

      }

      break ;

      case METH_SHD_GFA:{	/* Generalized Fractional Anisotropy index (SHD way) */

	SHD_calc_state *calc = dif->shd->calc ;
	int debug_gfa = 0;

	/* Check to make sure shd initialized.  If not, do so. */

	if( !dif->sta->shd_initialized ) init_shd( dif );

	/* Move complex input into dif->shd->calc->Alm[L][M] */

	zvtoAlm( calc->Alm, zs, calc->Lmax );
	if(debug_gfa) {
	  printf("DIFF_aniso_meth (GFA): Alm:\n");
	  printAlm(calc->Alm,calc->Lmax);
	}

	/* GFA */

	val[0] = alm2gfa( calc->Alm, calc->Lmax ) ;
	if(debug_gfa) printf("DIFF_aniso_meth: METH_SHD_GFA val[0] = %f\n",val[0]);

      }

      break ;

      case METH_SHD_GFAZ:{	/* Generalized Fractional Anisotropy index (SHD way) */

	SHD_calc_state *calc = dif->shd->calc ;
	int debug_gfa = 0;

	/* Check to make sure shd initialized.  If not, do so. */

	if( !dif->sta->shd_initialized ) init_shd( dif );

	/* Move complex input into dif->shd->calc->Alm[L][M] */

	zvtoAlm( calc->Alm, zs, calc->Lmax );
	if(debug_gfa) {
	  printf("DIFF_aniso_meth (GFA): Alm:\n");
	  printAlm(calc->Alm,calc->Lmax);
	}

	/* GFA */

	cval[0].r = alm2gfa( calc->Alm, calc->Lmax ) ;
	cval[0].i = 0.0;
	if(debug_gfa) printf("DIFF_aniso_meth: METH_SHD_GFA val[0] = %f\n",val[0]);

      }

      break ;

      case METH_SHD_GFAOZ:{	/* Generalized Fractional Anisotropy index (SHD way) */

	SHD_calc_state *calc = dif->shd->calc ;
	int debug_gfa = 0;

	/* Check to make sure shd initialized.  If not, do so. */

	if( !dif->sta->shd_initialized ) init_shd( dif );

	/* Move complex input into dif->shd->calc->Alm[L][M] */

	zvtoAlm( calc->Alm, zs, calc->Lmax );
	if(debug_gfa) {
	  printf("DIFF_aniso_meth (GFA): Alm:\n");
	  printAlm(calc->Alm,calc->Lmax);
	}

	/* GFA */

	cval[0].r = alm2gfao( calc->Alm, calc->Lmax ) ;
	cval[0].i = 0.0;
	if(debug_gfa) printf("DIFF_aniso_meth: METH_SHD_GFA val[0] = %f\n",val[0]);

      }

      break ;

      default:
        printf("DIFF_aniso_meth: Invalid method %i selected.\a\n",method);
	return ;

   }


   ncall++ ; return ;
}
