#ifndef DIF_STRUCTS_H
#define DIF_STRUCTS_H 
/* Diffusion Structures */

/* DTI STRUCTURES */

/*#include "afni.h"*/

#include <X11/Intrinsic.h>	/* for Boolean */

typedef struct DTI_prc_state {

  int   nimsl ;			/* a useful variable */
  int   tdim ;			/* diffusion tensor dimension */
  int   nten ;			/* number of diffusion tensor elements
				   (=tdim^2) */
  int   edim ;			/* eigenvalue vector dimension */

  float  **Dapp;		/* Apparent diffusion coefficient */
  float  **Bmat;		/* B-value matrix */
  float  **Evals;		/* Eigenvalues */

  float maskval ;		/* value below which processing is ignored */

  byte  *mask ;			/* byte mask 1Sept2002 */

  float   *fv ;			/* float work vector */
  double  *dv ;			/* double work vector */

  float  **D33;			/* 3x3 diffusion tensor 22-Oct-03 */

} DTI_prc_state ;

/* This is just for individual ev computations, not whole images */

typedef struct DTI_eig_state {	

  double *evals;		/* eigenvalues */
  double **evecs;		/* eigenvectors */
  double **dm;			/* double matrix workspace */
  double *dv;			/* double vector workspace */
  float  *fv;			/* float  vector workspace */
  int    nten;			/* # rows */
  int    nevals;		/* # eigenvalues (same as nten) */
  int    nr;			/* # rows */
  int    nc;			/* # columns */

} DTI_eig_state ;

/* This is just for individual lsq computations, not whole images */

typedef struct DTI_lsq_state {

  int   lsqdim ;		/* dimension of least-sq problem */

  int    nt; 
  double dt; 
  double dc; 
  float  *signal;
  float  *noise;
  float  *vec;
  float  *alpha;		/* fit coeffs */
  float  **tsim;
  int    nref; 
  int    nrts; 
  int    polort; 
  int    nrsin; 
  float  *sinper ;
  int    *sinharm ;
  int    ignore;
  char   *label ;

} DTI_lsq_state ;

typedef struct DTI_state {

  DTI_prc_state *prc ;	/* processing */
  DTI_eig_state *eig ;	/* eigensystem */
  DTI_lsq_state *lsq ;	/* least-squares */

} DTI_state ;

/* SHD STRUCTURES */

typedef struct SHD_calc_state { /* Calculations */

  short Lmax ;			/* maximum L to calculate */
  short Mmax ;			/* maximum M to calculate */
  short Lmin ;			/* minimum L to calculate */
  short Mmin ;			/* minimum M to calculate */
  short nl ;			/* number of L to calc */
  short nm ;			/* number of M to calc */
  short nlm ;			/* nl*nm - useful param */
  short neos ;			/* number of even/odd sum channels to calc (2) */

  short *Lvals ;		/* L values to calculate */
  short *Mvals ;		/* M values to calculate */

  short shd_grid_size ;		/* Grid size for SHT computation */

  Boolean full_mcalc_range ;	/* Flag to calculate -M:M */

  float    min_polr, max_polr;	/* min and max polar angles */
  float    min_azim, max_azim;	/* min and max azimu angles */
  int      npolr,nazim;		/* number of polar and azimu angles */

  int      npts ;		/* number of measured points on sphere */
  int      nvert ;		/* number of vertices measured on sphere */
  int      nb0 ;		/* number of b=0 acquisitions */
  float    *polr ;		/* polar angles */
  float    *azim ;		/* azimuthal angles */
  float    *sphdat ;		/* sphered data */
  float    *wt ;		/* voronoi weights */
  complex  ***Ylm;		/* spherical harmonics 
				   Ylm(l,m,pt[i]) 
				   where pt[i] = (theta[i],phi[i]) */
  float    **Ylm_scale;		/* Scaling factor for SHT */
  complex  **Alm;		/* spherical harmonic coefficients 
				   Alm(theta,phi)*/
  complex  **Olm;		/* spherical harmonic coefficients 
				   Alm(theta,phi)*/
  complex  *sumL ;		/* sum of Alm over M for each L (sumL[0,..,Lmax]) */
  float    *egyL ;		/* energy in L channels of Alm  (egyL[0,..,Lmax]) */
  float    **Dap;		/* Apparent diffusion tensor */

  float    *fwrk ;		/* float work array fwrk[nlm] */
  complex  *zwrk ;		/* complex work array fwrk[nlm] */
  float    *fv ;		/* float work vector  fv[nbvals*ndif] */
  float   **fm ;		/* float work matrix fv[nbvals][ndif] */
  complex  *zv ;		/* complex work vector cv[nbvals*ndif] */
  double   *dv ;		/* double work vector dv[nbvals*ndif] */

  /* Old measure stuff replaced by Voronoi...axe? */
  /* float    dpolr,dazim; */	/* polar and azimu increments */
  /* float    *dthdph; */	/* integration measure */

  /* SWD declarations. 16Jul04 (lrf) */

  complex ****Wlmr ;		/* Spherical waves
				   Wlmr(r,l,m,pt[i]) 
				   where pt[i] = (theta[i],phi[i]) */
  complex ***Almr ;		/* Spherical wave coefficients Almr(r,theta,phi) */
  float   *jrad ;	        /* radii for spherical bessel arg */

  float Rmin ;			/* min R to reconstruct */
  float Rmax ;			/* max R to reconstruct */
  int   nrad;			/* #radii for SWT */

} SHD_calc_state ;

typedef struct SHD_view_state {  /* Viewing */

  short Lmax ;			/* maximum L to view */
  short Mmax ;			/* maximum M to view */
  short Lmin ;			/* minimum L to view */
  short Mmin ;			/* minimum M to view */
  short nl ;			/* number of L to view */
  short nm ;			/* number of M to view */
  short nlm ;			/* nl*nm - useful param */

  short *Lvals ;		/* L values to view */
  short *Mvals ;		/* M values to view */

  Boolean full_mview_range ;	/* View -M:M */

} SHD_view_state ;

typedef struct SHD_state {

  SHD_view_state    *view ;	/* viewing */
  SHD_calc_state    *calc ;	/* processing */

} SHD_state ;

/* Large Diffusion Structures */

typedef struct DIF_status {

  int method ;			/* processing method */

  /* Tensor operations */

  int shd_performed ;		/* flag to indicate if shd has been done */
  int dti_performed ;		/* flag to indicate if dti has been done */
  int ftm_performed ;		/* flag to indicate if ftm has been done */

  int shd_initialized ;		/* flag to indicate if shd has been initialized */
  int dti_initialized ;		/* flag to indicate if dti has been initialized */
  int ftm_initialized ;		/* flag to indicate if ftm has been initialized */

  /* DTI scalar operations */

  int dti_ave_performed ;		/* flag to indicate if ave has been done */
  int dti_fai_performed ;		/* flag to indicate if fai has been done */
  int dti_rai_performed ;		/* flag to indicate if rai has been done */
  int dti_lat_performed ;		/* flag to indicate if lat has been done */
  int dti_sdv_performed ;		/* flag to indicate if sdv has been done */
  
  int dti_ave_initialized ;		/* flag to indicate if ave has been initialized */
  int dti_fai_initialized ;		/* flag to indicate if fai has been initialized */
  int dti_rai_initialized ;		/* flag to indicate if rai has been initialized */
  int dti_lat_initialized ;		/* flag to indicate if lat has been initialized */
  int dti_sdv_initialized ;		/* flag to indicate if sdv has been initialized */

  /* SHD scalar operations */

  int shd_shd_performed ;		/* flag to indicate if shd has been done */
  int shd_ftm_performed ;		/* flag to indicate if fib has been done */
  int shd_eos_performed ;		/* flag to indicate if eos has been done */
  int shd_fmi_performed ;		/* flag to indicate if fmi has been done */
  int shd_gra_performed ;		/* flag to indicate if gra has been done */
  int shd_gfa_performed ;		/* flag to indicate if gfa has been done */
  
  int shd_shd_initialized ;		/* flag to indicate if shd has been initialized */
  int shd_ftm_initialized ;		/* flag to indicate if fib has been initialized */
  int shd_eos_initialized ;		/* flag to indicate if eos has been initialized */
  int shd_fmi_initialized ;		/* flag to indicate if fmi has been initialized */
  int shd_gra_initialized ;		/* flag to indicate if gra has been initialized */
  int shd_gfa_initialized ;		/* flag to indicate if gfs has been initialized */

  int   dothresh ;		/* logical threshold */

  int overlay_chan ;		/* Diffusion overlay channel : 
				   dti = 0, shd = 1, ftm = 2 */
  int compute_chan ;		/* Diffusion computation channel : 
				   dti = 0, shd = 1, ftm = 2 */

} DIF_status ;

typedef struct IMG_state {

  char *psd_name ;		/* PSD internal name (psd_iname in rdbm.h) */

  int   nx, ny, nxy ;		/* x and y image size, and nxy = nx*ny */
  int   nsl ;			/* number of image slices */
  int   nim ;			/* total number of images = nsl*ndif*nbvals */

  int   nl ;			/* number of interleaves (needed since 
				   diagonal grads scaled by sqrt(2) */
  int   difmode ;		/* diffusion mode */
  int   nbvals ;		/* number of b-values */
  int   ndif ;			/* number of diffusion directions */
  int   nb0 ;			/* number of b=0 acquisitions */
  //int   ntess ;		/* Tesselation level  --- moved to Grad struct */
  //int   nvert ;		/* Number of vertices --- moved to Grad struct */

  int   ecco ;			/* SE eddy current compensation switch. 04-Feb-22 */
  int   bstep ;			/* B-value stepping */
  int   bslope ;		/* Increase of log-bstepping slope over */
  // int   polytope ;		/* Tesselation polytope type --- moved to Grad struct */
  float pwgd ;			/* Diffusion gradient width */
  float pwgd2 ;			/* Hyper diffusion gradient width */
  float pwdramp ;		/* Diffusion gradient ramp width */
  float Delta ;			/* Time between diffusion gradient centers */
  float Delta2 ;		/* Time between hyper diffusion gradient centers */

  float agdx1  ;		/* Amp of x-diff grad (G/mm) */
  float agdy1  ;		/* Amp of y-diff grad (G/mm) */
  float agdz1  ;		/* Amp of z-diff grad (G/mm) */

  float gdx1_amp  ;
  float gdx1_pw  ;
  float gdx2_amp  ;
  float gdx2_pw  ;

  float gdy1_amp  ;
  float gdy1_pw  ;
  float gdy2_amp  ;
  float gdy2_pw  ;

  float gdz1_amp  ;
  float gdz1_pw  ;
  float gdz2_amp  ;
  float gdz2_pw  ;

  float optm  ;			/* Mixing time (ms) */

  float *gdbstep ;              /* b-value(s), normalized to 1 */

  float *gdxpat ;               /* x diffusion grad pattern */
  float *gdypat ;               /* y diffusion grad pattern */
  float *gdzpat ;               /* z diffusion grad pattern */

  /* possibly redundant with gdipat above */

  float *GX ;			/* Actual x grad direction read from .GX file */
  float *GY ;			/* Actual y grad direction read from .GY file */
  float *GZ ;			/* Actual z grad direction read from .GZ file */

				/* 00_05_02 - new high ang res stuff*/
  int   drecon ;		/* diff recon mode: 1 = standard tensor
				                    2 = high angular res */

  float bsclx ;	                /* 21-Oct-03 - these are actually the bvalues */
  float bscly ;	                /* on each axis (should be the same!), and */
  float bsclz ;		        /* are calculated in setBscale (dti.c) and 
				   used explicitly in basser (dti.c) */

  float bval  ;			/* 04-Nov-03.  3dsfd now takes bval as input,
				   so we can output it directly. */

  int bval_available ;		/* 13-Apr-04.  Whether there is a known b-value
				   or it needs to be calculated. */
  int bpat_available ;		/* 13-Apr-04.  Whether there is a known b-pattern
				   or it needs to be calculated. 
				   This is currently always true for SPHERE 
				   sampling mode. */

  float *bvals  ;		/* 08-Jul-04.  Vector of bvalues. */

  Grad *dgrad ;			/* 12-Jul-04.  Gradient structure */

} IMG_state ;

typedef struct DIF_state {

  DIF_status        *sta ;	/* status of diff procedures */
  IMG_state         *img ;	/* imaging */
  DTI_state         *dti ;	/* diffusion tensor */
  SHD_state         *shd ;	/* spherical harmonic */
  FTM_state         *ftm ;	/* fiber tract mapping */

} DIF_state ;



typedef struct DSET_stats {

  int   count ;			/* # elements contributing to stats  */

  float min ;			/* min intensity                     */
  float max ;			/* max intensity                     */
  float mean ;			/* mean intensity                    */
  float sigma ;			/* standard deviation                */

  float *minar ;		/* min[sub-brick]                    */
  float *maxar ;		/* max[sub-brick]                    */
  float *meanar ;		/* mean[sub-brick]                   */
  float *sigmar ;		/* sigma[sub-brick]                  */

  float thresh ;		/* threshold                         */

  byte *mask ;			/* mask brick mask[nvox]             */

} DSET_stats ;

#endif
