
/**<alm2fai.c>*******************************************************

     Title: Alm TO mean squared Fractional Anisotropy

   Purpose: Compute mean squared fractional anisotropy from Alm

      Call: float alm2fai( complex **Alm, short Lmax )

     Input: complex **Alm[][] - SHT coefficients

    Output: returns float energy in L channels of A[l][m] if successful
            or NULLVAL if something went wrong

   History: returns float energy in L channels of A[l][m] if successful

***********************************************************************/

float alm2fai( complex **Alm, short Lmax )
{
  int debug = 1;

  float A00 = Alm[0][0].r ;	/* is real anyway */
  float A20 = Alm[2][0].r ;	/* is real anyway */
  float A22 = Alm[2][2].r ;	/* coeff is really 2*this but we've taken
				   that into account in scal22 */

  float scal00 = 1./sqrt( 1./( 4.*M_PI)) ;
  float scal20 =    sqrt( 5./(16.*M_PI)) ;
  float scal22 =    sqrt(15./( 4.*M_PI)) ;

  float sA00 = scal00 * A00 ;
  float sA20 = scal20 * A20 ;
  float sA22 = scal22 * A22 ;

  float Dx = sA00 - sA20 + sA22 ;
  float Dy = sA00 - sA20 - sA22 ;
  float Dz = sA00 + sA20 * 2.0 ;

  float Dxyz[3] ;

  float fvar, fmsq, rat, faival ;
#define fai_scal sqrt(3./2.) 

  Dxyz[0] = Dx ;
  Dxyz[1] = Dy ;
  Dxyz[2] = Dz ;

  fvar = vvar( Dxyz , 3 ) ;
  fmsq = vmsq( Dxyz , 3 ) ;
  faival = fai_scal * sqrt( (fvar/fmsq) ) ;

  if(debug) {
    BLANK_LINE ;
    PRINT_FV(Dxyz,3,"alm2fai: Dxyz");
    printf("alm2fai: faival = %f\n",faival);
  }

  return faival ;
}
