#ifndef _MRI_DIFFUSE_HEADER_
#define _MRI_DIFFUSE_HEADER_

/*****************************************************************************
  This software is copyrighted and owned by the Medical College of Wisconsin.
  See the file README.Copyright for details.
******************************************************************************/


#ifdef LINUX2
#include "/apps/afni/AFNI98/mrilib.h"
#include "/apps/afni/AFNI98/volpack.h"
#else
#include "/disk02/apps/afni/AFNI98/mrilib.h"
#include "/disk02/apps/afni/AFNI98/volpack.h"
#endif

/*============================================================================
  MDIF = MRI Diffuser, a set of routines for volume rendering 3D bricks.
  Built on top of VolPack, by Philippe Lacroute.
==============================================================================*/

/*--- prototypes ---*/

extern void * new_MDIF_diffuser(void) ;
extern void   destroy_MDIF_diffuser( void * ) ;

extern int  MDIF_set_graybytes( void * , MRI_IMAGE * ) ;
extern int  MDIF_set_opabytes ( void * , MRI_IMAGE * ) ;
extern int  MDIF_set_rgbbytes ( void * , MRI_IMAGE * ) ;
extern int  MDIF_set_rgbshorts( void * , MRI_IMAGE * ) ;
extern void MDIF_set_viewpoint( void * , float , float , float ) ;
extern void MDIF_set_size     ( void * , float , float , float ) ;

extern void MDIF_set_precalculation( void * ah , int ) ;

extern MRI_IMAGE * MDIF_diffuse( void * , int ) ;

extern int MDIF_needs_data( void * ah ) ;

extern void   MDIF_be_verbose( void * ) ;
extern void   MDIF_be_quiet  ( void * ) ;

extern void MDIF_set_rgbmap( void * , int , byte * , byte * , byte * ) ;
extern void MDIF_unset_rgbmap( void * ) ;

extern MRI_IMAGE * MDIF_rgb_to_cmap( MRI_IMAGE * ) ;

extern void MDIF_set_min_opacity( void * , float ) ;

/*--- typedefs, etc. ---*/

typedef struct {
   unsigned short rgb ;     /* packed into 15 bits */
   unsigned short alpha ;   /* opacity */
} rgbvox ;

#define EIGHT_TO_FIVE(z) ((z) >> 3)        /* maps 0..255 into 0..31  */
#define FIVE_TO_EIGHT(z) ((z) << 3)        /* maps 0..31  into 0..255 */

#define FIVE_TO_SHORT(a,b,c) ((a)<<10 | (b)<<5 | (c))
#define EIGHT_TO_SHORT(a)    (MDIF_MAX_COLORS + (a))

#define TFSINV 0.00390625
#define TTINV  0.03125
#define TFFINV (1.0/255.0)
#define TOINV  (1.0/31.0)

#define SHORT_to_rrr(s) TT_TO_ZO( ((s) & 0x7c00) >> 10 )
#define SHORT_to_ggg(s) TT_TO_ZO( ((s) & 0x03e0) >>  5 )
#define SHORT_to_bbb(s) TT_TO_ZO( ((s) & 0x001f)       )

#define SHORT_isgray(s) ( ( ((s) & 0x7c00) >> 10 == ((s) & 0x03e0) >> 5 ) &&  \
                          ( ((s) & 0x03e0) >>  5 == ((s) & 0x001f)      )    )

#define MDIF_MAX_CDIM   32
#define MDIF_MAX_GRAYS  256
#define MDIF_MAX_COLORS 32768                   /* = 32 * 32 * 32 */
#define TOT_COLORS (MDIF_MAX_COLORS+MDIF_MAX_GRAYS)

extern void init_MDIF_colortable(void) ;

/*------ lrf's diffusion stuff ---------------------*/

#define DMODE_RAI 0		/* Relative Anisotropy Index */
#define DMODE_VAR 1		/* Variance */
#define DMODE_SHD 2		/* Spherical Harmonic Transform */

/*---------------------------*/

#define MDIF_TYPE 9707312

#define PMODE_LOW    0
#define PMODE_MEDIUM 1
#define PMODE_HIGH   2

typedef struct {
   int type ;

   vpContext * vpc ;

   int nx,ny,nz , verbose , newopac,newvox , pmode ;
   float theta,phi,psi , sx,sy,sz ;

   int grayset , rgbset , opaset ;

   MRI_IMAGE * opim , * shim ;  /* opacity and shading */

   rgbvox * vox ;   /* encoded color + opacity */

   int     ncmap , newcmap ;
   float * cmap ;
   float   min_opacity ;
} MDIF_stuff ;

#define ISVALID_MDIF(ah) ( (ah) != NULL && (ah)->type == MDIF_TYPE )

#endif /* _MRI_DIFFUSE_HEADER_ */
