/**<sdv.c>*******************************************************

     Title: Spherical Diffusion Variance routines

   Purpose: Setup, allocate, compute spherical diffusion variance

    Author: Lawrence R. Frank

   History: 15-May-01 cp dti.c -> sdv.c

*/
#include "diffuse.h"

/*********************************************************************************

     Title: INITialize SDV

      Call: int init_sdv( DIF_state *dif )

   Purpose: Allocate space for sdv.c

     Input: DIF_state *dif

    Output: prc->fv(ndif) 

******************************************************************************/

int init_sdv( DIF_state *dif )
{
  DIF_status    *sta = dif->sta ;
  IMG_state     *img = dif->img ;
  DTI_prc_state *prc = dif->dti->dps ;

  prc->dv = dv_alloc( img->nbvals * img->ndif ) ; /* work space */

  if (prc->dv==NULL ) {
    sta->sdv_initialized = 0;
    return(0) ;
  } else {
    sta->sdv_initialized = 1;
    return(1) ;
  }
}

/******************************************************************************

     Title: Spherical Diffusion Variance

   Purpose: Compute spherical diffusion variance

      Call:

     Input: idata = magnitude image data from voxel with b=bval
	     bval = b-value

    Output: sdv = variance of apparent diffusion coefficient

    Algorithm: Frank, MRM (2001)

*********************************************************************************/

float sdv( float *ds , DIF_state *dif )
{
  int i ;
  float val ;
  IMG_state *img = dif->img ;
  DIF_status *sta = dif->sta ;
  DTI_prc_state *prc = dif->dti->dps ;

  if(ds[0]>sta->maskval) { /* mask all on a single threshold */

    for( i=0 ; i < img->ndif-1 ; i++ ) 
      prc->dv[i] = -log((double) (ds[i+1]/ds[0]));

    val = (float) dvvar( prc->dv, img->ndif ) ;

  } else {

    val = 0.;

  }

  return(val);

}
