/**<tstsph>**************************************************

  Title: TeST SPHarm routines

*/
#include "diffuse.h"

#define TSTFAC 0		/* test factorial */
#define TSTLEG 0		/* test Legendre polynomials */
#define TSTSPH 0		/* test Spherical Harmonics */
#define TSTSHT 0		/* test Spherical Harmonic Transform */
#define TSTCRD 1		/* test coordinates */

main()
{

  if(TSTCRD) {			/* Test coordinates*/

    int n;
    float x,y,z;
    cpoint cpt;
    spoint spt;

    printf("\nTesting coordinates\n");
    x = 2.3;
    y = 1.4;
    z = 4.5;

    printf("Calling newcpt with (x,y,z) = (%lf,%lf,%lf)\n",x,y,z);

    cpt = newcpt(x,y,z);
    printcpt(cpt," ");
    printf("Calling cpt2spt\n");
    spt = cpt2spt(cpt);
    printspt(spt," ");

    printf("TSTCRD: done !\n");

  }

  if(TSTFAC) {			/* Test Spherical Harmonics */

    int n;

    fprintf(stderr,"\nFactorial: Enter n > ");
    scanf("%i", &n);
    printf("Calling factorial with n = %i\n",n);
    printf("factorial %i = %g\n",n,factorial(n));

    printf("TSTFAC: done !\n");

  }

  if(TSTSHT) {			/* Test Spherical Harmonic Transform */
    int i,j;
    int nl,nm;
    int Lmin,Lmax,Mmin,Mmax;
    complex **data,**alm;
    int npolr,nazim;

    Lmin = 0; Lmax = 5;
    Mmax = Lmax;
    Mmin = -Mmax;

    nl = Lmax - Lmin + 1;
    nm = 2 * Mmax + 1;
    npolr = 10;
    nazim = 5;

    data = (complex **) calloc(npolr,sizeof(complex *));
    for(i=0; i<npolr; i++) data[i] = (complex *) calloc(nazim,sizeof(complex)); 

    alm = (complex **) calloc(nl,sizeof(complex *));
    for(i=0; i<nl; i++) alm[i] = (complex *) calloc(nm,sizeof(complex)); 

    printf("Calling Spherical Harmonic Transform with \n"
	   "lmax=%i, mmax=%i, nl=%i, nm=%i\n"
	   "npolr = %i, nazim = %i\n",Lmax,Mmax,nl,nm,npolr,nazim);

    for(i=0; i< npolr; i++) {
      for(j=0; j< nazim; j++) {
	data[i][j].r = i+1;
	data[i][j].i = j+1;
	printf("data[%i][%i] = (%lf,%lf)\n",i,j,data[i][j].r,data[i][j].i); 
      }
    }

    printf("Transforming ... \n");
    /* SpharmTransform(alm,data,Lmax,Mmax,npolr,nazim); */
    printf("...done!\n");

    printf("Back from Spherical Harmonic with output alm =\n");
    for(i=0; i< nl; i++) {
      for(j=0; j< nm; j++) {
	printf("alm[%i][%i] = (%lf,%lf)\n",i,j,alm[i][j].r,alm[i][j].i);
      }
    }

    printf("TSTSHT: done !\n");

  }

  if(TSTSPH) {			/* Test SphericalHarmonicY */

    int i,j;
    complex *y;
    float *polr,*azim;
    int l,m,npts;

    npts = 10;

    fprintf(stderr,"\nSpherical Harmonic: Enter l > ");
    scanf("%i", &l);
    fprintf(stderr,"Spherical Harmonic: Enter m > ");
    scanf("%i", &m);

    y = (complex *) calloc(npts,sizeof(complex));
    polr = (float *) calloc(npts,sizeof(float));
    azim = (float *) calloc(npts,sizeof(float));

    printf("Calling Spherical Harmonic with l=%i, m=%i, npts = %i\n",l,m,npts);

    SphericalHarmonicY(y,l,m,polr,azim,npts);

    printf("Back from Spherical Harmonic with output y =\n");
    for(i=0; i<npts; i++) {
	printf("y[%i] = (%lf,%lf)\n",i,y[i].r,y[i].i);
    }

    printf("TSTSPH: done !\n");
  }

  if(TSTLEG) {			/* Test Legendre */

    int i,j;
    float *x,*y;
    int l,m,n;

    n = 10;
    x = (float *) calloc((unsigned)n,sizeof(float));
    y = (float *) calloc((unsigned)n,sizeof(float));

    fprintf(stderr,"\nLegendre: Enter l > ");
    scanf("%i", &l);
    fprintf(stderr,"\nLegendre: Enter m > ");
    scanf("%i", &m);

    printf("Calling Legendre with l=%i and x =\n",l);
    for(i=0; i< n; i++) {
      x[i] = (float)i/(float)n;
      printf("x[%i] = %lf\n",i,x[i]);
    }

    Legendre(y,l,m,x,n);

    printf("Back from Legendre with output y =\n");
    for(i=0; i< n; i++) {
      printf("y[%i] = %lf\n",i,y[i]);
    }

    printf("TSTLEG: done !\n");
  }

}
