/**<tstsph>**************************************************

  Title: TeST SPHarm routines

*/
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/fcntl.h> 

#define TSTALM 0		/* test alm */
#define TSTYLM 1		/* test ylm */

typedef struct { float r, i ; }       complex  ; /* default */

complex ***ylm_alloc(int Lmax, int npts) ;
complex **alm_alloc(int Lmax) ;
complex pickalm( complex **Alm, 
		 short Lpick, short Mpick, 
		 short Lmax) ;

#define printYlm(Ylm,Lmax,nn) \
do{ int i,k,nl =   Lmax+1; \
    for(i=0; i<nl; i++) { \
     printf("\n"); \
     for(j=-i; j<=i; j++) { \
      for(k=0; k<nn; k++) { \
	printf("Ylm[%i][%2i][%i] = (%13.6g,%13.6g)\n",i,j,k,Ylm[i][j][k].r,Ylm[i][j][k].i); \
      } \
     }\
    }\
} while(0) ;

#define fillYlm(Ylm,Lmax,nn) \
do{   int i,j,k,nl =   Lmax+1; \
    for(k=0; k<nn; k++) { \
    for(i=0; i<nl; i++) { \
      for(j=-i; j<=i; j++) { \
	Ylm[i][j][k].r = i*k; \
	Ylm[i][j][k].i = j*k; \
      } \
    }\
    }\
} while(0) ;

#define printAlm(alm,Lmax) \
do{   int i,j,nl =   Lmax+1; \
    for(i=0; i<nl; i++) { \
      printf("\n"); \
      for(j=-i; j<=i; j++) { \
	printf("Alm[%i][%2i] = (%13.6g,%13.6g)\n",i,j,alm[i][j].r,alm[i][j].i); \
      } \
    }\
} while(0) ;

#define fillAlm(alm,Lmax) \
do{   int i,j,nl =   Lmax+1; \
    for(i=0; i<nl; i++) { \
      for(j=-i; j<=i; j++) { \
	alm[i][j].r = i; \
        alm[i][j].i = j; \
      } \
    }\
} while(0) ;

#define ABS(x)  (((x)<0) ? -(x) : (x))
#define CHECK_L(lval) (((lval)<=(Lmax)) ? (1) : (0))
#define CHECK_M(mval) (((ABS(mval))<=(Lmax)) ? (1) : (0))

main()
{

  if(TSTYLM) {			/* Test allocation */

    int i,j;
    int nl,nm,npts;
    int Lpick,Mpick;
    int Mmin,Mmax;
    int Lmin,Lmax = 2;
    complex ***ylm ;
    complex ztmp;

    Lmin =  0;
    Mmax =  Lmax;
    Mmin = -Mmax ;

    printf("\nTesting allocation:\n\n");

    npts = 2 ;
    nl = Lmax - Lmin + 1;
    nm = 2*nl - 1;

    printf("Allocating (nl,nm) = (%i,%i)\n"
	   " (Lmin,Lmax) =  (%i,%i)\n"
	   " (Mmin,Mmax) =  (%i,%i)\n"
	   " npts =  %i\n",
	   nl,nm,Lmin,Lmax,Mmin,Mmax,npts);

    ylm = ylm_alloc(Lmax,npts);

    printf("\nShowing results of ylm allocation:\n");
    printYlm(ylm,Lmax,npts);

    printf("\nFilling ylm for testing\n");
    fillYlm(ylm,Lmax,npts);
    printYlm(ylm,Lmax,npts);

    /*
    Lpick = 2;
    Mpick = -2;
    printf("\nTesting pickylm:\n");
    ztmp = pickylm(ylm,Lpick,Mpick,Lmax);
    printf("Retrieved value: Ylm[%i][%i] = (%lf,%lf)\n\n",Lpick,Mpick,ztmp.r,ztmp.i);
    */

    printf("\nTSTYLM: done !\n\n");

  }

  if(TSTALM) {			/* Test alm */

    int i,j;
    int nl,nm;
    int Lpick,Mpick;
    int Mmin,Mmax;
    int Lmin,Lmax = 2;
    complex **alm ;
    complex ztmp;

    Lmin =  0;
    Mmax =  Lmax;
    Mmin = -Mmax ;

    printf("\nTesting allocation:\n\n");

    nl = Lmax - Lmin + 1;
    nm = 2*nl - 1;

    printf("Allocating (nl,nm) = (%i,%i)\n"
	   " (Lmin,Lmax) =  (%i,%i)\n"
	   " (Mmin,Mmax) =  (%i,%i)\n",
	   nl,nm,Lmin,Lmax,Mmin,Mmax);

    alm = alm_alloc(Lmax);

    printf("Showing results allocation:\n\n");
    printAlm(alm,Lmax);

    printf("\nFilling alm for testing\n");
    fillAlm(alm,Lmax);
    printAlm(alm,Lmax);

    Lpick = 2;
    Mpick = -2;
    printf("\nTesting pickalm:\n");
    ztmp = pickalm(alm,Lpick,Mpick,Lmax);
    printf("Retrieved value: Alm[%i][%i] = (%lf,%lf)\n\n",Lpick,Mpick,ztmp.r,ztmp.i);

    printf("TSTALM: done !\n\n");

  }

}

complex ***ylm_alloc(int Lmax, int npts)
{
  complex ***ylm,***zzz;
  int i,j;
  int nl =   Lmax+1;
  int nm = 2*Lmax+1;

  zzz = (complex ***)calloc(nl,sizeof(complex **));
  ylm = (complex ***)calloc(nl,sizeof(complex **));

  for (j=0;j<nl;j++) {
    zzz[j] = (complex **)calloc(nm,sizeof(complex *));
    ylm[j] = (complex **)calloc(nm,sizeof(complex *));
    ylm[j] = zzz[j]+nl-1;
    for (i=0;i<nm;i++) {
      zzz[j][i] = (complex *)calloc(npts,sizeof(complex));
    }
  }

  free(zzz);
  return( ylm );
}

complex **alm_alloc(int Lmax)
{
  complex **alm,**zzz;
  int i;
  int nl =   Lmax+1;
  int nm = 2*Lmax+1;

  zzz = (complex **)calloc(nl,sizeof(complex *));
  alm = (complex **)calloc(nl,sizeof(complex *));

  for (i=0;i<nl;i++) {
    zzz[i] = (complex *)calloc(nm,sizeof(complex));
    alm[i] = zzz[i]+nl-1;
  }

  free(zzz);
  return( alm );
}

complex pickalm( complex **Alm, 
		 short Lpick, short Mpick, 
		 short Lmax)
{
  complex almpick ;

  if(!CHECK_L(Lpick)) printf("Invalid Lpick requested");
  if(!CHECK_M(Mpick)) printf("Invalid Mpick requested");
  
  almpick = Alm[Lpick][Mpick] ;

  return almpick;
}

