/**<init_dif_state.c>****************************************************

     Title: INITialize DIFfusion STATE

   Purpose: Initialize diffusion structure

      Call: DIF_state *init_dif_state( char *header )

   History: 02_01_06 lrf: adjust polytope for ses13 to match new sphses
            Make polytope consistent with hyper.e! (same as ses13 right now)

**************************************************************************/
#include "diffuse.h"

/* Processing initialization */

DIF_status *init_dif_status( DIF_state *dif )
{
  DIF_status *sta = NULL;

  sta = (DIF_status *) malloc (sizeof(DIF_status));

  sta->shd_performed = 0 ;
  sta->dti_performed = 0 ;
  sta->ftm_performed = 0 ;

  sta->shd_initialized = 0 ;
  sta->dti_initialized = 0 ;
  sta->ftm_initialized = 0 ;

  sta->dothresh = 0 ;
  sta->overlay_chan = -1 ;
  sta->compute_chan = -1 ;

  return (sta) ;
}

/* DTI processing initialization */

DTI_prc_state *init_dti_prc_state( IMG_state *img , DSET_stats *b0stats )
{
  int nn = 0;
  DTI_prc_state *prc = NULL;

  prc = (DTI_prc_state *) malloc (sizeof(DTI_prc_state));

  prc->nimsl = img->nbvals*img->dgrad->nvert;
  prc->tdim = TDIM ;	
  prc->nten = prc->tdim * prc->tdim;	

  prc->edim = prc->tdim ;   /* Number of eigenvalues is same as tensor dimension */
  prc->fv = NULL ;
  prc->dv = NULL ;

  prc->maskval = NULLVAL ;
  if(b0stats!=NULL)  prc->mask = b0stats->mask ;

  prc->Dapp  = NULL ;
  prc->Bmat = NULL ;

  prc->D33  = fm_alloc(3,3) ;	/* 3x3 diffusion tensor.  22-Oct-03 */

  return (prc) ;

}

/* Eigenvalue initialization */

DTI_eig_state *init_dti_eig_state( DTI_prc_state *prc )
{
  DTI_eig_state *eig = NULL;

  eig = (DTI_eig_state *) malloc (sizeof(DTI_eig_state));

  eig->nten    = prc->nten;
  eig->nevals  = prc->tdim;	/* makes more sense */

  eig->fv    = fv_alloc(prc->nten);
  eig->dv    = dv_alloc(prc->nten);

  eig->dm    = dm_alloc(prc->tdim,prc->tdim);
  eig->evecs = dm_alloc(prc->tdim,prc->tdim);
  eig->evals = dv_alloc(prc->tdim);
  eig->nr    = prc->tdim;
  eig->nc    = prc->tdim;

  return (eig) ;

}

/* Least squares initialization */

DTI_lsq_state *init_dti_lsq_state( DTI_prc_state *prc )
{
  DTI_lsq_state *lsq = NULL;
  
  int ir;

  double dc  = 0.0 ;
  double sample_dt = 1.0;	/* does this matter? */

  lsq = (DTI_lsq_state *) malloc (sizeof(DTI_lsq_state));

  lsq->nt     = prc->nimsl ;
  lsq->dt     = sample_dt ;
  lsq->dc     = dc ;

  /* n^2 - n + 1 = full - diag + (b=0) */
  lsq->lsqdim = prc->tdim * (prc->tdim - 1); 
  lsq->nrts   = lsq->lsqdim ;
  lsq->polort = 0 ;
  lsq->nrsin  = 0 ;

  lsq->ignore = 0 ;

  lsq->signal = (float *) malloc( sizeof(float) * lsq->nt ) ;
  if( lsq->signal == NULL )
      {fprintf(stderr,"\nmalloc error in init_lsq\n\a");exit(1);}

  lsq->noise  = (float *) malloc( sizeof(float) * lsq->nt ) ;
  if( lsq->noise == NULL )
      {fprintf(stderr,"\nmalloc error in init_lsq\n\a");exit(1);}

  lsq->label = (char *) malloc( sizeof(char) * LABEL_LEN ) ;
  if( lsq->label == NULL )
    {fprintf(stderr,"\nmalloc error in init_lsq\n\a");exit(1);}

  if(lsq->nrts!=0) {

    lsq->tsim = (float **) malloc( sizeof(float *) * lsq->nrts ) ;

    if( lsq->tsim == NULL )
      {fprintf(stderr,"\nmalloc error in init_lsq\n\a");exit(1);}

    for( ir=0 ; ir < lsq->nrts ; ir++ ){
      lsq->tsim[ir] = (float *) malloc( sizeof(float) * lsq->nt ) ;
      if( lsq->tsim[ir] == NULL )
	{fprintf(stderr,"\nmalloc error in init_lsq\n\a");exit(1);}
    }

  } else {
    lsq->tsim = NULL;
  }

  if(lsq->nrsin!=0) {
    lsq->sinharm = (int *) malloc( sizeof(int) * lsq->nrsin ) ;
    if( lsq->sinharm == NULL )
      {fprintf(stderr,"\nmalloc error in init_lsq\n\a");exit(1);}
    lsq->sinper  = (float *) malloc( sizeof(float) * lsq->nrsin ) ;
    if( lsq->sinper == NULL )
      {fprintf(stderr,"\nmalloc error in init_lsq\n\a");exit(1);}
  } else {
    lsq->sinharm = NULL;
    lsq->sinper  = NULL;
  }
  
  lsq->nref  = 0 ;		/* generated by lsqfit */
  lsq->alpha = NULL;		/* generated by lsqfit */

  return(lsq);

}

/* Not used here, but may elsewhere (?) */

void free_lsqstr( DTI_lsq_state *lsq, 
		  int    nt, 
		  double dt, 
		  int    nrts )
{
  int ir;

  free(lsq->signal) ;
  free(lsq->noise) ;
  free(lsq->sinper) ;
  free(lsq->sinharm) ;
  free(lsq->label) ;
  free(lsq->alpha) ;

  if( lsq->tsim != NULL ){
    for( ir=0 ; ir < nrts ; ir++ ) 
      if( lsq->tsim[ir] != NULL ) free(lsq->tsim[ir]) ;
    free(lsq->tsim) ;
  }

  free(lsq);
}

int scan_line_GE_EXCITE(char *line, IMG_state* img, Grad *dgrad) {
  char *program_name = "init_img_state_GE_EXCITE" ;
  int  MAXLEN = 256;
  char param_name[MAXLEN];
  char param_val[MAXLEN];
  int  local_debug = 0;
  int param_count = 0;

  printf("line: %s\n", line);
  if(local_debug) {printf("header_line = %s",line);fflush(stdout);}
    
  if(!strncmp(&line[0],"#",1)) {

    if(local_debug) {printf("(Comment line)\n");fflush(stdout);}

  } else {

    int print_params = 1;
    if (sscanf(line,"%s %s",param_name,param_val) < 2) {
      fprintf(stderr, "Ill-formed parameter line: %s\n", line);
      return(0);
    }
    printf("param_name: %s\n", param_name);
    if(local_debug) {printf("param_name = %s, param_val = %s\n",
			    param_name,param_val); fflush(stdout);}

    if(!strcmp(param_name,"psdname")) {
      strcpy(img->psd_name,param_val);
      if(print_params) printf("img->psd_name = %s\n",img->psd_name);
    } else if(!strcmp(param_name,"nx")) {
      img->nx = atoi(param_val) ;
      if(print_params) printf("img->nx = %i\n",img->nx);
    } else if(!strcmp(param_name,"ny")) {
      img->ny = atoi(param_val) ;
      if(print_params) printf("img->ny = %i\n",img->ny);
    } else if(!strcmp(param_name,"nsl")) {
      img->nsl = atoi(param_val) ;
      if(print_params) printf("img->nsl = %i\n",img->nsl);
    } else if(!strcmp(param_name,"dsamp")) {
      img->difmode = atoi(param_val) ;
      if(print_params) printf("img->difmode = %i\n",img->difmode);
    } else if(!strcmp(param_name,"ntess")) {
      dgrad->ntess = atoi(param_val) ;
      if(print_params) printf("dgrad->ntess = %i\n",dgrad->ntess);
    } else if(!strcmp(param_name,"polytope")) {
      dgrad->polytope = atoi(param_val) ;
      if(print_params) printf("dgrad->polytope = %i\n",dgrad->polytope);
	
      if(dgrad->polytope!=POLY_ICO) {
	printf("%s: Invalid input polytope type (%i)\n",
	       program_name,dgrad->polytope);
	printf("%s: Value must be %i\n",program_name,POLY_ICO);
	exit(0);
      }

    } else if(!strcmp(param_name,"nvert")) {
      dgrad->nvert = atoi(param_val) ;
      if(print_params) printf("dgrad->nvert = %i\n",dgrad->nvert);
    } else if(!strcmp(param_name,"nbvals")) {
      img->nbvals = atoi(param_val) ;
      if(print_params) printf("img->nbvals = %i\n",img->nbvals);
    } else if(!strcmp(param_name,"bstep")) {
      img->bstep = atoi(param_val) ;
      if(print_params) printf("img->bstep = %i\n",img->bstep);
    } else if(!strcmp(param_name,"bval")) {
      img->bval = atof(param_val) ;
      img->bval_available = 1 ;
      if(print_params) printf("img->bval = %f\n",img->bval);
    } else if(!strcmp(param_name,"nb0")) {
      img->nb0 = atoi(param_val) ;
      if(print_params) printf("img->nb0 = %i\n",img->nb0);
    }

    param_count++;
  }
  return param_count;
}

IMG_state *init_img_state_GE_EXCITE( char *headername ) 
{
  char *program_name = "init_img_state_GE_EXCITE" ;
  char tmp_header_name[256];
  int  debug = 1;
  int header_file_exists = 0;
  if (!strncmp(headername, "file:", 5)) {
    header_file_exists = 1;
    strncpy(tmp_header_name, headername+5, 255);
  }
  else {
    strcpy(tmp_header_name, "tempXXXXXX");
    mktemp(tmp_header_name);
    FILE *fp = fopen(tmp_header_name, "w");
    fprintf(fp, "%s", headername);
    fclose(fp);
    header_file_exists = 2;
  }

  FILE *fin;
  IMG_state *img = (IMG_state *) malloc (sizeof(IMG_state));

  printf("Inside %s:\n",program_name);

  int  MAXLEN = 256;
  char header_line[MAXLEN];
  int  param_count = 0;
  int  local_debug = 0;

  if (!(fin=fopen(tmp_header_name,"r"))) {
    if (header_file_exists) {
      printf("Can't open header file %s!\n",headername);
      return NULL;
    }
  } else {
    if(local_debug) {
      printf("Successfully opened header file %s\n",headername);
      fflush(stdout) ;
    }
  }

  /* load params into img */

  img->psd_name  = (char *)calloc(256, sizeof(char));
  img->bval_available = 0 ;

  Grad *dgrad = (Grad *) malloc(sizeof(Grad)) ;	/* 12-Jul-04 */
  img->dgrad = dgrad ;

  if (header_file_exists) {
    while (fgets(header_line, MAXLEN, fin) != NULL) {
      param_count += scan_line_GE_EXCITE(header_line, img, dgrad);
    }
  }
  else {
    printf("Error reading header information.\n");
/*     while (gets(headername) != NULL) { */
/*       param_count += scan_line_GE_EXCITE(headername, img, dgrad); */
/*     } */
  }

  if (header_file_exists > 1) {
    unlink(tmp_header_name);
  }

  if (!strcmp(img->psd_name,"")) {
    fprintf(stderr, "Pulse sequence name (psdname) was not defined in header information.\nWe are unable to load files without a pulse sequence name.\n");
    free(img);
    return(0);
  } else {
    printf("Continuing with pulse sequence %s.\n", img->psd_name);
  }

  /* Done loading params. */

  img->nxy  = img->nx * img->ny;
  img->ndif = dgrad->nvert + img->nb0;
  img->nim  = img->nsl * img->ndif * img->nbvals ;

  img->gdbstep  = (float *)calloc(img->nbvals, sizeof(float));

  img->gdxpat   = (float *)calloc(img->ndif, sizeof(float));
  img->gdypat   = (float *)calloc(img->ndif, sizeof(float));
  img->gdzpat   = (float *)calloc(img->ndif, sizeof(float));

  /* possibly redundant with gdipat above */

  printf("init_img_state: allocating G with n=%i)...\n",MAXNDIF);
  img->GX       = (float *)calloc(MAXNDIF, sizeof(float));
  img->GY       = (float *)calloc(MAXNDIF, sizeof(float));
  img->GZ       = (float *)calloc(MAXNDIF, sizeof(float));

  dgrad->gx       = (float *)calloc(dgrad->nvert, sizeof(float));
  dgrad->gy       = (float *)calloc(dgrad->nvert, sizeof(float));
  dgrad->gz       = (float *)calloc(dgrad->nvert, sizeof(float));
  dgrad->tris     = (int   *)calloc((unsigned)(2*dgrad->nvert),3*sizeof(int)); /* 17-Feb-04 */

  /* create array of b-values.  Jul 8, 2004 */
  img->bvals  = (float *)calloc(img->nbvals, sizeof(float));
  for (int j=0; j<img->nbvals; j++) img->bvals[j] = (j+1)*img->bval/img->nbvals;

  printf("Successfully read header file %s\n"
	 "%i parameters read\n",
	 headername,param_count); fflush(stdout);

  fclose(fin);
  return (img) ;
}

IMG_state *init_img_state_GE_LX( char *header ) 
{
  char *program_name = "init_img_state_GE_LX" ;
  int  debug = 1;

  /* Recon variables */

  int npixo2, nsamp, necho, ngrid;
  int rawimsize;
  int nframes, ntotal;
  int rhfrsize, rhnframes, rhnecho;
  int nbpi;			/* bytes per image  */
  int imrot, imtran;
  int sloffset,froffset;

  float xoff, yoff;		/* offsets in pixels  */
  float foff = 0;		/* total freq offset */
  float frcvr;			/* receiver frequency offset  */
  float gain = 1.0;	
  float dts, mapdt;
  float gfov, gtype, gamp, gslew; /* for genspiral  */
  int N, nl;			/* matrix size for genspiral */
  int bline;			/* number of baselines */
  int doconcat;			/* concatenated acquisitions */
  int ngap;			/* number of missing points in concat acq */
  float w0 = 0.0, wx = 0.0, wy = 0.0;
  float dwin = 2;		/* half length of Kaisser-Bessel window */
  float b = 8.0;		/* Kaisser-Bessel param  */
  float fermir;			/* radius of fermi window */
  float fermiw;			/* width of fermi window  */

  float rotmat[3][2];		/* gradient rotation matrix  */
  float bzero;			/* field, in Guass */

  RDB_SLICE_INFO_ENTRY datab[RDB_MAX_SLICES];

  /* Input variables */
  int im1       ;	/* first image frame */
  int nsl       ;	/* number of slices */
  int npix      ;	/* number of pixels */
  int navc      ;	/* do nav correction if set */
  int bmapflag  ;	/* do the b0 map if 1  */
  int domaxwell ;	/* do maxwell field correction if 1 */
  int nf2do     ;	/* number of time frames */
  int imgtyp    ;	/* output image type bitmap  */
  float zoomf   ;	/* zoom factor */
  float foffin  ;	/* offset from command line */
  int isl1      ;       /* first slices */
  int isl2      ;       /* last slice */
  
  /* Diffusion - lrf */
  int nbvals ;		/* number of b-values - lrf */
  int nb0 ;		/* number of b0 acquisitions - lrf */
  int ndif ;		/* number of diffusion directions - lrf */
  int difnex ;		/* number of diffusion averages (3dsfd) - lrf */
  int difmode ;		/* diffusion mode - lrf */
  int polytess ;	/* cat of polytope and ntess */
  int polytope ;	/* Tesselation polytope type */
  int ntess ;		/* diffusion sphere tesselation factor - lrf */
  int nvert ;		/* number of vertices in tesselated sphere - lrf */
  int ecco ;		/* se ecco - lrf. 04-Feb-22 */
  int bstep ;		/* B-value stepping - lrf */
  int bslope ;		/* Increase of log-bstepping slope over 1 - lrf */
  float bval  ;		/* bvalue (s/mm^2) */
  float pwgd  ;		/* Diffusion gradient width */
  float pwgd2  ;	/* Hyper diffusion gradient width */
  float Delta  ;	/* Time between diffusion gradient centers */
  float Delta2  ;	/* Time between hyper diffusion gradient centers */
  float pwdramp  ;	/* Diffusion gradient ramp width */
  float agdx1  ;	/* Amp of x-diff grad (G/cm) */
  float agdy1  ;	/* Amp of y-diff grad (G/cm) */
  float agdz1  ;	/* Amp of z-diff grad (G/cm) */
  float optm  ;		/* Mixing time (ms) */
  
  /* GP - lrf */
  int   gpmode ;	/* GP mode */
  int   ngpvals ;	/* Number of gp angles */
  float gpmax  ;	/* Maximum GP (% of PI) */
  
  char psd_name[256];	/* 02_01_06 - lrf */

  IMG_state *img = NULL;

  img = (IMG_state *) malloc (sizeof(IMG_state));
  Grad *dgrad = (Grad *) malloc(sizeof(Grad)) ;	/* 12-Jul-04 */
  img->dgrad = dgrad ;

  if(!strcmp(header,EMPTY)) {

    printf("No pfile header available\nReturning null state\n");
    return NULL ;

  } else {

    POOL_HEADER       *ph;
    RDB_HEADER_REC    *hr;
    MRIMAGEDATATYPE   *hr_im;	/* image header */
    EXAMDATATYPE      *hr_ex;	/* exam header */
    SERIESDATATYPE    *hr_se;	/* series header */

    ph    = (POOL_HEADER *)     header;
    hr    = (RDB_HEADER_REC *)  &(ph->rdb_hdr_rec);
    hr_im = (MRIMAGEDATATYPE *) &(ph->rdb_hdr_image);
    hr_se = (SERIESDATATYPE *)  &(ph->rdb_hdr_series);
    hr_ex = (EXAMDATATYPE *)    &(ph->rdb_hdr_exam);

    img->psd_name  = (char *)calloc(256, sizeof(char));
    strncpy(img->psd_name,hr_im->psd_iname,3);  /* Internal psd name */
    sprintf(img->psd_name,"hyp");

    nsl       = hr->rdb_hdr_nslices;    /* number of slices */
    rhnecho   = hr->rdb_hdr_nechoes;    /* number of echoes */
    rhfrsize  = hr->rdb_hdr_frame_size; /* a/d samples in file */
    nsamp     = rhfrsize;
    necho     = rhnecho;		/* number of echoes */
    doconcat  = (int) hr->rdb_hdr_user11;     /* do concat acq */
    if(doconcat)  nsamp = 2*nsamp;
    nl        = (int) hr->rdb_hdr_user4;      /* views */
    rhnframes = hr->rdb_hdr_nframes;    /* total number of views in slice */
    nframes   = (int) hr->rdb_hdr_user1;      /* number of temporal frames */
    rawimsize = hr->rdb_hdr_im_size;    /* picture size */
    nbpi      = nl*nsamp*4;	        /* num bytes per image */
    imrot     = hr->rdb_hdr_rotation;   /* image rotate */
    imtran    = hr->rdb_hdr_transpose;  /* image transpose */
    xoff      = hr->rdb_hdr_xoff;       /* x offset */
    yoff      = hr->rdb_hdr_yoff;	/* y offset */
    ngap      = (int) hr->rdb_hdr_user10;     /* concat gap */
    dts       = hr->rdb_hdr_user13;     /* sampling interval, usec */
    bmapflag  = (int) hr->rdb_hdr_user14;     /* set if b0 map data collected */
    mapdt     = hr->rdb_hdr_user15;     /* b0 map delta te, us  */
    gfov      = hr->rdb_hdr_user0;      /* gradient fov	*/
    N         = (int) hr->rdb_hdr_user3;      /* gradient design fov */
    gtype     = hr->rdb_hdr_user5;      /* gradient trajectory type  */
    gamp      = hr->rdb_hdr_user6;      /* gradient max, G/cm	*/
    gslew     = hr->rdb_hdr_user7;      /* gradient slew rate, mT/m/ms	*/
    bline     = hr->rdb_hdr_baseline_views; /* number of baselines */
    frcvr     = 1000*hr->rdb_hdr_user12; /* receiver center freqency, Hz  */

    bzero = (.1*hr->rdb_hdr_ps_mps_freq+hr->rdb_hdr_user19)*GAMMA;	/* Gauss  */

    if(MATCH_STR(img->psd_name,"ses") || 
       MATCH_STR(img->psd_name,"hyp") ||
       MATCH_STR(img->psd_name,"spdif")) 
      { 

      difmode = (int) hr->rdb_hdr_user20;	/* Diffusion mode */
      ndif    = (int) hr->rdb_hdr_user21;	/* Number of diffusion directions */
      nb0     = (int) hr->rdb_hdr_user22;	/* Number of b=0 acquisitions */
      nbvals  = (int) hr->rdb_hdr_user23;	/* Number of b-values */
      bstep   = (int) hr->rdb_hdr_user24;	/* B-value stepping */
      bslope  = (int) hr->rdb_hdr_user25;	/* Increase of log-bstepping slope over 1 */
      ntess   = (int) hr->rdb_hdr_user26;	/* Diffusion tesselation factor */
      nvert   = (int) hr->rdb_hdr_user27;	/* Number of vertices in tesselated polytope */
      polytope= (int) hr->rdb_hdr_user28;	/* Tesselation polytope type */
      ecco    = (int) hr->rdb_hdr_user29;	/* SE ecco switch. 04Feb22 */

      /* HW AHB! for spdif one time */
      if(0) {
      rawimsize = 128 ;
      ndif = 43;		
      ntess = 2;
      nvert = 42;
      difmode = 3;
      polytope = 2;
      nbvals = 1;
      nb0 = 1;
      nsl = 3;
      }
      /* HW AHB! */

      if(MATCH_STR(img->psd_name,"ses")) { /* 19-Aug-02 */
	pwgd    = hr->rdb_hdr_user30; /* Diffusion gradient width (us) */
	pwdramp = hr->rdb_hdr_user31; /* Diffusion gradient ramp width (us) */
	Delta   = hr->rdb_hdr_user32; /* Time between diff gradient centers (us) */
      } else if(MATCH_STR(img->psd_name,"hyp")||MATCH_STR(img->psd_name,"spdif")) { /* 19-Aug-02 */
	pwgd    = hr->rdb_hdr_user30; /* Diffusion gradient width (us) */
	pwgd2   = hr->rdb_hdr_user31; /* Diffusion hyper gradient width (us) */
	pwdramp = hr->rdb_hdr_user32; /* Diffusion gradient ramp width (us) */
	Delta   = hr->rdb_hdr_user38; /* Time between diff grad centers (us) */
	Delta2  = hr->rdb_hdr_user39; /* Time between hyper diff grad centers (us) */
      }

      agdx1   = hr->rdb_hdr_user33; /* Amp of x diffusion gradient (G/cm) */
      agdy1   = hr->rdb_hdr_user34; /* Amp of y diffusion gradient (G/cm) */
      agdz1   = hr->rdb_hdr_user35; /* Amp of z diffusion gradient (G/cm) */

      optm    = hr->rdb_hdr_user36; /* Mixing time (ms) */

      img->bval_available = 1 ;
      img->bval = 520;

      /*bval    = hr->rdb_hdr_user??; */ /* NEED TO DO THIS IN HYPER! b-value (s/mm^2) */

    } else if(MATCH_STR(img->psd_name,"3ds")) {	/* because img->psd_name only of length 3  */

      nb0    = 1;
      nbvals = 1;
      bstep  = 0;
      bslope = 1;
      nvert  = 0;
      ecco   = 0;

      difmode = (int) hr->rdb_hdr_user21;	/* Diffusion mode */
      polytess= (int) hr->rdb_hdr_user22;	/* cat of polytope and ntess */
      ndif    = (int) hr->rdb_hdr_user23;	/* Number of diffusion directions */
      difnex  = (int) hr->rdb_hdr_user24;	/* Number of diffusion averages */
      bval    = hr->rdb_hdr_user25; /* b-value (s/mm^2) */
      Delta   = hr->rdb_hdr_user26; /* Time between diff gradients (us) */
      pwgd    = hr->rdb_hdr_user27; /* Diffusion gradient width (us) */
      bval    = hr->rdb_hdr_user28; /* Diffusion gradient ramp width (us) */
      agdx1   = hr->rdb_hdr_user29; /* Amp of x diffusion gradient (G/cm) */
      agdy1   = hr->rdb_hdr_user30; /* Amp of y diffusion gradient (G/cm) */
      agdz1   = hr->rdb_hdr_user31; /* Amp of z diffusion gradient (G/cm) */

      img->bval_available = 1 ;

      { /* Retrieve polytope and ntess from polytess */
	int *iv = iparse(polytess,2) ;
	polytope = iv[0];	/* Tesselation polytope type */
	ntess    = iv[1]; 	/* Polytope tessellation level */
	free(iv) ;
      }

      if(debug) printf(" **************** Inside %s ***************\n"
		       " ndif = %i, bval = %f, difmode = %i\n",
		       program_name,ndif,bval,difmode);

    } else {
      printf("init_img_state (init_img_state): Unknown psd name %s ... exiting\n",
	     img->psd_name);
    }

    printf("pwgd = %lf us, pwdramp = %lf us, Delta = %lf us\n"
	   "ndif = %i, ntess = %i\n",
	 pwgd,pwdramp,Delta,ndif,ntess);

    /* GP user variables ruser30-ruser39 */

    gpmode  = (int) hr->rdb_hdr_user40;	/* GP mode */
    ngpvals = (int) hr->rdb_hdr_user41;	/* Number of gp angles */
    gpmax   = hr->rdb_hdr_user42;	/* Maximum GP (% of PI) */

  }

  img->nx       = rawimsize ;
  img->ny       = rawimsize ;
  img->nxy      = img->nx * img->ny ;
  img->nsl      = nsl ;
  img->nbvals   = nbvals ;
  img->bval     = bval ;	/* for single b-value.  Obviously need array for nbvals (later) */
  img->ndif     = ndif ;
  img->nim      = nsl * ndif * nbvals ;

  img->nb0      = nb0 ;
  img->nl       = nl ;
  img->difmode  = difmode ;	/* 0=none, 1=manual, 2=basser, 3=sphere */
  dgrad->ntess  = ntess ;
  dgrad->nvert  = nvert ;
  img->ecco     = ecco ;
  img->bstep    = bstep ;
  img->bslope   = bslope ;
  dgrad->polytope = polytope-MATCH_STR(img->psd_name,"ses") ; /* 02_01_06,lrf: ses13 now
							       inconsistent with sphses */
  //dgrad->polytope = polytope-1;	     /* for now, until we change hyper.e */
  img->pwgd     = pwgd    * US2SEC ; /* convert us -> sec */
  img->pwgd2    = pwgd2   * US2SEC ; /* convert us -> sec */
  img->Delta    = Delta   * US2SEC ; /* convert us -> sec */
  img->Delta2   = Delta2  * US2SEC ; /* convert us -> sec */
  img->pwdramp  = pwdramp * US2SEC ; /* convert us -> sec */

  img->agdx1    = agdx1/10. ;	/* amps in G/mm */
  img->agdy1    = agdy1/10. ;
  img->agdz1    = agdz1/10. ;

  img->optm     = optm ;

  img->gdbstep  = (float *)calloc(MAXNBVALS, sizeof(float));

  img->gdxpat   = (float *)calloc(MAXNDIF, sizeof(float));
  img->gdypat   = (float *)calloc(MAXNDIF, sizeof(float));
  img->gdzpat   = (float *)calloc(MAXNDIF, sizeof(float));

  /* possibly redundant with gdipat above */

  printf("init_img_state: allocating G with n=%i)...\n",MAXNDIF);
  img->GX       = (float *)calloc(MAXNDIF, sizeof(float));
  img->GY       = (float *)calloc(MAXNDIF, sizeof(float));
  img->GZ       = (float *)calloc(MAXNDIF, sizeof(float));

  dgrad->gx       = (float *)calloc(dgrad->nvert, sizeof(float));
  dgrad->gy       = (float *)calloc(dgrad->nvert, sizeof(float));
  dgrad->gz       = (float *)calloc(dgrad->nvert, sizeof(float));
  dgrad->tris     = (int   *)calloc((unsigned)(2*dgrad->nvert),3*sizeof(int)); /* 17-Feb-04 */

  printf("WARNING:  Untested bvalue array generation in init_img_state_GE_LX!\n");
  img->bvals  = (float *)calloc(img->nbvals, sizeof(float));
  for (int j=0; j<img->nbvals; j++) img->bvals[j] = (j+1)*img->bval/img->nbvals;

  return (img) ;

}

IMG_state *init_img_state_VARIAN( char *header ) 
{
  IMG_state *img = NULL;
  int ivp,NVP = 26;
  char param[NVP][80];
  char *line;

  img = (IMG_state *) malloc (sizeof(IMG_state));
  Grad *dgrad = (Grad *) malloc(sizeof(Grad)) ;	/* 12-Jul-04 */
  img->dgrad = dgrad ;

  if(!strcmp(header,EMPTY)) {

    printf("No pfile header available\nReturning null state\n");
    return NULL ;

  }

  /* image sizes */
  sscanf(header,
	 "%s %i"
	 "%s %i"
	 "%s %i"
	 "%s %i"
	 "%s %i"
	 "%s %i"
	 "%s %i"
	 "%s %i"
	 "%s %i"
	 "%s %i"
	 "%s %i"
	 "%s %i"
	 "%s %f"
	 "%s %f"
	 "%s %f"
	 "%s %f"
	 "%s %f"
	 "%s %f"
	 "%s %f"
	 "%s %f"
	 "%s %f"
	 "%s %f"
	 "%s %f"
	 "%s %f"
	 "%s %f"
	 "%s %f",
	 param[0],&img->nx,
	 param[1],&img->ny,
	 param[2],&img->nsl,
	 param[3],&img->nl,
	 param[4],&img->nbvals,
	 param[5],&img->difmode,
	 param[6],&img->nb0,
	 param[7],&dgrad->nvert,
	 param[8],&img->ndif,
	 param[9],&dgrad->polytope,
	 param[10],&dgrad->ntess,
	 param[11],&img->bstep,
	 param[12],&img->pwdramp,
	 param[13],&img->gdx1_amp,
	 param[14],&img->gdx1_pw,
	 param[15],&img->gdx2_amp,
	 param[16],&img->gdx2_pw,
	 param[17],&img->gdy1_amp,
	 param[18],&img->gdy1_pw,
	 param[19],&img->gdy2_amp,
	 param[20],&img->gdy2_pw,
	 param[21],&img->gdz1_amp,
	 param[22],&img->gdz1_pw,
	 param[23],&img->gdz2_amp,
	 param[24],&img->gdz2_pw,
	 param[25],&img->optm);

  img->pwdramp *= US2SEC ; /* convert us -> sec */
  for (ivp=0; ivp<NVP; ivp++) printf("param[%i] = %s\n",ivp,param[ivp]);

  /* other creations */
  img->nxy      = img->nx * img->ny ;
  img->nim      = img->nsl * img->ndif * img->nbvals ;
  img->bslope   = 1 ;
  img->ecco     = 0 ;
  img->bval_available = 0 ;

  /* allocations */

  img->gdbstep  = (float *)calloc(MAXNBVALS, sizeof(float));

  img->gdxpat   = (float *)calloc(MAXNDIF, sizeof(float));
  img->gdypat   = (float *)calloc(MAXNDIF, sizeof(float));
  img->gdzpat   = (float *)calloc(MAXNDIF, sizeof(float));

  /* possibly redundant with gdipat above */

  printf("init_img_state: allocating G with n=%i...\n",MAXNDIF);
  img->GX       = (float *)calloc(MAXNDIF, sizeof(float));
  img->GY       = (float *)calloc(MAXNDIF, sizeof(float));
  img->GZ       = (float *)calloc(MAXNDIF, sizeof(float));

  printf("init_img_state: allocating G with n=%i...\n",img->ndif);
  dgrad->gx       = (float *)calloc(dgrad->nvert, sizeof(float));
  dgrad->gy       = (float *)calloc(dgrad->nvert, sizeof(float));
  dgrad->gz       = (float *)calloc(dgrad->nvert, sizeof(float));
  dgrad->tris     = (int   *)calloc((unsigned)(2*dgrad->nvert),3*sizeof(int)); /* 17-Feb-04 */

  /* TRANSITIONAL HACK SECTION TO TEMPORARILY CREATED NEEDED VARS! */
  
  img->agdx1 = img->gdx1_amp ;
  img->agdy1 = img->gdy1_amp ;
  img->agdz1 = img->gdz1_amp ;
  img->pwgd  = img->gdz1_pw ;
  img->Delta = .01 ;

  /* end HACK section */

  /* User info */
  printf("img->nx = %i\n"
	 "img->ny = %i\n"
	 "img->nsl = %i\n"
	 "img->nl = %i\n"
	 "img->nbvals = %i\n"
	 "img->difmode = %i\n"
	 "img->nb0 = %i\n"
	 "dgrad->nvert = %i\n"
	 "img->ndif = %i\n"
	 "dgrad->polytope = %i\n"
	 "dgrad->ntess= %i\n"
	 "img->bstep = %i\n"
	 "img->pwdramp = %f\n"
	 "img->gdx1_amp = %f\n"
	 "img->gdx1_pw = %f\n"
	 "img->gdx2_amp = %f\n"
	 "img->gdx2_pw = %f\n"
	 "img->gdy1_amp = %f\n"
	 "img->gdy1_pw = %f\n"
	 "img->gdy2_amp = %f\n"
	 "img->gdy2_pw = %f\n"
	 "img->gdz1_amp = %f\n"
	 "img->gdz1_pw = %f\n"
	 "img->gdz2_amp = %f\n"
	 "img->gdz2_pw = %f\n"
	 "img->pwgd = %f\n"
	 "img->Delta = %f\n"
	 "img->pwgd2 = %f\n"
	 "img->Delta2 = %f\n"
	 "img->agdx1 = %f\n"
	 "img->agdy1 = %f\n"
	 "img->agdz1 = %f\n"
	 "img->optm = %f\n",
	 img->nx,
	 img->ny,
	 img->nsl,
	 img->nl,
	 img->nbvals,
	 img->difmode,
	 img->nb0,
	 dgrad->nvert,
	 img->ndif,
	 dgrad->polytope,
	 dgrad->ntess,
	 img->bstep,
	 img->pwdramp,
	 img->gdx1_amp,
	 img->gdx1_pw,
	 img->gdx2_amp,
	 img->gdx2_pw,
	 img->gdy1_amp,
	 img->gdy1_pw,
	 img->gdy2_amp,
	 img->gdy2_pw,
	 img->gdz1_amp,
	 img->gdz1_pw,
	 img->gdz2_amp,
	 img->gdz2_pw,
	 img->pwgd,
	 img->Delta,
	 img->pwgd2,
	 img->Delta2,
	 img->agdx1,
	 img->agdy1,
	 img->agdz1,
	 img->optm);

  return(img) ;
}

/* Imaging initialization */

IMG_state *init_img_state( char *header, char *scanner_type ) 
{
  IMG_state *img = NULL;

  if(!strcmp(scanner_type,"SCANNER_TYPE_VARIAN"))
    img = init_img_state_VARIAN(header) ;

  if(!strcmp(scanner_type,"SCANNER_TYPE_GE_LX"))
    img = init_img_state_GE_LX(header) ;

  if(!strcmp(scanner_type,"SCANNER_TYPE_GE_EXCITE"))
    img = init_img_state_GE_EXCITE(header) ;

  return(img);
}

/* Diffusion state initialization */

DTI_state *init_dti_state( IMG_state *img , DSET_stats *b0stats )
{
  DTI_state *dti = (DTI_state *) malloc (sizeof(DTI_state)) ;

  if(img==NULL) {
    dti = NULL ;
  } else {
    dti->prc = init_dti_prc_state( img , b0stats ) ;
    dti->eig = init_dti_eig_state( dti->prc ) ;
    dti->lsq = init_dti_lsq_state( dti->prc ) ;
  }
  return (dti) ;
}

/* Diffusion state initialization */

DIF_state *init_dif_state( char *header, DSET_stats *b0stats, char *scanner_type )
{
  int nn = 0;
  DIF_state *dif = NULL;

  dif = (DIF_state *) malloc (sizeof(DIF_state));

  printf("scanner_type = %s\n",scanner_type);

  dif->sta = init_dif_status( dif ) ;

  if(header==NULL) {

    printf("init_dif_state:  Header is NULL!  Can't initialize init_img_state!\n");
    printf("...proceed at own risk!...\n");

  } else {

    if(!strcmp(scanner_type,EMPTY)) {

	dif->img = init_img_state_GE_LX( header ) ;

    } else {

      if(!strcmp(scanner_type,"SCANNER_TYPE_GE_LX")) {
	dif->img = init_img_state_GE_LX( header ) ;
      }
      if(!strcmp(scanner_type,"SCANNER_TYPE_GE_EXCITE")) {
	dif->img = init_img_state_GE_EXCITE( header ) ;
	if (!dif->img) {
	  free(dif);
	  return(0);
	}
      }
      if(!strcmp(scanner_type,"SCANNER_TYPE_VARIAN")) {
	dif->img = init_img_state_VARIAN( header ) ;
      }

    }

  }

  dif->dti = init_dti_state( dif->img , b0stats ) ;
  dif->shd = init_shd_state( dif->img ) ;
  dif->ftm = init_ftm_state( dif->dti, dif->shd ) ;

  return (dif) ;
}
