/**<init_SHD_state.c>****************************************************

     Title: INITialize SHD STATE

   Purpose: Initialize SHD structure

      Call: SHD_state *init_SHD_state( short lmax_calc , 
                                       short mmax_calc ,
				       short shd_grid_size ,
				       Boolean full_m_range )

**************************************************************************/
#include "diffuse.h"

/* Viewing State */

SHD_view_state *init_shd_view_state( SHD_calc_state *calc )
{
  SHD_view_state *view = (SHD_view_state *) malloc (sizeof(SHD_view_state));

  /* Full range -M:M */

  view->full_mview_range = calc->full_mcalc_range ;

  /* Min, Max, and Number of L values for viewing */

  view->Lmax = calc->Lmax ;
  view->Lmin = calc->Lmin ;
  view->nl   = calc->nl   ;

  /* Min, Max, and Number of M values for viewing */

  view->Mmax = calc->Mmax ;
  view->Mmin = calc->Mmin ;
  view->nm   = calc->nm   ;

  printf("init_shd_view_state, view (nl,nm) = (%i,%i)\n",
	 view->nl,view->nm) ;

  /* useful param - nl*nm */

  view->nlm  = view->nl * view->nm ;

  /* Array of L values for viewing */

  view->Lvals = (short *) malloc (sizeof(short) * view->nl) ;
  if( view->Lvals == NULL ) {
    printf("init_SHD_state:  Can't malloc workspace!\n");
    return NULL;
  }

  /* Array of M values for viewing */

  view->Mvals = (short *) malloc (sizeof(short) * view->nm) ;
  if( view->Mvals == NULL ) {
    printf("init_SHD_state:  Can't malloc workspace!\n");
    return NULL;
  }

  return (view) ;
}

/* Calculation State */

SHD_calc_state *init_shd_calc_state( IMG_state *img )
{
  int ii ;

  SHD_calc_state *calc = (SHD_calc_state *) malloc (sizeof(SHD_calc_state));

  /* Full range -M:M */

  calc->full_mcalc_range = CALC_full_mcalc_range_DEF ;

  /* Computational grid size */

  calc->shd_grid_size = CALC_shd_grid_size_DEF ;

  calc->npolr = calc->shd_grid_size ;
  calc->nazim = calc->shd_grid_size ;

  /* Min, Max, and Number of L values for calculations */

  calc->Lmax = CALC_Lmax_DEF ;
  calc->Lmin = CALC_Lmin_DEF ;
  calc->nl   = CALC_nl_DEF   ;

  /* Min, Max, and Number of M values for calculations */

  calc->Mmax = CALC_Mmax_DEF ;
  calc->Mmin = CALC_Mmin_DEF ;
  calc->nm   = CALC_nm_DEF   ;

  /* useful param - nl*nm */

  calc->nlm  = calc->nl * calc->nm ;

  /* Array of L and Mvalues for calculations */

  calc->Lvals = NULL ;
  calc->Mvals = NULL ;

  return (calc) ;
}

/* SHD state initialization */

SHD_state *init_shd_state( IMG_state *img )
{
  SHD_state *shd = (SHD_state *) malloc (sizeof(SHD_state)) ;

  shd->calc = init_shd_calc_state( img ) ;
  shd->view = init_shd_view_state( shd->calc ) ;

  return (shd) ;
}

