/**<lsqfit_dif.c>*******************************************************

     Title: Least SQuare FITting

   Purpose: Calculate regression coefficients "a" 
            relating data d to model F by d = a F;

      Call: void lsqfit_dif ( LSQStruct *lsqstr, float *data ) 

   Returns: LSQStruct *lsqstr (my Least SQuares STRUCTure)

    Author: Lawrence R. Frank

   History: 24-Feb-99 modified RWCox's plug_lsqfit.c 
                      for my own evil purposes.
            01-Nov-99 copied my ~/C/noise/lsqfit_pnr.c to lsqfit_dif.c
            02-Nov-99 return fit coeffs in newly created lsqstr->alpha[nref]

 Algorithm: This uses RWCox's Cholesky factorization routines

*/
#include "diffuse.h"

#define NRMAX_SIN 2
#define NRMAX_TS  2
#define HARM_MAX  11

#define DBLEVEC(ll) (double *) malloc( sizeof(double) * (ll) )
#define DISCARD(xx) if( xx != NULL ){free(xx); xx = NULL;}
#define CLEANUP     {DISCARD(cc); DISCARD(rr);}

static int polort=1 , ignore=3 , nrsin=0 , nrts=0 , initialize=1 ;

static char lbuf[4096] ;
static char sbuf[256] ;

void lsqfit_dif( DTI_lsq_state *lsqstr, float *data ) 
{
   int        nt = lsqstr->nt ;
   double     dt = lsqstr->dt ;
   float *signal = lsqstr->signal ;
   float  *noise = lsqstr->noise ;
   float  **tsim = lsqstr->tsim ;
   int      nrts = lsqstr->nrts ;
   int    ignore = lsqstr->ignore ;
   char   *label = lsqstr->label ;

   int nlen , nref ;

   static int nlen_old = -666 , nref_old = -666 ;
   static double dt_old = -666.666 ;
   static float **ref = NULL ;
   static double *dch = NULL ;

   int   ir , ii , ks, jh ;
   float fac , tm , val ;
   float *fit , *tsar ;

   int debug = 1;

   /*** compute how many ref functions are ordered ***/

   nref = nrts ;

   /*** do nothing if not enough data to fit ***/

   nlen = nt - ignore ;

   /* if( nlen <= nref ) { */
   if( nlen < nref ) {		/* I think this is ok, since no const vec, but check! */
            fprintf(stderr,
		    "Note enough data to fit in lsqfit_dif: (nlen=%i) < (nref=%i)\n\a",
		    nlen,nref) ;
	    return ;  /* do nothing if not enough data to fit */
   }

#define CC(i,j) cc[(i)+(j)*nref]

   /** if data vectors are new length,
       or have a new number of reference vectors,
       or have a new time step and need sinusoids,
       or the initialize flag is set,
       then reinitialize reference vectors and Choleski factor **/

   if( nlen != nlen_old || nref != nref_old ||
       initialize       || (dt != dt_old && nrsin > 0) ){

      /* free old storage */

      if( ref != NULL ){
         for( ir=0 ; ir < nref_old ; ir++ ) if( ref[ir] != NULL ) free(ref[ir]) ;
         free(ref) ;
      }
      if( dch != NULL ) free(dch) ;

      /* make space for ref vectors */

      ref = (float **) malloc( sizeof(float *) * nref ) ;
      if( ref == NULL ){fprintf(stderr,"\nmalloc error in plug_lsqfit\n\a");exit(1);}
      for( ir=0 ; ir < nref ; ir++ ){
         ref[ir] = (float *) malloc( sizeof(float) * nlen ) ;
         if( ref[ir] == NULL ) {fprintf(stderr,"\nmalloc error in plug_lsqfit\n\a");exit(1);}
      }
      nlen_old = nlen ;
      nref_old = nref ;
      dt_old   = dt ;

      /**** fill ref vectors ****/

      ir = 0 ;

      /* r(t) = timeseries files */

      for( ks=0 ; ks < nrts ; ks++ ){
         if( tsim[ks] == NULL ){
            initialize = 1 ;
            fprintf(stderr,"Inadequate time series #%d in lsqfit_dif\n\a",ks+1) ;
            return ;
         }
         tsar = tsim[ks] ;
         for( ii=0 ; ii < nlen ; ii++ ) {
	   ref[ir][ii] = tsar[ii+ignore] ;
	   if(debug) printf(" ref[%i][%i] = %f\n",ir,ii,ref[ir][ii]);
	 }
         ir++ ;
      }

      /* Cholesky-ize */

      dch = startup_lsqfit_dif( nlen , NULL , nref , ref ) ;
      if( dch == NULL ){
         initialize = 1 ;
         fprintf(stderr,"Choleski error in lsqfit_dif\n\a") ;
         return ;
      }

      initialize = 0 ;
   }

   /** find least squares fit coefficients **/

   fit = delayed_lsqfit_dif( nlen , data+ignore , nref , ref , dch ) ;

   /* 99_11_02 - lrf: save coeffs */
   lsqstr->nref  = nref;
   if( lsqstr->alpha == NULL ) 
     lsqstr->alpha = (float *) malloc( sizeof(float) * nref ) ; 
   for( ir=0 ; ir < nref ; ir++ ) lsqstr->alpha[ir] = fit[ir] ;

   lsqstr->dc = fit[0] * ref[0][0] ;
   for( ii=0 ; ii < nlen ; ii++ ){
      val = 0.0 ;
      for( ir=0 ; ir < nref ; ir++ ) val += fit[ir] * ref[ir][ii] ;

      /* vec[ii+ignore] = (dofit) ? val : data[ii+ignore] - val ; */
      signal[ii+ignore] = val ;
       noise[ii+ignore] = data[ii+ignore] - val ;
   }

   /** Create label  **/

   if( label != NULL ){  /* assemble this 1 line at a time from sbuf */

      label[0] = '\0' ;   /* make this a 0 length string to start */

      /** for each reference, make a string into sbuf **/

      ir = 0 ;
      sprintf(sbuf,"Coef of 1 = %g\n" , fit[ir++] ) ;
      strcat(label,sbuf) ;

      for( ks=0 ; ks < nrts ; ks++ ){
         sprintf(sbuf,"Coef of ts %i = %g\n" , ks+1 , fit[ir++] ) ;
         strcat(label,sbuf) ;
      }

   }

   free(fit) ;
   return ;
}

/* All unspoiled Bob Cox stuff below this point */

/*----------------------------------------------------------------
   Similar to above, but only produce the Choleski decomposition
   and weight scaled ref vectors for later use in delayed_lsqfit_dif.
   This is to be used when fitting several data vectors to the
   same references with the same weight factors.

 veclen = length of vectors
 wt     = array holding weight for each data point (length veclen)
          [if NULL, then a vector of all 1's is used]
 nref   = number of reference vectors, each of length veclen;
          must have 1 <= nref <= veclen
 ref    = array of pointers to reference vectors, so that
          ref[i][k] is the k-th component of the i-th reference,
          for i=0..nref-1, k=0..veclen-1

   Return value is a (double *) which points to malloc-ed memory
   for the Choleksi decomposition.  It should later be passed
   to delayed_lsqfit_dif.  If wt != NULL, then ref[ii][jj] is
   scaled by wt[jj] as well.

   If NULL is returned, an error occured.
------------------------------------------------------------------*/

double *startup_lsqfit_dif( int veclen, float *wt, int nref, float *ref[] )
{
   char *program_name = "startup_lsqfit_dif" ;
   int    ii , jj , kk ;
   double *cc = NULL ;
   double sum ;
   int debug = 0 ;

   if( nref < 1 || veclen < nref || ref == NULL ){
      fprintf(stderr,"*** Illegal inputs to %s\n",program_name) ;
      return NULL ;
   }

   /*** form coefficient matrix into cc */

   cc = DBLEVEC(nref*nref) ;
   if( cc == NULL ){
      fprintf(stderr,"Can't malloc workspace in %s\n",program_name) ;
      return NULL ;
   }

   if( wt != NULL ){
      for( jj=0 ; jj < nref ; jj++ ){
         for( ii=0 ; ii <= jj ; ii++ ){
            sum = 0.0 ;
            for( kk=0 ; kk < veclen ; kk++ ) sum += ref[ii][kk] * ref[jj][kk] * wt[kk] ;
            CC(ii,jj) = CC(jj,ii) = sum ;
         }
      }
   } else {
      for( jj=0 ; jj < nref ; jj++ ){
         for( ii=0 ; ii <= jj ; ii++ ){
            sum = 0.0 ;
            for( kk=0 ; kk < veclen ; kk++ ) sum += ref[ii][kk] * ref[jj][kk] ;
            CC(ii,jj) = CC(jj,ii) = sum ;
         }
      }
   }

   /*** Choleski decompose cc ***/

   for( ii=0 ; ii < nref ; ii++ ){
      for( jj=0 ; jj < ii ; jj++ ){
         sum = CC(ii,jj) ;
         for( kk=0 ; kk < jj ; kk++ ) sum -= CC(ii,kk) * CC(jj,kk) ;
         CC(ii,jj) = sum / CC(jj,jj) ;
      }
      sum = CC(ii,ii) ;
      for( kk=0 ; kk < ii ; kk++ ) sum -= CC(ii,kk) * CC(ii,kk) ;
      if(debug) fprintf(stderr,"sum = %f\n",sum) ;fflush(stdout);
      if( sum <= 0.0 ){
         free(cc) ;
         fprintf(stderr,"Choleski factorization failure in %s\n",program_name) ;
         return NULL ;
      }
      CC(ii,ii) = sqrt(sum) ;
   }

   /*** scale ref by wt, if desired ***/

   if( wt != NULL ){
      for( ii=0 ; ii < nref ; ii++ )
         for( jj=0 ; jj < veclen ; jj++ ) ref[ii][jj] *= wt[jj] ;
   }

   return cc ;
}

/**------------------------------------------------------------------
  Given the data from startup_lsqfit, finish the job.
------------------------------------------------------------------**/

float *delayed_lsqfit_dif( int veclen ,
			   float *data , int nref , float *ref[] , double *cc )
{
   char *program_name = "delayed_lsqfit_dif" ;
   int    ii , jj ;
   float  *alpha = NULL ;
   double *rr = NULL ;
   register double sum ;

   if( nref < 1 || veclen < nref ||
       data == NULL || ref == NULL || cc == NULL ) return NULL ;

   /*** form RHS vector into rr ***/

   rr = DBLEVEC(nref) ; if( rr == NULL ) return NULL ;

   for( ii=0 ; ii < nref ; ii++ ){
      sum = 0.0 ;
      for( jj=0 ; jj < veclen ; jj++ ) sum += ref[ii][jj] * data[jj] ;
      rr[ii] = sum ;
   }

   /*** forward solve ***/

   for( ii=0 ; ii < nref ; ii++ ){
      sum = rr[ii] ;
      for( jj=0 ; jj < ii ; jj++ ) sum -= CC(ii,jj) * rr[jj] ;
      rr[ii] = sum / CC(ii,ii) ;
   }

   /*** backward solve ***/

   for( ii=nref-1 ; ii >= 0 ; ii-- ){
      sum = rr[ii] ;
      for( jj=ii+1 ; jj < nref ; jj++ ) sum -= CC(jj,ii) * rr[jj] ;
      rr[ii] = sum / CC(ii,ii) ;
   }

   /*** put result into alpha ***/

   alpha = (float *) malloc( sizeof(float) * nref ) ; if( alpha == NULL ) return NULL ;
   for( ii=0 ; ii < nref ; ii++ ) alpha[ii] = rr[ii] ;

   /*** cleanup and exit ***/

   free(rr) ;
   return alpha ;
}
