/**<shd.c>*******************************************************

     Title: Spherical Harmonic Decomposition

   Purpose: Decompose diffusion data with SHT

    Author: Lawrence R. Frank

   History: 22-May-01 initial version
            06-Jan-02 spherical weights computation 
*/
#include "diffuse.h"

/**<init_shd_scalars.c>*******************************************************

     Title: INITialize SCALARS

      Call: int init_shd_scalars( DIF_state *dif )

   Purpose: Allocate space for shd_scalars.c

     Input: DIF_state *dif

    Output: prc->fv(ndif) 

******************************************************************************/

int init_shd_scalars( int METH_SHD_SCALAR , DIF_state *dif )
{
  DIF_status    *sta = dif->sta ;
  IMG_state     *img = dif->img ;
  /* SHD_prc_state *prc = dif->dti->prc ;*/ /* doesn't exist but might want in future */
  int ival ;

  switch ( METH_SHD_SCALAR ) {

      default: 
	printf("\ashd_scalars: Unrecognized value %i of METH_SHD_SCALAR! \n",
	       METH_SHD_SCALAR);
	printf("...returning...\n");
	return(-1);

      case METH_SHD_FIB:
	
	/* doesn't exist but might want in future */
	/* if (prc->dv==NULL ) {
	  ival = 0 ;
	} else {
	  ival = 1 ;
	  }*/

	ival = 1 ;		/* tmp */
	sta->shd_ftm_initialized = ival;
	break;

      case METH_SHD_EOS:

	/* doesn't exist but might want in future */
	/* if (prc->dv==NULL ) {
	  ival = 0 ;
	} else {
	  ival = 1 ;
	  }*/

	ival = 1 ;		/* tmp */
	sta->shd_eos_initialized = ival;
	break;

      case METH_SHD_FMI:

	/* doesn't exist but might want in future */
	/* if (prc->dv==NULL ) {
	  ival = 0 ;
	} else {
	  ival = 1 ;
	  }*/

	ival = 1 ;		/* tmp */
	sta->shd_fmi_initialized = ival;
	break;

      case METH_SHD_GRA:

	/* doesn't exist but might want in future */
	/* if (prc->dv==NULL ) {
	  ival = 0 ;
	} else {
	  ival = 1 ;
	  }*/

	ival = 1 ;		/* tmp */
	sta->shd_gra_initialized = ival;
	break;

      case METH_SHD_GFA:

	/* doesn't exist but might want in future */
	/* if (prc->dv==NULL ) {
	  ival = 0 ;
	} else {
	  ival = 1 ;
	  }*/

	ival = 1 ;		/* tmp */
	sta->shd_gfa_initialized = ival;
	break;
  }

  return ( ival );
}

/**<init_swd.c>*******************************************************

     Title: INITialize SWD

      Call: int init_shd( DIF_state *dif )

   Purpose: Allocate space for spherical wave stuff

     Input: DIF_state *dif

    Output: 

***********************************************************************/

int init_swd( DIF_state *dif )
{
  char *program_name = "init_swd";
  int debug  = 1 ;
  int debug2 = 0 ;

  IMG_state       *img = dif->img ;
  Grad          *dgrad = img->dgrad ;
  SHD_calc_state *calc = dif->shd->calc ;

  calc->nrad = img->nbvals ;	/* WHAT IS THIS SUPPOSED TO BE?? */
  calc->Rmin = 0.1 ;		/* WHAT IS THIS SUPPOSED TO BE?? */
  calc->Rmax = 1.0 ;		/* WHAT IS THIS SUPPOSED TO BE?? */

  calc->Wlmr = wlmr_alloc(calc->Lmax,calc->nrad,dgrad->nvert) ;
  calc->Almr = almN_alloc(calc->Lmax,calc->nrad) ;
  calc->jrad = fv_alloc(calc->nrad);
      
  /* Construct radii for spherical bessel arg */
  vlinsp(calc->jrad,calc->Rmin,calc->Rmax,calc->nrad);
      
  printf("%s: Constructing Spherical Waves ...\n",program_name);fflush(stdout);
  
  if(debug) {
    printf("%s: calc->nrad = %i\n"
	   "    (calc->Lmin,calc->Lmax) = (%i,%i)\n"
	   "    dgrad->nvert = %i\n",
	   program_name,
	   calc->nrad,
	   calc->Lmin,calc->Lmax,
	   dgrad->nvert);
    fflush(stdout);
  }

  SphericalWavesWlmr( calc->Wlmr, calc->jrad, calc->nrad, 
		      calc->Lmin,calc->Lmax, 
		      calc->polr, calc->azim, dgrad->nvert) ;

  if(debug2) printWlmr(calc->Wlmr,calc->Lmax,dgrad->nvert,calc->nrad);

}

/**<init_shd.c>*******************************************************

     Title: INITialize SHD

      Call: int init_shd( DIF_state *dif )

   Purpose: Allocate space for shd.c

     Input: DIF_state *dif

    Output: 

***********************************************************************/

int init_shd( DIF_state *dif )
{
  char *program_name = "init_shd";
  int i ;
  int debug = 0 ;

  SHD_calc_state *calc = dif->shd->calc ;

  /* Still need some (but not all!) dti stuff for Dapp */

  DIF_status    *sta = dif->sta ;
  IMG_state     *img = dif->img ;
  Grad        *dgrad = img->dgrad ;
  DTI_prc_state *prc = dif->dti->prc ;
  DTI_lsq_state *lsq = dif->dti->lsq ;

  prc->Dapp  = fm_alloc( img->nxy  , prc->nten  ); 
  prc->Bmat = fm_alloc( img->ndif*img->nbvals , lsq->lsqdim );
  prc->Evals = NULL ;
  prc->fv    = NULL ;
  prc->dv    = NULL ;

  sta->dti_initialized = 0;	/* because last three NULL's will cause
				   crash if we call DTI next! */

  /* L and M ranges set from control box
     prior to init_shd by call to
     get_spharm_control_params */

  /* Check input */

  if(debug) printf("%s: calc->Lmax = %i, calc->Mmax = %i\n",
		   program_name,calc->Lmax,calc->Mmax);

  if (calc->Lmax < 0) {
    printf("Error! SpharmTransform: \a");
    printf("Lmax=%i must be >= 0 \n",calc->Lmax);
    return -1;
  }
  if (calc->Mmax > calc->Lmax) {
    printf("Error! SpharmTransform: \a");
    printf("Mmax=%i not in range [0,Lmax] = [%i,%i]\n",
	   calc->Mmax,0,calc->Lmax);
    return -1;
  }

  /* Number of L values for calculations */

  calc->nl = calc->Lmax + 1 ; /* 0,...,Lmax */

  /* Number of M values for calculations */

  if ( calc->full_mcalc_range ) {
    calc->nm = 2*calc->Mmax + 1 ; /* -Mmax,...,0,...,Mmax */
  } else {
    calc->nm = calc->Mmax + 1 ;   /*           0,...,Mmax */
  }

  /* Calculate number of components in Alm array (it's _not_ nl*nl!!!) */
  calc->nlm = 0 ;
  for(i=0; i<calc->nl; i++) {
    calc->nlm += 2*i + 1 ;
  }

  if(debug) printf("%s: calc->nl = %i, calc->nm = %i, calc->nlm = %i\n",
		   program_name,calc->nl,calc->nm,calc->nlm);

  /* Array of L values for calculations */

  calc->Lvals = (short *) malloc (sizeof(short) * calc->nl) ;
  if( calc->Lvals == NULL ) {
    printf("init_SHD_state:  Can't malloc workspace!\n");
    return -1;
  }
  for(i=0; i<calc->nl; i++) calc->Lvals[i] = i;

  /* Array of M values for calculations */

  calc->Mvals = (short *) malloc (sizeof(short) * calc->nm) ;
  if( calc->Mvals == NULL ) {
    printf("init_SHD_state:  Can't malloc workspace!\n");
    return -1;
  }
  for(i=0; i<calc->nm; i++) calc->Mvals[i] = i;

  /* Even/odd L sum channels (probably always = 2) */
  calc->neos = 2;		/* 02_03_19 - for METH_SHD_EOS */

  /* Create angular ranges */

  calc->min_polr = 0. ;
  calc->max_polr = PI ;
  calc->min_azim = 0. ;
  calc->max_azim = TWOPI ;

  calc->nvert = dgrad->nvert;	/* number of points on sphere = ndif */
  calc->npts  = dgrad->nvert;	/* redundant - need?? (ndif is nvert + 1) */
  calc->nb0   = img->nb0;	/* number of b=0 measurements */

  /* Careful!  This is what it used to be!  
    There seems to be a difference between requested and
    the number in the gradient file!! */
  /* done in sphere_parms now!!! */

  /* calc->polr = fv_alloc( calc->npolr ) ; */ /*     polar angles */
  /* calc->azim = fv_alloc( calc->nazim ) ; */ /* azimuthal angles */

  /* vlinsp( calc->polr , calc->min_polr , calc->max_polr , calc->npolr );
     vlinsp( calc->azim , calc->min_azim , calc->max_azim , calc->nazim ); */

  /* The following is now done with Voronoi weights.  Axe when ready */

  /* Compute measure dtheta dphi (ie, dpolr dazim) 
     I've gotten rid of calc->dazim, calc->dpolr, calc->dthdph 
     in dif_structs.h */

  /* calc->dazim  = (calc->max_azim - calc->min_azim)/(calc->nazim-1.);
     calc->dpolr  = (calc->max_polr - calc->min_polr)/(calc->npolr-1.);
     calc->dthdph = fv_alloc( calc->npts ) ;
     for(i=0; i<calc->npts; i++) 
     calc->dthdph[i] = sin(calc->polr[i]) * calc->dpolr * calc->dazim; 
     } */

  /* Determine the spherical coordinates 
     from the input gradient directions.
     Determine the weights for the SHT. */
 
  sphere_parms( dif ) ;

  /* Determine the spherical harmonic transform
     scaling coefficients. */
 
  calc->Ylm_scale = rlm_alloc( calc->Lmax ); 
  ylm_scale( dif ); 

  /* Allocate for spherical harmonics */

  calc->Ylm = ylm_alloc( calc->Lmax, calc->npts ) ;

  if(debug) printYlm(calc->Ylm,calc->Lmax, calc->npts ) ;

  printf("\nComputing spherical harmonics:\n"
	 "(Lmin,Lmax)=(%i,%i) , (Mmin,Mmax)=(%i,%i)\n",
	 calc->Lmin,calc->Lmax,calc->Mmin,calc->Mmax);
  printf("nl = %i, nm = %i, nlm = %i\n", calc->nl, calc->nm, calc->nlm);

  /* Calculate spherical harmonics Ylm */

  SphericalHarmonicsYlm( calc->Ylm, calc->Lmin, calc->Lmax, 
			 calc->polr, calc->azim, calc->npts);

  if(debug) printYlm(calc->Ylm,calc->Lmax, calc->npts ) ;

  /* Allocate for spherical harmonic coefficients Alm */

  calc->Alm = alm_alloc( calc->Lmax ) ;

  /* Allocate for sumL, the sum of Alm over M for each L (sumL[0,..,Lmax]) */

  calc->sumL = zv_alloc( calc->nl) ;

  /* Allocate for egyL, the energy in the l'th channel of Alm (sumL[0,..,Lmax]) */

  calc->egyL = fv_alloc( calc->nl) ;

  /* Allocate for apparent diffusion tensor */

  calc->Dap = fm_alloc( calc->nl, calc->nm ); 

  /* Float work array */
  calc->zwrk = zv_alloc( calc->nlm ) ;/* the length of this was chosen _arbitrarily_ !! */
  calc->fwrk = fv_alloc( calc->nlm ) ;/* the length of this was chosen _arbitrarily_ !! */
  calc->zv   = zv_alloc( img->nbvals * img->ndif ) ; /* work space */
  calc->fv   = fv_alloc( img->nbvals * img->ndif ) ; /* work space */
  calc->fm   = fm_alloc( img->nbvals , img->ndif ) ; /* work space */
  calc->dv   = dv_alloc( img->nbvals * img->ndif ) ; /* work space */

  /* Initialize SWD stuff.  16Jul04, lrf */

  if(img->nbvals>1) {
    printf("%s: Number of bvalues %i > 1: Initializing swd\n",program_name,img->nbvals);
    init_swd(dif) ;
  }

  /* Check output */

  if (calc->polr==NULL ||
      calc->azim==NULL ||
      calc->wt  ==NULL ||
      calc->Dap ==NULL || 
      calc->Alm ==NULL || 
      calc->Ylm ==NULL) 
    {
      printf("%s: Can't initialize!\n\a",program_name);
      sta->shd_initialized = 0;
      return(0) ;

    } else {

      sta->shd_initialized = 1;
      return(1) ;
    }

}

/**<ylm_scale.c>*******************************************************

     Title: Ylm SCALE coefficients for SHT

   Purpose: 

      Call: int ylm_scale( DIF_state *dif ) ;

     Input: 
    Output: 

 Algorithm: 

  Return value is 0 if an error transpired, or 1 if all went OK.

***************************************************************************/

int ylm_scale( DIF_state *dif ) 
{
  int ll,mm;
  int debug = 0;

  SHD_calc_state *calc = dif->shd->calc ;

  for( ll=calc->Lmin ; ll<=calc->Lmax ; ll++ ) {
    for( mm=-ll ; mm<=ll ; mm++ ) {
      calc->Ylm_scale[ll][mm] = YLM_COEF(ll,ABS(mm)) ;
      if(debug) printf("calc->Ylm_scale[%i][%i] = %g\n",ll,mm,calc->Ylm_scale[ll][mm]);
    }
  }

  return(1);
}

/**<sphere_parms.c>*******************************************************

     Title: SPHERE PARaMeterS

   Purpose: Convert vector of diffusion measurements ds[n]
            into vector of measurements on the sphere dsp[{polr[n],azim[n]}]
	    as a function of the polar and azimuthal angles, as determined
	    by the gradient directions.

      Call: int sphere_parms( DIF_state *dif ) ;

     Input:  ds[npts] = measured dw signal
             dif->calc->polr[npts] = polar angles of measurements
             dif->calc->azim[npts] = azimuthal angles of measurements

             dif->calc->polr_interp[npts] = polar angles to interpolate onto
             dif->calc->azim_interp[npts] = azimu angles to interpolate onto

    Output: dsp[npts] = 

 Algorithm: Convert input gradient vectors (root.GX,root.GY,root.GZ)
            into unit spherical polar coordinates (1,polar,azim)
	    and interpolate onto spherical surface by computing the
	    Voronoi area associated with each point.  This is done 
	    using the routine "sphere_voronoi_angles", written by
	    RWCox himself, which computes the convex hull triangular
	    facets between the points.

  Return value is 0 if an error transpired, or 1 if all went OK.

***************************************************************************/

int sphere_parms( DIF_state *dif ) 
{
  IMG_state      *img  = dif->img ;
  SHD_calc_state *calc = dif->shd->calc ;
  cpoint cpt ;
  spoint spt ;
  int i,ii ;
  double thresh = 1.e-5 ;

  /* Allocate for sphere parms */

  calc->wt     = fv_alloc( calc->nvert ); 
  calc->sphdat = fv_alloc( calc->nvert ); /* this will be used later to hold
					     the measurements from the voxel
					     under consideration */
  calc->polr   = fv_alloc( calc->nvert ); 
  calc->azim   = fv_alloc( calc->nvert ); 

  /* Compute angles from gradient directions */

  for(i=0; i<calc->nvert; i++) {
    ii  = i - 1;
    cpt = newcpt( img->GX[i], img->GY[i], img->GZ[i] );
    spt = cpt2spt( cpt );
    calc->polr[i] = spt.pol ;
    calc->azim[i] = spt.azi ;
    if((double) ABS(spt.rad-1.0)>thresh) {
      sprintf(tmpbuf,"Radius %g determined from gradients not unity (diff=%g)!\n",
	      spt.rad,ABS(spt.rad-1.0));
      WARNING(tmpbuf) ;
    }
  }

  /* Compute weights */

  if(!sphere_voronoi_angles( calc->nvert, calc->polr, calc->azim, &calc->wt )) 
    {
      sprintf(tmpbuf,"Error in sphere_voronoi_angles");
      NEAR_FATAL_ERROR(tmpbuf) ;
    }

  return(1);

}
