#ifndef _MCW_3DDIFMAKER_HEADER_
#define _MCW_3DDIFMAKER_HEADER_

#include "afni.h"

#ifndef FREEUP
#define FREEUP(x) do{if((x) != NULL){free((x)); (x)=NULL;}}while(0)
#endif

/* 
   The following two are for using a single number as indeces:
   Example:  INDX1(29) = 2, INDX2(29) = 9 
*/
#define INDX1(x) ((int) floor((x)/10.))
#define INDX2(x) ((int) fmod((x),10.))
#define TENSOR_METH(x)     INDX1((x)) /* Tensor method (DTI or SHD) */
#define TENSOR_SUB_METH(x) INDX2((x)) /* Tensor sub-method (RAI,FIB0, etc) */

/* Diffusion processing methods 
   The first method in each category is now the actual tensor
   processing method:
           METH_DTI_DTI = full diffusion tensor calculation
           METH_SHD_SHD = full spherical harmonic transform */

#define METH_DTI_OFFSET 10	
#define METH_SHD_OFFSET 20	

#define METH_DTI  10		/* Diffusion Tensor Imaging 
				   (shorthand for METH_DTI_DTI) */
#define METH_SHD  20		/* Spherical Harmonic Decomposition
				   (shorthand for METH_SHD_SHD) */
#define METH_SWD  30		/* Spherical Wave Decomposition
				   (shorthand for METH_SWD_SWD) */

/* These are of the form (METH_DTI,METHOD) 
   ie, METH_DTI=1,METH_DTI_RAI=2 -> 12
   So retrieval takes place as:  INDX1(12)=1 (ie METH_DTI), 
                                 INDX2(12)=2 (ie METH_DTI_RAI) */

#define N_DTI_SCALAR_METH 5	/* all but first below */
#define METH_DTI_DTI  10	/* Diffusion Tensor calculation */
#define METH_DTI_AVE  11	/* average diffusion coefficient */
#define METH_DTI_FAI  12	/* fractional anisotropy index */
#define METH_DTI_RAI  13	/* relative anisotropy index */
#define METH_DTI_LAT  14	/* lattice anisotropy index */
#define METH_DTI_SDV  15	/* spherical diffusion variance */

static char *DTI_OUTPUT_PREFIX_CHOICES[ N_DTI_SCALAR_METH + 1] 
                     = { "DTI","AVE","FAI","RAI","LAT","SDV" } ;

#define N_SHD_SCALAR_METH 5	/* all but first below */
#define METH_SHD_SHD   20	/* Spherical Harmonic Decomposition */
#define METH_SHD_FIB   21	/* Fiber channels */
#define METH_SHD_EOS   22	/* Even/Odd L Sums */
#define METH_SHD_FMI   23	/* Fractional Multifiber Index */
#define METH_SHD_GRA   24	/* Generalized Relative Anisotropy index */
#define METH_SHD_GFA   25	/* Generalized Fractional Anisotropy index */
#define METH_SHD_GFAZ   26	/* Generalized Fractional Anisotropy index 
                                   (complex) */
#define METH_SHD_GFAOZ   27	/* Original Generalized Fractional Anisotropy
				   index, but stored as complex */

/* static int METH_SCALAR_DIM[] = {NL*NM,3,1}; */

static char *SHD_OUTPUT_PREFIX_CHOICES[ N_SHD_SCALAR_METH + 1] 
                     = { "SHD","FIB","EOS","FMI","GRA","GFA"} ;

#define N_SWD_SCALAR_METH 5	/* all but first below */
#define METH_SWD_SWD   30	/* Spherical Harmonic Decomposition */
#define METH_SWD_FIB   31	/* Fiber channels */
#define METH_SWD_EOS   32	/* Even/Odd L Sums */
#define METH_SWD_FMI   33	/* Fractional Multifiber Index */
#define METH_SWD_GRA   34	/* Generalized Relative Anisotropy index */
#define METH_SWD_GFA   35	/* Generalized Fractional Anisotropy index */

/* static int METH_SCALAR_DIM[] = {NL*NM,3,1}; */

static char *SWD_OUTPUT_PREFIX_CHOICES[ N_SWD_SCALAR_METH + 1] 
                     = { "SWD","FIB","EOS","FMI","GRA","GFA"} ;

/* Function prototypes */

extern THD_3dim_dataset * 
MAKER_4D_to_typed_difbuc( Widget wcancel , /* needed for cancelling computation */
			  Widget shell ,   /* needed for meter */
			  THD_3dim_dataset * old_dset , byte *mask ,
			  char * new_prefix , int new_datum ,
			  generic_func * user_func ,
			  int method , int *cancelcode ,
			  void * user_data ) ;

extern THD_3dim_dataset *
MAKER_4D_to_typed_swdbuc( Widget wcancel , /* 17-Jul-04 */
			  Widget shell ,
			  THD_3dim_dataset *old_dset ,
			  byte *mask ,
			  char *new_prefix , 
			  int new_datum ,
			  generic_func *user_func ,
			  int method , 
			  int *cancelcode ,
			  void *user_data ) ;

void DIFF_aniso_meth( int     method , 
		      float   fs[] , 
		      complex cs[] , 
		      void    *udp , 
		      int     nbrick , 
		      float   *val ,
		      complex *cval ) ;

#endif
