/**<tstdif>**************************************************

  Title: TeST DIFtor routines

*/
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/fcntl.h> 

#include "./dif_patterns.h"
#include "../vector/matrix.h"

/*! 3x3 matrix of floats. */

typedef struct { float mat[3][3] ; } THD_mat33 ;
static THD_mat33 tempA_mat33 ;

/* Parse an integer inum into n separate numbers
   eg, if inum = 43512, then n = 5, and
   iv[0-4]={4,3,5,1,2} */
int *iparse(int inum, int n)
{
  int i ;

  int *iv = (int *) calloc(n,sizeof(int));

  for(i=0; i<n; i++) {
    iv[i] = floor( (inum % (int) pow(10,n-i))/pow(10,n-1-i) );
  }

  return(iv) ;
}

/* Concatenate integer from n separate numbers
   eg, if iv={4,3,5,1,2}, then n=5 and
   inum = 43512 */
int icat(int *iv,int n)
{
  int i ;
  int inum = 0;

  for(i=0; i<n; i++) {
    inum += iv[i] * pow(10,n-i-1) ;
  }

  return(inum) ;
}
/******************************************************************************

     Title: BASSER mode diffusion tensor reconstruction

   Purpose: Compute diffusion tensor for a single voxel

      Call: float *basser( float *D , float *data, DIF_state *dif )

     Input: D[3][3] = diffusion tensor 
            data[7] = data[0] (b=0), data[1-6] (6 encoded directions)
    Output: D = diffusion tensor subvector (ie, 9 element vector)

 Algorithm: Basser, JMR SerB 103:247-254 (1994)

******************************************************************************/

THD_mat33 basserD( float *data, float bval )
{
  int   i;
  float S0 = data[0];		/* b=0 assumed to be in first slot */
  float Si ;			/* 6 diff directions (data[1-6]) */

  THD_mat33 sum_mat33,tmp_mat33 ;

  CLEAR_MAT33(sum_mat33) ;

  for (i=0; i<6; i++) {

    Si = (log(S0) - log(data[i+1])) / bval ;
    printf("S[%i] = %f\n",i,Si);
    MAT2THD(tmp_mat33,basser_dual[i]) ;
    SCALE_MAT33(tmp_mat33,Si);
    PRINT_MAT33("basserD: data",tmp_mat33);
    ACCUM_MAT33(sum_mat33,tmp_mat33) ;
  }
  PRINT_MAT33("basserD: sum",sum_mat33);

  return (sum_mat33) ;
}

#define TSTCHR 0
#define TSTMOD 0
#define TSTPAT 1
#define TSTFAI 1

#define RFA_SCALE sqrt(3./2.)

main()
{
  int i,j,l,m,n;

  if(TSTFAI) {			/* Test fai */

    float trc,det,detD,fa,ra ;
    THD_mat33 D,I;

    IDENTITY_MAT33(I) ;
    
    float bval = 500.;
    static float fs[] = {100., 1., 2., 3., 4., 5., 6.};

    D = basserD( fs, bval ) ;
    PRINT_MAT33("D",D);
    trc = TRACE_MAT33(D) ;
    SCALE_MAT33(I,trc/.3) ;
    SUB_MAT33(D,D,I) ;
    
    fa = RFA_SCALE * DET_MAT33(D) / trc ; /* fractional anisotropy */
    printf("fa = %f\n",fa);
  }

  if(TSTMOD) {			/* Test mod */
    int k = 43512 ;
    int n = 5;			/* length of k */
    int inum,*iv ;

    printf("original number = %i\n",k);

    iv = iparse(k,n) ;
    
    printf("Parsed number:\n");
    for(i=0; i<n; i++) {
      printf("iv = %i\n",iv[i]);
    }

    inum = icat(iv,n) ;
    printf("Concated number: %i\n",inum);

  }
  if(TSTPAT) {			/* Test patterns */
    char bstr[80] ;
    float val ;
    THD_mat33 tmp_mat33,fmat33,ident33,I ;

    n = 6 ;

    IDENTITY_MAT33(I);
    PRINT_MAT33("identity",I);

    for(i=0; i<n; i++) {
      printf("\n");
      sprintf(bstr,"basser_dual[%i]",i+1) ;
      MAT2THD(tmp_mat33,basser_dual[i]);
      PRINT_MAT33(bstr,tmp_mat33);
    }

    printf("\n");
    for(i=0; i<n; i++) {
      MAT2THD(tmp_mat33,basser_dual[i]);
      SCALE_MAT33(tmp_mat33,1);
      ACCUM_MAT33(fmat33,tmp_mat33) ;
    }
    sprintf(bstr,"accumlated matrix") ;
    PRINT_MAT33(bstr,fmat33);

    val = DET_MAT33(fmat33) ;
    printf("val = %f\n",val);

    /* CLEAR_MAT33(fmat33);
       IDENTITY_MAT33(ident33);
       smat33 = ADD_MAT33(ident33,basser_dual[i]);
       PRINT_MAT33("test of ADD_MAT33",smat33); */

  }

  if(TSTCHR) {			/* Test trig */

    float *x,*y1,*y2;
    n = 10;
    x = (float *) calloc((unsigned)n,sizeof(float));
    y1 = (float *) calloc((unsigned)n,sizeof(float));
    y2 = (float *) calloc((unsigned)n,sizeof(float));

    printf("Calling vcos(x),vsin(x) =\n");
    for(i=0; i< n; i++) {
      x[i] = (float)i/(float)n;
      printf("x[%i] = %lf\n",i,x[i]);
    }

    printf("TSTCHR: done !\n");
  }

}
