/**<PrintPfileHeaderSummary>**

     Title: PRINT PFILE HEADER SUMMARY

    Author: R Birn, NIH

   History: 02_01_06 - lrf Return dimensions string to use for to3d
*/
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/fcntl.h>
#include "toolsdata.h"
#include "imagedb.h"
#include "rdbm.h"

#define RMB_DEBUG 0

/* Pulse sequence type definitions */

#define HYPER_SEQ 0
#define   STE_SEQ 1
#define    SE_SEQ 2
#define    GE_SEQ 3

void PrintPfileHeaderSummary( char *header, char *dimstr )
{
   POOL_HEADER       *ph;
   RDB_HEADER_REC    *hr;
   MRIMAGEDATATYPE   *hr_im;                 /* image header */
   EXAMDATATYPE      *hr_ex;                 /* exam header */
   SERIESDATATYPE    *hr_se;                 /* series header */
   
   int i;
   
   char *start_x_ras, *start_y_ras;
   char *end_x_ras, *end_y_ras;
   float start_x_loc, start_y_loc;
   float end_x_loc, end_y_loc;
   
   float tlhc_R, tlhc_A, tlhc_S;
   float brhc_R, brhc_A, brhc_S;
   
   char *dset_type;
   char *freq_dir;
   
   float tr;
   
   ph    = (POOL_HEADER     *) header;
   hr    = (RDB_HEADER_REC  *) &(ph->rdb_hdr_rec);
   hr_im = (MRIMAGEDATATYPE *) &(ph->rdb_hdr_image);
   hr_se = (SERIESDATATYPE  *) &(ph->rdb_hdr_series);
   hr_ex = (EXAMDATATYPE    *) &(ph->rdb_hdr_exam);
   
   printf("\n");
   printf("#####################################################\n");
   printf("#                SUMMARY INFORMATION                 \n");
   printf("#                                                    \n");
   printf("# Scan Date: %s\n", hr->rdb_hdr_scan_date);
   printf("# Scan Time: %s\n", hr->rdb_hdr_scan_time);
   printf("# Hospital:  %s\n", hr_ex->hospname);
   printf("#                                                    \n");
   printf("# Patient ID:   %s\n", hr_ex->patid);
   printf("# Patient Name: %s\n", hr_ex->patname);
   printf("#                                                    \n");
   printf("# Exam #: %i\n", hr_ex->ex_no);
   printf("# Exam Description: %s\n", hr_ex->ex_desc);
   printf("#                                                    \n");
   printf("# Series #: %i\n", hr_se->se_no);
   printf("# Series Description: %s\n", hr_se->se_desc);
   printf("# Protocol: %s\n", hr_se->prtcl);
   printf("#                                                    \n");
   printf("# PSD name: %s\n", hr_im->psd_iname);
   printf("#                                                    \n");
   printf("#####################################################\n");
   
   
   /*** Compute geometry information ***/
   printf("\n");
   
   /*---Plane---*/
   printf("Scan plane:           ");
   switch(hr_im->plane) {
      default: printf("Unknown");
      case 2:  printf("Axial"); break;
      case 4:  printf("Sagittal"); break;
      case 8:  printf("Coronal"); break;
   }
   printf("\n");
   
   /*---Slice thickness---*/
   printf("Slice thickness (mm): %f\n", hr_im->slthick);
   
   /*---Field of View---*/
   printf("Field of View (mm):   %f\n\n", hr_im->dfov);
   
   /*---In-plane dimensions---*/
   
   /* for NIH epi */
   tlhc_R = hr_im->tlhc_R;
   tlhc_A = hr_im->tlhc_A;
   tlhc_S = hr_im->tlhc_S;
   brhc_R = hr_im->brhc_R;
   brhc_A = hr_im->brhc_A;
   brhc_S = hr_im->brhc_S;
   
   switch(hr_im->plane){
   
      case 2:       /*axial*/     
         if(hr_im->freq_dir == 1){  /*R-L*/
	    start_x_ras = (brhc_R >  0) ? "R" : "L";
	    start_x_loc = fabs(brhc_R);
	    end_x_ras =   (tlhc_R >  0) ? "R" : "L";
	    end_x_loc =   fabs(tlhc_R);
	    
	    start_y_ras = (brhc_A >  0) ? "A" : "P";
	    start_y_loc = fabs(brhc_A);
	    end_y_ras =  (tlhc_A >  0) ? "A" : "P";
	    end_y_loc =  fabs(tlhc_A);
	    
	    freq_dir = "R/L";
	    
	 }
	 else if(hr_im->freq_dir == 2){   /*A-P*/
	    start_x_ras = (tlhc_A >  0) ? "A" : "P";
	    start_x_loc = fabs(tlhc_A);
	    end_x_ras =   (brhc_A >  0) ? "A" : "P";
	    end_x_loc =   fabs(brhc_A);
	    
	    start_y_ras = (brhc_R >  0) ? "R" : "L";
	    start_y_loc = fabs(brhc_R);
	    end_y_ras =  (tlhc_R >  0) ? "R" : "L";
	    end_y_loc =  fabs(tlhc_R);
	    
	    freq_dir = "A/P";
	    
	 }
	 break;
	 
      case 4:    /*sagittal*/        
         if(hr_im->freq_dir == 1){   /*A-P*/
	    
	    start_x_ras = (brhc_A >  0) ? "A" : "P";
	    start_x_loc = fabs(brhc_A);
	    end_x_ras =  (tlhc_A >  0) ? "A" : "P";
	    end_x_loc =  fabs(tlhc_A);
	    
	    start_y_ras = (brhc_S >  0) ? "S" : "I";
	    start_y_loc = fabs(brhc_S);
	    end_y_ras =   (tlhc_S >  0) ? "S" : "I";
	    end_y_loc =   fabs(tlhc_S);
	    
	    freq_dir = "A/P";
	    
	 }
	 else if(hr_im->freq_dir == 2){   /*S-I*/
	    
	    start_x_ras = (tlhc_S >  0) ? "S" : "I";
	    start_x_loc = fabs(tlhc_S);
	    end_x_ras =   (brhc_S >  0) ? "S" : "I";
	    end_x_loc =   fabs(brhc_S);
	    
	    start_y_ras = (brhc_A >  0) ? "A" : "P";
	    start_y_loc = fabs(brhc_A);
	    end_y_ras =  (tlhc_A >  0) ? "A" : "P";
	    end_y_loc =  fabs(tlhc_A);
	    
	    freq_dir = "S/I";
	    
	 }
	 break;
	 
      case 8:       /*coronal*/     
         if(hr_im->freq_dir == 1){   /*R-L*/
	    
	    start_x_ras = (tlhc_R >  0) ? "R" : "L";
	    start_x_loc = fabs(tlhc_R);
	    end_x_ras =   (brhc_R >  0) ? "R" : "L";
	    end_x_loc =   fabs(brhc_R);
	    
	    start_y_ras = (brhc_S >  0) ? "S" : "I";
	    start_y_loc = fabs(brhc_S);
	    end_y_ras =  (tlhc_S >  0) ? "S" : "I";
	    end_y_loc =  fabs(tlhc_S);
	    
	    freq_dir = "R/L";
	    
	    
	 }
	 else if(hr_im->freq_dir == 2){   /*S-I*/
	    
	    start_x_ras = (brhc_S >  0) ? "S" : "I";
	    start_x_loc = fabs(brhc_S);
	    end_x_ras =   (tlhc_S >  0) ? "S" : "I";
	    end_x_loc =   fabs(tlhc_S);
	    
	    start_y_ras = (brhc_R >  0) ? "R" : "L";
	    start_y_loc = fabs(brhc_R);
	    end_y_ras =  (tlhc_R >  0) ? "R" : "L";
	    end_y_loc =  fabs(tlhc_R);
	    
	    freq_dir = "S/I";
	    
	    
	 }
	 break;
   }
   
   printf("X: ");
   printf("%s %.1f - %s %.1f\n", start_x_ras, start_x_loc,end_x_ras, end_x_loc);
   
   printf("Y: ");
   printf("%s %.1f - %s %.1f\n", start_y_ras, start_y_loc, end_y_ras, end_y_loc);
   
   /*---Slice direction---*/
   
   printf("Z: ");
   printf("%c %.1f - %c %.1f\n\n", hr_se->start_ras, fabs(hr_se->start_loc), hr_se->end_ras, fabs(hr_se->end_loc));
   
   /*---dataset type---*/
   dset_type = (char *)malloc(sizeof(char)*4);
   printf("Pulse sequence name: ");
   if( strncmp(hr_im->psdname,"hyper",5) == 0) {
      dset_type = "epan";
      printf("hyper\n");
      printf("Pulse sequence type: ");
      switch((int)hr->rdb_hdr_user16) {
        default: printf("Unknown");
        case HYPER_SEQ: printf("Hyperecho\n");       break;
        case   STE_SEQ: printf("Stimulated Echo\n"); break;
        case    SE_SEQ: printf("Spin Echo\n");       break;
        case    GE_SEQ: printf("Gradient Echo\n");   break;
      }
   }
   if( strncmp(hr_im->psdname,"3dsfd",5) == 0) {
      dset_type = "epan";
      printf("3dsfd");
   }
   if( strncmp(hr_im->psdname,"jbepi",5) == 0) {
      dset_type = "epan";
      printf("jbepi");
   }
   if( strncmp(hr_im->psdname,"spep",4) == 0) {
      dset_type = "epan";
      printf("spep");
      if( hr->rdb_hdr_user0 == 0 ) printf(" (spiral)");
      if( hr->rdb_hdr_user0 == 1 ) printf(" (epi)");
      if( hr->rdb_hdr_user0 == 2 ) printf(" (3d spiral fse)");
   }
   if( strncmp(hr_im->psdname,"jbspep",6) == 0) {
      dset_type = "epan";
      printf("jbspep");
      if( hr->rdb_hdr_user0 == 0 ) printf(" (spiral)");
      if( hr->rdb_hdr_user0 == 1 ) printf(" (epi)");
      if( hr->rdb_hdr_user0 == 2 ) printf(" (3d spiral fse)");
   }
   if( strncmp(hr_im->psdname,"epi3",5) == 0) {
      dset_type = "epan";
      printf("epi3 (NIH epi)");
   }
   printf("\n");
   
   printf("Freq. enc. dir: %s\n", freq_dir); 
   
   /*---TR---*/
   /* tr = (hr_im->sctime)/(1000.0*hr->rdb_hdr_nframes);
      printf("TR:             %.1f ms\n", tr); */
   tr = (float) hr_im->tr/1000.;
   printf("TR:             %.1f ms\n", (float)hr_im->tr/1000.);
   printf("TE1:            %.1f ms\n", (float)hr_im->te/1000.);
   if( strncmp(hr_im->psdname,"hyper",5) == 0) {
      switch((int)hr->rdb_hdr_user16) {
        default: 
        case HYPER_SEQ: 
	  printf("TE2:            %.1f ms\n", (float)hr->rdb_hdr_user37/1000.);
	  break;
        case   STE_SEQ: 
	  printf("TM:             %.1f ms\n", (float)hr->rdb_hdr_user36/1000.);
	  break;
      }
   }
   
   /*---AFNI to3d command line---*/

   printf("\n");
   printf("to3d ");
   printf("-time:tz %i %i %i alt+z ", hr->rdb_hdr_nframes, hr->rdb_hdr_nslices, (int)tr);
   printf("-%s ", dset_type);
   printf("-xFOV %.1f%s-%.1f%s ", start_x_loc, start_x_ras, end_x_loc, end_x_ras);
   printf("-yFOV %.1f%s-%.1f%s ", start_y_loc, start_y_ras, end_y_loc, end_y_ras);
   printf("-zSLAB %.1f%c-%.1f%c ", fabs(hr_se->start_loc), hr_se->start_ras, fabs(hr_se->end_loc), hr_se->end_ras);
   printf("\n\n");
   
   if(dimstr!=NULL) {
     sprintf(dimstr,"-xFOV %.1f%s-%.1f%s " 
	     "-yFOV %.1f%s-%.1f%s "
	     "-zSLAB %.1f%c-%.1f%c ",  
	     start_x_loc, start_x_ras, end_x_loc, end_x_ras, 
	     start_y_loc, start_y_ras, end_y_loc, end_y_ras,
	     fabs(hr_se->start_loc), hr_se->start_ras, 
	     fabs(hr_se->end_loc), hr_se->end_ras);
   }
}

