/**<dif2afni.c>***************************************************************

      Title: DIFfusion TO AFNI

    Purpose: Convert diffusion images collected with ses13.psd
             into AFNI 3D file

     Author: Lawrence R. Frank
             (c) University of California, San Diego

    History: 00_11_18 cp stack.c (ECW) -> dif2afni.c
                      system call to to3d

      Notes: This program is basically a combination of
             ECW's stack.c with a system call to RWC's to3d.c
             with some additional available input.

             I've disabled stack options '-behead' and '-rm'

*/
#include "d2a.h"

#define MAXNDIF  3000   /* Maximum number of diffusion directions.
                           Defined in grses.c */
#define debug 0

typedef struct {
  char   filename[80];
  char   tpattern[80];
  char   fmat[100];
  char   orient[10];
  int    nsl; 
  int    ndif; 
  float  tr ;
  char   dim[256]; /* rjt */
} AFNIStruct;

char hfile[80], *pfile;
FILE *fin,*fout;

main(int argc,char *argv[])
{
        int need_help = 0;
        int i,j,n,sl1,nsl,dif1,ndif,nopt,remove,ntype,done;
        int imsize,sl,dif,nx,ny,head,behead,headsize,datasize,attach;
        int file_exists(),file_size(),get_old_header();
        long int ier,isize,gsize;
        short int *im,*ihead,*idata;
        char rmcmd[80],*inroot,*outroot;
        char fname[80],dname[80],hname[80],fnum[80],head_id[80];
        FILE *fd;
        int write_file(), read_file();
        void instructions();
        void send_help() ;
        char to3d_str[10000];   /* command line for program to3d */
        float TR = 1.0;
        char gdxname[80],gdyname[80],gdzname[80];
        float gdx[MAXNDIF],gdy[MAXNDIF],gdz[MAXNDIF];
        AFNIStruct afnistr;
        int open_files(char *, char *);
        int write_header(char *, char * ) ;
        char header[RDB_HEADER_SIZE_BYTES];
        char img_extent[256];  /* rjt */

        /* System remove call */
        sprintf(rmcmd,"/bin/rm -f ");

        /* Initialize some stuff */

        sl1 = 1;
        nsl = 0;
        dif1 = 1;
        ndif = 0;
        inroot = argv[argc-1];
        outroot = inroot;
        remove = 0;
        behead = 0;
        head = 0;
        attach = 0;

        /* parse arguments */

        nopt = 0;
        if(argc==2) {
          if (!strncmp(argv[1], "-help", 5)) { send_help( ); exit(0); }
          else { instructions(); exit(0); }
        }

        for (i = 1; i < argc-1; i++) { /* ------- Options ------- */
                if (!strncmp(argv[i], "-sl1", 4)) {    /* First slice */
                        if (++i >= argc) { instructions(); exit(0); }
                        sl1 = atoi(argv[i]);
                        nopt++; nopt++;
                }
                else if (!strncmp(argv[i], "-nsl", 4)) {    /* Number of slices */
                        if (++i >= argc) { instructions(); exit(0); }
                        nsl = atoi(argv[i]);
                        nopt++; nopt++;
                }
                else if (!strncmp(argv[i], "-tr", 3)) {    /* Repetition time (ms) */
                        if (++i >= argc) { instructions(); exit(0); }
                        TR = atof(argv[i]);
                        nopt++; nopt++;
                }
                else if (!strncmp(argv[i], "-oname", 6)) {  /* Outfile root */
                        if (++i >= argc) { instructions(); exit(0); }
                        outroot = argv[i];
                        nopt++; nopt++;
                }
                else if (!strncmp(argv[i], "-head", 7)) {  /* Write header from Pfile */
                        if (++i >= argc) { instructions(); exit(0); }
                        pfile = argv[i];
                        head = 1;
                        nopt++; nopt++;
                }
                else {
                        printf("Option %s is not recognized\n",argv[i]);
                        instructions();
                        exit(0);
                }
                /* First dif */
                /* else if (!strncmp(argv[i], "-dif1", 5)) {    
                   if (++i >= argc) { instructions(); exit(0); }
                   dif1 = atoi(argv[i]);
                   nopt++; nopt++;
                   } */
                   /* Number of difs */
                /* else if (!strncmp(argv[i], "-ndif", 5)) {    
                   if (++i >= argc) { instructions(); exit(0); }
                   ndif = atoi(argv[i]);
                   nopt++; nopt++;
                   } */
                /* Remove image files */
                /* else if (!strncmp(argv[i], "-rm", 3)) {  
                   if (i >= argc) { instructions(); exit(0); }
                   remove = 1;
                   nopt++;
                   } */
                /* Behead P-file */
                /* else if (!strncmp(argv[i], "-behead", 7)) { 
                   if (i >= argc) { instructions(); exit(0); }
                   behead = 1;
                   nopt++;
                   } */

        } /* end of options */

        if (argc-nopt < 2){
                instructions();
                exit(0);
        }

        /* Print out header from pfile */

        if(head) {
          printf("Generating header from pfile %s\n",pfile);
          sprintf(hfile,"%s.header",outroot); /* Header name */

          if(!open_files(pfile,header)) exit (0);

        /* Fill in FOV information for AFNI structure - rjt */
          PrintPfileHeaderSummary( header, img_extent );
          printf("Image extent: %s\n",img_extent);

          if(!write_header(hfile,header)) {
            printf("\aError writing file %s!\n",hfile);
            exit (1);
          } else {
            printf("Wrote file  %s\n",hfile);
          }
        }

        /* Get image name type and file size 
           ntype: 1=root.xx.dxx 
                  2=root.xxx 
                  3=root.dxx */
        printf("nsl = %i\n",nsl);

        ntype = 0;
        sprintf(fname,"%s.%02d.d%02d",inroot,sl1,dif1);
        printf("fname = %s\n",fname);
        if (isize = file_size(fname)) ntype = 1;
        if (!ntype && ndif<2) {
                sprintf(fname,"%s.%03d",inroot,sl1);
                if (isize = file_size(fname)) ntype = 2;
                printf("fname = %s\n",fname);
        }
        if (!ntype) {
                sprintf(fname,"%s.d%02d",inroot,dif1);
                if (isize = file_size(fname)) ntype = 3;
                nsl = 1;
                printf("fname = %s\n",fname);
        }
        if (!ntype){
                printf("\aUnknown image type = %i! Exiting...\n",ntype);
                instructions();
                exit(0);
        }
        imsize = isize/sizeof(short int);

        /* Set nx, ny, and head based on file length */
        switch(imsize) {
                case 4096:
                        nx = 64;          /* raw image formats */
                        ny = 64;
                        head = 0;
                        break;
                case 9216:
                        nx = 96;
                        ny = 96;
                        head = 0;
                        break;
                case 16384:
                        nx = 128;
                        ny = 128;
                        head = 0;
                        break;
                case 65536:
                        nx = 256;
                        ny = 256;
                        head = 0;
                        break;
                case 72704:                /* GE 4.x formats */
                        nx = 256;
                        ny = 256;
                        head = 14336;
                        sprintf(head_id,"ge4x_im");
                        break;
                case 269312:
                        nx = 512;
                        ny = 512;
                        head = 14336;
                        sprintf(head_id,"ge4x_im");
                        break;
                case 19456:                /* Siemens formats */
                        nx = 128;
                        ny = 128;
                        head = 6144;
                        sprintf(head_id,"vis_im");
                        break;
                case 68608:
                        nx = 256;
                        ny = 256;
                        head = 6144;
                        sprintf(head_id,"vis_im");
                        break;
                case 265216:
                        nx = 512;
                        ny = 512;
                        head = 6144;
                        sprintf(head_id,"vis_im");
                        break;
        }
        behead *= head;   /* only behead if header exists */
        attach = head;   /* attach header if one exists */

        /* Get nsl */

        if (!nsl) {
                done = 0;
                i = sl1;
                while (!done) {
                        strcpy(fname,inroot);
                        if (ntype == 1) sprintf(fnum,".%02d.d%02d",i,dif1);
                        else if (ntype == 2) sprintf(fnum,".%03d",i);
                        strcat(fname,fnum);
                        if (file_exists(fname)) i++;
                        else done = 1;
                }
                nsl = --i - sl1 + 1;
        }
        printf("Found %i slices\n",nsl);
        
        /* Get ndif */

        if (ntype == 2) ndif = 1;
        if (!ndif) {
                done = 0;
                i = dif1;
                while (!done) {
                        strcpy(fname,inroot);
                        if (ntype == 1) sprintf(fnum,".%02d.d%02d",sl1,i);
                        else if (ntype == 3) sprintf(fnum,".d%02d",i);
                        strcat(fname,fnum);
                        if (file_exists(fname)) i++;
                        else done = 1;
                }
                ndif = --i - dif1 + 1;
        }
        printf("Found %i diffusion directions\n",ndif);
        
        /* Gradient direction files.
           They are always ".01", per dell.c 
           They are saved from dell.c as ascii
           but we want to convert them to text 
           and save them as AFNI *.1D files */

        sprintf(gdxname,"%s.01.gdx",inroot);
        sprintf(gdyname,"%s.01.gdy",inroot);
        sprintf(gdzname,"%s.01.gdz",inroot);
        gsize = ndif * sizeof(float);
        read_file(&ier,gdxname,&gsize,gdx);
        read_file(&ier,gdyname,&gsize,gdy);
        read_file(&ier,gdzname,&gsize,gdz);

        if(debug) {
          for (dif=dif1-1;dif<dif1+ndif-1;dif++) { /* note offset of 0 */
            printf("Diff dir %i: (gdx,gdy,gdz) =
(%f,%f,%f)\n",dif,gdx[dif],gdy[dif],gdz[dif]);
          }       
        }

        /* Write out gradient files in AFNI *.1D format */

        /* X grad */
        /* sprintf(gdxname,"%s_gdx.1D",outroot); */
        sprintf(gdxname,"%s.GDX",outroot);
        printf("Writing xdiff gradient file to %s\n",gdxname);
        if ((fd = fopen(gdxname,"w")) == NULL) {
                printf("Problem opening %s\n",gdxname);
                instructions();
                exit(0);
        }
        for (dif=dif1-1;dif<dif1+ndif-1;dif++) {
          fprintf(fd,"%g\n",gdx[dif]);
        }
        fclose(fd);

        /* Y grad */

        /* sprintf(gdyname,"%s_gdy.1D",outroot); */
        sprintf(gdyname,"%s.GDY",outroot);
        printf("Writing ydiff gradient file to %s\n",gdyname);
        if ((fd = fopen(gdyname,"w")) == NULL) {
                printf("Problem opening %s\n",gdyname);
                instructions();
                exit(0);
        }
        for (dif=dif1-1;dif<dif1+ndif-1;dif++) {
          fprintf(fd,"%g\n",gdy[dif]);
        }
        fclose(fd);

        /* Z grad */

        /* sprintf(gdzname,"%s_gdz.1D",outroot); */
        sprintf(gdzname,"%s.GDZ",outroot);
        printf("Writing zdiff gradient file to %s\n",gdzname);
        if ((fd = fopen(gdzname,"w")) == NULL) {
                printf("Problem opening %s\n",gdzname);
                instructions();
                exit(0);
        }
        for (dif=dif1-1;dif<dif1+ndif-1;dif++) {
          fprintf(fd,"%g\n",gdz[dif]);
        }
        fclose(fd);

        /* allocate arrays */

        headsize = head/sizeof(short int);
        datasize = imsize - headsize;
        im = (short int *)calloc(imsize,sizeof(short int));
        ihead = (short int *)calloc(headsize,sizeof(short int));
        if (behead) idata = (short int *)calloc(ndif*nsl*datasize,sizeof(short int));
        else idata = (short int *)calloc(ndif*nsl*imsize,sizeof(short int));

        /* Read in images */

        n = 0;
        for (dif=dif1;dif<dif1+ndif;dif++) {
          for (sl=sl1;sl<sl1+nsl;sl++) {
            strcpy(fname,inroot);
            if (ntype == 1) sprintf(fnum,".%02d.d%02d",sl,dif);
            else if (ntype == 2) sprintf(fnum,".%03d",sl);
            else if (ntype == 3) sprintf(fnum,".d%02d",dif);
            strcat(fname,fnum);
            if (behead) {
              read_file(&ier,fname,&isize,im);
              if (n==0) for (i=0;i<headsize;i++) ihead[i] = im[i];
              j = n*datasize;
              for (i=0;i<datasize;i++) idata[j+i] = im[headsize+i];
            }
            else read_file(&ier,fname,&isize,idata+n*imsize);
            if (ier) {
              instructions();
              exit(0);
            }
            n++;
          }     
        }
        if (head && !behead) for (i=0;i<headsize;i++) ihead[i] = idata[i];

        /* Write data block */

        strcpy(dname,outroot);
        sprintf(fnum,".brik");
        strcat(dname,fnum);
        if (behead) isize = nsl * ndif * datasize * sizeof(short int);
        else isize = nsl * ndif * imsize * sizeof(short int);
        write_file(&ier,dname,&isize,idata);
        if (ier) {
          instructions();
          exit(0);
        }

        /* Get old header if called for */

        if (!head) attach = get_old_header(inroot,head_id,ihead,&headsize);

        /* Write info to header */

        strcpy(hname,outroot);
        sprintf(fnum,".head");
        strcat(hname,fnum);
        if ((fd = fopen(hname,"w")) == NULL) {
                printf("Problem opening %s\n",hname);
                instructions();
                exit(0);
        }
        fprintf(fd,"#Standard UCSD/MRI header Revision 1.00 97.03.22 Eric C. Wong\n");
        fprintf(fd,"#                         Revision 2.00 00.11.19 L.R. Frank\n");
        fprintf(fd,"#\n");
        fprintf(fd,"#Header contains comments and attributes\n");
        fprintf(fd,"#in any order.\n");
        fprintf(fd,"#\n");
        fprintf(fd,"#Comment lines start with #\n");
        fprintf(fd,"#End of generated header marked by 'end'\n");
        fprintf(fd,"#Data is 4D short int and is in the order [xyzt] for now\n");
        fprintf(fd,"#Attribute fields are one per line and are of the form:\n");
        fprintf(fd,"#    attribute:value:value...\n");
        fprintf(fd,"#\n");
        fprintf(fd,"#Attributes defined thus far are:\n");
        fprintf(fd,"#\n");
        fprintf(fd,"#Required:\n");
        fprintf(fd,"#nx:int       Number of points in X direction\n");
        fprintf(fd,"#ny:int       Number of points in Y direction\n");
        fprintf(fd,"#nz:int       Number of points in Z direction\n");
        fprintf(fd,"#nt:int       Number of points in T direction\n");
        fprintf(fd,"#\n");
        fprintf(fd,"#Optional: (Default in absence shown in [brackets])\n");
        fprintf(fd,"#nz1:int[1]      Slice number of first slice\n");
        fprintf(fd,"#nt1:int[1]      First time point\n");
        fprintf(fd,"#head:int[0]     Included image header size in bytes\n");
        fprintf(fd,"#attach:int:header_type[none]   attached header size in bytes and
type\n");
        fprintf(fd,"#        header_type = [ge4x_raw,ge4x_im,vis_im]\n");
        fprintf(fd,"#\n");
        fprintf(fd,"nx:%d\n",nx);
        fprintf(fd,"ny:%d\n",ny);
        fprintf(fd,"nz:%d\n",nsl);
        fprintf(fd,"nt:%d\n",ndif);
        if (sl1>1) fprintf(fd,"nz1:%d\n",sl1);
        if (dif1>1) fprintf(fd,"nt1:%d\n",dif1);
        if (head && !behead) fprintf(fd,"head:%d\n",head);
        if (attach) fprintf(fd,"attach:%d:%s\n",headsize*sizeof(short int),head_id);
        fprintf(fd,"end\n");
        if (attach) fwrite(ihead,sizeof(short int),headsize,fd);
        fclose(fd);

        free(idata);
        
        if (remove) {
                for (dif=dif1;dif<dif1+ndif;dif++) {
                        for (sl=sl1;sl<sl1+nsl;sl++) {
                                strcpy(fname,rmcmd);
                                strcat(fname,inroot);
                                if (ntype == 1) sprintf(fnum,".%02d.d%02d",sl,dif);
                                else if (ntype == 2) sprintf(fnum,".%03d",sl);
                                else if (ntype == 3) sprintf(fnum,".d%02d",dif);
                                strcat(fname,fnum);
                                system(fname);
                        }       
                }
        }

        /* Construct afnistr */

        strcpy (afnistr.filename, outroot);
        strcpy (afnistr.orient, "RAI");
        afnistr.nsl  = nsl;
        afnistr.ndif = ndif;
        afnistr.tr   = TR;
        strcpy (afnistr.dim, img_extent); /* rjt */
        strcpy (afnistr.tpattern, "seqplus");
        sprintf(afnistr.fmat, "3D:0:0:%i:%i:%i:%s.brik",nx,ny,nsl*ndif,outroot);
        if(debug) {
          printf("afnistr.filename = %s\n",afnistr.filename);
          printf("afnistr.orient = %s\n",afnistr.orient);
          printf("afnistr.tpattern = %s\n",afnistr.tpattern);
          printf("afnistr.fmat = %s\n",afnistr.fmat);
          printf("afnistr.tr = %f\n",afnistr.tr);
          printf("afnistr.nsl = %i\n",afnistr.nsl);
          printf("afnistr.ndif = %i\n",afnistr.ndif);
        }

        /*----- Construct command line for program to3d -----*/
                /* added dimension information - rjt */ 
        sprintf(to3d_str,"to3d -prefix %s -orient %s %s -omri -time:zt %d %d %.2f %s %s",
                afnistr.filename,
                afnistr.orient,
                afnistr.dim,
                afnistr.nsl,
                afnistr.ndif,
                afnistr.tr,
                afnistr.tpattern,
                afnistr.fmat);

        if(debug) printf("to3d_str = %s\n",to3d_str); 

        /*----- Invoke program to3d to write out AFNI dataset -----*/

        printf("Writing AFNI dataset ");
        printf("into %s\n", afnistr.filename);
        system (to3d_str); 

        /* Remove stack header */
        strcpy(fname,rmcmd);
        strcat(fname,hname);
        system(fname);
        /* Remove stack data */
        strcpy(fname,rmcmd);
        strcat(fname,dname);
        system(fname);

        printf("...done!\n");
}

int file_exists(char fname[])
{
        int     i;
        FILE *fd;                               /* file descriptor  */

        if ((fd = fopen(fname, "r")) != NULL) {
                fclose(fd);
                i = 1;
        }
        else i = 0;

        return i;
}

int file_size(char fname[])
{
        int     fp;                             /* file descriptor  */
        int     fsize;                          /* file size  */
        struct stat file_stat;                  /* status structure */

        if ((fp = open(fname, O_RDONLY)) <= 0)  /* file must exist */
        {
                return 0;
        }

        fstat(fp, &file_stat);                  /* get file size in bytes   */

        fsize = file_stat.st_size;

        close(fp);

        return fsize;
}

int get_old_header(char *fname,char *htype,short int *ar,int *n)
{
        int i,done;
        FILE *fd;
        char str[100];

        strcpy(str,fname);
        strcat(str,".head");
        if ((fd = fopen(str, "r")) == NULL) return 0;

        done = 0;
        do {            /* look for attach: field */
                if (feof(fd)) return 0;
                fgets(str,98,fd);
                if (!strncmp(str, "attach", 6)) {
                        sscanf(str,"attach:%d:%s",n,htype);
                        *n /= sizeof(short int);
                        done = 1;
                }
                else if (!strncmp(str, "end", 3)) return 0;
        } while (!done);

        done = 0;
        do {                           /* go to beginning of header */
                if (feof(fd)) return 0;
                fgets(str,98,fd);
                if (!strncmp(str, "end", 3)) done = 1;
        } while (!done);
        
        i = fread(ar,sizeof(short int),*n,fd);
        fclose(fd);
        if (i<*n) *n = i;
        return 1;
}

/* ----------------------------------------------------------------
   Write file subroutine. A.Jesmanowicz, MCW 1989
        ier   : return error :  1 - opening problem,
                                2 - negative length.
        fname : file name.
        size  : size of the file.
        arr   : file array.
   --------------------------------------------------------------*/
void write_iqm(long int *ier,char fname[],long int *size,char arr[])
{
        int     isize = *size;
        int     fp;                             /* file descriptor  */

        {       int i=0;                        /* cut junk from the name */
                while(fname[i] != 32 && fname[i]) i++;
                fname[i] = 0;
        }

        if(isize < 0)                           /* size has to be real */
        {       *ier=2;                         /* or ier = 2.         */
                return;
        }

        if ((fp = open(fname, O_WRONLY|O_CREAT|O_EXCL,0644)) <= 0)
        {       *ier=1;                         /* file must not exist */
                return;                         /* or ier = 1.     */
        }

        *ier=0;                                 /* no error : ier=0 */
        write(fp, arr, isize);                  /* write whole file */
        close(fp);
        return;  
}

/* Report for write_iqm  */

int write_iqm_error( long int *ier , char *fname )
{
  char *errstr;

  if(*ier==1) errstr = "FILE EXISTS";
  if(*ier==2) errstr = "FILE SIZE < 0";

  if (*ier!=0) {
    printf("ERROR WRITING FILE %s! %s\n",fname,errstr);
  }

  return(*ier);

}

int write_file( long int *ier,
                char     fname[],
                long int *size,
                char     arr[] )
{
  write_iqm(ier,fname,size,arr);
  return(write_iqm_error(ier,fname));
}

void read_iqm(long int *ier,char fname[],long int *size,char arr[])
{
        int     isize = *size;
        int     fp;                             /* file descriptor  */
        struct stat file_stat;                  /* status structure */

        {       int i=0;                        /* cut junk from the name */
                while(fname[i] != 32 && fname[i]) i++;
                fname[i] = 0;
        }

        if ((fp = open(fname, O_RDONLY)) <= 0)  /* file must exist */
        {       *ier=1;                         /* or ier = 1.     */
                return;
        }

        fstat(fp, &file_stat);                  /* get file size in bytes   */

        if(file_stat.st_size > isize && isize)  /* file can not be too long */
        {       *ier=2;                         /* or ier = 2.              */
                return;
        }
        *size =  file_stat.st_size;             /* return file size */

        read(fp, arr, file_stat.st_size);       /* read whole file  */
        close(fp);
        *ier=0;                                 /* no error : ier=0 */
        return;
}

/* Report for read_iqm  */

int read_iqm_error( long int *ier , char *fname )
{
  char *errstr;

  if(*ier==1) errstr = "FILE DOES NOT EXIST";
  if(*ier==2) errstr = "FILE SIZE TOO LONG";

  if (*ier!=0) {
    printf("ERROR READING FILE %s! %s\n",fname,errstr);
  }

  return(*ier);

}

int read_file( long int *ier,
               char     fname[],
               long int *size,
               char     arr[] )
{
  read_iqm(ier,fname,size,arr);
  return(read_iqm_error(ier,fname));
}


int open_files(char *fname, char *header)
{
  if (!(fin=fopen(fname,"r"))) {
    printf("Can't open rawfile %s!\n",fname);
    return(0);
  }

  /* read raw hdr */

  if (fread(header, sizeof(char), RDB_HEADER_SIZE_BYTES, fin) 
      != RDB_HEADER_SIZE_BYTES)
    printf("header too short!\n");

#ifdef Linux
   printf("Linux system detected - swapping bytes in header ...\n");
   SwapPfileHeader(header);
#endif

  return(1);
}

int write_header(char *fname, char *header )
{
  if (!(fout=fopen(fname,"w"))) {
    printf("Can't open rawfile %s!\n",fname);
    return(0);
  }

  if (fwrite(header, sizeof(char), RDB_HEADER_SIZE_BYTES, fout) 
      != RDB_HEADER_SIZE_BYTES)
    printf("header too short!\n");

  return(1);
}

void instructions()
{
  printf("\nCreates an AFNI 3D dataset from a 3D+diffusion data set.\n"
         "Images are assumed to be in root.slice.diff format\n"
         "\n"
         "Usage: dif2afni [options] inroot\n"
         "\n"
         "Tag Options: \n\n"
         "\t-tr        n     [1]       repetition time in ms\n\n"
         "I/O Options:\n\n"
         "\t-sl1       n     [1]       first slice\n"
         "\t-nsl       n     [all]     number of slices\n"
         "\t-head      Pfile           write header from Pfile\n"
         "\t-oname     fname [inroot]  output file name\n\n"
         "For more help, type 'dif2afni -help'\n\n"
         );
         /* "\t-dif1   n[1]           first time point\n" */
         /* "\t-ndif   n[all]         number of diffusion directions\n" */
         /* "\t-behead               remove image headers before stacking\n"
            "\t-rm                   remove image files after stacking\n"); */
  return;
}

void send_help( )
{
  printf("\nExample: \n\n"
         "If you have reconstructed images 'im.(slices).(diff)'\n"
         "from pfile P12345.7, you can create an afni dataset\n"
         "called 'brain+orig' with header information retrieved\n"
         "from the pfile by typing:\n\n"
         "dif2afni -head P12345.7 -oname brain im\n\n");
}
