/* print_header.h */

#ifndef _PRINT_HEADER_
#define _PRINT_HEADER_

#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#define RECON_FLAG		/* for the next header include */
#include "imagedb.h"
#include "rdbm.h"

#define GAMMA 4257.0            /* proton gyromagnetic ratio in Hz/gauss */

  RDB_HEADER_REC raw;
  char header[RDB_HEADER_SIZE_BYTES];

  int nviews, npixo2, nsamp, nslc, necho, ngrid;
  int rawimsize;
  int rawnslc;
  int nframes, ntotal;
  int rhfrsize, rhnframes, rhnecho;
  int nbpi;  		/* bytes per image  */
  int imrot, imtran;
  int sloffset,froffset;
  float xoff, yoff;		/* offsets in pixels  */
  float foff;			/* total freq offset */
  float frcvr;			/* receiver frequency offset  */
  float gain;
  float dts, mapdt;
  float gfov, gtype, gamp, gslew;	/* for genspiral  */
  int N, nl;			/* matrix size for genspiral */
  int bline;			/* number of baselines */
  int doconcat;			/* concatenated acquisitions */
  int ngap;			/* number of missing points in concat acq */
  float w0, wx, wy;
  float dwin;			/* half length of Kaisser-Bessel window */
  float b;			/* Kaisser-Bessel param  */
  float fermir;			/* radius of fermi window */
  float fermiw;			/* width of fermi window  */
  int isl, ifr, iech, ibval, idif, igp;

  /* Input variables */
  int im1       ;	/* first image frame */
  int nsl       ;	/* number of slices */
  int npix      ;	/* number of pixels */
  int navc      ;	/* do nav correction if set */
  int bmapflag  ;	/* do the b0 map if 1  */
  int domaxwell ;	/* do maxwell field correction if 1 */
  int nf2do     ;	/* number of time frames */
  int imgtyp    ;	/* output image type bitmap  */
  float zoomf   ;	/* zoom factor */
  float foffin  ;	/* offset from command line */
  int isl1      ;	/* first slices */
  int isl2	;       /* last slice */

  /* Diffusion - lrf */
  int nbvals ;		/* number of b-values - lrf */
  int nb0 ;		/* number of b0 acquisitions - lrf */
  int ndif ;		/* number of diffusion directions - lrf */
  int difmode ;		/* diffusion mode - lrf */
  int difnex ;		/* number of averages - lrf */
  int ntess ;		/* diffusion sphere tesselation factor - lrf */
  int nvert ;		/* number of vertices in tesselated sphere - lrf */
  int dome ;		/* dome = half sphere - lrf */
  int bmode ;		/* B-value stepping - lrf */
  int bslope ;		/* Increase of log-bstepping slope over 1 - lrf */
  int polytope ;	/* Tesselation polytope type */
  float pwgd  ;		/* Diffusion gradient width */
  float Delta  ;	/* Time between diffusion gradient centers */
  float pwdramp  ;	/* Diffusion gradient ramp width */
  float agdx1  ;	/* Amp of x-diff grad (G/cm) */
  float agdy1  ;	/* Amp of y-diff grad (G/cm) */
  float agdz1  ;	/* Amp of z-diff grad (G/cm) */
  float optm  ;		/* Mixing time (ms) */

  float Gamma = 2*M_PI*GAMMA;

  char dimstr[256];		/* 02_01_06 - lrf */

/* Prototypes */
int print_header(char *fname) ;

#endif
