#include <string.h>
#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/fcntl.h>
#include "toolsdata.h"
#include "imagedb.h"
#include "rdbm.h"

#define RMB_DEBUG 1

/* ----------------------- */
int swap_2( char *arR, int nr )
/* ----------------------- */
{
   int i;
   char ctmp;

   for (i=0; i < nr; i+=2){
     ctmp     = arR[i+1];
     arR[i+1] = arR[i];
     arR[i]   = ctmp;
   }
}

/* ----------------------- */
int swap_4( char *arR, int nr )
/* ----------------------- */
{
   int i;
   char ctmp2, ctmp3;

   for (i=0; i < nr; i+=4){
     ctmp3    = arR[i+3];
     ctmp2    = arR[i+2];
     arR[i+2] = arR[i+1];
     arR[i+3] = arR[i];
     arR[i]   = ctmp3;
     arR[i+1] = ctmp2;
   }
}

void SwapPfileHeader(char * header){

   POOL_HEADER       *ph;
   RDB_HEADER_REC    *hr;
   MRIMAGEDATATYPE   *hr_im;                 /* image header */
   EXAMDATATYPE      *hr_ex;                 /* exam header */
   SERIESDATATYPE    *hr_se;                 /* series header */
   
   int i;

   ph = (POOL_HEADER *) header;
   hr = (RDB_HEADER_REC *) &(ph->rdb_hdr_rec);
   hr_im = (MRIMAGEDATATYPE *) &(ph->rdb_hdr_image);
   hr_se = (SERIESDATATYPE *) &(ph->rdb_hdr_series);
   hr_ex = (EXAMDATATYPE *) &(ph->rdb_hdr_exam);

   /*################## RDB REC HEADER ######################*/

   swap_4(&hr->rdb_hdr_rdbm_rev, 4);
   swap_4(&hr->rdb_hdr_run_int, 4); 		/* Rdy pkt Run Number */
   swap_2(&hr->rdb_hdr_scan_seq, 2); 		/* Rdy pkt Sequence Number */
   swap_2(&hr->rdb_hdr_file_contents, 2); 	/* Data type 0=emp 1=nrec 2=rw 	0, 1, 2 */
   swap_2(&hr->rdb_hdr_lock_mode, 2); 		/* unused */
   swap_2(&hr->rdb_hdr_dacq_ctrl, 2); 		/* rhdacqctrl bit mask		15 bits */
   swap_2(&hr->rdb_hdr_recon_ctrl, 2); 		/* rhrcctrl bit mask 		15 bits */
   swap_2(&hr->rdb_hdr_exec_ctrl, 2); 		/* rhexecctrl bit mask 		15 bits */
   swap_2(&hr->rdb_hdr_scan_type, 2); 		/* bit mask 			15 bits */
   swap_2(&hr->rdb_hdr_data_collect_type, 2); 	/* rhtypebit mask		15 bits */
   swap_2(&hr->rdb_hdr_data_format, 2); 	/* rhformatbit mask 		15 bits */
   swap_2(&hr->rdb_hdr_recon, 2); 		/* rhrecon proc-a-son recon	0 - 100 */
   swap_2(&hr->rdb_hdr_datacq, 2); 		/* rhdatacq proc-a-son dacq */
   swap_2(&hr->rdb_hdr_npasses, 2); 		/* rhnpasses passes for a scan0 - 256 */
   swap_2(&hr->rdb_hdr_npomp, 2); 		/* rhnpomp pomp group slices	1,2 */
   swap_2(&hr->rdb_hdr_nslices, 2); 		/* rhnslices slices in a pass	0 - 256 */
   swap_2(&hr->rdb_hdr_nechoes, 2); 		/* rhnecho echoes of a slice	1 - 32 */
   swap_2(&hr->rdb_hdr_navs, 2); 		/* rhnavs num of excitiations	1 - 32727 */
   swap_2(&hr->rdb_hdr_nframes, 2); 		/* rhnframes yres 0 - 1024 */
   swap_2(&hr->rdb_hdr_baseline_views, 2);	/* rhbline baselines 0 - 1028 */
   swap_2(&hr->rdb_hdr_hnover, 2); 		/* rhhnover overscans 0 - 1024 */
   swap_2(&hr->rdb_hdr_frame_size, 2); 		/* rhfrsize xres 0 - 1024 */
   swap_2(&hr->rdb_hdr_point_size, 2); 		/* rhptsize 2 - 4 */
   swap_2(&hr->rdb_hdr_vquant, 2); 		/* rhvquant 3d volumes	1 */
   swap_2(&hr->rdb_hdr_cheart, 2); 		/* RX Cine heart phases 1 - 32 */
   swap_4(&hr->rdb_hdr_ctr, 4); 			/* RX Cine TR in sec	0 - 3.40282e38*/
   swap_4(&hr->rdb_hdr_ctrr, 4); 			/* RX Cine RR in sec	0 - 30.0 */
   swap_2(&hr->rdb_hdr_initpass, 2); 		/* rhinitpass allocate passes0 - 32767 */
   swap_2(&hr->rdb_hdr_incrpass, 2); 		/* rhincrpass tps autopauses	0 - 32767 */
   swap_2(&hr->rdb_hdr_method_ctrl, 2); 		/* rhmethod0=recon, 1=psd	0, 1 */
   swap_2(&hr->rdb_hdr_da_xres, 2);   		/* rhdaxres 0 - 1024 */
   swap_2(&hr->rdb_hdr_da_yres, 2);   		/* rhdayres 0 - 2049 */
   swap_2(&hr->rdb_hdr_rc_xres, 2);   		/* rhrcxres 0 - 1024 */
   swap_2(&hr->rdb_hdr_rc_yres, 2);   		/* rhrcyres 0 - 1024 */
   swap_2(&hr->rdb_hdr_im_size, 2);   		/* rhimsize 0 - 512 */
   swap_4(&hr->rdb_hdr_rc_zres, 4);   		/* power of 2 > rhnslices	0 - 128 */
   swap_4(&hr->rdb_hdr_raw_pass_size, 4);   	/* rhrawsize 0 - 2147483647*/
   swap_4(&hr->rdb_hdr_sspsave, 4);   		/* rhsspsave 0 - 2147483647*/
   swap_4(&hr->rdb_hdr_udasave, 4);   		/* rhudasave 0 - 2147483647*/
   swap_4(&hr->rdb_hdr_fermi_radius, 4);   	/* rhfermr fermi radius		0 - 3.40282e38*/
   swap_4(&hr->rdb_hdr_fermi_width, 4);   		/* rhfermw fermi width		0 - 3.40282e38*/
   swap_4(&hr->rdb_hdr_fermi_ecc, 4);   		/* rhferme fermi excentiricty	0 - 3.40282e38*/
   swap_4(&hr->rdb_hdr_clip_min, 4);   		/* rhclipmin 4x IP limit		+-16383 */
   swap_4(&hr->rdb_hdr_clip_max, 4);   		/* rhclipmax 4x IP limit		+-16383 */
   swap_4(&hr->rdb_hdr_default_offset, 4);   	/* rhdoffset default offset = 0	+-3.40282e38 */
   swap_4(&hr->rdb_hdr_xoff, 4);   		/* rhxoff scroll img in x 	+-256 */
   swap_4(&hr->rdb_hdr_yoff, 4);   		/* rhyoff scroll img in y	+-256 */
   swap_4(&hr->rdb_hdr_nwin, 4);   		/* rhnwin hecho window width	0 - 256 */
   swap_4(&hr->rdb_hdr_ntran, 4);   		/* rhntran hecho trans width	0 - 256 */
   swap_4(&hr->rdb_hdr_scalei, 4);   		/* PS rhscalei			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_scaleq, 4);   		/* PS rhscaleqdef = 0		+-3.40282e38 */
   swap_2(&hr->rdb_hdr_rotation, 2);   		/* RX 0 90 180 270 deg		0 - 3 */
   swap_2(&hr->rdb_hdr_transpose, 2);   		/* RX 0, 1 n / y transpose 	0 - 1*/
   swap_2(&hr->rdb_hdr_kissoff_views, 2);   	/* rhblank zero image views	0 - 512 */
   swap_2(&hr->rdb_hdr_slblank, 2);   		/* rhslblankslice blank 3d	0 - 128 */ 
   swap_2(&hr->rdb_hdr_gradcoil, 2);   		/* RX 0=off 1=Schnk 2=Rmr	0 - 2 */
   swap_2(&hr->rdb_hdr_ddaover, 2);   		/* rhddaover unused */
   swap_2(&hr->rdb_hdr_sarr, 2);   		/* SARR bit mask 		15 bits */
   swap_2(&hr->rdb_hdr_fd_tr, 2);   		/* SARR feeder timing info */
   swap_2(&hr->rdb_hdr_fd_te, 2);   		/* SARR feeder timing info */
   swap_2(&hr->rdb_hdr_fd_ctrl, 2);   		/* SARR control of feeder */
   swap_2(&hr->rdb_hdr_algor_num, 2);   	/* SARR df decimation ratio */
   swap_2(&hr->rdb_hdr_fd_df_dec, 2);   	/* SARR which feeder algor */

   swap_2(&hr->rdb_hdr_dab[0].start_rcv, 2);   	/* rhdab0s rhdab0e st, stp rcv 	0 - 15 */
   swap_2(&hr->rdb_hdr_dab[0].stop_rcv, 2);   	/* rhdab0s rhdab0e st, stp rcv 	0 - 15 */
   swap_2(&hr->rdb_hdr_dab[1].start_rcv, 2);   	/* rhdab0s rhdab0e st, stp rcv 	0 - 15 */
   swap_2(&hr->rdb_hdr_dab[1].stop_rcv, 2);   	/* rhdab0s rhdab0e st, stp rcv 	0 - 15 */
   swap_2(&hr->rdb_hdr_dab[2].start_rcv, 2);   	/* rhdab0s rhdab0e st, stp rcv 	0 - 15 */
   swap_2(&hr->rdb_hdr_dab[2].stop_rcv, 2);   	/* rhdab0s rhdab0e st, stp rcv 	0 - 15 */
   swap_2(&hr->rdb_hdr_dab[3].start_rcv, 2);   	/* rhdab0s rhdab0e st, stp rcv 	0 - 15 */
   swap_2(&hr->rdb_hdr_dab[3].stop_rcv, 2);   	/* rhdab0s rhdab0e st, stp rcv 	0 - 15 */

   swap_4(&hr->rdb_hdr_user0, 4);   		/* rhuser0 			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_user1, 4);   		/* rhuser1 			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_user2, 4);   		/* rhuser2 			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_user3, 4);   		/* rhuser3 			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_user4, 4);   		/* rhuser4 			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_user5, 4);   		/* rhuser5 			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_user6, 4);   		/* rhuser6 			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_user7, 4);   		/* rhuser7 			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_user8, 4);   		/* rhuser8 			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_user9, 4);   		/* rhuser9 			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_user10, 4);   		/* rhuser10			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_user11, 4);   		/* rhuser11			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_user12, 4);   		/* rhuser12			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_user13, 4);   		/* rhuser13			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_user14, 4);   		/* rhuser14			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_user15, 4);   		/* rhuser15			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_user16, 4);   		/* rhuser16			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_user17, 4);   		/* rhuser17			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_user18, 4);   		/* rhuser18			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_user19, 4);   		/* rhuser19			+-3.40282e38 */
   swap_4(&hr->rdb_hdr_v_type, 4);		/* rhvtypebit mask		31 bits */
   swap_4(&hr->rdb_hdr_v_coefxa, 4);		/* RX x flow direction control	0 - 4 */
   swap_4(&hr->rdb_hdr_v_coefxb, 4);		/* RX x flow direction control        0 - 4 */
   swap_4(&hr->rdb_hdr_v_coefxc, 4);		/* RX x flow direction control        0 - 4 */
   swap_4(&hr->rdb_hdr_v_coefxd, 4);		/* RX x flow direction control        0 - 4 */
   swap_4(&hr->rdb_hdr_v_coefya, 4);		/* RX y flow direction control        0 - 4 */
   swap_4(&hr->rdb_hdr_v_coefyb, 4);		/* RX y flow direction control        0 - 4 */
   swap_4(&hr->rdb_hdr_v_coefyc, 4);		/* RX y flow direction control        0 - 4 */
   swap_4(&hr->rdb_hdr_v_coefyd, 4);		/* RX y flow direction control        0 - 4 */
   swap_4(&hr->rdb_hdr_v_coefza, 4);		/* RX z flow direction control        0 - 4 */
   swap_4(&hr->rdb_hdr_v_coefzb, 4);		/* RX z flow direction control        0 - 4 */
   swap_4(&hr->rdb_hdr_v_coefzc, 4);		/* RX z flow direction control        0 - 4 */
   swap_4(&hr->rdb_hdr_v_coefzd, 4);		/* RX z flow direction control        0 - 4 */
   swap_4(&hr->rdb_hdr_vm_coef1, 4);		/* RX weight for mag image 1  0 - 1 */
   swap_4(&hr->rdb_hdr_vm_coef2, 4);		/* RX weight for mag image 2  0 - 1 */
   swap_4(&hr->rdb_hdr_vm_coef3, 4);		/* RX weight for mag image 3  0 - 1 */
   swap_4(&hr->rdb_hdr_vm_coef4, 4);		/* RX weight for mag image 4  0 - 1 */
   swap_4(&hr->rdb_hdr_v_venc, 4);   		/* RX vel encodeing cm / sec	0.001 - 5000 */
   swap_4(&hr->spectral_width, 4);   		/* specwidthfilter width kHz	500 - 3355432 */
   swap_2(&hr->csi_dims, 2);   			/* spectro */
   swap_2(&hr->xcsi, 2);   			/* rhspecrescsix		2 - 64 */
   swap_2(&hr->ycsi, 2);   			/* rhspecrescsiy		2 - 64 */
   swap_2(&hr->zcsi, 2);   			/* spectro */
   swap_4(&hr->roilenx, 4);   			/* RX x csi volume dimension */
   swap_4(&hr->roileny, 4);   			/* RX y csi volume dimension */
   swap_4(&hr->roilenz, 4);   			/* RX z csi volume dimension */
   swap_4(&hr->roilocx, 4);   			/* RX x csi volume center */
   swap_4(&hr->roilocy, 4);   			/* RX y csi volume center */
   swap_4(&hr->roilocz, 4);   			/* RX z csi volume center */
   swap_4(&hr->numdwell, 4);   			/* specdwells			0 - 3.40282e38*/
   swap_4(&hr->rdb_hdr_ps_command, 4);		/* PS internal use only       */
   swap_4(&hr->rdb_hdr_ps_mps_r1, 4);		/* PS MPS R1 setting	       1 - 7 */
   swap_4(&hr->rdb_hdr_ps_mps_r2, 4);		/* PS MPS R2 setting	       1 - 30 */
   swap_4(&hr->rdb_hdr_ps_mps_tg, 4);		/* PS MPS Transmit gain setting       0 - 200*/
   swap_4(&hr->rdb_hdr_ps_mps_freq, 4);   	/* PS MPS Center frequency hz	+-3.40282e38 */
   swap_4(&hr->rdb_hdr_ps_aps_r1, 4);		/* PS APS R1 setting	       1 - 7 */
   swap_4(&hr->rdb_hdr_ps_aps_r2, 4);		/* PS APS R2 setting	       1 - 30 */
   swap_4(&hr->rdb_hdr_ps_aps_tg, 4);		/* PS APS Transmit gain setting       0 - 200*/
   swap_4(&hr->rdb_hdr_ps_aps_freq, 4); 	/* PS APS Center frequency hz	+-3.40282e38 */
   swap_4(&hr->rdb_hdr_ps_scalei, 4);   	/* PS rational scaling 		+-3.40282e38 */
   swap_4(&hr->rdb_hdr_ps_scaleq, 4);   	/* PS unused */			
   swap_4(&hr->rdb_hdr_ps_snr_warning, 4); 	/* PS noise test 0=16 1=32 bits	0, 1 */
   swap_4(&hr->rdb_hdr_ps_aps_or_mps, 4);  	/* PS prescan order logic	0 - 5 */
   swap_4(&hr->rdb_hdr_ps_mps_bitmap, 4);  	/* PS bit mask			4 bits*/

   swap_4(&hr->rdb_hdr_ps_filler1, 4);		/* PS filler */
   swap_4(&hr->rdb_hdr_ps_filler2, 4);	 	/* PS filler */
   for(i=0; i<16; i++){
      swap_4(&hr->rdb_hdr_rec_noise_mean[i], 4);   	/* PS mean noise each receiver +-3.40282e38 */
      swap_4(&hr->rdb_hdr_rec_noise_std[i], 4);   	/* PS noise calc for muti rec	+-3.40282e38 */
   }
   swap_2(&hr->halfecho, 2);   				/* spectro full, half echo 0, 1 */
   /* 858 bytes */

   /* New fields 02-19-92 */
   swap_2(&hr->rdb_hdr_im_size_y, 2);   		/* rh???? 			0 - 512 */
   swap_4(&hr->rdb_hdr_data_collect_type1, 4);   	/* rh???? bit mask		31 bits */
   swap_4(&hr->rdb_hdr_freq_scale, 4);   		/* rh???? freq k-space step +-3.40282e38 */
   swap_4(&hr->rdb_hdr_phase_scale, 4);   		/* rh???? freq k-space step +-3.40282e38 */
   /* 14 bytes */

   swap_2(&hr->rdb_hdr_ovl, 2);   		/* rhovl - overlaps for MOTSA */ 

   /* Phase Correction Control Param. */
   swap_2(&hr->rdb_hdr_pclin, 2);   	/* Linear Corr. 0:off, 1:linear, 2:polynomial */
   swap_2(&hr->rdb_hdr_pclinnpts, 2);   /* fit number of points */
   swap_2(&hr->rdb_hdr_pclinorder, 2);  /* fit order */
   swap_2(&hr->rdb_hdr_pclinavg, 2);   	/* linear phase corr avg 0:off, 1:on */
   swap_2(&hr->rdb_hdr_pccon, 2);   	/* Const Corr. 0:off, 1:Ky spec., 2:polyfit(2/ilv), 3:polyfit(1/ilv) */
   swap_2(&hr->rdb_hdr_pcconnpts, 2);   /* fit number of points */
   swap_2(&hr->rdb_hdr_pcconorder, 2);  /* fit order */
   swap_2(&hr->rdb_hdr_pcextcorr, 2);   /* external correction file 0:don't use, 1: use */
   swap_2(&hr->rdb_hdr_pcgraph, 2);   	/* Phase Correction coef. image 0:off, 1:linear & constant */
   swap_2(&hr->rdb_hdr_pcileave, 2);   	/* Interleaves to use for correction: 0=all, 1=only first */
   swap_2(&hr->rdb_hdr_hdbestky, 2);   	/* bestky view for fractional Ky scan */
   swap_2(&hr->rdb_hdr_pcctrl, 2);   	/* phase correction research control */
   swap_2(&hr->rdb_hdr_pcthrespts, 2);  /* 2..512 adjacent points */
   swap_2(&hr->rdb_hdr_pcdiscbeg, 2);   /* 0..512 beginning point to discard */
   swap_2(&hr->rdb_hdr_pcdiscmid, 2);   /* 0..512 middle point to discard */
   swap_2(&hr->rdb_hdr_pcdiscend, 2);  	/* 0..512 ending point to discard */
   swap_2(&hr->rdb_hdr_pcthrespct, 2);  /* Threshold percentage */
   swap_2(&hr->rdb_hdr_pcspacial, 2);   /* Spacial best ref scan index 0..512 */
   swap_2(&hr->rdb_hdr_pctemporal, 2);  /* Temporal best ref scan index 0..512 */
   swap_2(&hr->rdb_hdr_pcspare, 2);   	/* spare for phase correction */
   swap_2(&hr->rdb_hdr_ileaves, 2);   	/* Nunmber of interleaves */
   swap_2(&hr->rdb_hdr_kydir, 2);   	/* Ky traversal dircetion 0: top-down, 1:center out */
   swap_2(&hr->rdb_hdr_alt, 2);   	/* Alt read sign 0=no, 1=odd/even, 2=pairs */
   swap_2(&hr->rdb_hdr_reps, 2);   	/* Number of scan repetitions */
   swap_2(&hr->rdb_hdr_ref, 2);   	/* Ref Scan 0: off 1: on */
   swap_4(&hr->rdb_hdr_pcconnorm, 4);   /* Constant S term normalization factor */
   swap_4(&hr->rdb_hdr_pcconfitwt, 4);  /* Constant polyfit weighting factor */
   swap_4(&hr->rdb_hdr_pclinnorm, 4);   /* Linear S term normalization factor */
   swap_4(&hr->rdb_hdr_pclinfitwt, 4);  /* Linear polyfit weighting factor */
   swap_4(&hr->rdb_hdr_pcbestky, 4);   	/* Best Ky location */

   /* VRG Filter param */
   swap_4(&hr->rdb_hdr_vrgf, 4);   	/* control word for VRG filter */
   swap_4(&hr->rdb_hdr_vrgfxres, 4);   	/* control word for VRGF final x resolution */

   /* Bandpass AsymmetryCorrection Param. */
   swap_4(&hr->rdb_hdr_bp_corr, 4);   		/* control word for bandpass asymmetry */
   swap_4(&hr->rdb_hdr_recv_freq_s, 4); 	/* starting frequency (+62.5) */
   swap_4(&hr->rdb_hdr_recv_freq_e, 4);   	/* ending frequency (-62.5) */
   swap_4(&hr->rdb_hdr_hniter, 4);   		/* Selects the number of*/

   /* iterations used in homodyne processing */
   swap_4(&hr->rdb_hdr_fast_rec, 4);   	/* Added for homodyne II, tells if*/

   /* the fast receiver is being used*/
   /* and the lpf setting of the fast*/
						/* receiver, 0: fast receiver off,*/
						/* 1 - 5: lpf settings */
   swap_4(&hr->rdb_hdr_refframes, 4);   	/* total # of frames for ref scan */
   swap_4(&hr->rdb_hdr_refframep, 4);   	/* # of frames per pass for a ref scan */
   swap_4(&hr->rdb_hdr_scnframe, 4);   		/* total # of frames for a entire scan */
   swap_4(&hr->rdb_hdr_pasframe, 4);   		/* # of frames per pass */
   swap_4(&hr->rdb_hdr_user_usage_tag, 4);   	/* for spectro */
   swap_4(&hr->rdb_hdr_user_fill_mapMSW, 4);   	/* for spectro */
   swap_4(&hr->rdb_hdr_user_fill_mapLSW, 4);   	/* for Spectro */
   swap_4(&hr->rdb_hdr_user20, 4);  		/* all following usercv are for spectro */
   swap_4(&hr->rdb_hdr_user21, 4);
   swap_4(&hr->rdb_hdr_user22, 4);
   swap_4(&hr->rdb_hdr_user23, 4);
   swap_4(&hr->rdb_hdr_user24, 4);
   swap_4(&hr->rdb_hdr_user25, 4);
   swap_4(&hr->rdb_hdr_user26, 4);
   swap_4(&hr->rdb_hdr_user27, 4);
   swap_4(&hr->rdb_hdr_user28, 4);
   swap_4(&hr->rdb_hdr_user29, 4);
   swap_4(&hr->rdb_hdr_user30, 4);
   swap_4(&hr->rdb_hdr_user31, 4);
   swap_4(&hr->rdb_hdr_user32, 4);
   swap_4(&hr->rdb_hdr_user33, 4);
   swap_4(&hr->rdb_hdr_user34, 4);
   swap_4(&hr->rdb_hdr_user35, 4);
   swap_4(&hr->rdb_hdr_user36, 4);
   swap_4(&hr->rdb_hdr_user37, 4);
   swap_4(&hr->rdb_hdr_user38, 4);
   swap_4(&hr->rdb_hdr_user39, 4);
   swap_4(&hr->rdb_hdr_user40, 4);
   swap_4(&hr->rdb_hdr_user41, 4);
   swap_4(&hr->rdb_hdr_user42, 4);
   swap_4(&hr->rdb_hdr_user43, 4);
   swap_4(&hr->rdb_hdr_user44, 4);
   swap_4(&hr->rdb_hdr_user45, 4);
   swap_4(&hr->rdb_hdr_user46, 4);
   swap_4(&hr->rdb_hdr_user47, 4);
   swap_4(&hr->rdb_hdr_user48, 4);
   swap_2(&hr->rdb_hdr_pcfitorig, 2);   	/* Adjust view indexes if set so bestky view = 0 */
   swap_2(&hr->rdb_hdr_pcshotfirst, 2);   	/* First view within an echo group used for fit*/
   swap_2(&hr->rdb_hdr_pcshotlast, 2);   	/* Last view within an echo group used for fit */
   swap_2(&hr->rdb_hdr_pcmultegrp, 2);   	/* If = 1, force pts from other egrps to be used */
   swap_2(&hr->rdb_hdr_pclinfix, 2);   		/* If = 2, force slope to be set to pclinslope */

   /* If = 1, neg readout slope = pos readout slope */
   swap_2(&hr->rdb_hdr_pcconfix, 2);   		/* If = 2, force slope to be set to pcconslope */

   /* If = 1, neg readout slope = pos readout slope */
   swap_4(&hr->rdb_hdr_pclinslope, 4);   	/* Value to set lin slope to if forced */
   swap_4(&hr->rdb_hdr_pcconslope, 4);   	/* Value to set con slope to if forced */
   swap_2(&hr->rdb_hdr_pccoil, 2);   		/* If 1,2,3,4, use that coil's results for all */

   /* Variable View Sharing */
   swap_2(&hr->rdb_hdr_vvsmode, 2);   		/* Variable view sharing mode */
   swap_2(&hr->rdb_hdr_vvsaimgs, 2);   		/* number of original images */
   swap_2(&hr->rdb_hdr_vvstr, 2);   		/* TR in microseconds */
   swap_2(&hr->rdb_hdr_vvsgender, 2);   	/* gender: male or female */

   /* 3D Slice ZIP */
   swap_2(&hr->rdb_hdr_zip_factor, 2);   	/* Slice ZIP factor: 0=OFF, 2, or 4 */

   /* Maxwell Term Correction Coefficients */
   swap_4(&hr->rdb_hdr_maxcoef1a, 4);   	/* Coefficient A for flow image 1 */
   swap_4(&hr->rdb_hdr_maxcoef1b, 4);   	/* Coefficient B for flow image 1 */
   swap_4(&hr->rdb_hdr_maxcoef1c, 4);   	/* Coefficient C for flow image 1 */
   swap_4(&hr->rdb_hdr_maxcoef1d, 4);   	/* Coefficient D for flow image 1 */
   swap_4(&hr->rdb_hdr_maxcoef2a, 4);   	/* Coefficient A for flow image 2 */
   swap_4(&hr->rdb_hdr_maxcoef2b, 4);   	/* Coefficient B for flow image 2 */
   swap_4(&hr->rdb_hdr_maxcoef2c, 4);   	/* Coefficient C for flow image 2 */
   swap_4(&hr->rdb_hdr_maxcoef2d, 4);   	/* Coefficient D for flow image 2 */
   swap_4(&hr->rdb_hdr_maxcoef3a, 4);   	/* Coefficient A for flow image 3 */
   swap_4(&hr->rdb_hdr_maxcoef3b, 4);   	/* Coefficient B for flow image 3 */
   swap_4(&hr->rdb_hdr_maxcoef3c, 4);   	/* Coefficient C for flow image 3 */
   swap_4(&hr->rdb_hdr_maxcoef3d, 4);   	/* Coefficient D for flow image 3 */
   swap_4(&hr->rdb_hdr_ut_ctrl, 4);   		/* System utility control variable */
   swap_2(&hr->rdb_hdr_dp_type, 2);   		/* EPI II diffusion control cv */
   swap_2(&hr->rdb_hdr_arw, 2);   		/* Arrhythmia rejection window(percentage:1-100)*/
   swap_2(&hr->rdb_hdr_vps, 2);			/* View Per Segment for FastCine */ 
   swap_2(&hr->rdb_hdr_mcReconEnable, 2);   	/* N-Coil recon map */

   /*#################### ACQ TABLE HEADER #############################*/
   
   /*#################### SERIES HEADER ################################*/

   swap_2(&hr_se->se_uniq, 2);            /*The Make-Unique Flag*/

   swap_2(&hr_se->se_exno, 2);            /*Exam Number*/
   swap_2(&hr_se->se_no , 2);             /*Series Number*/
   swap_4(&hr_se->se_datetime, 4);        /*Allocation Series Data/Time stamp*/
   swap_4(&hr_se->se_actual_dt, 4);       /*Actual Series Data/Time stamp*/
   swap_2(&hr_se->se_typ , 2);            /*Series Type*/
   swap_2(&hr_se->se_source , 2);         /*Series from which prescribed*/
   swap_2(&hr_se->se_plane , 2);          /*Most-like Plane (for L/S)*/
   swap_2(&hr_se->scan_type , 2);         /*Scout or Axial (for CT)*/
   swap_4(&hr_se->position, 4);           /*Patient Position*/
   swap_4(&hr_se->entry, 4);              /*Patient Entry*/

   swap_4(&hr_se->lmhor, 4);              /*Horizontal Landmark*/

   swap_2(&hr_se->se_contrast , 2);       /*Non-zero if > 0 image used contrast(L/S)*/

   swap_4(&hr_se->start_loc, 4);          /*First scan location (L/S)*/

   swap_4(&hr_se->end_loc, 4);            /*Last scan location (L/S)*/
   swap_2(&hr_se->se_pseq , 2);           /*Last Pulse Sequence Used (L/S)*/
   swap_2(&hr_se->se_sortorder , 2);      /*Image Sort Order (L/S)*/
   swap_4(&hr_se->se_lndmrkcnt, 4);       /*Landmark Counter*/
   swap_2(&hr_se->se_nacq , 2);           /*Number of Acquisitions*/
   swap_2(&hr_se->xbasest , 2);           /*Starting number for baselines*/
   swap_2(&hr_se->xbaseend, 2);           /*Ending number for baselines*/
   swap_2(&hr_se->xenhst, 2);             /*Starting number for enhanced scans*/
   swap_2(&hr_se->xenhend, 2);            /*Ending number for enhanced scans*/

   swap_4(&hr_se->se_lastmod, 4);         /*Date/Time of Last Change*/

   swap_4(&hr_se->se_delta_cnt, 4);       /*Indicates number of updates to header*/
   swap_4(&hr_se->se_pds_a, 4);           /*PixelData size - as stored*/
   swap_4(&hr_se->se_pds_c, 4);           /*PixelData size - Compressed*/
   swap_4(&hr_se->se_pds_u, 4);           /*PixelData size - UnCompressed*/
   swap_4(&hr_se->se_checksum, 4);        /*Series Record checksum*/
   swap_4(&hr_se->se_complete, 4);        /*Series Complete Flag*/
   swap_4(&hr_se->se_numarch, 4);         /*Number of Images Archived*/
   swap_4(&hr_se->se_imagect, 4);         /*Last Image Number Used*/
   swap_4(&hr_se->se_numimages, 4);       /*Number of Images Existing*/

   swap_4(&hr_se->se_numunimg, 4);        /*Number of Unstored Images*/

   swap_4(&hr_se->se_toarchcnt, 4);       /*Number of Unarchived Images*/

   swap_4(&hr_se->echo1_alpha, 4);        /*Echo 1 Alpha Value*/
   swap_4(&hr_se->echo1_beta, 4);         /*Echo 1 Beta Value*/
   swap_2(&hr_se->echo1_window, 2);       /*Echo 1 Window Value*/
   swap_2(&hr_se->echo1_level, 2);        /*Echo 1 Level Value*/
   swap_4(&hr_se->echo2_alpha, 4);        /*Echo 2 Alpha Value*/
   swap_4(&hr_se->echo2_beta, 4);         /*Echo 2 Beta Value*/
   swap_2(&hr_se->echo2_window, 2);       /*Echo 2 Window Value*/
   swap_2(&hr_se->echo2_level, 2);        /*Echo 2 Level Value*/
   swap_4(&hr_se->echo3_alpha, 4);        /*Echo 3 Alpha Value*/
   swap_4(&hr_se->echo3_beta, 4);         /*Echo 3 Beta Value*/
   swap_2(&hr_se->echo3_window, 2);       /*Echo 3 Window Value*/
   swap_2(&hr_se->echo3_level, 2);        /*Echo 3 Level Value*/
   swap_4(&hr_se->echo4_alpha, 4);        /*Echo 4 Alpha Value*/
   swap_4(&hr_se->echo4_beta, 4);         /*Echo 4 Beta Value*/
   swap_2(&hr_se->echo4_window, 2);       /*Echo 4 Window Value*/
   swap_2(&hr_se->echo4_level, 2);        /*Echo 4 Level Value*/
   swap_4(&hr_se->echo5_alpha, 4);        /*Echo 5 Alpha Value*/
   swap_4(&hr_se->echo5_beta, 4);         /*Echo 5 Beta Value*/
   swap_2(&hr_se->echo5_window, 2);       /*Echo 5 Window Value*/
   swap_2(&hr_se->echo5_level, 2);        /*Echo 5 Level Value*/
   swap_4(&hr_se->echo6_alpha, 4);        /*Echo 6 Alpha Value*/
   swap_4(&hr_se->echo6_beta, 4);         /*Echo 6 Beta Value*/
   swap_2(&hr_se->echo6_window, 2);       /*Echo 6 Window Value*/
   swap_2(&hr_se->echo6_level, 2);        /*Echo 6 Level Value*/
   swap_4(&hr_se->echo7_alpha, 4);        /*Echo 7 Alpha Value*/
   swap_4(&hr_se->echo7_beta, 4);         /*Echo 7 Beta Value*/
   swap_2(&hr_se->echo7_window, 2);       /*Echo 7 Window Value*/
   swap_2(&hr_se->echo7_level, 2);        /*Echo 7 Level Value*/
   swap_4(&hr_se->echo8_alpha, 4);        /*Echo 8 Alpha Value*/
   swap_4(&hr_se->echo8_beta, 4);         /*Echo 8 Beta Value*/
   swap_2(&hr_se->echo8_window, 2);       /*Echo 8 Window Value*/
   swap_2(&hr_se->echo8_level, 2);        /*Echo 8 Level Value*/


   
   /*#################### EXAM HEADER ##################################*/

   swap_2(&hr_ex->ex_uniq, 2);        /*The Make-Unique Flag*/

   swap_2(&hr_ex->ex_no, 2);          /*Exam Number*/

   swap_2(&hr_ex->detect, 2);         /*Detector Type*/

   swap_4(&hr_ex->numcells, 4);       /*Number of cells in det*/
   swap_4(&hr_ex->zerocell, 4);       /*Cell number at theta*/
   swap_4(&hr_ex->cellspace, 4);      /*Cell spacing*/
   swap_4(&hr_ex->srctodet, 4);       /*Distance from source to detector*/
   swap_4(&hr_ex->srctoiso, 4);       /*Distance from source to iso*/
   swap_2(&hr_ex->tubetyp, 2);        /*Tube type*/
   swap_2(&hr_ex->dastyp, 2);         /*DAS type*/
   swap_2(&hr_ex->num_dcnk, 2);       /*Number of Decon Kernals*/
   swap_2(&hr_ex->dcn_len, 2);        /*Number of elements in a Decon Kernal*/
   swap_2(&hr_ex->dcn_density, 2);    /*Decon Kernal density*/
   swap_2(&hr_ex->dcn_stepsize, 2);   /*Decon Kernal stepsize*/
   swap_2(&hr_ex->dcn_shiftcnt, 2);   /*Decon Kernal Shift Count*/

   swap_4(&hr_ex->magstrength, 4);    /*Magnet strength (in gauss)*/
   swap_2(&hr_ex->patage, 2);         /*Patient Age (years, months or days)*/
   swap_2(&hr_ex->patian, 2);         /*Patient Age Notation*/
   swap_2(&hr_ex->patsex, 2);         /*Patient Sex*/
   swap_4(&hr_ex->patweight, 4);      /*Patient Weight*/
   swap_2(&hr_ex->trauma, 2);         /*Trauma Flag*/
   swap_4(&hr_ex->ex_datetime, 4);    /*Exam date/time stamp*/
   swap_2(&hr_ex->ex_format, 2);      /*Exam Format*/

   /*swap_8(&hr_ex->firstaxtime, 8);*/    /*Start time(secs) of first axial in exam*/

   swap_4(&hr_ex->ex_lastmod, 4);     /*Date/Time of Last Change*/
   swap_2(&hr_ex->protocolflag, 2);   /*Non-Zero indicates Protocol Exam*/

   swap_4(&hr_ex->ex_delta_cnt, 4);   /*Indicates number of updates to header*/
   swap_4(&hr_ex->ex_checksum, 4);    /*Exam Record Checksum*/
   swap_4(&hr_ex->ex_complete, 4);    /*Exam Complete Flag*/
   swap_4(&hr_ex->ex_seriesct, 4);    /*Last Series Number Used*/
   swap_4(&hr_ex->ex_numarch, 4);     /*Number of Series Archived*/
   swap_4(&hr_ex->ex_numseries, 4);   /*Number of Series Existing*/

   swap_4(&hr_ex->ex_numunser, 4);    /*Number of Unstored Series*/

   swap_4(&hr_ex->ex_toarchcnt, 4);   /*Number of Unarchived Series*/

   swap_4(&hr_ex->ex_prospcnt, 4);    /*Number of Prospective/Scout Series*/

   swap_4(&hr_ex->ex_modelnum, 4);    /*Last Model Number used*/
   swap_4(&hr_ex->ex_modelcnt, 4);    /*Number of ThreeD Models*/

   swap_2(&hr_ex->ex_stat, 2);        /*Patient Status*/
   swap_2(&hr_ex->study_status, 2);   /*indicates if study has complete info(DICOM/genesis)*/


   
   /*#################### IMAGE HEADER #################################*/

   swap_2(&hr_im->im_uniq, 2);            /*The Make-Unique Flag*/

   swap_2(&hr_im->im_exno, 2);            /*Exam number for this image*/
   swap_2(&hr_im->im_seno, 2);            /*Series Number for this image*/
   swap_2(&hr_im->im_no, 2);              /*Image Number*/

   swap_4(&hr_im->im_datetime, 4);        /*Allocation Image date/time stamp*/
   swap_4(&hr_im->im_actual_dt, 4);       /*Actual Image date/time stamp*/
   swap_4(&hr_im->sctime, 4);             /*Duration of scan*/
   swap_4(&hr_im->slthick, 4);            /*Slice Thickness (mm)*/
   swap_2(&hr_im->imatrix_X, 2);          /*Image matrix size - X*/
   swap_2(&hr_im->imatrix_Y, 2);          /*Image matrix size - Y*/
   swap_4(&hr_im->dfov, 4);               /*Display field of view - X (mm)*/
   swap_4(&hr_im->dfov_rect, 4);          /*Display field of view - Y (if different)*/
   swap_4(&hr_im->dim_X, 4);              /*Image dimension - X*/
   swap_4(&hr_im->dim_Y, 4);              /*Image dimension - Y*/
   swap_4(&hr_im->pixsize_X, 4);          /*Image pixel size - X*/
   swap_4(&hr_im->pixsize_Y, 4);          /*Image pixel size - Y*/
   swap_2(&hr_im->contmode, 2);           /*Image Contrast Mode*/
   swap_2(&hr_im->serrx, 2);              /*Series from which prescribed*/
   swap_2(&hr_im->imgrx, 2);              /*Image from which prescribed*/
   swap_2(&hr_im->screenformat, 2);       /*Screen Format(8/16 bit)*/
   swap_2(&hr_im->plane, 2);              /*Plane Type*/

   swap_4(&hr_im->scanspacing, 4);        /*Spacing between scans (mm?)*/
   swap_2(&hr_im->im_compress, 2);        /*Image compression type for allocation*/
   swap_2(&hr_im->im_scouttype, 2);       /*Scout Type (AP or lateral)*/


   swap_4(&hr_im->loc, 4);                /*Image location*/
   swap_4(&hr_im->ctr_R, 4);              /*Center R coord of plane image*/
   swap_4(&hr_im->ctr_A, 4);              /*Center A coord of plane image*/
   swap_4(&hr_im->ctr_S, 4);              /*Center S coord of plane image*/
   swap_4(&hr_im->norm_R, 4);             /*Normal R coord*/
   swap_4(&hr_im->norm_A, 4);             /*Normal A coord*/
   swap_4(&hr_im->norm_S, 4);             /*Normal S coord*/
   swap_4(&hr_im->tlhc_R, 4);             /*R Coord of Top Left Hand Corner*/
   swap_4(&hr_im->tlhc_A, 4);             /*A Coord of Top Left Hand Corner*/
   swap_4(&hr_im->tlhc_S, 4);             /*S Coord of Top Left Hand Corner*/
   swap_4(&hr_im->trhc_R, 4);             /*R Coord of Top Right Hand Corner*/
   swap_4(&hr_im->trhc_A, 4);             /*A Coord of Top Right Hand Corner*/
   swap_4(&hr_im->trhc_S, 4);             /*S Coord of Top Right Hand Corner*/
   swap_4(&hr_im->brhc_R, 4);             /*R Coord of Bottom Right Hand Corner*/
   swap_4(&hr_im->brhc_A, 4);             /*A Coord of Bottom Right Hand Corner*/
   swap_4(&hr_im->brhc_S, 4);             /*S Coord of Bottom Right Hand Corner*/
   swap_4(&hr_im->tr, 4);                 /*Pulse repetition time(usec)*/
   swap_4(&hr_im->ti, 4);                 /*Pulse inversion time(usec)*/
   swap_4(&hr_im->te, 4);                 /*Pulse echo time(usec)*/
   swap_4(&hr_im->te2, 4);                /*Second echo echo (usec)*/
   swap_2(&hr_im->numecho, 2);            /*Number of echoes*/
   swap_2(&hr_im->echonum, 2);            /*Echo Number*/
   swap_4(&hr_im->tbldlta, 4);            /*Table Delta*/
   swap_4(&hr_im->nex, 4);                /*Number of Excitations*/
   swap_2(&hr_im->contig, 2);             /*Continuous Slices Flag*/
   swap_2(&hr_im->hrtrate, 2);            /*Cardiac Heart Rate (bpm)*/
   swap_4(&hr_im->tdel, 4);               /*Delay time after trigger (msec)*/
   swap_4(&hr_im->saravg, 4);             /*Average SAR*/
   swap_4(&hr_im->sarpeak, 4);            /*Peak SAR*/
   swap_2(&hr_im->monsar, 2);             /*Monitor SAR flag*/
   swap_2(&hr_im->trgwindow, 2);          /*Trigger window (% of R-R interval)*/
   swap_4(&hr_im->reptime, 4);            /*Cardiac repetition time*/
   swap_2(&hr_im->imgpcyc, 2);            /*Images per cardiac cycle*/
   swap_2(&hr_im->xmtgain, 2);            /*Actual Transmit Gain (.1 db)*/
   swap_2(&hr_im->rcvgain1, 2);           /*Actual Receive Gain Analog (.1 db)*/
   swap_2(&hr_im->rcvgain2, 2);           /*Actual Receive Gain Digital (.1 db)*/
   swap_2(&hr_im->mr_flip, 2);            /*Flip Angle for GRASS scans (deg.)*/

   swap_4(&hr_im->mindat, 4);             /*Minimum Delay after Trigger (uSec)*/
   swap_2(&hr_im->cphase, 2);             /*Total Cardiac Phase prescribed*/
   swap_2(&hr_im->swappf, 2);             /*Swap Phase/Frequency Axis*/
   swap_2(&hr_im->pauseint, 2);           /*Pause Interval (slices)*/

   swap_4(&hr_im->pausetime, 4);          /*Pause Time*/
   swap_4(&hr_im->obplane, 4);            /*Oblique Plane*/
   swap_4(&hr_im->slocfov, 4);            /*Slice Offsets on Freq axis*/
   swap_4(&hr_im->xmtfreq, 4);            /*Center Frequency (0.1 Hz)*/
   swap_4(&hr_im->autoxmtfreq, 4);        /*Auto Center Frequency (0.1 Hz)*/
   swap_2(&hr_im->autoxmtgain, 2);        /*Auto Transmit Gain (0.1 dB)*/
   swap_2(&hr_im->prescan_r1, 2);         /*PreScan R1 - Analog*/
   swap_2(&hr_im->prescan_r2, 2);         /*PreScan R2 - Digital*/

   swap_4(&hr_im->user_bitmap, 4);        /*Bitmap defining user CVs*/
   swap_2(&hr_im->cenfreq, 2);            /*Center Frequency Method*/
   swap_2(&hr_im->imode, 2);              /*Imaging Mode*/
   swap_4(&hr_im->iopt, 4);               /*Imaging Options*/
   swap_2(&hr_im->pseq, 2);               /*Pulse Sequence*/
   swap_2(&hr_im->pseqmode, 2);           /*Pulse Sequence Mode*/


   swap_4(&hr_im->psd_datetime, 4);       /*PSD Creation Date and Time*/

   swap_2(&hr_im->ctyp, 2);               /*Coil Type*/

   swap_2(&hr_im->surfctyp, 2);           /*Surface Coil Type*/
   swap_2(&hr_im->surfcext, 2);           /*Extremity Coil Flag*/

   swap_4(&hr_im->rawrunnum, 4);          /*RawData Run Number*/
   swap_4(&hr_im->cal_fldstr, 4);         /*Calibrated Field Strength (x10 uGauss)*/
   swap_2(&hr_im->supp_tech, 2);          /*SAT fat/water/none*/

   swap_4(&hr_im->vbw, 4);                /*Variable Bandwidth (Hz)*/
   swap_2(&hr_im->slquant, 2);            /*Number of slices in this scan group*/
   swap_2(&hr_im->gpre, 2);               /*Graphically prescribed*/
   swap_4(&hr_im->intr_del, 4);           /*Interimage/interloc delay (uSec)*/
   swap_4(&hr_im->user0, 4);              /*User Variable 0*/
   swap_4(&hr_im->user1, 4);              /*User Variable 1*/
   swap_4(&hr_im->user2, 4);              /*User Variable 2*/
   swap_4(&hr_im->user3, 4);              /*User Variable 3*/
   swap_4(&hr_im->user4, 4);              /*User Variable 4*/
   swap_4(&hr_im->user5, 4);              /*User Variable 5*/
   swap_4(&hr_im->user6, 4);              /*User Variable 6*/
   swap_4(&hr_im->user7, 4);              /*User Variable 7*/
   swap_4(&hr_im->user8, 4);              /*User Variable 8*/
   swap_4(&hr_im->user9, 4);              /*User Variable 9*/
   swap_4(&hr_im->user10, 4);             /*User Variable 10*/
   swap_4(&hr_im->user11, 4);             /*User Variable 11*/
   swap_4(&hr_im->user12, 4);             /*User Variable 12*/
   swap_4(&hr_im->user13, 4);             /*User Variable 13*/
   swap_4(&hr_im->user14, 4);             /*User Variable 14*/
   swap_4(&hr_im->user15, 4);             /*User Variable 15*/
   swap_4(&hr_im->user16, 4);             /*User Variable 16*/
   swap_4(&hr_im->user17, 4);             /*User Variable 17*/
   swap_4(&hr_im->user18, 4);             /*User Variable 18*/
   swap_4(&hr_im->user19, 4);             /*User Variable 19*/
   swap_4(&hr_im->user20, 4);             /*User Variable 20*/
   swap_4(&hr_im->user21, 4);             /*User Variable 21*/
   swap_4(&hr_im->user22, 4);             /*User Variable 22*/
   swap_4(&hr_im->user23, 4);             /*Projection Angle*/
   swap_4(&hr_im->user24, 4);             /*Concat Sat Type Flag*/


   swap_4(&hr_im->im_lastmod, 4);         /*Date/Time of Last Change*/
   swap_4(&hr_im->im_pds_a, 4);           /*PixelData size - as stored*/
   swap_4(&hr_im->im_pds_c, 4);           /*PixelData size - Compressed*/
   swap_4(&hr_im->im_pds_u, 4);           /*PixelData size - UnCompressed*/
   swap_4(&hr_im->im_checksum, 4);        /*AcqRecon record checksum*/
   swap_4(&hr_im->im_archived, 4);        /*Image Archive Flag*/
   swap_4(&hr_im->im_complete, 4);        /*Image Complete Flag*/
   swap_2(&hr_im->satbits, 2);            /*Bitmap of SAT selections*/
   swap_2(&hr_im->scic, 2);               /*Surface Coil Intensity Correction Flag*/
   swap_2(&hr_im->satxloc1, 2);           /*R-side SAT pulse loc rel to lndmrk*/
   swap_2(&hr_im->satxloc2, 2);           /*L-side SAT pulse loc rel to lndmrk*/
   swap_2(&hr_im->satyloc1, 2);           /*A-side SAT pulse loc rel to lndmrk*/
   swap_2(&hr_im->satyloc2, 2);           /*P-side SAT pulse loc rel to lndmrk*/
   swap_2(&hr_im->satzloc1, 2);           /*S-side SAT pulse loc rel to lndmrk*/
   swap_2(&hr_im->satzloc2, 2);           /*I-side SAT pulse loc rel to lndmrk*/
   swap_2(&hr_im->satxthick, 2);          /*Thickness of X-axis SAT pulse*/
   swap_2(&hr_im->satythick, 2);          /*Thickness of Y-axis SAT pulse*/
   swap_2(&hr_im->satzthick, 2);          /*Thickness of Z-axis SAT pulse*/
   swap_2(&hr_im->flax, 2);               /*Phase contrast flow axis*/
   swap_2(&hr_im->venc, 2);               /*Phase contrast velocity encoding*/
   swap_2(&hr_im->thk_disclmr, 2);        /*Slice Thickness*/
   swap_2(&hr_im->ps_flag, 2);            /*Auto/Manual Prescan flag*/
   swap_2(&hr_im->ps_status, 2);          /*Bitmap of changed values*/
   swap_2(&hr_im->image_type, 2);         /*Magnitude, Phase, Imaginary, or Real*/
   swap_2(&hr_im->vas_collapse, 2);       /*Collapse Image*/
   swap_4(&hr_im->user23n, 4);            /*User Variable 23*/
   swap_4(&hr_im->user24n, 4);            /*User Variable 24*/
   swap_2(&hr_im->proj_alg, 2);           /*Projection Algorithm*/

   swap_4(&hr_im->x_axis_rot, 4);         /*X Axis Rotation*/
   swap_4(&hr_im->y_axis_rot, 4);         /*Y Axis Rotation*/
   swap_4(&hr_im->z_axis_rot, 4);         /*Z Axis Rotation*/
   swap_4(&hr_im->thresh_min1, 4);        /*Lower Range of Pixels 1*/
   swap_4(&hr_im->thresh_max1, 4);        /*Upper Range of Pixels 1*/
   swap_4(&hr_im->thresh_min2, 4);        /*Lower Range of Pixels 2*/
   swap_4(&hr_im->thresh_max2, 4);        /*Upper Range of Pixels 2*/
   swap_2(&hr_im->echo_trn_len, 2);       /*Echo Train Length for Fast Spin Echo*/
   swap_2(&hr_im->frac_echo, 2);          /*Fractional Echo - Effective TE Flag*/
   swap_2(&hr_im->prep_pulse, 2);         /*Preporatory Pulse Option*/
   swap_2(&hr_im->cphasenum, 2);          /*Cardiac Phase Number*/
   swap_2(&hr_im->var_echo, 2);           /*Variable Echo Flag*/
   swap_2(&hr_im->img_window, 2);         /*Window Value*/
   swap_2(&hr_im->img_level, 2);          /*Level Value*/
   swap_4(&hr_im->slop_int_1, 4);         /*Number of 3D Slabs*/
   swap_4(&hr_im->slop_int_2, 4);         /*Slice Locs Per 3D Slab*/
   swap_4(&hr_im->slop_int_3, 4);         /*# of Slice Locs on Each Slab Which Overlap Neighbors*/
   swap_4(&hr_im->slop_int_4, 4);         /*Image Filtering 0.5/0.2T*/
   swap_4(&hr_im->slop_int_5, 4);         /*Integer Slop Field 5*/
   swap_4(&hr_im->slop_float_1, 4);       /*Float Slop Field 1*/
   swap_4(&hr_im->slop_float_2, 4);       /*Float Slop Field 2*/
   swap_4(&hr_im->slop_float_3, 4);       /*Float Slop Field 3*/
   swap_4(&hr_im->slop_float_4, 4);       /*Float Slop Field 4*/
   swap_4(&hr_im->slop_float_5, 4);       /*Float Slop Field 5*/
   swap_2(&hr_im->scanactno, 2);          /*Scan Acquisition Number*/
   swap_2(&hr_im->vasflags, 2);           /*Magnitude Weighting Flag*/
   swap_4(&hr_im->vencscale, 4);          /*Scale Weighted Venc*/
   swap_2(&hr_im->integrity, 2);          /*GE Image Integrity*/

   swap_4(&hr_im->fphase, 4);             /*Number Of Phases*/
   swap_2(&hr_im->freq_dir, 2);           /*Frequency Direction*/
   swap_2(&hr_im->vas_mode, 2);           /*Vascular Mode*/
   swap_2(&hr_im->dont_use_1, 2);         /*This field is not used*/
   swap_2(&hr_im->dont_use_2, 2);         /*This field is not used*/
   swap_2(&hr_im->dont_use_3, 2);         /*This field is not used*/
   swap_2(&hr_im->pscopts, 2);            /*bitmap of prescan options*/
   swap_2(&hr_im->asoffsetx, 2);          /*gradient offset in X-direction*/
   swap_2(&hr_im->asoffsety, 2);          /*gradient offset in Y-direction*/
   swap_2(&hr_im->asoffsetz, 2);          /*gradient offset in Z-direction*/
   swap_2(&hr_im->unoriginal, 2);         /*identifies image as original or unoriginal*/
   swap_2(&hr_im->interleaves, 2);        /*number of EPI shots*/
   swap_2(&hr_im->effechospace, 2);       /*effective echo spacing for EPI*/
   swap_2(&hr_im->viewsperseg, 2);        /*views per segment*/
   swap_2(&hr_im->rbpm, 2);               /*respiratory rate, breaths per min*/
   swap_2(&hr_im->rtpoint, 2);            /*respiratory trigger point as percent of max.*/
   swap_2(&hr_im->rcvrtype, 2);           /*type of receiver used*/
   swap_4(&hr_im->dbdt, 4);               /*peak rate of change of gradient field, tesla/sec*/
   swap_4(&hr_im->dbdtper, 4);            /*limit in units of percent of theoretical curve*/
   swap_4(&hr_im->estdbdtper, 4);         /*PSD estimated limit in units of percent*/
   swap_4(&hr_im->estdbdtts, 4);          /*PSD estimated limit in Teslas/sec*/
   swap_4(&hr_im->saravghead, 4);         /*Avg head SAR*/
   swap_4(&hr_im->neg_scanspacing, 4);    /*Negative scan spacing for overlap slices*/
   swap_4(&hr_im->offsetfreq, 4);         /*Offset Frequency - Mag.Transfer*/
   swap_4(&hr_im->user_usage_tag, 4);     /*Defines how following user CVs are to be filled in*/
                                                     /*Default value = 0x00000000*/
                                                     /*GE range = 0x00000001 - 0x7fffffff*/
                                                     /*Research = 0x80000000 - 0xffffffff*/
   swap_4(&hr_im->user_fill_mapMSW, 4);   /*Define what process fills in the user CVs, ifcc or TIR*/
   swap_4(&hr_im->user_fill_mapLSW, 4);   /*Define what process fills in the user CVs, ifcc or TIR*/
   swap_4(&hr_im->user25, 4);             /*User Variable 25*/
   swap_4(&hr_im->user26, 4);             /*User Variable 26*/
   swap_4(&hr_im->user27, 4);             /*User Variable 27*/
   swap_4(&hr_im->user28, 4);             /*User Variable 28*/
   swap_4(&hr_im->user29, 4);             /*User Variable 29*/
   swap_4(&hr_im->user30, 4);             /*User Variable 30*/
   swap_4(&hr_im->user31, 4);             /*User Variable 31*/
   swap_4(&hr_im->user32, 4);             /*User Variable 32*/
   swap_4(&hr_im->user33, 4);             /*User Variable 33*/
   swap_4(&hr_im->user34, 4);             /*User Variable 34*/
   swap_4(&hr_im->user35, 4);             /*User Variable 35*/
   swap_4(&hr_im->user36, 4);             /*User Variable 36*/
   swap_4(&hr_im->user37, 4);             /*User Variable 37*/
   swap_4(&hr_im->user38, 4);             /*User Variable 38*/
   swap_4(&hr_im->user39, 4);             /*User Variable 39*/
   swap_4(&hr_im->user40, 4);             /*User Variable 40*/
   swap_4(&hr_im->user41, 4);             /*User Variable 41*/
   swap_4(&hr_im->user42, 4);             /*User Variable 42*/
   swap_4(&hr_im->user43, 4);             /*User Variable 43*/
   swap_4(&hr_im->user44, 4);             /*User Variable 44*/
   swap_4(&hr_im->user45, 4);             /*User Variable 45*/
   swap_4(&hr_im->user46, 4);             /*User Variable 46*/
   swap_4(&hr_im->user47, 4);             /*User Variable 47*/
   swap_4(&hr_im->user48, 4);             /*User Variable 48*/
   swap_4(&hr_im->slop_int_6, 4);         /*Integer Slop Field 6*/
   swap_4(&hr_im->slop_int_7, 4);         /*Integer Slop Field 7*/
   swap_4(&hr_im->slop_int_8, 4);         /*Integer Slop Field 8*/
   swap_4(&hr_im->slop_int_9, 4);         /*Integer Slop Field 9*/
   
}
