/***********************************************************************
 * File: 	dst.h
 * Programmer:	Ning Kang
 * Version of:	6/8/2004
 * --------------------------------------------------------------------
 * Description:
 * - Serial version  
 * - int and float version  
 ***********************************************************************/

 
/***********************************************************************
 * Header files
 ***********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <errno.h>
                                // header files needed for IDA.
#include "sundialstypes.h"      // definitions of real, integer, boolean type
#include "sundialsmath.h"
#include "ida.h"                // main IDA header file                       
#include "idaspgmr.h"           // use IDASPGMR linear solver each internal step
#include "iterativ.h"           // contains the enum for preconditioning types 
#include "nvector_serial.h"     // definitions of type N_Vector, macro NV_DATA_S

#include "dst_interface.h"

/***********************************************************************
 * Some macro definitions for conditional inclusion.
 ***********************************************************************/
#define ILU0
#define FORTRAN_UNDERSCORE      // both SunOS and HP-UX need appended 
                                // underscore for FORTRAN routine name
#define DEBUG_PRINT
//#define DEBUG_PRINT_SEED_NODE
#define TIMER

typedef float realtype_;        // for reading the tensor data

/***********************************************************************
 * Input and output files 
 ***********************************************************************/
//#define DE              // data format as of Germany (one single file)
//#define DT_SCALE      // under DE: need to scale tensor values to mm^2/sec?
#define AU            // data format as of Australia (six files)

//#define ASCII_DATA_FILE  // input tensor data is ASCII or binary (default)

#define FT_PREFIX    "ft"

//#define FILE_INDEX "index"
#ifndef STR_LEN
#define STR_LEN 256
#endif

#define FILE_DIFF_PROFILE "diff.vtk"

/***********************************************************************
 * NX, NY, NZ are the dimensions of the 3D diffusion kernel.
 * NEQ is the number of unknowns.
 ***********************************************************************/
#define NX_KERNEL 11//17//7//17//9//11
#define NY_KERNEL 11//17//7//17//9//11
#define NZ_KERNEL 7//7//13//8//13//7
#define NEQ NX_KERNEL*NY_KERNEL*NZ_KERNEL
               // for rank-4 DT
#define NX_KERNEL_DT4 11//17//7//17//9//11
#define NY_KERNEL_DT4 11//17//7//17//9//11
#define NZ_KERNEL_DT4 7//7//13//8//13//7
#define NEQ_DT4 NX_KERNEL_DT4*NY_KERNEL_DT4*NZ_KERNEL_DT4

/***********************************************************************
 * Pathway connection thresholds.
 ***********************************************************************/
                        /* diffusion volume ratio for determining 
			   concentration threshold */  
//#define DIFFVOL 0.08     
#define DIFFSIZ 68
#define DIFFSIZ_DT4 68
                       // criteria set C = {c1 c2 c3 c4 c5}
#define DIST_RATIO dti->distRatio  //0.8 // distance ratio
#define TENSOR_FA dti->tensorFA    //0.2  // fractional anisotropy index
#define IP_VECTORS dti->ipVectors  //0.8 // angles between two neighbor vectors
#define IP_SEED_VEC_NODE dti->ipSeedVecNode  //0.8 // inner product thresholds

#define FRONT_MAX_NODES 5  //4// threshold for number of voxels on front
#define FRONT_MAX_NODES_ROOT 10 //4// threshold for front voxels of root seeds

#define IP_SEED_VEC  IP_SEED_VEC_NODE      
#define IP_SEED_NODE IP_SEED_VEC_NODE  
#define IP_NODE_VEC  IP_SEED_VEC_NODE

#define LEVEL_ZERO 0
                        /* MIN_LEVEL must <= actual lowest level */
#define MIN_LEVEL 5     /* set minimum level to remove very short tracks */

#define DWI_B0_MAX 1.0e6
#define DWI_B0_LO dti->b0_lo
#define DWI_B0_HI dti->b0_hi

                        /* specify ROI for track end points */
//#define SET_END_ROI
#define ROI_X_LO 85
#define ROI_X_HI 170
#define ROI_Y_LO 0
#define ROI_Y_HI 115//125
#define ROI_Z_LO 30
#define ROI_Z_HI 70
                        /* if a seed point falls into this erroneous zone,
			   tracking will not go on from it. */
//#define SET_ERR_ZONE
#define ZONE_X_LO 0//95
#define ZONE_X_HI 0//132//115//105
#define ZONE_Y_LO 0
#define ZONE_Y_HI 0//256//120
#define ZONE_Z_LO 0//70//70
#define ZONE_Z_HI 0//147//80

//#define SET_ERR_ZONE_1
#define ZONE_1_X_LO 0//142//125
#define ZONE_1_X_HI 0//256
#define ZONE_1_Y_LO 0
#define ZONE_1_Y_HI 0//256
#define ZONE_1_Z_LO 0
#define ZONE_1_Z_HI 0//147

/***********************************************************************
 * Parameters for ftvtk.c
 ***********************************************************************/
#ifndef FALSE
  #define FALSE 0
#endif  
#ifndef TRUE
  #define TRUE 1
#endif

#define B0_BRAIN_VOLUME_THRESHOLD 400
                        /* number of approximated points for each  
                           segment (bounded by two original data points) */
#define APPR_PTS_NUM  5   //10//3  
#define SPLINE_ORDER 3         /* the spline order */
#define NUM_PP 4        /* the number of polynomial pieces */

/***********************************************************************
 * Parameters for GMRES method and IDA solver. 
 ***********************************************************************/
#define MAX_L 10                // maximum Krylov subspace dimension
#define MAX_RS 10               // maximum number of restarts used in GMRES.
#define EPLI_FAC 1.0e-2		// A factor in the linear iteration convergence
				// test constant. Set it to be 0.0 to use the
				// default. (default = 1.0)
   			        // The value of t, at which a computed solution
                                // is desired.
//#define T_OUT args->diffIntegTime  //   0.005//0.005//0.1//1//0.1//0.03 
#define I_TASK ONE_STEP_TSTOP   // the type of task to be used.
//#define I_TASK NORMAL_TSTOP     // the type of task to be used.
#define I_TOL SS                // the type of tolerances to be used.
#define R_TOL 1.0e-3            // relative tolerance, scalar
#define A_TOL 1.0e-3            // absolute tolerance, scalar

                                /* parameters for optional input */
#define MAX_ORD 5               // maximum order to be used. Default = 5.
#define MAX_STEP 500            // maximum number of internal steps to be taken 
                                // to reach tout. Default = 500.
                                // initial step size.
//#define H_INIT T_OUT//0.005//0.1//0.1//0.05//0.001              
                                // Maximum absolute value of step size allowed.
#define H_MAX 0.1//0.4//0.05//0.5//0.01         
                                // Default = infinity.
#define N_CONFAC 0.2		// factor in nonlinear convergence test for use
				// during integration. (default = 1)
 
/***********************************************************************
 * Parameters for ILU0 and ILUT preconditioners.
 ***********************************************************************/
#define LFIL 1
#define ILUT_TOL 1.0e-2
//#define NNZ_MAX ( NEQ*(19+LFIL*2) ) 
#define NNZ_MAX ( NEQ*20 ) 
#define NNZ_MAX_DT4 ( NEQ_DT4*50 ) 
#define IWK NNZ_MAX             // The length of arrays alu and jlu in ILUT.
#define IWK_DT4 NNZ_MAX_DT4

/***********************************************************************
 * Parameters for processing tensor data.
 ***********************************************************************/
#ifndef DTXYZ_NUM
#define DTXYZ_NUM 6             // number of diffusion tensor components
#endif
                    // components from one voxel saved consecutively
//#define DTXYZ(i,j) (*(dtxyz+i*DTXYZ_NUM+j)) 
                   // voxels from one component saved consecutively
#define DTXYZ(id,component) dtxyz[component][id]
               // macro for 4th-order diffusion tensor
#define DTXYZ_4TH(id,component) dt4Elem[component][id]

#define NEI_NUM 26
                                // zero tensor component threshold
//#define ZERO_DTXYZ 1.0e-15

#define DT_THRESHOLD 1.0e-4     // for getting the scale factor
#define SF_CONST 1.0e3          // The scale factor constant

/***********************************************************************
 * Miscellaneous.
 ***********************************************************************/
#ifdef DT_SCALE
  #define TENSOR_SCALE 1.0e6
#else
  #define TENSOR_SCALE 1
#endif

#define WHITE 101
#define GREY 102
#define DARK_GREY 103
#define BLACK 104

#define SEED_PEAK 1.0           /* initial concentration for seed node */

#define OUT_OF_GRID -1
#define NO_FRONT_AT_ROOT -2

#ifndef DST_SUCCESS
#define DST_SUCCESS 0     // the same as IDA_SUCCESS
#endif

#define ELEM_SWAP(a,b) { register float t=(a);(a)=(b);(b)=t; }


/***********************************************************************
 * User data structure for storing information
 * for each voxel in the diffusion kernel.
 ***********************************************************************/
struct kernel_voxel_
{
    int gridID;                 // id# in the global grid, in natural order
    int id;                     // id# in the kernel
    int i, j, k;                // Cartesian indeces of the kernel voxel
    realtype dist;              // distance to its parent
    realtype fa;                // fraction anisotropy index
    float b0;                   // b0 image
//    realtype md;                // mean diffusivity
    int zeroTensor;             // whether all components of tensor are zero
    realtype eigvec[3];           // principal eigenvector 
    struct kernel_voxel_ *next, *tail;
};
typedef struct kernel_voxel_ *Kernel_Voxel_;
/*
               // for 4th-order DT
struct kernel_voxel_dt4_
{
    int gridID;                 // id# in the global grid, in natural order
    int id;                     // id# in the kernel
    int i, j, k;                // Cartesian indeces of the kernel voxel
    realtype dist;              // distance to its parent
    realtype gfa;                // fraction anisotropy index
    int zeroTensor;             // whether all components of tensor are zero
    struct kernel_voxel_ *next, *tail;
};
typedef struct kernel_voxel_dt4_ *KernelVoxel_dt4_;
*/

/***********************************************************************
 * User data structure, mainly store info about
 * the coefficient matrix with CSR format and some geometric
 * data for the diffusion kernel.
 ***********************************************************************/
                   // structure tag "__Diff_kernel_" 
		   // added for gcc to do C++ name mangling
typedef struct __Diff_Kernel_    
{
    realtype *aa;               // Matrix stored with CSR format: aa, ja, ia
    int *ia, *ja;               
    int neq,                    // number of unknows or size of kernel
        nnz,                    // number of non-zero elements 
	nx, ny, nz;             // dimensions of the kernel
    realtype dx, dy, dz;        // delta x, delta y, and delta z
                        /* coefficients in cutting plane equation ax+by+cz+d=0, 
			   (pa,pb,pc) is a unit normal vector of the plane */
    realtype pa, pb, pc, pd;
    realtype *sf;               // scale factor 
    Kernel_Voxel_ *voxel;
    int seedID,                 // id# of the seed
        seedI, seedJ, seedK;
    int median;
    realtype cThreshold;        // threshold for zero concentration
    realtype *diffval;          // work array for determining threshold
} *Diff_Kernel_;


/***********************************************************************
 * User data structure for ILU precondtioners (ILU0 and ILUT),
 * including parameters required for using SPARSKITS package. 
 ***********************************************************************/
typedef struct __ILU_
{
    realtype *alu;              // L/U matrix: alu, jlu, ju.
    int *jlu, *ju;
                                // Required ILUT parameters in SPARSKITS 
    realtype tol;               // Threshold in L and U matrix.
    int lfil,                   // Fill-in parameter
        iwk,                    // The minimum length of arrays alu and jlu
	ierr;                   // Error message
    int *jr, *jwu, *jwl;        // Integer work array with length n
    realtype *wu, *wl;          // Real work arrays of length n+1 and n, resp
} *ILU_;


/***********************************************************************
 * User data structure for DAE solver - IDA, including 
 * variables and parameters used in the SUNDIALS package.
 ***********************************************************************/
typedef struct __DAE_
{
    M_Env machEnv;
    N_Vector yVector,           // Solution vector y
        ypVector,               // Yprime vector y' 
        rVector,                // Residual vector r
        constraintsVector,      // Constraints vector for solution values. 
        idVector;               // A vector which states a given element to be
                                // either algebraic or differential
    void *mem;                  // The pointer to the memory allocated by 
                                // function 'IDAMalloc'.

    int itol,                   // The type of tolerances used
	itask;                  // The type of tasks used
    realtype rtol,              // Relative tolerance scalar
	atol;                   // Absolute tolerance scalar

    long int *iopt;             // The user-allocated array (of size OPT_SIZE,
                                // given by the package) that will hold
				// optional integer inputs and outputs.
    realtype *ropt;             // The user-allocated array (of size OPT_SIZE,
                                // given by the package) that will hold
				// optional real inputs and outputs.
    booleantype optIn;          // A flag indicating if there are any optional
                                // inputs from the user in arrays 'iopt'&'ropt'

    realtype t0,                // Initial value of t, the independent variable
        tout,                   // The next value of t, at which a computed
	                        // solution is desired. 
        t,                      // Actual value of t corresponding to the
	                        // solution vector y
        epliFactor;             // Linear iteration convergence test constant
	
    int maxKDim,                // maximum Krylov subspace dimension
        maxRestart;             // Maximum number of restarts to be used in the
				// GMRES.
    realtype initPeak;
    int loop;
} *DAE_;


/***********************************************************************
 * User data structure for storing information
 * for each voxel in the 3D data set.
 ***********************************************************************/
struct voxel_
{
    int i, j, k;                // Cartesian indeces of the voxel.
    int color;                  // whether the voxel has been visited or not.   
    int level;                  // which level the voxel is on in the BFS graph
    int id;                   // voxel index
    struct voxel_ *next, *tail;
    struct voxel_ *parent, *child;
};
typedef struct voxel_ *Voxel_;


/***********************************************************************
 * User data structure for DTI information
 ***********************************************************************/
typedef struct __DTI_
{
    int num, nx, ny, nz;        // sizes of the DTI data set
    int seedID, 
        seedI, seedJ, seedK;    // current node indeces for diffusion
    int dtxyzNum;               // number of diffusion tensor components
    realtype dx, dy, dz;        // delta x, delta y, and delta z
    realtype *dtxyz[DTXYZ_NUM];  // diffusion tensor components 
    float *fa, *md;             // FA and MD
    float *b0;                  // b0 image in DWI
    float *eigvec[3];            // principal eigenvector
    Voxel_ *voxel;
    float tensorFA, distRatio, ipVectors, ipSeedVecNode;// tracking thresholds
    float b0_lo, b0_hi;         // threshold for b0 image
               // for 4th-order DT
    float *dt4_gfa; // GFA
    float *dt4Elem[DT4_ELEM];
    float dt4_GFA, dt4_distRatio, dt4_ipVectors, dt4_integTime;
} *DTI_;


/***********************************************************************
 * User data structure for ftvtk.c
 ***********************************************************************/
                                /* a fiber path */    
typedef struct __PATH_
{
    int *ptIndex;               // indeces of points on the path
    int numPts;                 // number of points on the path
} *PATH_;                       

struct trimmedFT_
{
//    int ptOffset;  // offset in arrays of data coordinates
    int pathOffset; // offset in the array of paths 
    int ptIdxDisp;  // redisplace the point indices in this path
    struct trimmedFT_ *next;
};
typedef struct trimmedFT_ *TrimmedFT_;
                                /* fiber tract in VTK format */    
typedef struct __FT_
{
    int origSize,               // number of data points before approximation
        size,                   // number of data points after approximation
	numPaths;               // number of fiber paths
    int *ptn;                   // sequence number
//    int *origPtn;               // original sequence number
    float *ptx, *pty, *ptz;       // data coordinates
    float *ptFA, *ptMD;          // fa and md values
    int *origX, *origY, *origZ; // data coordinates before approximation
    float *origFA, *origMD;    // original fa and md values
    int *origPts;               // original number of points on each path
    PATH_ path;                 // fiber paths
    TrimmedFT_ tft_head;   // head pointer to linked list of retrieved paths
    int trimmedNumPaths;   // number of paths going through target region
    int trimmedSize;       // number of points in retrieved paths
} *FT_;
                   // for hash table to retrieve paths through target 
struct targetPt_
{
    int x, y, z;
    struct targetPt_ *next;
};
typedef struct targetPt_ *TargetPt_;  

/*
int *_MyArray; 
_MyArray=(int *)malloc(M*N*sizeof(int)); 
#define MyArray(x, y) (*(_MyArray+x*N+y)) 
int **array1 = (int **)malloc(nrows * sizeof(int *));
for(i = 0; i < nrows; i++)
    array1[i] = (int *)malloc(ncolumns * sizeof(int));
*/


/***********************************************************************
 * The C function prototypes.
 ***********************************************************************/
                                // File: datainit.c
Diff_Kernel_ kernelData_new( void *argsTable, int neq, int nnz, int nx, 
    int ny, int nz, int diffSize );
void kernelData_free( void *kernelData );
ILU_ iluData_new( int neq, int nnz, int iwk );
void iluData_free( void *iluData );
DAE_ daeData_new( void *argsTable );
void daeData_free( void *daeData );
DTI_ dtiData_new( void *argsTable );
void dtiData_free( void *dtiData );
DAE_ daeData_new_DT4( void *argsTable );
DTI_ dtiData_new_DT4( void *argsTable );
void initDTIvoxel( void *dtiData );
void resetDTIvoxel( void *dtiData );

                                // File: matrix.c
void getCoeffMatrixStructure( void *kernelData );
int getCoeffMatrixEntry( void *kernelData, void *dtiData );
int nodeIndex( int i, int j, int k, int nx, int nxy );
int nodeIndex_zero( int i, int j, int k, int nx, int nxy );
                                // File:diffkernel.c
int kernelToGridMapping( void *kernelData, void *dtiData );
int kernelToGridMapping_DT4( void *kernelData, void *dtiData );
void diffThreshold( void *kernelData, void *daeData );

                                // File: diffsim.c
int rootNodeDiffSim( void *kernelData, void *daeData, void *iluData );
int leafNodeDiffSim( void *kernelData, void *daeData, void *iluData );
int rootNodeDiffSim_DT4( void *kernelData, void *daeData, void *iluData );
int leafNodeDiffSim_DT4( void *kernelData, void *daeData, void *iluData );

                                // File: fibertract.c
int fiberTracking( void *kernelData, void *iluData, void *daeData, 
    void *dtiData, void *argsTable );
int setTractFile( char *file, unsigned int len, char *dir, char *prefix, 
    int seqno );
void cuttingPlaneCoeff( void *dtiData, void *kernelData );
int inForwardDiffVol( void *kernelData, void *kernelVoxel );
realtype distance( void *kernelData, int i, int j, int k );
Kernel_Voxel_ sortDistance( void *voxelList, int listSize, void *voxelData );
void insertionSort( realtype *numbers, int array_size, int *numIDs );
void insert( void *voxelList, void *voxelData );
void insertSeed( void *voxelList, void *voxelNode );
void setSeedNode( void *dtiData, void *voxelData );

                        // file: ftvtk.c 
int fiberVTK( void *argsTable, char *dir, int spline_order, int num_pp );
 
                        // file: l2spline.f 
extern "C" void l2knts_( float*, int*, int*, float*, int* ); 
extern "C" void l2appr_( float*, int*, int*, float*, float*, float*, int*, 
    float*, float*, float* );
extern "C" void bsplpp_( float*, float*, int*, int*, float*, float*, float*, 
    int* );
extern "C" void l2err_( int*, float*, float*, int*, float*, float*, float*, 
    float*, float*, float*, int*, int* );
extern "C" void ppval_( float*, float*, float*, int*, int*, float*, int* );

                           // file: ssparse.f
/***********************************************************************
 * Function prototypes from SPARSKITS.
 * They are mainly the preconditioner-related functions.
 ***********************************************************************/
extern "C" void ilut_( int*, realtype*, int*, int*, int*, realtype*, 
    realtype*, int*, int*, int*, realtype*, realtype*, int*, int*, int*, int* );
extern "C" void ilu0_( int*, realtype*, int*, int*, realtype*, int*, int*, 
    int*, int* );
extern "C" void csort_( int*, realtype*, int*, int*, int*, int* );
extern "C" void lusol_( int*, realtype*, realtype*, realtype*, int*, int* );
extern "C" void matsort_( int*, realtype*, int*, int* );
                                // otherwise
/*
void ilut( int*, realtype*, int*, int*, int*, realtype*, realtype*, int*, 
    int*, int*, realtype*, realtype*, int*, int*, int*, int* );	
void ilu0( int*, realtype*, int*, int*, realtype*, int*, int*, int*, int* );
void csort( int*, realtype*, int*, int*, int*, int* );
void lusol( int*, realtype*, realtype*, realtype*, int*, int* );
void matsort( int*, realtype*, int*, int* );
*/

               // file: matrixDT4.c
void getCoeffMatrixStructure_DT4( void *kernelData );
int getCoeffMatrixEntry_DT4( void *kernelData, void *dtiData );
void nodeIndex_decode( int index, int *i, int *j, int *k, int nx, int ny );

               // file: fibertractDT4.c
int fiberTracking_DT4( void *kernelData, void *iluData, 
    void *daeData, void *dtiData, void *argsTable );













