#define DST_DATA_DIR      "for_dst/" //"noise_res/"  //"res/"
#define STR_LEN 50
#define DTXYZ_NUM 6     // number of distinct elements in 2nd-order DT
#define DT4_ELEM 15 // number of distinct elements in 4th-order DT

/***********************************************************************
 * data structure for DST arguments table
 ***********************************************************************/
#ifndef DST_ARGS_TABLE_
#define DST_ARGS_TABLE_
typedef struct __Args_Table_
{
    int nvox, nx, ny, nz;
    float dx, dy, dz;          // positive spacing
    float dxOrig, dyOrig, dzOrig;// original spacing, might be negative.
    int roiSize;               // size of drawing area
    int *roiPoints[3];         // starting points in drawing area
    float *dt[DTXYZ_NUM]; // tensor components
    float *fa, *md;      // FA and MD
    float *eigvec[3];    // principal eigenvector
    float *dwi_b0;       // b0 image in dwi dataset for computing brain volume
    char tractbase[STR_LEN];    // Base name for the computed tracks
    int numTractFiles;          // number of generated files for computed tracks
    char tractFileName[STR_LEN];      // tract file name in vtk format
    int totalNumNodes;         // total number of nodes in paths
    int totalNumPaths;         // total number of fiber paths
    float tensorFA, distRatio, ipVectors, ipSeedVecNode;// tracking thresholds
    float diffIntegTime;  // integration time for solving diffusion equation
    float b0_lo, b0_hi;  // threshold for b0 image
               // file for storing fiber statistics
    char fiberStatsFileName[STR_LEN];
    int roiSizeTarget;    // size of target drawing area
    int *roiPtsTarget[3];  // points in target area
               // for 4th-order DT
    float *dt4[DT4_ELEM]; // 4th-order DT elements
    float dt4_GFA, dt4_distRatio, dt4_transAngle, dt4_integTime;//thresholds
} *Args_Table_;
#endif

#ifndef FT_VTK_SUFFIX
#define FT_VTK_SUFFIX "_ft.vtk"
#endif

#ifndef FT_STATS_SUFFIX
#define FT_STATS_SUFFIX "_ft.stats"
#endif

#ifndef ISO_VOXEL
#define ISO_VOXEL "_isovox"  
#endif

#ifndef ORIG
#define ORIG "+orig"
#endif

#ifndef FT_DT4_VTK_SUFFIX
#define FT_DT4_VTK_SUFFIX "_ftDT4.vtk"
#endif

#ifndef FT_DT4_STATS_SUFFIX
#define FT_DT4_STATS_SUFFIX "_ftDT4.stats"
#endif

               // interface for 2nd-order DT
Args_Table_ diffsimFT_interface( int nvox, int *dims, float *voxSize, 
    float *tensor[], float *fa, float *md, float *eigvec[], float *threshold,
    float *dwi_b0, int numSeeds, int *seedsXYZ[], 
    char *seedsFileName, int targetSize, int *targetXYZ[] );
               // interface for 4th-order DT
Args_Table_ diffsimFT_DT4_interface( int nvox, int *dims, 
    float *voxSize, float *tensor[], float *tensor4th[], float *gfa, 
    float *threshold, int numSeeds, int *seedsXYZ[], char *seedsFileName );

