#include "dst.h"

static void csrMatrix_DT4( void *kernelData );
static void csrEntry_DT4( void *kernelData, void *dtiData, 
  int nx, int ny, int nz );
static int csrIndex_zero( int i, int j, int k, int id, void *kernelData );

void getCoeffMatrixStructure_DT4( void *kernelData )
{
    Diff_Kernel_ kernel;
    int neq;

    kernel = (Diff_Kernel_)kernelData;
    neq = kernel->neq;
                                // Initialize ia and ja arrays in CSR format
    csrMatrix_DT4( kernel );
                                // Re-allocate memory
    kernel->nnz = *(kernel->ia + neq) - 1;
printf("***************nnz=%d nnz-max=%d\n",kernel->nnz,NNZ_MAX_DT4);    
    kernel->aa = (realtype *)realloc( kernel->aa, 
        kernel->nnz * sizeof( realtype ) );
    kernel->ja = (int *)realloc( kernel->ja, kernel->nnz * sizeof( int ) );
                                // Sort the matrix structure to be the column-
                                // increasing CSR format.
#ifdef ILU0
    #ifdef FORTRAN_UNDERSCORE
        matsort_( &neq, kernel->aa, kernel->ja, kernel->ia );
    #else
        matsort( &neq, kernel->aa, kernel->ja, kernel->ia );
    #endif
#endif

}

int getCoeffMatrixEntry_DT4( void *kernelData, void *dtiData )
{
    Diff_Kernel_ kernel;
    DTI_ dti;

    kernel = (Diff_Kernel_)kernelData;
    dti = (DTI_)dtiData;

    if( kernelToGridMapping_DT4( kernel, dti ) != 0 ) return 1;
                                // Fill in entries in the aa array
    csrEntry_DT4( kernel, dti, kernel->nx, kernel->ny, kernel->nz );
    return 0;
}

void csrMatrix_DT4( void *kernelData )
{
  Diff_Kernel_ kernel;
  int nx, ny, nz, nxy;
  int *ia, *ja;
  int i, j, k, counter, id;

  kernel = (Diff_Kernel_)kernelData;
  ia = kernel->ia;
  ja = kernel->ja;
  nx = kernel->nx;
  ny = kernel->ny;
  nz = kernel->nz;
  nxy = nx*ny;

  counter = 1;
  for( k = 1; k <= nz; ++k )
  for( j = 1; j <= ny; ++j )
  for( i = 1; i <= nx; ++i )
  {
               // (0,0,0)
    id = nodeIndex( i,j,k,nx,nxy );
    ia[id-1] = counter;
    ja[counter-1] = id;
    counter ++;
               // Since the Dirichlet boundary condition is used,
	       // only the diagonal entries are needed for boundary points.
    if( i==1 || j==1 || k==1 || i==nx || j==ny || k==nz ) continue;
    
               // (1,0,0)
    if( i < nx )
    { ja[counter-1] = nodeIndex( i+1,j,k,nx,nxy ); counter++; }
               // (2,0,0)
    if( i < nx-1 ) 
    { ja[counter-1] = nodeIndex( i+2,j,k,nx,nxy ); counter++; }
               // (-1,0,0)
    if( i > 1 )  
    { ja[counter-1] = nodeIndex( i-1,j,k,nx,nxy ); counter++; }
               // (-2,0,0)
    if( i > 2 )
    { ja[counter-1] = nodeIndex( i-2,j,k,nx,nxy ); counter++; }
               // (2,1,0)
    if( i < nx-1 && j < ny )
    { ja[counter-1] = nodeIndex( i+2,j+1,k,nx,nxy ); counter++; }
               // (2,-1,0)
    if( i < nx-1 && j > 1 )
    { ja[counter-1] = nodeIndex( i+2,j-1,k,nx,nxy ); counter++; }
               // (1,1,0)
    if( i < nx && j < ny )
    { ja[counter-1] = nodeIndex( i+1,j+1,k,nx,nxy ); counter++; }
               // (1,-1,0)
    if( i < nx && j > 1 )
    { ja[counter-1] = nodeIndex( i+1,j-1,k,nx,nxy ); counter++; }
               // (-1,1,0)
    if( i > 1 && j < ny )
    { ja[counter-1] = nodeIndex( i-1,j+1,k,nx,nxy ); counter++; }
               // (-1,-1,0)
    if( i > 1 && j > 1 )
    { ja[counter-1] = nodeIndex( i-1,j-1,k,nx,nxy ); counter++; }
               // (-2,1,0)
    if( i > 2 && j < ny )
    { ja[counter-1] = nodeIndex( i-2,j+1,k,nx,nxy ); counter++; }
               // (-2,-1,0)
    if( i > 2 && j > 1 )
    { ja[counter-1] = nodeIndex( i-2,j-1,k,nx,nxy ); counter++; }
               // (2,0,1)
    if( i < nx-1 && k < nz )
    { ja[counter-1] = nodeIndex( i+2,j,k+1,nx,nxy ); counter++; }
               // (2,0,-1)
    if( i < nx-1 && k > 1 )
    { ja[counter-1] = nodeIndex( i+2,j,k-1,nx,nxy ); counter++; }
               // (1,0,1)
    if( i < nx && k < nz )
    { ja[counter-1] = nodeIndex( i+1,j,k+1,nx,nxy ); counter++; }
               // (1,0,-1)
    if( i < nx && k > 1 )
    { ja[counter-1] = nodeIndex( i+1,j,k-1,nx,nxy ); counter++; }
               // (-1,0,1)
    if( i > 1 && k < nz )
    { ja[counter-1] = nodeIndex( i-1,j,k+1,nx,nxy ); counter++; }
               // (-1,0,-1)
    if( i > 1 && k > 1 )
    { ja[counter-1] = nodeIndex( i-1,j,k-1,nx,nxy ); counter++; }
               // (-2,0,1)
    if( i > 2 && k < nz )
    { ja[counter-1] = nodeIndex( i-2,j,k+1,nx,nxy ); counter++; }
               // (-2,0,-1)
    if( i > 2 && k > 1 )
    { ja[counter-1] = nodeIndex( i-2,j,k-1,nx,nxy ); counter++; }
               // (0,1,0)
    if( j < ny )
    { ja[counter-1] = nodeIndex( i,j+1,k,nx,nxy ); counter++; }
               // (0,-1,0)
    if( j > 1 )
    { ja[counter-1] = nodeIndex( i, j-1,k,nx,nxy ); counter++; }
               // (0,0,1)
    if( k < nz )
    { ja[counter-1] = nodeIndex( i,j,k+1,nx,nxy ); counter++; }
               // (0,0,-1)
    if( k > 1 )
    { ja[counter-1] = nodeIndex( i,j,k-1,nx,nxy ); counter++; }
               // (1,1,1)
    if( i < nx && j < ny && k < nz )
    { ja[counter-1] = nodeIndex( i+1,j+1,k+1,nx,nxy ); counter++; }
               // (1,1,-1)
    if( i < nx && j < ny && k > 1 )
    { ja[counter-1] = nodeIndex( i+1,j+1,k-1,nx,nxy ); counter++; }
               // (1,-1,1)
    if( i < nx && j > 1 && k < nz )
    { ja[counter-1] = nodeIndex( i+1,j-1,k+1,nx,nxy ); counter++; }
               // (1,-1,-1)
    if( i < nx && j > 1 && k > 1 )
    { ja[counter-1] = nodeIndex( i+1,j-1,k-1,nx,nxy ); counter++; }
               // (0,1,1)
    if( j < ny && k < nz )
    { ja[counter-1] = nodeIndex( i,j+1,k+1,nx,nxy ); counter++; }
               // (0,1,-1)
    if( j < ny && k > 1 )
    { ja[counter-1] = nodeIndex( i,j+1,k-1,nx,nxy ); counter++; }
               // (0,-1,1)
    if( j > 1 && k < nz )
    { ja[counter-1] = nodeIndex( i,j-1,k+1,nx,nxy ); counter++; }
               // (0,-1,-1)
    if( j > 1 && k > 1 )	       
    { ja[counter-1] = nodeIndex( i,j-1,k-1,nx,nxy ); counter++; }
               // (-1,1,1)
    if( i > 1 && j < ny && k < nz )
    { ja[counter-1] = nodeIndex( i-1,j+1,k+1,nx,nxy ); counter++; }
               // (-1,1,-1)
    if( i > 1 && j < ny && k > 1 )
    { ja[counter-1] = nodeIndex( i-1,j+1,k-1,nx,nxy ); counter++; }
               // (-1,-1,1)
    if( i > 1 && j > 1 && k < nz )
    { ja[counter-1] = nodeIndex ( i-1,j-1,k+1,nx,nxy ); counter++; }
               // (-1,-1,-1)
    if( i > 1 && j > 1 && k > 1 )
    { ja[counter-1] = nodeIndex ( i-1,j-1,k-1,nx,nxy ); counter++; }
               // (1,2,0)
    if( i < nx && j < ny-1 )
    { ja[counter-1] = nodeIndex( i+1,j+2,k,nx,nxy ); counter++; }
               // (-1,2,0)
    if( i > 1 && j < ny-1 )
    { ja[counter-1] = nodeIndex( i-1,j+2,k,nx,nxy ); counter++; }
               // (1,-2,0)
    if( i < nx && j > 2 )
    { ja[counter-1] = nodeIndex( i+1,j-2,k,nx,nxy ); counter++; }
               // (-1,-2,0)
    if( i > 1 && j > 2 )
    { ja[counter-1] = nodeIndex( i-1,j-2,k,nx,nxy ); counter++; }
               // (1,0,2)
    if( i < nx && k < nz-1 )
    { ja[counter-1] = nodeIndex( i+1,j,k+2,nx,nxy ); counter++; }
               // (-1,0,2)
    if( i > 1 && k < nz-1 )
    { ja[counter-1] = nodeIndex( i-1,j,k+2,nx,nxy ); counter++; }
               // (1,0,-2)
    if( i < nx && k > 2 )
    { ja[counter-1] = nodeIndex( i+1,j,k-2,nx,nxy ); counter++; }
               // (-1,0,-2)
    if( i > 1 && k > 2 )
    { ja[counter-1] = nodeIndex( i-1,j,k-2,nx,nxy ); counter++; }
               // (0,2,0)
    if( j < ny-1 )
    { ja[counter-1] = nodeIndex( i,j+2,k,nx,nxy ); counter++; }
               // (0,-2,0)
    if( j > 2 )
    { ja[counter-1] = nodeIndex( i,j-2,k,nx,nxy ); counter++; }
               // (0,2,1)
    if( j < ny-1 && k < nz )
    { ja[counter-1] = nodeIndex( i,j+2,k+1,nx,nxy ); counter++; }
               // (0,2,-1)
    if( j < ny-1 && k > 1 )
    { ja[counter-1] = nodeIndex( i,j+2,k-1,nx,nxy ); counter++; }
               // (0,-2,1)
    if( j > 2 && k < nz )
    { ja[counter-1] = nodeIndex( i,j-2,k+1,nx,nxy ); counter++; }
               // (0,-2,-1)
    if( j > 2 && k > 1 )
    { ja[counter-1] = nodeIndex( i,j-2,k-1,nx,nxy ); counter++; }
               // (0,1,2)
    if( j < ny && k < nz -1 )
    { ja[counter-1] = nodeIndex( i,j+1,k+2,nx,nxy ); counter++; }
               // (0,-1,2)
    if( j > 1 && k < nz-1 )
    { ja[counter-1] = nodeIndex( i,j-1,k+2,nx,nxy ); counter++; }
               // (0,1,-2)
    if( j < ny && k > 2 )
    { ja[counter-1] = nodeIndex( i,j+1,k-2,nx,nxy ); counter++; }
               // (0,-1,-2)
    if( j > 1 && k > 2 )
    { ja[counter-1] = nodeIndex( i,j-1,k-2,nx,nxy ); counter++; }
               // (0,0,2)
    if( k < nz-1 )
    { ja[counter-1] = nodeIndex( i,j,k+2,nx,nxy ); counter++; }
               // (0,0,-2)
    if( k > 2 )
    { ja[counter-1] = nodeIndex( i,j,k-2,nx,nxy ); counter++; }
  }  

  ia[nxy*nz] = counter;
}

               // fill in entries in the coefficient matrix
void csrEntry_DT4( void *kernelData, void *dtiData, 
  int nx, int ny, int nz )
{
  Diff_Kernel_ kernel;
  DTI_ dti;
  float **dtxyz, **dt4Elem, *aa, *sf, dx, dy, dz;
  int i, j, k, index, id, nxy, neq, *ia, *ja;
  float Dxx[nx][ny][nz], Dyy[nx][ny][nz], Dzz[nx][ny][nz], 
        Dxy[nx][ny][nz], Dxz[nx][ny][nz], Dyz[nx][ny][nz];
  float Dxxxx[nx][ny][nz], Dyyyy[nx][ny][nz], Dzzzz[nx][ny][nz], // 1
        Dxxxy[nx][ny][nz], Dxxxz[nx][ny][nz], Dyyyx[nx][ny][nz], // 4
	Dyyyz[nx][ny][nz], Dzzzx[nx][ny][nz], Dzzzy[nx][ny][nz], // 4
        Dxxyy[nx][ny][nz], Dxxzz[nx][ny][nz], Dyyzz[nx][ny][nz], // 6
        Dxxyz[nx][ny][nz], Dyyxz[nx][ny][nz], Dzzxy[nx][ny][nz]; // 12
  float a[16], b[11], c[10]; // the 1st element	of each array is left unused.
  double dxx, dyy, dzz, dxy, dxz, dyz;    

  kernel = (Diff_Kernel_)kernelData;
  neq = kernel->neq;
  aa = kernel->aa; ia = kernel->ia; ja = kernel->ja;
  sf = kernel->sf;
  dx = kernel->dx; dy = kernel->dy; dz = kernel->dz;
  dti = (DTI_)dtiData;
  dtxyz = dti->dtxyz;
  dt4Elem = dti->dt4Elem;

  nxy = nx*ny;
  dxx = dx*dx; dxy = dx*dy; dxz = dx*dz;
  dyy = dy*dy; dyz = dy*dz; dzz = dz*dz;

               // retrieve the tensor components
  for( i = 0; i < nx; ++ i )
  for( j = 0; j < ny; ++ j )
  for( k = 0; k < nz; ++ k )
  {
               // node index in the diffusion kernel
    id = nodeIndex( i+1,j+1,k+1,nx,nxy ) - 1;
               // node index in the grid
    id = kernel->voxel[id]->gridID;
    if( id == OUT_OF_GRID )
    {
      Dxx[i][j][k] = 0; Dyy[i][j][k] = 0; Dzz[i][j][k] = 0;
      Dxy[i][j][k] = 0; Dxz[i][j][k] = 0; Dyz[i][j][k] = 0;
      Dxxxx[i][j][k] = 0; Dyyyy[i][j][k] = 0; Dzzzz[i][j][k] = 0;
      Dxxxy[i][j][k] = 0; Dxxxz[i][j][k] = 0; Dyyyx[i][j][k] = 0; 
      Dyyyz[i][j][k] = 0; Dzzzx[i][j][k] = 0; Dzzzy[i][j][k] = 0; 
      Dxxyy[i][j][k] = 0; Dxxzz[i][j][k] = 0; Dyyzz[i][j][k] = 0;
      Dxxyz[i][j][k] = 0; Dyyxz[i][j][k] = 0; Dzzxy[i][j][k] = 0;
    }
    else
    {
      Dxx[i][j][k] = DTXYZ(id, 0);    
      Dyy[i][j][k] = DTXYZ(id, 1);
      Dzz[i][j][k] = DTXYZ(id, 2);
      Dxy[i][j][k] = DTXYZ(id, 3);
      Dxz[i][j][k] = DTXYZ(id, 4);
      Dyz[i][j][k] = DTXYZ(id, 5);
#ifdef SET_DT4_ZERO      
      Dxxxx[i][j][k] = 0;//DTXYZ_4TH( id, 0 );
      Dxxxy[i][j][k] = 0;//DTXYZ_4TH( id, 1 );
      Dxxxz[i][j][k] = 0;//DTXYZ_4TH( id, 2 );  
      Dxxyy[i][j][k] = 0;//DTXYZ_4TH( id, 3 );
      Dxxzz[i][j][k] = 0;//DTXYZ_4TH( id, 4 );
      Dxxyz[i][j][k] = 0;//DTXYZ_4TH( id, 5 );  
      Dyyyx[i][j][k] = 0;//DTXYZ_4TH( id, 6 );
      Dzzzx[i][j][k] = 0;//DTXYZ_4TH( id, 7 );
      Dyyxz[i][j][k] = 0;//DTXYZ_4TH( id, 8 );  
      Dzzxy[i][j][k] = 0;//DTXYZ_4TH( id, 9 );
      Dyyyy[i][j][k] = 0;//DTXYZ_4TH( id, 10 );
      Dyyyz[i][j][k] = 0;//DTXYZ_4TH( id, 11 );  
      Dyyzz[i][j][k] = 0;//DTXYZ_4TH( id, 12 );
      Dzzzy[i][j][k] = 0;//DTXYZ_4TH( id, 13 );
      Dzzzz[i][j][k] = 0;//DTXYZ_4TH( id, 14 );
#else      
      Dxxxx[i][j][k] = DTXYZ_4TH( id, 0 );
      Dxxxy[i][j][k] = DTXYZ_4TH( id, 1 );
      Dxxxz[i][j][k] = DTXYZ_4TH( id, 2 );  
      Dxxyy[i][j][k] = DTXYZ_4TH( id, 3 );
      Dxxzz[i][j][k] = DTXYZ_4TH( id, 4 );
      Dxxyz[i][j][k] = DTXYZ_4TH( id, 5 );  
      Dyyyx[i][j][k] = DTXYZ_4TH( id, 6 );
      Dzzzx[i][j][k] = DTXYZ_4TH( id, 7 );
      Dyyxz[i][j][k] = DTXYZ_4TH( id, 8 );  
      Dzzxy[i][j][k] = DTXYZ_4TH( id, 9 );
      Dyyyy[i][j][k] = DTXYZ_4TH( id, 10 );
      Dyyyz[i][j][k] = DTXYZ_4TH( id, 11 );  
      Dyyzz[i][j][k] = DTXYZ_4TH( id, 12 );
      Dzzzy[i][j][k] = DTXYZ_4TH( id, 13 );
      Dzzzz[i][j][k] = DTXYZ_4TH( id, 14 );  
#endif      
    }
  }
               // Fill out the array for coefficient matrix.
	       // The Dirichlet boundary condition is used,
	       // on which the concentration is zero.
  memset( aa, 0, kernel->nnz*sizeof( float ) ); 
  memset( sf, 0, neq*sizeof( float ) ); 
  for( i = 0; i < nx; ++ i )
  for( j = 0; j < ny; ++ j )
  for( k = 0; k < nz; ++ k )
  {
    id = nodeIndex( i+1,j+1,k+1,nx,nxy ) - 1;
               // Set the entry for boundary points to 1.
    if( i==0 || j==0 || k==0 || i==nx-1 || j==ny-1 || k==nz-1 )
    {
      index = csrIndex_zero( i,j,k,id,kernel );
      aa[index] = 1;
      sf[id] = 1;
      continue;
    }
               // The rest is to fill out entries for interior points.
	       // We assume an extra point outside of the boundary, 
	       // on which the concentration is also zero. Thus the
	       // central difference scheme works for all interior points.
	       // ----------------------------------------------------
               // find the scale factor for the row
//  int dtmax;	     
//    dtmax = MAX( MAX( ABS(Dxxxx[i][j][k]),
//      ABS(Dyyyy[i][j][k]) ), ABS(Dzzzz[i][j][k]) );
//    if( dtmax <= DT_THRESHOLD ) 
//      sf[id] = SF_CONST;
//    else 
//      sf[id] = 1.0/dtmax;
    sf[id] = 1;
               // calculate the intermediate coefficients: a, b, c
    a[1] = Dxxxx[i][j][k];
    a[2] = Dxxxy[i][j][k]*4;
    a[3] = Dxxxz[i][j][k]*4;
    a[4] = Dxxyy[i][j][k]*6;
    a[5] = Dxxzz[i][j][k]*6;
    a[6] = Dxxyz[i][j][k]*12;
    a[7] = Dyyyx[i][j][k]*4;
    a[8] = Dzzzx[i][j][k]*4;
    a[9] = Dyyxz[i][j][k]*12;
    a[10]= Dzzxy[i][j][k]*12;
    a[11]= Dyyyy[i][j][k];
    a[12]= Dyyyz[i][j][k]*4;
    a[13]= Dyyzz[i][j][k]*6;
    a[14]= Dzzzy[i][j][k]*4;
    a[15]= Dzzzz[i][j][k];

    b[1] = (Dxxxx[i+1][j][k] - Dxxxx[i-1][j][k])/(2*dx)
         + (Dxxxy[i][j+1][k] - Dxxxy[i][j-1][k])/(2*dy)
         + (Dxxxz[i][j][k+1] - Dxxxz[i][j][k-1])/(2*dz);
    b[2] = (Dxxxy[i+1][j][k] - Dxxxy[i-1][j][k])/(2*dx) * 3
         + (Dxxyy[i][j+1][k] - Dxxyy[i][j-1][k])/(2*dy) * 3
         + (Dxxyz[i][j][k+1] - Dxxyz[i][j][k-1])/(2*dz) * 3;
    b[3] = (Dxxxz[i+1][j][k] - Dxxxz[i-1][j][k])/(2*dx) * 3
         + (Dxxyz[i][j+1][k] - Dxxyz[i][j-1][k])/(2*dy) * 3
         + (Dxxzz[i][j][k+1] - Dxxzz[i][j][k-1])/(2*dz) * 3;
    b[4] = (Dxxyy[i+1][j][k] - Dxxyy[i-1][j][k])/(2*dx) * 3
         + (Dyyyx[i][j+1][k] - Dyyyx[i][j-1][k])/(2*dy) * 3
         + (Dyyxz[i][j][k+1] - Dyyxz[i][j][k-1])/(2*dz) * 3;
    b[5] = (Dxxzz[i+1][j][k] - Dxxzz[i-1][j][k])/(2*dx) * 3
         + (Dzzxy[i][j+1][k] - Dzzxy[i][j-1][k])/(2*dy) * 3
         + (Dzzzx[i][j][k+1] - Dzzzx[i][j][k-1])/(2*dz) * 3;
    b[6] = (Dxxyz[i+1][j][k] - Dxxyz[i-1][j][k])/(2*dx) * 6
         + (Dyyxz[i][j+1][k] - Dyyxz[i][j-1][k])/(2*dy) * 6
	 + (Dzzxy[i][j][k+1] - Dzzxy[i][j][k-1])/(2*dz) * 6;
    b[7] = (Dyyyx[i+1][j][k] - Dyyyx[i-1][j][k])/(2*dx)
         + (Dyyyy[i][j+1][k] - Dyyyy[i][j-1][k])/(2*dy)
	 + (Dyyyz[i][j][k+1] - Dyyyz[i][j][k-1])/(2*dz);
    b[8] = (Dyyxz[i+1][j][k] - Dyyxz[i-1][j][k])/(2*dx) * 3
         + (Dyyyz[i][j+1][k] - Dyyyz[i][j-1][k])/(2*dy) * 3
	 + (Dyyzz[i][j][k+1] - Dyyzz[i][j][k-1])/(2*dz) * 3;
    b[9] = (Dzzxy[i+1][j][k] - Dzzxy[i-1][j][k])/(2*dx) * 3
         + (Dyyzz[i][j+1][k] - Dyyzz[i][j-1][k])/(2*dy) * 3
	 + (Dzzzy[i][j][k+1] - Dzzzy[i][j][k-1])/(2*dz) * 3;
    b[10]= (Dzzzx[i+1][j][k] - Dzzzx[i-1][j][k])/(2*dx)
         + (Dzzzy[i][j+1][k] - Dzzzy[i][j-1][k])/(2*dy)
	 + (Dzzzz[i][j][k+1] - Dzzzz[i][j][k-1])/(2*dz);

    c[1] = Dxx[i][j][k];
    c[2] = Dxy[i][j][k]*2;
    c[3] = Dxz[i][j][k]*2;
    c[4] = (Dxx[i+1][j][k] - Dxx[i-1][j][k])/(2*dx)
         + (Dxy[i][j+1][k] - Dxy[i][j-1][k])/(2*dy)
	 + (Dxz[i][j][k+1] - Dxz[i][j][k-1])/(2*dz);
    c[5] = Dyy[i][j][k];
    c[6] = Dyz[i][j][k]*2;
    c[7] = (Dxy[i+1][j][k] - Dxy[i-1][j][k])/(2*dx)
         + (Dyy[i][j+1][k] - Dyy[i][j-1][k])/(2*dy)
	 + (Dyz[i][j][k+1] - Dyz[i][j][k-1])/(2*dz);
    c[8] = Dzz[i][j][k];
    c[9] = (Dxz[i+1][j][k] - Dxz[i-1][j][k])/(2*dx)
         + (Dyz[i][j+1][k] - Dyz[i][j-1][k])/(2*dy)
	 + (Dzz[i][j][k+1] - Dzz[i][j][k-1])/(2*dz);

               // (0,0,0): diagonal entry
    index = csrIndex_zero( i,j,k,id,kernel );
    aa[index] = 6*a[1]/(dxx*dxx) + 4*a[4]/(dxx*dyy) + 4*a[5]/(dxx*dzz)
              + 6*a[11]/(dyy*dyy) + 4*a[13]/(dyy*dzz) + 6*a[15]/(dzz*dzz)
	      - 2*c[1]/dxx - 2*c[5]/dyy - 2*c[8]/dzz;

    	       // off-diagonal entries
//    for( i > 1 && i < nx-2 && j > 1 && j < ny-2 && k > 1 && k < nz-2 )
   
               // (1,0,0)
      index = csrIndex_zero( i+1,j,k,id,kernel );
      aa[index] = -4*a[1]/(dxx*dxx) - 2*a[4]/(dxx*dyy) - 2*a[5]/(dxx*dzz)
                - b[1]/(dx*dxx) - b[4]/(dx*dyy) - b[5]/(dx*dzz)
		+ c[1]/dxx + c[4]/(2*dx);
               // (2,0,0)
    if( i < nx-2 )
    {
      index = csrIndex_zero( i+2,j,k,id,kernel );
      aa[index] = a[1]/(dxx*dxx) + b[1]/(2*dxx*dx);
    }      
               // (-1,0,0)
      index = csrIndex_zero( i-1,j,k,id,kernel );
      aa[index] = -4*a[1]/(dxx*dxx) - 2*a[4]/(dxx*dyy) - 2*a[5]/(dxx*dzz)
                + b[1]/(dx*dxx) + b[4]/(dx*dyy) + b[5]/(dx*dzz)
		+ c[1]/dxx - c[4]/(2*dx);
	       // (-2,0,0)
    if( i > 1 )
    { 
      index = csrIndex_zero( i-2,j,k,id,kernel );
      aa[index] = a[1]/(dxx*dxx) - b[1]/(2*dx*dxx); 
    }
               // (2,1,0)
    if( i < nx-2 )	       
    { 
      index = csrIndex_zero( i+2,j+1,k,id,kernel );
      aa[index] = a[2]/(4*dxx*dxy);
    }      
               // (2,-1,0)
    if( i < nx-2 )
    {
      index = csrIndex_zero( i+2,j-1,k,id,kernel );
      aa[index] = -a[2]/(4*dxx*dxy);
    }
               // (1,1,0)
      index = csrIndex_zero( i+1,j+1,k,id,kernel );
      aa[index] = -a[2]/(2*dxx*dxy) + a[4]/(dxx*dyy) - a[7]/(2*dxy*dyy)
                - a[10]/(2*dxy*dzz) + b[2]/(2*dxx*dy) + b[4]/(2*dx*dyy)
		+ c[2]/(4*dxy);
	       // (1,-1,0)
      index = csrIndex_zero( i+1,j-1,k,id,kernel );
      aa[index] = a[2]/(2*dxx*dxy) + a[4]/(dxx*dyy) + a[7]/(2*dxy*dyy)
                + a[10]/(2*dxy*dzz) - b[2]/(2*dxx*dy) + b[4]/(2*dx*dyy)
		- c[2]/(4*dxy);
	       // (-1,1,0)
      index = csrIndex_zero( i-1,j+1,k,id,kernel );
      aa[index] = a[2]/(2*dxx*dxy) + a[4]/(dxx*dyy) + a[7]/(2*dxy*dyy)
                + a[10]/(2*dxy*dzz) + b[2]/(2*dxx*dy) - b[4]/(2*dx*dyy)
		- c[2]/(4*dxy);
	       // (-1,-1,0)	
      index = csrIndex_zero( i-1,j-1,k,id,kernel );
      aa[index] = -a[2]/(2*dxx*dxy) + a[4]/(dxx*dyy) - a[7]/(2*dxy*dyy)
                - a[10]/(2*dxy*dzz) - b[2]/(2*dxx*dy) - b[4]/(2*dx*dyy)
		+ c[2]/(4*dxy);
	       // (-2,1,0)
    if( i > 1 )
    {
      index = csrIndex_zero( i-2,j+1,k,id,kernel );
      aa[index] = -a[2]/(4*dxx*dxy);
    }
	       // (-2,-1,0)
    if( i > 1 )
    {
      index = csrIndex_zero( i-2,j-1,k,id,kernel );
      aa[index] = a[2]/(4*dxx*dxy);
    }
               // (2,0,1)
    if( i < nx-2 )
    {
      index = csrIndex_zero( i+2,j,k+1,id,kernel );
      aa[index] = a[3]/(4*dxx*dxz);
    }
               // (2,0,-1)
    if( i < nx-2 )
    {
      index = csrIndex_zero( i+2,j,k-1,id,kernel );
      aa[index] = -a[3]/(4*dxx*dxz);
    }
               // (1,0,1)
      index = csrIndex_zero( i+1,j,k+1,id,kernel );
      aa[index] = -a[3]/(2*dxx*dxz) + a[5]/(dxx*dzz) - a[8]/(2*dxz*dzz)
                - a[9]/(2*dxy*dyz) + b[3]/(2*dxx*dz) + b[5]/(2*dx*dzz)
		+ c[3]/(4*dxz);
               // (1,0,-1)
      index = csrIndex_zero( i+1,j,k-1,id,kernel );
      aa[index] = a[3]/(2*dxx*dxz) + a[5]/(dxx*dzz) + a[8]/(2*dxz*dzz)
                + a[9]/(2*dxy*dyz) - b[3]/(2*dxx*dz) + b[5]/(2*dx*dzz)
		- c[3]/(4*dxz);
               // (-1,0,1)
      index = csrIndex_zero( i-1,j,k+1,id,kernel );
      aa[index] = a[3]/(2*dxx*dxz) + a[5]/(dxx*dzz) + a[8]/(2*dxz*dzz)
                + a[9]/(2*dxy*dyz) + b[3]/(2*dxx*dz) - b[5]/(2*dx*dzz)
		- c[3]/(4*dxz);
               // (-1,0,-1)
      index = csrIndex_zero( i-1,j,k-1,id,kernel );
      aa[index] = -a[3]/(2*dxx*dxz) + a[5]/(dxx*dzz) - a[8]/(2*dxz*dzz)
                - a[9]/(2*dxy*dyz) - b[3]/(2*dxx*dz) - b[5]/(2*dx*dzz)
		+ c[3]/(4*dxz);
	       // (-2,0,1)
    if( i > 1 )
    {
      index = csrIndex_zero( i-2,j,k+1,id,kernel );
      aa[index] = -a[3]/(4*dxx*dxz);
    }  
	       // (-2,0,-1)
    if( i > 1 )
    {
      index = csrIndex_zero( i-2,j,k-1,id,kernel );
      aa[index] = a[3]/(4*dxx*dxz);
    }
               // (0,1,0)
      index = csrIndex_zero( i,j+1,k,id,kernel );
      aa[index] = -2*a[4]/(dxx*dyy) - 4*a[11]/(dyy*dyy) - 2*a[13]/(dyy*dzz)
                - b[2]/(dxx*dy) - b[7]/(dyy*dy) - b[9]/(dy*dzz)
		+ c[5]/dyy + c[7]/(2*dy);
               // (0,-1,0)
      index = csrIndex_zero( i,j-1,k,id,kernel );
      aa[index] = -2*a[4]/(dxx*dyy) - 4*a[11]/(dyy*dyy) - 2*a[13]/(dyy*dzz)
                + b[2]/(dxx*dy) + b[7]/(dyy*dy) + b[9]/(dy*dzz)
		+ c[5]/dyy - c[7]/(2*dy);
	       // (0,0,1)
      index = csrIndex_zero( i,j,k+1,id,kernel );
      aa[index] = -2*a[5]/(dxx*dzz) - 2*a[13]/(dyy*dzz) - 4*a[15]/(dzz*dzz)
                - b[3]/(dxx*dz) - b[8]/(dyy*dz) - b[10]/(dzz*dz)
		+ c[8]/dzz + c[9]/(2*dz);
	       // (0,0,-1)
      index = csrIndex_zero( i,j,k-1,id,kernel );
      aa[index] = -2*a[5]/(dxx*dzz) - 2*a[13]/(dyy*dzz) - 4*a[15]/(dzz*dzz)
                + b[3]/(dxx*dz) + b[8]/(dyy*dz) + b[10]/(dzz*dz)
		+ c[8]/dzz - c[9]/(2*dz);
	       // (1,1,1)	
      index = csrIndex_zero( i+1,j+1,k+1,id,kernel );
      aa[index] = a[6]/(4*dxx*dyz) + a[9]/(4*dxy*dyz) + a[10]/(4*dxy*dzz)
                + b[6]/(8*dxy*dz);
	       // (1,1,-1)	
      index = csrIndex_zero( i+1,j+1,k-1,id,kernel );
      aa[index] = -a[6]/(4*dxx*dyz) - a[9]/(4*dxy*dyz) + a[10]/(4*dxy*dzz)
                - b[6]/(8*dxy*dz);
               // (1,-1,1)
      index = csrIndex_zero( i+1,j-1,k+1,id,kernel );
      aa[index] = -a[6]/(4*dxx*dyz) + a[9]/(4*dxy*dyz) - a[10]/(4*dxy*dzz)
                - b[6]/(8*dxy*dz);
               // (1,-1,-1)
      index = csrIndex_zero( i+1,j-1,k-1,id,kernel );
      aa[index] = a[6]/(4*dxx*dyz) - a[9]/(4*dxy*dyz) - a[10]/(4*dxy*dzz)
                + b[6]/(8*dxy*dz);
               // (0,1,1)
      index = csrIndex_zero( i,j+1,k+1,id,kernel );
      aa[index] = -a[6]/(2*dxx*dyz) - a[12]/(2*dyy*dyz) + a[13]/(dyy*dzz)
                - a[14]/(2*dyz*dzz) + b[8]/(2*dyy*dz) + b[9]/(2*dy*dzz)
		+ c[6]/(4*dyz);
               // (0,1,-1)
      index = csrIndex_zero( i,j+1,k-1,id,kernel );
      aa[index] = a[6]/(2*dxx*dyz) + a[12]/(2*dyy*dyz) + a[13]/(dyy*dzz)
                + a[14]/(2*dyz*dzz) - b[8]/(2*dyy*dz) + b[9]/(2*dy*dzz)
		- c[6]/(4*dyz);
               // (0,-1,1)
      index = csrIndex_zero( i,j-1,k+1,id,kernel );
      aa[index] = a[6]/(2*dxx*dyz) + a[12]/(2*dyy*dyz) + a[13]/(dyy*dzz)
                + a[14]/(2*dyz*dzz) + b[8]/(2*dyy*dz) - b[9]/(2*dy*dzz)
		- c[6]/(4*dyz);
               // (0,-1,-1)
      index = csrIndex_zero( i,j-1,k-1,id,kernel );
      aa[index] = -a[6]/(2*dxx*dyz) - a[12]/(2*dyy*dyz) + a[13]/(dyy*dzz)
                - a[14]/(2*dyz*dzz) - b[8]/(2*dyy*dz) - b[9]/(2*dy*dzz)
		+ c[6]/(4*dyz);
               // (-1,1,1)
      index = csrIndex_zero( i-1,j+1,k+1,id,kernel );
      aa[index] = a[6]/(4*dxx*dyz) - a[9]/(4*dxy*dyz) - a[10]/(4*dxy*dzz)
                - b[6]/(8*dxy*dz);
               // (-1,1,-1)
      index = csrIndex_zero( i-1,j+1,k-1,id,kernel );
      aa[index] = -a[6]/(4*dxx*dyz) + a[9]/(4*dxy*dyz) - a[10]/(4*dxy*dzz)
                + b[6]/(8*dxy*dz);
               // (-1,-1,1)
      index = csrIndex_zero( i-1,j-1,k+1,id,kernel );
      aa[index] = -a[6]/(4*dxx*dyz) - a[9]/(4*dxy*dyz) + a[10]/(4*dxy*dzz)
                + b[6]/(8*dxy*dz);
               // (-1,-1,-1)
      index = csrIndex_zero( i-1,j-1,k-1,id,kernel );
      aa[index] = a[6]/(4*dxx*dyz) + a[9]/(4*dxy*dyz) + a[10]/(4*dxy*dzz)
                - b[6]/(8*dxy*dz);
     	       // (1,2,0)
    if( j < ny-2 )
    {
      index = csrIndex_zero( i+1,j+2,k,id,kernel );	       
      aa[index] = a[7]/(4*dxy*dyy);
    }  
     	       // (-1,2,0)
    if( j < ny-2 )
    {
      index = csrIndex_zero( i-1,j+2,k,id,kernel );	       
      aa[index] = -a[7]/(4*dxy*dyy);
    }
     	       // (1,-2,0)
    if( j > 1 )
    {
      index = csrIndex_zero( i+1,j-2,k,id,kernel );	       
      aa[index] = -a[7]/(4*dxy*dyy);
    }  
     	       // (-1,-2,0)
    if( j > 1 )
    {
      index = csrIndex_zero( i-1,j-2,k,id,kernel );	       
      aa[index] = a[7]/(4*dxy*dyy);
    }  
               // (1,0,2)
    if( k < nz-2 )
    {
      index = csrIndex_zero( i+1,j,k+2,id,kernel );
      aa[index] = a[8]/(4*dxz*dzz);
    }
               // (-1,0,2)
    if( k < nz-2 )
    {
      index = csrIndex_zero( i-1,j,k+2,id,kernel );
      aa[index] = -a[8]/(4*dxz*dzz);
    }
               // (1,0,-2)
    if( k > 1 )
    {
      index = csrIndex_zero( i+1,j,k-2,id,kernel );
      aa[index] = -a[8]/(4*dxz*dzz);
    }  
               // (-1,0,-2)
    if( k > 1 )
    {
      index = csrIndex_zero( i-1,j,k-2,id,kernel );
      aa[index] = a[8]/(4*dxz*dzz);
    }  
               // (0,2,0)
    if( j < ny-2 )
    {
      index = csrIndex_zero( i,j+2,k,id,kernel );
      aa[index] = a[11]/(dyy*dyy) + b[7]/(2*dyy*dy);
    }  
               // (0,-2,0)
    if( j > 1 )
    {
      index = csrIndex_zero( i,j-2,k,id,kernel );
      aa[index] = a[11]/(dyy*dyy) - b[7]/(2*dyy*dy);
    }
               // (0,2,1)
    if( j < ny-2 )
    {
      index = csrIndex_zero( i,j+2,k+1,id,kernel );
      aa[index] = a[12]/(4*dyy*dyz);
    }  
               // (0,2,-1)
    if( j < ny-2 )
    {
      index = csrIndex_zero( i,j+2,k-1,id,kernel );
      aa[index] = -a[12]/(4*dyy*dyz);
    }  
               // (0,-2,1)
    if( j > 1 )
    {
      index = csrIndex_zero( i,j-2,k+1,id,kernel );
      aa[index] = -a[12]/(4*dyy*dyz);
    }  
               // (0,-2,-1)
    if( j > 1 )
    {
      index = csrIndex_zero( i,j-2,k-1,id,kernel );
      aa[index] = a[12]/(4*dyy*dyz);
    }  
               // (0,1,2)
    if( k < nz-2 )
    {
      index = csrIndex_zero( i,j+1,k+2,id,kernel );
      aa[index] = a[14]/(4*dyz*dzz);
    }  
               // (0,-1,2)
    if( k < nz-2 )
    {
      index = csrIndex_zero( i,j-1,k+2,id,kernel );
      aa[index] = -a[14]/(4*dyz*dzz);
    }  
               // (0,1,-2)
    if( k > 1 )
    {
      index = csrIndex_zero( i,j+1,k-2,id,kernel );
      aa[index] = -a[14]/(4*dyz*dzz);
    }  
               // (0,-1,-2)
    if( k > 1 )
    {
      index = csrIndex_zero( i,j-1,k-2,id,kernel );
      aa[index] = a[14]/(4*dyz*dzz);
    }  
               // (0,0,2)
    if( k < nz-2 )
    {
      index = csrIndex_zero( i,j,k+2,id,kernel );
      aa[index] = a[15]/(dzz*dzz) + b[10]/(2*dzz*dz);
    }  
               // (0,0,-2)
    if( k > 1 )
    {
      index = csrIndex_zero( i,j,k-2,id,kernel );
      aa[index] = a[15]/(dzz*dzz) - b[10]/(2*dzz*dz);
    }
  }
               // Apply the scale factor to each row.
  for( i = 0; i < neq; ++ i )
  for( k = ia[i]-1; k < ia[i+1]-1; ++ k )
    aa[k] *= sf[i]; 
}


/***********************************************************************
 * Find the index of the entry in the aa array
 * which corresponds to that in the ja array.
 ***********************************************************************/
int csrIndex_zero( int i, int j, int k, int id, void *kernelData )
{
    int index, nx, ny, nxy;
    Diff_Kernel_ kernel;
    int *ia, *ja;        
    
    kernel = (Diff_Kernel_)kernelData;
    ia = kernel->ia;
    ja = kernel->ja;
    nx = kernel->nx;
    ny = kernel->ny;
    nxy = nx*ny;
    
    for( index = ia[id]-1; index < ia[id+1]-1; ++ index )
      if( ja[index] == nodeIndex( i+1,j+1,k+1,nx,nxy ) )
        break;

    return index;
}

// Decode the mapping from a Cartesian coordinate triple to a single integer.
// The coordinate starts from 0.
void nodeIndex_decode( int index, int *i, int *j, int *k, int nx, int ny )
{
    *i = index % nx;
    index /= nx;
    *j = index % ny;
    *k = index / ny;
}

