/*
 * plug_diff.h
 *
 */

#define ADP_MAJOR_VER "0"
#define TWOPI 6.28318530717958647692
#define RAD_TO_DEG (360./TWOPI)
#define DEG_TO_RAD (TWOPI/360.)

/* We want diffusion in units of micron^2/ms 
   so that diffusivity of water is approx = 1  */

/* macros */

#define MIN(a,b) (((a)>(b)) ? (b) : (a))
#define MAX(a,b) (((a)<(b)) ? (b) : (a))
#define ABS(x)  (((x)<0) ? -(x) : (x))

#define BETWEEN(x,a,b)  (((x) >= (a)) && ((x)<=(b)))
#define EVEN(x) (((x) % 2) ? (0) : (1))
#define ODD(x)  (((x) % 2) ? (1) : (0))

#define UWA(x) ( (x) - TWOPI )
#define UWB(x) ( (x) + TWOPI )
#define UNWRAP(x) ( ((x) > M_PI) ? UWA((x)) : ((x) <-M_PI) ? UWB((x)) : (x) )

/*#define log2(x)	    ((log(x)) * (M_LOG2E))*/
#define LOG2(n)     (rint(log2((double)n)))
#define ISPOW2(n)   ((fmod(rint(log2((double)n)),log2((double)n)) < 1.e-14) ? (1) : (0))
#define MAKEPOW2(n) ((int) pow(2.,ceil(log2((double)n))))

#define nint(f)		((int)(((f)>0)?(f+0.5):(f-0.5)))
#define lnint(f)	((long)(((f)>0)?(f+0.5):(f-0.5)))
/*#define sign(f,g)	(((g)>0)?(f):(-(f)))*/

#define PRINT_ZV(v,n,label) \
do{ int i; \
    for(i=0; i<(n); i++) { \
        printf("%s[%i] = (%13.6g,%13.6g)\n",(label),i,(v)[i].r,(v)[i].i); \
    }\
} while(0) 

#define PRINT_FV(v,n,label) \
do{ int i; \
    for(i=0; i<(n); i++) { \
        printf("%s[%i] = %13.6g\n",(label),i,(v)[i]); \
    }\
} while(0) 

#define PRINT_IV(v,n,label) \
do{ int i; \
    for(i=0; i<(n); i++) { \
        printf("%s[%i] = %d\n",(label),i,(v)[i]); \
    }\
} while(0) 

			/* end macros */

/* stuff for reading pfile headers */

#define RECON_FLAG		/* for the next header include */
#include "d2a/rdbm.h"
RDB_HEADER_REC rawhead;
/*char header[RDB_HEADER_SIZE_BYTES];*/


#include "vtk/vtk_all.h"		/* vtk - Feb 20, 2004 */

//Handle old versions of VTK
#ifndef vtkFloatingPointType
#define vtkFloatingPointType vtkFloatingPointType
typedef float vtkFloatingPointType;
#endif

#include "./vtk/vtk_Utils.h"
#include "./vtk/vtk_window.h"
#include "./vtk/vtk_mapsignal.h"
#include "./vtk/vtk_mapvectors.h"

#include "afni.h"

/* My local routine headers */

#include "./ftdst/dst_interface.h"  // header file for DST fiber tracking

#include "./d2a/d2a.h"
#include "./splash/lrf_splash.h"
#include "./vector/vector.h"
#include "./vector/matrix.h"
#include "./utils/utils.h"	/* includes datatypes.h */
//#include "./diffuse/diffuse.h"	/* includes fiber.h */

/* Diffusion computation */

#define DWI2DT_COMPUTE 1
#define SHD_COMPUTE 2
#define SWD_COMPUTE 4		/* 17-Jul-04 */
#define DTI_COMPUTE 8
#define FTM1_COMPUTE 1
#define FTM2_COMPUTE 2

/* Diffusion overlay */

#define DTI_OVERLAY 1
#define SHD_OVERLAY 2
#define FTM_OVERLAY 4

#ifdef EXPERIMENTAL
#define NBUT_DIFC 3  /* Diffusion compute buttons */
static char *DIFC_labels[NBUT_DIFC] = { "Diffusion Tensor", "Spherical Harmonic \nDecomposition", "Spherical Wave\nDecomposition" } ;
#define NBUT_DIFO 3  /* Diffusion overlay buttons */
static char *DIFO_labels[NBUT_DIFO] = { "DTI" , "SHD" , "FTM" } ;
#else
#define NBUT_DIFC 2  /* Diffusion compute buttons */
static char *DIFC_labels[NBUT_DIFC] = { "Diffusion Tensor", "Spherical Harmonic \nDecomposition" } ;
#define NBUT_DIFO 2  /* Diffusion overlay buttons */
static char *DIFO_labels[NBUT_DIFO] = { "DTI" , "SHD" } ;
#endif

/* FTM 10-July-03 */
#define NBUT_FTMC 2  /* Fiber compute buttons */
static char *FTMC_labels[NBUT_FTMC] = { "Fiber method 1" , "Fiber method 2" } ;

#define EMPTY_LABEL "Empty"
#define EMPTY EMPTY_LABEL

#define UCSD_LOGO
#define CORNY_LOGO

#undef  FREEUP
#define FREEUP(x) do{ if((x) != NULL){free((x)); (x)=NULL;} } while(0)

extern void LRF_splashup  (void) ; 

/* Macro to set sensitivity for a rowcol.
   This was derived from RWC's AV_SENSITIZE. */

#define RC_SENSITIZE(wrowcol,sss)                           \
   do{ Boolean sen = (Boolean) sss ;                        \
      if( wrowcol != NULL ) {                               \
      int exp = (XtIsSensitive(wrowcol) != sen) ;           \
      XtSetSensitive(wrowcol,sen);                          \
      if( exp ) MCW_expose_widget(wrowcol) ;                \
   } } while(0)

/* Macro to set sensitivity for a bbox.
   This was derived from RWC's AV_SENSITIZE.
*/

#define BB_SENSITIZE(bb,sss)                           \
   do{ Boolean sen = (Boolean) sss ;                        \
      if( bb->wrowcol != NULL ) {                               \
      int exp = (XtIsSensitive(bb->wrowcol) != sen) ;           \
      XtSetSensitive(bb->wrowcol,sen);                          \
      if( exp ) MCW_expose_widget(bb->wrowcol) ;                \
   } } while(0)

THD_3dim_dataset *write_fbricks_to_fim( PLUGIN_interface  *plint , 
					int nsbricks, /* # sub-bricks */
					int nsbsize,  /* sub-brick size */
					THD_3dim_dataset  *old_dset ,
					float **fout , /* array of input floats */
					char  *new_prefix , 
					Boolean OverWriteMode ) ;

#define DSET_VIEWCODE(ds) (((ds)->dblk!=NULL && (ds)->dblk->diskptr!=NULL) \
                       ? ((ds)->dblk->diskptr->viewcode) : "\0" )

#define DSET_DIFF_TAG(ds) 

MCW_bbox * new_LRF_bbox( Widget parent ,
                         int num_but , char * label_but[] ,
                         int bb_type , int bb_frame ,
			 int bb_orient ,
                         XtCallbackProc cb , XtPointer cb_data ) ;

#define PRINT_DSTATS(ds)                                 \
  do{ if((ds)==NULL) { \
        printf("\nPRINT_DSTAT: Structure doesn't exist!\n");  \
      } else { \
        printf(" Count = %i\n",(ds)->count);  \
        printf("   Min = %f\n",(ds)->min);   \
        printf("   Max = %f\n",(ds)->max);   \
        printf("  Mean = %f\n",(ds)->mean);   \
        printf("   Std = %f\n",(ds)->sigma);  \
        printf("Thresh = %f\n",(ds)->thresh); \
      }; \
  } while(0)

/* Print out dataset_link */

#define PRINT_DSL(dslink)                                 \
  do{ if((dslink)==NULL) { \
        printf("\nPRINT_DSL: Dataset Link doesn't exist!\n");  \
      } else { \
        int id ; \
	printf("\n      ndsl = %i\n",(ndsl));  \
        for( id=0 ; id < ndsl ; id++ ){ \
	  printf("  title[%i] = %s\n",id,(dsl)[id].title) ; \
	  printf("   date[%i] = %s\n",id,(dsl)[id].idcode.date) ; \
	  printf("id_code[%i] = %s\n\n",id,(dsl)[id].idcode.str) ; \
        } \
      }; \
  } while(0)

#define GET_LOC(loc,ijk,ds) \
do{int ix,jy,kz; \
   THD_fvec3 fv; \
   IJK_TO_THREE((ijk),ix,jy,kz,(ds)->daxes->nxx,(ds)->daxes->nxx*(ds)->daxes->nyy); \
   fv = THD_3dind_to_3dmm((ds),TEMP_IVEC3(ix,jy,kz)); \
   (loc)[0] = fv.xyz[0] ; \
   (loc)[1] = fv.xyz[1] ; \
   (loc)[2] = fv.xyz[2] ; \
} while(0);

#define PRINT_LOC(loc,ijk,ds) \
do{int ix,jy,kz;\
   IJK_TO_THREE((ijk),ix,jy,kz,(ds)->daxes->nxx, (ds)->daxes->nxx*(ds)->daxes->nyy); \
   fprintf(stderr,"Voxel(%d,%d,%d) at location (%f,%f,%f)\n", \
   ix,jy,kz,(loc)[0],(loc)[1],(loc)[2]); \
} while(0);
