/* cp afn_splash.c -> lrf_splash.c */

#include "afni.h"
#include "lrf_splash.h"

#ifdef NO_FRIVOLITIES

void LRF_splashdown(void){ return; }  /* for party poopers */
void LRF_splashup  (void){ return; }

#else  /*================================ for party animals !!!!!!!!!!!!!!!!!!*/

#include "lrf_splash.h"     /* contains the RLE image data */

static void * lrf_SPLASH_popup_image( void * , MRI_IMAGE * ) ;
static MRI_IMAGE * lrf_SPLASH_decode26( int , int , int , char ** ) ;

static MRI_IMAGE * imspl = NULL ;
static void * handle = NULL ;

#define USE_FADING

#ifdef AFNI_DEBUG
#  define USE_TRACING
#endif
#include "dbtrace.h"

/*----------------------------------------------------------------------------*/

void LRF_splashdown(void)
{
   PLUGIN_impopper * ppp = (PLUGIN_impopper *) handle ;

ENTRY("LRF_splashdown") ;

   if( handle != NULL ){
#ifdef USE_FADING
      float max_splash = 3.0 ;
      char * hh = getenv("AFNI_SPLASHTIME") ;
      if( hh != NULL ) max_splash = strtod(hh,NULL) ;
      if( max_splash > 0.0 ){
         if( imspl != NULL ){  /* fade gently away */
            byte * bspl ; int ii , nv , kk ; double et ;
            bspl = MRI_BYTE_PTR(imspl) ; nv = imspl->nvox ; et = COX_clock_time() ;
            for( kk=0 ; kk < 10 ; kk++ ){
#if 0
               for( ii=0 ; ii < nv ; ii++ ) bspl[ii] *= 0.92 ;
#else
               for( ii=0 ; ii < nv ; ii++ ) bspl[ii] = (15*bspl[ii])>>4 ;
#endif
               lrf_SPLASH_popup_image(handle,imspl) ;
               drive_MCW_imseq( ppp->seq , isqDR_reimage , (XtPointer) 0 ) ;
               if( COX_clock_time()-et > 1.1 ) break ;
            }
         }
      }
#endif
      lrf_SPLASH_popup_image(handle,NULL); myXtFree(handle) ; /* get rid of window */
   }
   mri_free(imspl) ; imspl = NULL ;
   EXRETURN ;
}

/*----------------------------------------------------------------------------*/

void LRF_splashup(void)
{
   PLUGIN_impopper * ppp ;
   MRI_IMAGE * imov ;
   int    dd,ee ;
   char   bb ;
   byte * bspl ;
   static int first=1 , nov , dnov ;

ENTRY("LRF_splashup") ;

   /*--- create splash image ---*/

   if( ! PLUTO_popup_open(handle) ){
      mri_free(imspl) ;
      imspl = lrf_SPLASH_decode26( NX_stevechin, NY_stevechin, NLINE_stevechin, BAR_stevechin ) ;

      if( first ){
         nov  =    (lrand48() >> 8) % LRF_NOVER  ;
         dnov = 2*((lrand48() >> 8) % 2) - 1 ;
      }
      nov  = (nov+dnov+LRF_NOVER) % LRF_NOVER ;
      imov = lrf_SPLASH_decode26( lrf_xover[nov], lrf_yover[nov], lrf_lover[nov], lrf_bover[nov] ) ;

      mri_overlay_2D( imspl, imov, LRF_IXOVER, LRF_JYOVER ) ; mri_free(imov) ;

      handle = lrf_SPLASH_popup_image( handle, imspl ) ;
#ifndef USE_FADING
      mri_free(imspl) ; imspl = NULL ;
#endif

      /* modify image display properties */

      ppp = (PLUGIN_impopper *) handle ;

      if( first ){ dd = MWM_DECOR_BORDER ;
                   ee = 0 ;
      } else     { dd = MWM_DECOR_BORDER | MWM_DECOR_TITLE | MWM_DECOR_MENU ;
                   ee = MWM_FUNC_MOVE | MWM_FUNC_CLOSE ;
      }

      XtVaSetValues( ppp->seq->wtop ,
                       XmNx , (GLOBAL_library.dc->width-NX_stevechin)/2 ,
                       XmNy , 100 ,
                       XmNmwmDecorations , dd ,
                       XmNmwmFunctions   , ee ,
                     NULL ) ;

      /* actually popup image display */

      drive_MCW_imseq( ppp->seq , isqDR_realize   , NULL                     ) ;
      drive_MCW_imseq( ppp->seq , isqDR_onoffwid  , (XtPointer) isqDR_offwid ) ;
      drive_MCW_imseq( ppp->seq , isqDR_clearstat , NULL                     ) ;
#if 0
      drive_MCW_imseq( ppp->seq , isqDR_reimage   , (XtPointer) 0            ) ;
#endif

      /* some super-frivolities */

      if( !first ){
         drive_MCW_imseq( ppp->seq , isqDR_title     , (XtPointer) "AFNI!" ) ;
         drive_MCW_imseq( ppp->seq , isqDR_imhelptext,
                          (XtPointer) " \n"
                                      " The new AFNI logo is the only image \n"
                                      " returned from Mars by the ill-fated \n"
                                      " Polar Lander mission.    How such a \n"
                                      " structure could have been formed is \n"
                                      " being researched by NASA's top men. \n"
                        ) ;
      }

   /*--- destroy splash image ---*/

   } else {
      ppp = (PLUGIN_impopper *) handle ;

      /* bring splash window to the top */

      if( ISQ_REALZ(ppp->seq) )
         XMapRaised( XtDisplay(ppp->seq->wtop) , XtWindow(ppp->seq->wtop) ) ;

      LRF_splashdown() ;  /* off with their heads */
   }

   first = 0 ; EXRETURN ;
}

/*-----------------------------------------------------------------------
  The following is adapted from PLUTO_popup_image() in afni_plugin.c
-------------------------------------------------------------------------*/

static void * lrf_SPLASH_popup_image( void * handle , MRI_IMAGE * im )
{
   PLUGIN_impopper * imp = (PLUGIN_impopper *) handle ;

ENTRY("lrf_SPLASH_popup_image") ;

   /*-- input image is NULL ==> popdown, if applicable --*/

   if( im == NULL ){
      if( imp != NULL )
         drive_MCW_imseq( imp->seq , isqDR_destroy , NULL ) ;

      RETURN ((void *) imp) ;
   }

   /*-- input = no popper handle ==> create one --*/

   if( imp == NULL ){
      imp      = myXtNew(PLUGIN_impopper) ;
      imp->seq = NULL ; imp->im  = NULL ;
   }

   /*-- input = non-null image ==> replace image --*/

   mri_free( imp->im ) ;                   /* toss old copy */
   imp->im = mri_to_mri( im->kind , im ) ; /* make new copy */

   /*-- input = inactive popper handle ==> activate it --*/

   if( imp->seq == NULL )
      imp->seq = open_MCW_imseq( GLOBAL_library.dc ,
                                 PLUGIN_imseq_getim , (XtPointer) imp ) ;

   /*-- unlike PLUTO_popup_image, actual popup is left to caller --*/

   RETURN ((void *) imp) ;
}

/*--------------------------------------------------------------------------
  Decode the 26 data into an image
----------------------------------------------------------------------------*/

static MRI_IMAGE * lrf_SPLASH_decode26( int nx, int ny , int nl , char ** im26 )
{
   MRI_IMAGE * im ;
   byte * bim ;
   int ii , jj , cc,rr , dd,ee ;
   char bb ;

ENTRY("lrf_SPLASH_decode26") ;

   if( nx < 3 || ny < 3 || nl < 3 || im26 == NULL ) RETURN(NULL) ;

   im  = mri_new( nx , ny , MRI_byte ) ;
   bim = MRI_BYTE_PTR(im) ;

   /* decode the RLE image data into a real image array */

   cc = rr = 0 ;
   for( ii=0 ; ii < im->nvox && rr < nl ; ){
      bb = im26[rr][cc++] ; if( bb == '\0' ) break ;
      if( bb >= 'A' && bb <= 'Z' ){
         jj = bb - 'A' ; bim[ii++] = map26[jj] ;
      } else {
         dd = bb - '0' ; bb = im26[rr][cc++] ; if( bb == '\0' ) break ;
         jj = bb - 'A' ;
         for( ee=0 ; ee < dd && ii < im->nvox ; ee++ )
            bim[ii++] = map26[jj] ;
      }
      if( im26[rr][cc] == '\0' ){ cc = 0 ; rr++ ; }
   }

   RETURN(im) ;
}
#endif /* NO_FRIVOLITIES */
