/*<alloc.c>***************************************************************

   Title: ALLOCation

*/
#include "alloc.h"

/* ----------------------------- matrix allocations --------------------- */

/*<cm_alloc.c>****************************************************************/

char **cm_alloc(int nr, int nc)
{
  char **cm;
  int i;

  cm = (char **)calloc(nr,sizeof(char *));

  for (i=0;i<nr;i++) cm[i] = (char *)calloc(nc,sizeof(char));

  return( cm );
}

/*<sm_alloc.c>****************************************************************/

short **sm_alloc(int nr, int nc)
{
  short **sm;
  int i;

  sm = (short **)calloc(nr,sizeof(short *));

  for (i=0;i<nr;i++) sm[i] = (short *)calloc(nc,sizeof(short));

  return( sm );
}

/*<fm_alloc.c>****************************************************************/

float **fm_alloc(int nr, int nc)
{
  float **fm;
  int i;

  fm = (float **)calloc(nr,sizeof(float *));

  for (i=0;i<nr;i++) fm[i] = (float *)calloc(nc,sizeof(float));

  return( fm );
}

/*<dm_alloc.c>****************************************************************/

double **dm_alloc(int nr, int nc)
{
  double **dm;
  int i;

  dm = (double **)calloc(nr,sizeof(double *));

  for (i=0;i<nr;i++) dm[i] = (double *)calloc(nc,sizeof(double));

  return( dm );
}


/*<_mm_free.c>****************************************************************/

void _mm_free( char **mm, int nr, int nc )
{
  int i;

  for (i=0;i<nr;i++) free(mm[i]) ;

  free((char *)mm);

}

/* ----------------------------- tensor allocations --------------------- */

/*<st_alloc.c>****************************************************************/

short ***st_alloc( int ns, int nr, int nc )
{
  short ***st;
  int i;

  st = (short ***)calloc(ns,sizeof(short **));

  for (i=0;i<ns;i++) st[i] = sm_alloc(nr,nc) ;

  return( st );
}

/*<ft_alloc.c>****************************************************************/

float ***ft_alloc( int ns, int nr, int nc )
{
  float ***ft;
  int i;

  ft = (float ***)calloc(ns,sizeof(float **));

  for (i=0;i<ns;i++) ft[i] = fm_alloc(nr,nc) ;

  return( ft );
}

/*<dt_alloc.c>****************************************************************/

double ***dt_alloc( int ns, int nr, int nc )
{
  double ***dt;
  int i;

  dt = (double ***)calloc(ns,sizeof(double **));

  for (i=0;i<ns;i++) dt[i] = dm_alloc(nr,nc) ;

  return( dt );
}

/*<_tt_free.c>****************************************************************/

void _tt_free( char ***tt, int ns, int nr, int nc )
{
  int i;

  for (i=0;i<ns;i++) mm_free(tt[i],nr,nc) ;

  free((char *)tt);
}

/* ----------------------------- ntuple allocations --------------------- */

/*<ftupleN_alloc.c>**********************************************************

  Title: Float nTUPLE in N dimensions

  x[ns][nr][nc][ncomp]

*/
float ****ftupleN_alloc( int ns, int nr, int nc, int ncomp )
{
  float ****ft;
  int i;

  ft = (float ****)calloc(ns,sizeof(float ***));

  for (i=0;i<ns;i++) ft[i] = ft_alloc(nr,nc,ncomp) ;

  return( ft );
}

/*<dtupleN_alloc.c>**********************************************************

  Title: Double nTUPLE in N dimensions

  x[ns][nr][nc][ncomp]

*/
double ****dtupleN_alloc( int ns, int nr, int nc, int ncomp )
{
  double ****dt;
  int i;

  dt = (double ****)calloc(ns,sizeof(double ***));

  for (i=0;i<ns;i++) dt[i] = dt_alloc(nr,nc,ncomp) ;

  return( dt );
}

/*<ftuple3_free.c>**********************************************************

  Title: Float nTUPLE in 3 dimensions

  x[ns][nr][nc][ncomp]

*/
void _tupleN_free( char ****ft, int ns, int nr, int nc, int ncomp )
{
  int i;

  for (i=0;i<ns;i++) tt_free(ft[i],nr,nc,ncomp) ;

  free((char *)ft);
}

/*-------------------------------------------------------------------

  Can the following be made obsolete by using the _mm_free above?

-------------------------------------------------------------------*/

/**<fm_free.c>********************************************************

       Title:  Float Matrix FREE

     Purpose:  Deallocate float matrix d[m][n]

*/
void fm_free( float **fm, int nrow, int ncol )
{
  static int ir ;

  if( fm != NULL ){
    for( ir=0 ; ir < nrow ; ir++ ) if( fm[ir] != NULL ) free(fm[ir]) ;
    free(fm) ;
  }

}

/**<dm_free.c>********************************************************

       Title:  Double Matrix FREE

     Purpose:  Deallocate double matrix d[m][n]

*/
void dm_free( double **dm, int nrow, int ncol )
{
  static int ir ;

  if( dm != NULL ){
    for( ir=0 ; ir < nrow ; ir++ ) if( dm[ir] != NULL ) free(dm[ir]) ;
    free(dm) ;
  }

}

/**<im_free.c>********************************************************

       Title:  Int Matrix FREE

     Purpose:  Deallocate int matrix d[m][n]

*/
void im_free( int **im, int nrow, int ncol )
{
  static int ir ;

  if( im != NULL ){
    for( ir=0 ; ir < nrow ; ir++ ) if( im[ir] != NULL ) free(im[ir]) ;
    free(im) ;
  }

}

/**<sm_free.c>********************************************************

       Title:  Short Matrix FREE

     Purpose:  Deallocate short matrix d[m][n]

*/
void sm_free( short **sm, int nrow, int ncol )
{
  static int ir ;

  if( sm != NULL ){
    for( ir=0 ; ir < nrow ; ir++ ) if( sm[ir] != NULL ) free(sm[ir]) ;
    free(sm) ;
  }

}

/**<cm_free.c>********************************************************

       Title:  Char Matrix FREE

     Purpose:  Deallocate char matrix d[m][n]

*/
void cm_free( char **cm, int nrow, int ncol )
{
  static int ir ;

  if( cm != NULL ){
    for( ir=0 ; ir < nrow ; ir++ ) if( cm[ir] != NULL ) free(cm[ir]) ;
    free(cm) ;
  }

}
