/*-------------------------------------------------------------------------
   create a new MCW_bbox:
       parent    = parent Widget
       num_but   = number of buttons (from 1 to MCW_MAX_BB)
       label_but = array of strings of button labels
       bb_type   = MCW_BB_check      -> check box (any may be on)
                   MCW_BB_radio_one  -> radio box, exactly one will be on
                   MCW_BB_radio_zero -> radio box, zero or one will be on
       bb_frame  = MCW_BB_noframe    -> no frame around box
                 = MCW_BB_frame      -> put frame around box
       bb_orient = LRF_BB_vert       -> vertical
                 = LRF_BB_horiz      -> horizontal
       cb        = Callback procedure for Disarm (NULL for none)
       cb_data   = data to pass to Callback procedure (NULL for none)
---------------------------------------------------------------------------*/

#include "utils.h"

MCW_bbox * new_LRF_bbox( Widget parent ,
                         int num_but , char * label_but[] ,
                         int bb_type , int bb_frame ,
			 int bb_orient ,
                         XtCallbackProc cb , XtPointer cb_data )
{
   MCW_bbox * bb ;
   int ib , initial_value ;
   Widget rc_parent ;
   Arg wa[30] ;  int na ;
   Pixel  fg_pix ;

   if( num_but <= 0 || num_but >= 32 ){
      fprintf(stderr,"\n*** illegal new_MCW_bbox has %d buttons\n",num_but) ;
      EXIT(1) ;
   }

   bb = (MCW_bbox *) XtMalloc( sizeof(MCW_bbox) ) ;

   bb->nbut      = num_but ;
   initial_value = 0 ;

   /***--- create Frame, if desired ---***/

   switch( bb_frame ){

      case MCW_BB_frame:
         rc_parent = bb->wtop = bb->wframe =
            XtVaCreateManagedWidget(
               "frame" , xmFrameWidgetClass , parent ,
                   XmNshadowType , XmSHADOW_ETCHED_IN ,
                   XmNinitialResourcesPersistent , False ,
               NULL ) ;
         break ;

      case MCW_BB_noframe:
      default:
        rc_parent  = parent ;
        bb->wframe = NULL ;
        break ;
   }

   /***--- create RowColumn to hold the buttons ---***/

#define MAX_PER_COL 8

   na = 0 ;

   switch( bb_orient ){
      case LRF_BB_vert:
	XtSetArg( wa[na] , XmNpacking    , XmPACK_TIGHT )                ; na++ ;
         break ;

      case LRF_BB_horiz:
	XtSetArg( wa[na] , XmNpacking    , XmPACK_COLUMN )               ; na++ ;
	XtSetArg( wa[na] , XmNnumColumns , num_but ) ; na++ ;
         break ;
      default:
	XtSetArg( wa[na] , XmNpacking    , XmPACK_TIGHT )                ; na++ ;
        break ;
   }

   XtSetArg( wa[na] , XmNmarginHeight , 0 ) ; na++ ;  /* squash things in */
   XtSetArg( wa[na] , XmNmarginWidth  , 0 ) ; na++ ;
   XtSetArg( wa[na] , XmNspacing      , 1 ) ; na++ ;

   XtSetArg( wa[na] , XmNinitialResourcesPersistent , False ) ; na++ ;

   if( bb_type == MCW_BB_radio_zero || bb_type == MCW_BB_radio_one ){

      XtSetArg( wa[na] , XmNradioBehavior , True ) ; na++ ;

      if( bb_type == MCW_BB_radio_one ){
         initial_value = 1 ;
         XtSetArg( wa[na] , XmNradioAlwaysOne , True ) ; na++ ;
      } else {
         XtSetArg( wa[na] , XmNradioAlwaysOne , False ) ; na++ ;
      }
   }

   bb->wrowcol = XtCreateWidget(
                   "dialog" , xmRowColumnWidgetClass , rc_parent ,
                   wa , na ) ;

   if( bb->wframe == NULL ) bb->wtop = bb->wrowcol ;  /* topmost widget */

   XtVaGetValues( bb->wtop , XmNforeground , &fg_pix , NULL ) ;

   /***--- create the buttons ---***/

   for( ib=0 ; ib < num_but ; ib++ ){
      bb->wbut[ib] = XtVaCreateManagedWidget(
                        "dialog" , xmToggleButtonWidgetClass , bb->wrowcol ,
                           LABEL_ARG(label_but[ib]) ,
			   XmNorientation  , XmHORIZONTAL ,
                           XmNmarginHeight  , 0 ,
                           XmNmarginWidth   , 0 ,
                           XmNselectColor   , fg_pix ,  /* 04 Nov 1996 */
                           XmNrecomputeSize , False ,
                           XmNtraversalOn   , False ,
                           XmNinitialResourcesPersistent , False ,
                        NULL ) ;

      if( cb != NULL )
         XtAddCallback( bb->wbut[ib] , XmNdisarmCallback , cb , cb_data ) ;

   }
   for( ib=num_but ; ib < MCW_MAX_BB ; ib++ ) bb->wbut[ib] = NULL ;

   MCW_set_bbox( bb , initial_value ) ;
   XtManageChild( bb->wrowcol ) ;

   bb->parent = bb->aux = NULL ;
   return bb ;
}
