/* from mrilib.h */

typedef struct MRI_IMAGE {
         int nx,ny,nz , nt,nu,nv,nw ,   /* dimensions of array */
             nxy,nxyz,nxyzt , nvox  ,   /* nx*ny , nxy*nz , nxyz*nt , # voxels */
             pixel_size ;               /* bytes per pixel */
         MRI_TYPE kind ;                /* one of the type codes above */
         MRI_DATA im ;                  /* pointer to pixel data */
         char * name ;                  /* string attached; may be NULL */
         float dx,dy,dz, dt,du,dv,dw ,  /* physical pixel dimensions, if != 0 */
               xo,yo,zo, to,uo,vo,wo  ; /* offsets for dimensions */
#ifdef USE_MRI_LABELS
         char xlab[MRI_LABEL_SIZE] ,    /* labels for each dimension */
              ylab[MRI_LABEL_SIZE] , zlab[MRI_LABEL_SIZE] ,
              tlab[MRI_LABEL_SIZE] , ulab[MRI_LABEL_SIZE] ,
              vlab[MRI_LABEL_SIZE] , wlab[MRI_LABEL_SIZE]  ;
#endif

#ifdef USE_MRI_DELAY
         char * fname ;
         int foffset , fondisk ;
#endif
} MRI_IMAGE ;

/* from mrilib.h */

typedef struct MRI_IMARR {
      int num , nall ;
      MRI_IMAGE ** imarr ;
} MRI_IMARR ;

/* from afni.h */

typedef struct {
   MCW_DC * dc ;                                  /* display context for everyone */
   THD_sessionlist * sslist ;                     /* all sessions viewable */
   MRI_IMARR * timeseries ;                       /* all timeseries available */
   Three_D_View * controllers[MAX_CONTROLLERS] ;  /* all controllers available */
   MCW_interruptables interruptables ;            /* windows and widgets */

   MCW_function_list registered_0D ;              /* registered functions */
   MCW_function_list registered_1D ;
   MCW_function_list registered_2D ;

   int controller_lock , ignore_lock ;
   int have_dummy_dataset ;
   int sesstrail ;                                /* 23 Oct 1998 */

   THD_coorder cord ;

#ifdef ALLOW_PLUGINS
   struct AFNI_plugin_array * plugins ;           /* plugins */
#endif

   PBAR_palette_table * gpt ;

   int time_lock ;                                /* 03 Nov 1998 */

   int hints_on ;                                 /* 01 Aug 1999 */

   float fim_bkthr_perc ;                         /* 02 Jun 1999 */

   MCW_function_list registered_fim ;             /* 30 Jan 2000 */

} AFNI_library_type ;

/* from afni_plugin.c */

void PLUTO_register_timeseries( char * cname , MRI_IMAGE * tsim )
{
   MRI_IMAGE * qim ;

ENTRY("PLUTO_register_timeseries") ;

   if( tsim != NULL ){
      qim = mri_to_float( tsim ) ;  /* a copy */
      mri_add_name( cname , qim ) ; /* the name */
      AFNI_add_timeseries( qim ) ;  /* give it to AFNI */
   }
   EXRETURN ;
}

/* from mri_add_name.c */

void mri_add_name( char * str , MRI_IMAGE * im )
{
   int ll ;

   if( im->name != NULL ){ free( im->name ) ; im->name = NULL ; }

   if( str == NULL ) return ;

   ll = strlen(str) ; if( ll <= 0 ) return ;

   im->name = (char * ) malloc( ll+1 ) ;
   strcpy( im->name , str ) ;
   return ;
}

/* from afni.c */

void AFNI_add_timeseries( MRI_IMAGE * tsim )
{
ENTRY("AFNI_add_timeseries") ;

   if( tsim != NULL ){
      POPDOWN_timeseries_chooser ;
      ADDTO_IMARR(GLOBAL_library.timeseries,tsim) ;
   }
   EXRETURN ;
}

/* from mrilib.h */

#define ADDTO_IMARR(name,imm) \
   do{ int nn , iq ;   \
       if( (name)->num == (name)->nall ){   \
          nn = (name)->nall = 1.1*(name)->nall + INC_IMARR ;  \
          (name)->imarr = realloc( (name)->imarr,sizeof(MRI_IMAGE *)*nn ); \
          for( iq=(name)->num ; iq < (name)->nall ; iq++ ) (name)->imarr[iq] = NULL ; } \
       nn = (name)->num ; ((name)->num)++ ;   \
       (name)->imarr[nn] = (imm) ; break ; } while(0)


GLOBAL_library.timeseries->num
