/**<tstvec>**************************************************

  Title: TeST VECtor routines

*/
#include "vector.h"

#define TSTTRIG 0		/* test trig functions */
#define TSTZVEC 0		/* test complex vector functions */
#define TSTFVEC 1		/* test float vector functions */
#define TSTCHR  0		/* test float vector functions */
#define TESTYLM 0		/* test ylm normalization coeffs */

/*#define YLM_COEF(l,m) (((2*L+1)/(4.*PI))*(factorial(l-m)/factorial(l+m)))*/
/*===<pinput.c>========================================================

     Title: Parameter INPUT

   Purpose: Prompt user for numerical input

      Call: float pinput(char *pname, float pdef)
      
     Input: char *pname = parameter name for prompt
            float pdef  = parameter default value

    Return: float value of input

   History: 02_11_15 - lrf

*/
float pinput(char *pname, float pdef)
{
  char  line[100];
  int   nn;
  float fval;

  printf(" Enter %s [%g]: ",pname,pdef);
  fgets(line,sizeof(line),stdin);
  nn = sscanf(line,"%f",&fval);

  if(nn<0) fval = pdef;

  return(fval) ;
}

main()
{
  int i,j,l,m,n;


  if(TESTYLM) {
    int L=5, M=4;
    /*printf("YLM_COEFF(%i,%i) = %g\n",L,M,YLM_COEF(L,M));*/
  }

  if(TSTCHR) {
    char *label_list[] = {"L=0","L=1","L=2","L=3","L=4","L=5"};
    char list[10][80],ctmp[80] ;
    int lmax=5 ;

    for(i=0; i<=lmax; i++) {
      sprintf(ctmp,"L=%2i\n",i);
      strcat(list[i],ctmp);
    }

    for(i=0; i<=lmax; i++) {
      printf("list[%i] = %s\n",i,list[i]);
    }

  }

  if(TSTTRIG) {			/* Test trig */

    float *x,*y1,*y2;
    n = 10;
    x = (float *) calloc((unsigned)n,sizeof(float));
    y1 = (float *) calloc((unsigned)n,sizeof(float));
    y2 = (float *) calloc((unsigned)n,sizeof(float));

    printf("Calling vcos(x),vsin(x) =\n");
    for(i=0; i< n; i++) {
      x[i] = (float)i/(float)n;
      printf("x[%i] = %lf\n",i,x[i]);
    }

    vcos(y1,x,n);
    vsin(y2,x,n);

    printf("Back from vcos,vsin with output =\n");
    for(i=0; i<n; i++) {
      printf("(y1[%i],y2[%i]) = (%lf,%lf)\n",i,i,y1[i],y2[i]);
    }

    printf("TSTTRIG: done !\n");
  }

  if(TSTZVEC) {			/* Test complex vector */

    complex *z1,*z2;
    n = 10;
    z1 = (complex *) calloc((unsigned)n,sizeof(complex));
    z2 = (complex *) calloc((unsigned)n,sizeof(complex));

    printf("Calling zvexp =\n");
    for(i=0; i< n; i++) {
      z1[i].r = 1.0;
      z1[i].i = (float)i/(float)n;
      printf("z[%i] = (%lf,%lf)\n",i,z1[i].r,z1[i].i);
    }

    zvexp(z2,z1,n);

    printf("Calling zvexp =\n");
    for(i=0; i< n; i++) {
      printf("z[%i] = (%lf,%lf)\n",i,z2[i].r,z2[i].i);
    }

    printf("TSTZVEC: done !\n");
  }

  if(TSTFVEC) {			/* Test complex vector */

    float *x,a;
    float xmin,xmax ;
    int   nx;
    n = 10;
    x = (float *) calloc((unsigned)n,sizeof(float));

    printf("Calling zvscal =\n");
    for(i=0; i< n; i++) {
      x[i] = (float)i/(float)n;
      printf("x[%i] = %lf\n",i,x[i]);
    }

    printf("Calling vscal =\n");
    a = 4.5;
    vscal(x,a,x,n);

    for(i=0; i< n; i++) {
      printf("x[%i] = %lf\n",i,x[i]);
    }

    xmin = pinput("xmin",0.0); 
    xmax = pinput("xmax",5.0); 
    nx   = (int) pinput("nx",3); 
    if(nx>n) {
      printf("nx must be less than n.  Setting to n\n");
      nx=n; 
    }

    printf("Calling vlinsp: xmin = %f, xmax = %f, nx = %i\n",xmin,xmax,nx);
    vlinsp(x,xmin,xmax,nx);

    for(i=0; i< nx; i++) {
      printf("x[%i] = %lf\n",i,x[i]);
    }

    printf("TSTFVEC: done !\n");
  }


}
