/* vector.h */
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <sys/time.h>
#include <time.h>

#ifndef TYPEDEF_byte
#define TYPEDEF_byte
typedef unsigned char byte ;
#endif

#ifndef TYPEDEF_complex
#define TYPEDEF_complex
typedef struct { float r, i ; } complex ;
#endif

#ifndef PI
#  define PI 3.141592653589793238462643
#endif

#ifndef TWOPI
#  define TWOPI 6.283185307179586476925286     /* 2 * PI */
#endif

#ifndef PI2
#  define PI2 1.57079632679490     /* PI/2 */
#endif

#ifndef MAX
#define MAX(x, y) ( (x) > (y) ? (x) : (y) )
#endif

#ifndef MIN
#define MIN(x, y) ( (x) > (y) ? (y) : (x) )
#endif

#define MABS(x)	    (((x)<0) ? -(x) : (x))
#define SIGN2(f,g)  (((g)>0)?(f):(-(f)))
#define SIGN(a)     ((a) >= 0 ? 1 : -1)
#define FMOD(x,y)   ((x)-(y)*floor((x)/(y)))

/************************* some complex math *********************************/

#define SQR(x)   ((x)*(x))
#define CUBE(x)  ((x)*(x)*(x))
#define ZSQR(z)  (SQR(z.r)+SQR(z.i))
#define ZABS(z)  sqrt(ZSQR(z))
#define ZARG(z)  ( ((z).r!=0.0 || (z).i!=0.0) ? atan2((z).i,(z).r) : 0.0 )

/* complex z = 0 */
#define ZCLR(z) ((z).r=0.0,(z).i=0.0)

/* complex z = y */
#define ZCPY(z,y) ((z).r=(y).r,(z).i=(y).i)

/* complex u += v */
#define ZADDTO(u,v) ((u).r += (v).r, (u).i += (v).i)

/* complex u *= a */
#define ZSCAL(u,a) ((u).r *= (a), (u).i *= (a))

/****************************************************************************/

#define M_FREE(x,m,n) \
  do{ int i; \
      if( (x) != NULL ){ \
	for( i=0; i<(n); i++ ) if( (x)[i] != NULL ) free((x)[i]) ; \
	free((x)) ; \
      } \
    } while(0) ;

/* should probably be in timeutils */
long int clock_seed( void );

double factorial(int N) ;

/* Integer vector routines */

short *svlinsp( short *x, short f1, short f2, int n ) ;
short smnmx( short *x , int n , int *nmin , int *nmax ) ;
void sran( short *a , short *amin , int *lamin ,
	              short *amax , int *lamax , int n ) ;

/* Float vector routines */

float *vdiff( float *dx, float *x, int n);
float *vgrad( float *gx, float *x, int n);
float *unwrap( float *p, int n, float *mask );
float *vtrn2( float *z, int nx, int ny );

double vcor( float *x, float *y, int n) ;
float *vrnd( float *x, int n, char *dist ) ;
float *vsin(float *z, float *x, int n) ;
float *vcos(float *z, float *x, int n) ;
float *vscal(float *z, float a, float *x, int n) ;
float *vmove(float *z, float *x, int N);
float *vrev( float *z , float *x , int N ) ;
float *vlinsp(float *x,float d1,float d2,int n) ;
float vmsq( float *x, int n ) ;
float vstd( float *x, int n ) ;
float vvar( float *x, int n ) ;
float vsum( float *x, int n ) ;
float vmean( float *x, int n ) ;
float vmnmx(float *x, int n, int *nmin, int *nmax) ;
void vran( float *a , float *amin , int *lamin ,
	              float *amax , int *lamax , int n ) ;
int vclip( register float *z, register float *x, register int n,
	   register float xmin, register float xmax) ;
float *vclr( register float *x, register int n );
float *vfill( register float *x, float a, register int n ) ;

double  *fm2dv( double  *dvec, float **fmat, int nr, int nc) ;
double **fv2dm( double **dmat, float  *fvec, int nr, int nc) ;

float *vrndn(float *x,int n) ;

/* Double vector routines */

double *dvrndn(double *x,int n) ;
double *dvmove( double *z, double *x , int N ) ;
double *dvadd( double *z, double *x, double a, double *y, int N ) ;
double *dvscal(double *z, double a, double *x, double b , int N ) ;
double dvmsq( double *x, int n ) ;
double dvsum( double *x, int n ) ;
double dvmean( double *x, int n ) ;
double dvvar( double *x, int n ) ;
double dvmnmx(double *x, int n, int *nmin, int *nmax) ;

/* Complex routines */

complex zclr( complex z ) ;
complex zrect(complex z) ;
complex zcnjg(complex a) ;
complex zmuli(complex a) ;
double  zabs(complex a) ;
complex zexp(complex z) ;
complex zdiv(complex a,complex b) ;
complex zscal(double d, complex z) ;
complex zmul(complex a, complex b) ;
complex zmulc(complex a,complex b) ;
complex zadd(complex a, complex b) ;
complex zsub(complex a, complex b) ;
complex zcplx(double r, double i) ;
complex zadd(complex a, complex b) ;

/* Complex vector routines */

complex zvsum(complex *x,int n) ;
complex zvmean( complex *x, int n) ;
double  zvstd( complex *x, int n );
complex zvcov(complex *x, complex *y, int n) ;
complex zvcc(complex *x,complex *y,int n) ;

complex *zvadd(complex *z, complex *x, complex a, register complex *y, register int N );
float *zvabs( register float *z, complex *x, register int N) ;
complex  zvcor(complex *x, complex *y, register int n) ;
complex zvconv(complex *x,complex *y, register int n) ;
complex *zvscal(complex *z,complex a, register complex *x, register int N);
complex *zvcplx( complex *z, float *real, float *imag, register int N );
complex *zvzap( complex *z , int n ) ;
complex *zvzap3( complex *z, int nr, int nc, int nz ) ;
complex *zvzap2( complex *z, int nr, int nc ) ;
complex *zvzap1( complex *z, int nr, int nc, int rowcol ) ;
complex *zvpolr(complex *z, complex *x,int N) ;
complex *zvrect(complex *z,complex *x,int N) ;
complex *zvexp(complex *z, complex *x, int n) ;
complex *zvclr(complex *x , register int N ) ;
complex *zvmove(complex *z, complex *x, int N) ;
complex *zvrev(register complex *z,register complex *x,register int N) ;
complex *zvtori(register float *real, register float *imag,complex *z, register int N);
complex *zvtrn2( complex *z, int nx, int ny ) ;
complex *zvmul( complex *z, register complex *x, register complex *y, register int n );
complex *zvmulc( complex *z, register complex *x, register complex *y, register int n );
complex *zvdiv( complex *z, register complex *x, register complex *y, register int N) ;

/* double routines */
double   *ADD_DVEC3( double *z, double *x , double a , double *y ) ;
double *SCALE_DVEC3( double *z, double a , double *x ) ;
double   *INC_DVEC3( double *z, double a , double *x ) ;

/* float routines */
/*float *SCALE_FVEC3( float *z, float a , float *x ) ;*/

/* Subtraction of two 3-vectors: x-y */
#define SUB_VEC3(z,x,y) ((z)[0]=(x)[0]-(y)[0] , (z)[1]=(x)[1]-(y)[1] , (z)[2]=(x)[2]-(y)[2])

/* Scale vector y and put in x: x = a*y */
#define SCALE_VEC3(x,a,y) \
                  ((x)[0]=(a)*(y)[0],\
		   (x)[1]=(a)*(y)[1],\
		   (x)[2]=(a)*(y)[2])
/* x -> -x */
#define NEGATE_VEC3(x) ((x)[0]*=(-1),(x)[1]*=(-1),(x)[2]*=(-1))

/* Dot product between two vectors: x.y */
#define DOT_VEC3(x,y) ((x)[0]*(y)[0] + (x)[1]*(y)[1] + (x)[2]*(y)[2])

/* 3-Vector norm */
#define NORM_VEC3(x) sqrt(DOT_VEC3(x,x)) 

/* Load components into vec3 */
#define LOAD_VEC3(v,a,b,c) ((v)[0] = (a), (v)[1] = (b), (v)[2] = (c))

/* Copy vec3 "fromv" into "tov" */
#define COPY_VEC3( tov, fromv ) \
        ((tov)[0] = (fromv)[0], (tov)[1] = (fromv)[1], (tov)[2] = (fromv)[2])

#define PRINT_VEC3(str,fv) \
   fprintf(stderr,"%s: %13.6g %13.6g %13.6g\n",(str),(fv)[0],(fv)[1],(fv)[2])
