/**<vmnmx>**************************************************

    Title: Float Vector MiN/MaX

  Purpose: Find the minimum and maximum values,
           and their respective indices, of a float vector.
 
     Call: float vmnmx(float *x, int n, int *nmin, int *nmax)

    Input: x = float vector to examine
           n = length(x)

   Output:  *nmin = index of mininum value in x[]
            *nmax = index of maxinum value in x[]

	    Also value of function is the maximum of the
	    absolute values of the vector points.
	    If nmin=0 or nmax=0, then does not return these pointer values
	    If n<=0, returns immediately with value 0 and *nmin = *nmax = 0.

  History: Musicus, circa 1980
*/
#define MABS(x)		(((x)<0) ? -(x) : (x))
#define MAX(x,y)	(((x)<(y)) ? (y) : (x))

float vmnmx(float *x, int n, int *nmin, int *nmax)
{
	float *xold = x ;
	register float xmin,xmax ;
	register int nnmin,nnmax ;
	register int count ;

	if(n<=0)
	{
		if(nmin) *nmin = 0 ;
		if(nmax) *nmax = 0 ;
		return(0.) ;
	}
	nnmin = nnmax = 0 ;
	xmin = xmax = *x++ ;
	n-- ;
	count = 1 ;
	if(n>0)
	{
		do {
			if(*x < xmin) {
				nnmin = count++ ;
				xmin = *x++ ;
			} else if(*x > xmax) {
				nnmax = count++ ;
				xmax = *x++ ;
			} else {
				count++ ;
				x++ ;
			}
		} while( --n > 0 ) ;
	}
	if(nmin) *nmin = nnmin ;
	if(nmax) *nmax = nnmax ;
	xmax = MABS(xmax) ;
	xmin = MABS(xmin) ;
	return(MAX(xmax,xmin)) ;
}
