/**<vrnd>*************************************************

    Title: Float Vector RaNDom sequence

  Purpose: Vector random number generator

     Call: float *vrnd( float *x, int n, char *dist )

    Input: x[n] = malloc-ed float array
             n  = length of x
	   dist = "uniform" -> uniform distribution
	          "normal"  -> Gaussian (normal) distribution
*/
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "vector"

float *vrnd( float *x, int n, char *dist )
{
  int i;
  long int clock_seed( void );
  double gasdev( void );
  float *xold = x;

  srand48(clock_seed());

  if(!strcmp(dist,"uniform")) {

    for(i=0; i<n; i++) xold[i] = (float) drand48();

  } else if(!strcmp(dist,"normal")) {

    for(i=0; i<n; i++) xold[i] = (float) gasdev();

  } else {

    printf("vrnd: Unknown distribution requested!\n\a");
    exit(0);

  }
  return(xold);
}
/*
      Ref: Numerical Recipes, pg 289

    Notes: Uses unix c-library function drand48()
           for generation of uniformly distributed
	   variates, rather than num rec ran1().
*/

double gasdev( void )
{
  static int iset = 0;
  static float gset;
  double fac,rsq,v1,v2;

  if (iset == 0) {
    do {
      v1 = 2.0 * drand48() - 1.0;
      v2 = 2.0 * drand48() - 1.0;
      rsq = v1 * v1 + v2 * v2;
    } while (rsq >= 1.0 || rsq == 0.0);

    fac = sqrt(-2.0 * log(rsq)/rsq);
    gset = v1 * fac;
    iset = 1;
    return v2 * fac;
  } else {
    iset = 0;
    return gset;
  }
}

long int clock_seed( void )
{
  long   int iseed;
  static char time_line[80];
  struct tm *times;
  long	 time_sec;

  time (&time_sec);
  times = localtime (&time_sec);

  sprintf (time_line, "%2d%2d%2d\n",
	   times->tm_hour,
	   times->tm_min,
	   times->tm_sec);
  
  iseed = (long) atof(time_line);
  return (iseed);
}
