/**<vscal.c>****************************************************

     Title: Vector SCALe

   Purpose: Scales float vector x[] by a giving z[]

      Call: float *vscal(float *z, float a, float *x, int n)

     Input: x = vector to scale
	    a = scale factor
	    n = length(x)

    Output: z[i] = a*x[i] for i=0,...,n-1 if z<=x
                          for i=n-1,...,0 if z>x

    Returns pointer to z[] array
 
    Works correctly regardless of how x[] and z[] might overlap

    History: Musicus, circa 1980
*/
#include "vector.h"

float *vscal(float *z, float a, float *x, int n)
{
	float *zold = z;

	if(n>0)
	{
		if(z<=x)
		{
			do {
				*z++ = a * *x++ ;
			} while( --n > 0) ;
		}
		else
		{
			z += n ;
			x += n ;
			do {
				*--z = a * *--x ;
			} while( --n > 0) ;
		}
	}
	return(zold) ;
}

