/**<zvcor.c>****************************************************

     Title: Complex Vector CORrelation

   Purpose: Correlates two complex vectors

      Call: complex zvcor( complex *x, complex *y, int n )

     Input: x,y = input complex vectors
	    n = length(x)

    Output: z = x[0]*cnjg(y[0]) +...+ x[N-1]*cnjg(y[N-1])

     Notes: Handles case where x=y specially, taking half the time

   History: Musicus, circa 1989
*/
#include "vector.h"

complex zvcor( complex *x, complex *y, int n )
{
	register float *px = (float*)x ;
	register float *py = (float*)y ;

	register double sumr = 0.;
	register double sumi = 0.;
	complex ztemp ;

	if(n>0)
	{
		if(px==py)
		{
			do {
				sumr += *px++ * *py++ ;
				sumr += *px++ * *py++ ;
			} while(--n > 0) ;
		}
		else
		{
			do {
				sumr += *px * *py ;
				sumi -= *px++ * *(py+1) ;
				sumi += *px * *py++ ;
				sumr += *px++ * *py++ ;
			} while(--n > 0) ;
		}
	}

	ztemp.r = sumr ;
	ztemp.i = sumi ;
	return(ztemp) ;
}
