/**<zvscal.c>****************************************************

     Title: Complex Vector SCALe

   Purpose: Scales complex vector x[] by a giving z[]

            z[i] = a*x[i] for i=0,...,n-1 if z<=x
	                  for i=n-1,...,0 if z>x

      Call: complex *zvscal(complex *z, complex a, complex *x, int N)

    Output: Returns pointer to x[]

     Notes: Works correctly regardless of how x[] and z[] might overlap
            Twice as fast if a is real or pure imaginary
*/
#include "vector.h"

complex *zvscal(complex *z,complex a, register complex *x, register int N)
{
	register float *pz,*px ;

	if(N>0)
	{
		if(z <= x)
		{
			pz = (float*)z ;
			if(a.i == 0)
			{
				px = (float*)x ;
				do {
					*pz++ = a.r * *px++ ;
					*pz++ = a.r * *px++ ;
				} while( --N > 0) ;
			}
			else if(a.r == 0)
			{
				do {
					*pz++ = -a.i * x->i ;
					*pz++ = a.i * x->r ;
					x++ ;
				} while( --N > 0) ;
			}
			else
			{
				do {
					*pz++ = a.r * x->r - a.i * x->i ;
					*pz++ = a.r * x->i + a.i * x->r ;
					x++ ;
				} while( --N > 0) ;
			}
		}
		else
		{
			pz = (float*)(z+N) ;
			x += N ;
			if(a.i == 0)
			{
				px = (float*)x ;
				do {
					*--pz = a.r * *--px ;
					*--pz = a.r * *--px ;
				} while( --N > 0) ;
			}
			else if(a.r == 0)
			{
				do {
					--x ;
					*--pz = a.i * x->r ;
					*--pz = -a.i * x->i ;
				} while( --N > 0) ;
			}
			else
			{
				do {
					--x ;
					*--pz = a.r*x->i + a.i*x->r ;
					*--pz = a.r*x->r - a.i*x->i ;
				} while( --N > 0) ;
			}
		}
	}
	return(z) ;
}
