#ifndef __vtkParametricSHD_h
#define __vtkParametricSHD_h

#include "vtkParametricFunction.h"
#include "vtk_Utils.h"

class VTK_COMMON_EXPORT ParametricSHD : public vtkParametricFunction {
 public:
  vtkTypeRevisionMacro(ParametricSHD, ParametricSHD);
  void PrintSelf(ostream& os, vtkIndent indent);
  static ParametricSHD *New();

  virtual int GetDimension() {return 2;}

  // Description:
  // Set/Get the scaling factor for the x-axis. Default = 1.
  vtkSetMacro(XRadius,double);
  vtkGetMacro(XRadius,double);

  // Description:
  // Set/Get the scaling factor for the y-axis. Default = 1.
  vtkSetMacro(YRadius,double);
  vtkGetMacro(YRadius,double);

  // Description:
  // Set/Get the scaling factor for the z-axis. Default = 1.
  vtkSetMacro(ZRadius,double);
  vtkGetMacro(ZRadius,double);

  vtkSetMacro(lmax,int);
  vtkGetMacro(lmax,int);

  void SetAlm( complex **alm, int lmax ) ;

  virtual void Evaluate(double uvw[3], double Pt[3], double Duvw[9]);
  virtual double EvaluateScalar(double uvw[3], double Pt[3], double Duvw[9]);

protected:
  ParametricSHD();
  ~ParametricSHD();

  // Variables
  double XRadius;
  double YRadius;
  double ZRadius;

  int lmax;
  complex **Alm;
/*   double N1; */
/*   double N2; */

private:
  ParametricSHD(const ParametricSHD&);  // Not implemented.
  void operator=(const ParametricSHD&);  // Not implemented.

};

#endif
