/*=======================================================================
  Program:   Launch VTK window
  Language:  C++
  Date:      $Date: 2004/03/03 02:48:57 $
  Version:   $Revision: 1.5 $
  Author:    Cyrus Jam
=========================================================================*/

//
// Work in Progress 
//

#define VTK_RWI_MESSAGE \
{printf("\nActivating rendering window interactor ...\n" \
	"(Hit 'q' in window to exit)\n");fflush(stdout);}

#include "vtk_window.h"

/* Global ... */
vtkXRenderWindowInteractor *iren,*irenv ;
vtkXOpenGLRenderWindow     *renWin,*renvWin ;
vtkRenderer                *ren1,*renv ;
vtkPolyDataMapper          *dataMapper,*vdataMapper ;
vtkActor                   *dataActor,*vdataActor ;
vtkOutlineFilter           *outline ;

/* Global vector data for different modes ... */ 
vtkStructuredGrid *vdata ;

/* Processing switches */ 
Boolean DO_STREAM       = 0 ;	/* Streamline */
Boolean DO_STUBE        = 0 ;	/* Streamtube */
Boolean DO_VORTICITY    = 1 ;	/* Vorticity calc for streamline */ 
Boolean DO_RIBBON       = 0 ;	/* Ribbon filter */
Boolean DO_LINE_WIDGET  = 1;	/* Line widget   */ 
Boolean DO_PLANE_WIDGET = 0;	/* Plane widget   */ 

/* Widget setup */
Widget setup_widgets(Widget,vtkXOpenGLRenderWindow*);
Widget setup_vector_widgets(Widget,vtkXOpenGLRenderWindow*);

#define BETWEEN(x,a,b)  (((x) >= (a)) && ((x)<=(b)))
#define SET_LIMS(lims,xmin,xmax)  ((lims)[0]=(xmin),(lims)[1]=(xmax))
static float flims[2] ;
/*===<pinput.c>========================================================
 
     Title: Parameter INPUT
 
   Purpose: Prompt user for numerical input
 
      Call: float pinput(char *pname, float pdef)
       
     Input: char *pname = parameter name for prompt
            float pdef  = parameter default value
 
    Return: float value of input
 
   History: 02_11_15 - lrf
 
*/
float pinput(char *pname, float pdef, float *plims)
{
  char  line[100];
  int   nn;
  float fval;
 
  int mistakes = 0;
  int too_many_mistakes = 10;
 
  do {
 
    printf(" Enter %s [%g] {%g,%g}: ",pname,pdef,plims[0],plims[1]);
    fgets(line,sizeof(line),stdin);
    nn = sscanf(line,"%f",&fval);

    if(nn<0) fval = pdef;

    if(!BETWEEN(fval,plims[0],plims[1])) {
      printf("\n ***** Parameter '%s' must be between %f and %f\a\n",
             pname,plims[0],plims[1]) ;
      mistakes++ ;
      if(mistakes>too_many_mistakes) {
        printf("You've had your chance!  Later ...\a\n");
        exit(0);
      }
    }

  }  while(!BETWEEN(fval,plims[0],plims[1])) ;

  return(fval) ;
}

/** Callbacks ************************************************************/

/* Line widget */

class vtkLWCallback : public vtkCommand
{
public:
  static vtkLWCallback *New() 
    { return new vtkLWCallback; }
  virtual void Execute(vtkObject *caller, unsigned long, void*)
    {
      vtkLineWidget *lineWidget = reinterpret_cast<vtkLineWidget*>(caller);
      lineWidget->GetPolyData(this->PolyData);
      this->PolyData->Print(cout) ;
      lineWidget->Print(cout) ;
      this->Actor->VisibilityOn();
    }
  vtkLWCallback():PolyData(0),Actor(0) {}
  vtkPolyData *PolyData;
  vtkActor *Actor;
};

/* Plane widget */

class vtkPWCallback : public vtkCommand
{
public:
  static vtkPWCallback *New() 
    { return new vtkPWCallback; }
  virtual void Execute(vtkObject *caller, unsigned long, void*)
    {
      vtkPlaneWidget *planeWidget = reinterpret_cast<vtkPlaneWidget*>(caller);
      planeWidget->GetPolyData(this->PolyData);
      this->Actor->VisibilityOn();
    }
  vtkPWCallback():PolyData(0),Actor(0) {}
  vtkPolyData *PolyData;
  vtkActor *Actor;
};

/** end callbacks ************************************************************/

/*===<DisplayVectorField.c>==================================================

  Title: DISPLAY VECTOR FIELD
  Purpose: 

*/
void DisplayVectorField(void)
{
  char *program_name = "DisplayVectorField";
  int debug = 0 ;
  Boolean FIX_ARROW_LENGTHS = 0;
  double Arrow_Scale_Factor = 1.0 ; // Why???

  if(vdata==NULL) {
    fprintf(stderr,"%s: vdata is NULL!\n",program_name);
    return ;
  }

  if(debug) {
    fprintf(stderr,"Inside %s\n",program_name);
    vdata->Print(cout) ;
  }

  /***************** Construct arrows *****************/

  /* source data = vtkGlyphSource2D */

  vtkGlyphSource2D *arrow = vtkGlyphSource2D::New();
  arrow->SetGlyphTypeToArrow();  
  arrow->SetScale(.5);
  arrow->FilledOff();

  /* actual arrows = vtkGlyph3D */

  vtkGlyph3D *glyph = vtkGlyph3D::New();
  glyph->SetSource(arrow->GetOutput());

  /* scaling mystery */

  glyph->SetScaleModeToScaleByVector();
  glyph->SetScaleMode(VTK_SCALE_BY_VECTOR);
  if(FIX_ARROW_LENGTHS) {
    glyph->ClampingOn();
    Arrow_Scale_Factor *= 200. ; // this is 1/.005
  }
  printf("Arrow_Scale_Factor = %f\n",Arrow_Scale_Factor);
  glyph->SetScaleFactor(Arrow_Scale_Factor);
  glyph->SetColorModeToColorByScalar();	/* use reflectivity scalars for color */
  glyph->SetVectorModeToUseVector();
  glyph->SetInput(vdata);
  glyph->OrientOn();
  glyph->Modified() ;

  /* Done creating arrows, add them ... */
  vdataMapper->SetInput(glyph->GetOutput());

  /***************** end arrow construction *****************/

  /*** Colormap ***/ 
  float range[2];
  vdata->GetScalarRange(range);
  if(debug) fprintf(stderr,"range = (%f,%f)\n",range[0],range[1]) ;
  range[0] = 0;			// Peg lower limit to zero
  vtkLookupTable *lut = CreateStandardColorTable( range ) ;

  /* Done creating lookup table, add it ... */
  vdataMapper->SetLookupTable(lut); 

  /*** end colormap ***/ 

  /*** Outline data ***/ 

  if(outline==NULL) outline = vtkOutlineFilter::New();

  outline->SetInput(vdata);
  vtkPolyDataMapper *outlineMapper = vtkPolyDataMapper::New();
  outlineMapper->SetInput(outline->GetOutput());
  vtkActor *outlineActor = vtkActor::New();
  outlineActor->SetMapper(outlineMapper);
  outlineActor->GetProperty()->SetColor(0,0,0);

  /* Done outlining, add it ... */
  renv->AddActor(outlineActor); 

  /*** end outline ***/ 

  /* actor, renderer ...*/ 
  vdataActor->SetMapper(vdataMapper) ;
  renv->AddActor(vdataActor) ;
  renv->ResetCamera() ;
  renvWin->Render();  
}

void Launch_vtk_vector_window(Widget shell)
{

  renvWin = vtkXOpenGLRenderWindow::New();
  renv = vtkRenderer::New();

  Widget vtk_vector_widg = setup_vector_widgets(shell,renvWin);
  
  /* X Window stuff end */

  fprintf( stderr, ":) Launch vtk vector window\n" );	

  vdataMapper = vtkPolyDataMapper::New(); // data mapper
  vdataActor = vtkActor::New();	// actor
  vdataActor->SetMapper(vdataMapper);

  renv->SetBackground( 0.1, 0.2, 0.4 );	// renderer
  renvWin->AddRenderer( renv );	// render win 
  renvWin->SetSize( 300, 300 );

  // x win
  //vtkXRenderWindowInteractor *irenv = vtkXRenderWindowInteractor::New();
  irenv = vtkXRenderWindowInteractor::New();
  irenv->SetRenderWindow(renvWin);
  irenv->SetWidget(vtk_vector_widg);

  renvWin->Render();
  irenv->Initialize(XtWidgetToApplicationContext(vtk_vector_widg)) ;
  
  fprintf( stderr, ":( exiting vtk window\n" );	

}

void Send_Data_to_vtk_vector_window_old( vtkPolyData *data )
{
  char *program_name = "Send_Data_to_vtk_vector_window" ;
  int debug = 0 ;

  if(debug) {
    fprintf(stderr,"Inside %s\n",program_name);
    data->Print(cout) ;
  }

  vdata = vtkPolyData::New();
  vdata->ShallowCopy(data) ;

}

void Send_Data_to_vtk_vector_window( vtkPolyData *data )
{
  char *program_name = "Send_Data_to_vtk_vector_window" ;
  int debug = 0 ;

  if(debug) {
    fprintf(stderr,"Inside %s\n",program_name);
    data->Print(cout) ;
  }

  vdata = vtkPolyData::New();
  vdata->ShallowCopy(data) ;

}

void Send_Data_to_vtk_window( vtkPolyData *data )
{
  printf("Inside Send_Data_to_vtk_window:\n"); fflush(stdout);
  dataMapper->SetInput( data );
  dataActor->SetMapper( dataMapper );
  ren1->AddActor( dataActor );
  ren1->ResetCamera() ;
  renWin->Render();  
}

void Launch_vtk_window(Widget shell)
{

  //vtkXOpenGLRenderWindow *renWin = vtkXOpenGLRenderWindow::New();
  //vtkRenderer *ren1 = vtkRenderer::New();
  renWin = vtkXOpenGLRenderWindow::New();
  ren1 = vtkRenderer::New();

  Widget vtk_widg = setup_widgets(shell,renWin);
  
  /* X Window stuff end */

  fprintf( stderr, ":) Launch vtk window\n" );	
  
  //
  // In this example we terminate the pipeline with a mapper process object.
  // (Intermediate filters such as vtkShrinkPolyData could be inserted in
  // between the source and the mapper.)  We create an instance of
  // vtkPolyDataMapper to map the polygonal data into graphics primitives. We
  // connect the output of the cone souece to the input of this mapper.
  //
  //vtkPolyDataMapper *dataMapper = vtkPolyDataMapper::New();
  dataMapper = vtkPolyDataMapper::New();
  //dataMapper->SetInput( data );

  //
  // Create an actor to represent the data. The actor orchestrates rendering
  // of the mapper's graphics primitives. An actor also refers to properties
  // via a vtkProperty instance, and includes an internal transformation
  // matrix. We set this actor's mapper to be dataMapper which we created
  // above.
  //
  //vtkActor *dataActor = vtkActor::New();
  dataActor = vtkActor::New();
  //dataActor->SetMapper( dataMapper );

  //
  // Create the Renderer and assign actors to it. A renderer is like a
  // viewport. It is part or all of a window on the screen and it is
  // responsible for drawing the actors it has.  We also set the background
  // color here.
  //
  //ren1->AddActor( dataActor );
  ren1->SetBackground( 0.1, 0.2, 0.4 );

  //
  // Finally we create the render window which will show up on the screen.
  // We put our renderer into the render window using AddRenderer. We also
  // set the size to be 300 pixels by 300.
  //

  renWin->AddRenderer( ren1 );
  renWin->SetSize( 300, 300 );

  // we typecast to an X specific interactor
  // Since we have decided to make this an X program
  //vtkXRenderWindowInteractor *iren = vtkXRenderWindowInteractor::New();
  iren = vtkXRenderWindowInteractor::New();
  iren->SetRenderWindow(renWin);
  iren->SetWidget(vtk_widg);

  renWin->Render();
  iren->Initialize(XtWidgetToApplicationContext(vtk_widg)) ;

  //
  // Free up any objects we created. All instances in VTK are deleted by
  // using the Delete() method.
  //

  /*dataMapper->Delete();
    dataActor->Delete();
    ren1->Delete();
    renWin->Delete(); */
  
  fprintf( stderr, ":( exiting vtk window\n" );	

}

/* quit callback */

void quit_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
  if(vdata!=NULL) vdata->Delete() ;
  XtUnmapWidget( (Widget)client_data ) ; 
}

/* vector field callback */

void vfield_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
  char *program_name = "vfield_cb" ;
  int debug = 0 ;

  if(debug) {
    fprintf(stderr,"Inside %s\n",program_name);
    vdata->Print(cout);
  }

  DisplayVectorField() ;

}

/* compute streamlines */
void compute_streamlines(Widget w, XtPointer client_data, XtPointer call_data)
{
  char *program_name = "compute_streamlines" ;
  int debug = 0 ;

  if(debug) {
    fprintf(stderr,"Inside %s: DO_STREAM=%i, DO_STUBE=%i\n",
	    program_name,DO_STREAM,DO_STUBE);
    vdata->Print(cout);
  }

  /*** Outline data ***/ 

  if(outline==NULL) outline = vtkOutlineFilter::New();

  outline->SetInput(vdata);
  vtkPolyDataMapper *outlineMapper = vtkPolyDataMapper::New();
  outlineMapper->SetInput(outline->GetOutput());
  vtkActor *outlineActor = vtkActor::New();
  outlineActor->SetMapper(outlineMapper);
  outlineActor->GetProperty()->SetColor(0,0,0);

  renv->AddActor(outlineActor);   /* Done outlining, add it ... */

  /*** done outlining data ***/ 

  /* Set up streamlining players */ 

  vtkStreamLine     *streamLine       = vtkStreamLine::New();
  vtkPolyDataMapper *streamLineMapper = vtkPolyDataMapper::New();
  vtkActor          *streamLineActor  = vtkActor::New();

  vtkTubeFilter     *streamTube       = vtkTubeFilter::New();
  vtkPolyDataMapper *streamTubeMapper = vtkPolyDataMapper::New();
  vtkActor          *streamTubeActor  = vtkActor::New();

  vtkRibbonFilter    *streamRibbon    = vtkRibbonFilter::New();

  /******** Probing widgets ********/ 

  vtkLineWidget   *lineWidget = vtkLineWidget::New();
  vtkPlaneWidget *planeWidget = vtkPlaneWidget::New();

  /******** Probing sources ********/ 

  vtkPolyData    *seeds = vtkPolyData::New();
  vtkProbeFilter *probe = vtkProbeFilter::New();

  /* Integration params */ 

  /*
  float maxTime    = 100.0 ;
  float IntStepLen = 0.2 ;
  float StepLen    = 0.0001 ;
  */
  float TermSpeed  = 0.002 ;

  SET_LIMS(flims,1.0,1000.0) ;
  float maxTime = pinput("Streamline max time",100.0,flims);
  SET_LIMS(flims,0.01,1.0) ; 
  float IntStepLen = pinput("Streamline integration step length",0.2,flims);
  SET_LIMS(flims,0.00001,0.1) ;
  float StepLen = pinput("Streamline step length",0.0001,flims);

  /* Graphics params */ 

  float Stream_LineWidth = 1.0 ;
  float Ribbon_Width     = 0.025 ;
  float Tube_Radius      = 1.25 ;

  /* The StepLength parameter controls the time increment used to
     generate individual points along the streamline(s). Smaller
     values result in more line primitives but smoother
     streamlines. The StepLength instance variable is defined in
     terms of time (i.e., the distance that the particle travels in
     the specified time period). Thus, the line segments will be
     smaller in areas of low velocity and larger in regions of high
     velocity. 

     IntegrationStepLength, defined by the superclass vtkStreamer,
     is used to control integration step size and is expressed as a
     fraction of the cell length.) */

  /* Set up the integrator */ 

  vtkRungeKutta4 *integ = vtkRungeKutta4::New();
  streamLine->SetIntegrator(integ) ;
  streamLine->SetMaximumPropagationTime(maxTime) ;
  streamLine->SetNumberOfThreads(1) ; 
  streamLine->SetStepLength(StepLen) ; 
  streamLine->SetIntegrationStepLength(IntStepLen) ; 
  streamLine->SetIntegrationDirectionToIntegrateBothDirections() ; 
  streamLine->SetTerminalSpeed(TermSpeed) ;

  if(DO_VORTICITY) {
    streamLine->SpeedScalarsOff() ;
    streamLine->OrientationScalarsOn() ;
    streamLine->VorticityOn() ;
  } else {
    streamLine->SpeedScalarsOn() ;
    streamLine->OrientationScalarsOff() ;
    streamLine->VorticityOff() ;
  }

  /* Load up the data ... */

  streamLine->SetInput(vdata);	

  /* Set source for appropriate probing widget */ 

  if(DO_LINE_WIDGET) {
      streamLine->SetSource(seeds) ;
  }

  streamLine->Print(cout) ;

  if(DO_STUBE) {
    printf("\nStreamtubing...\n");

    streamTube->SetRadius(Tube_Radius) ;
    streamTube->SetNumberOfSides(12) ;
    streamTube->UseDefaultNormalOn() ;
      
    // Source
    // I don't think this cleaning does anything!
    vtkCleanPolyData *cleanStream = vtkCleanPolyData::New();
    cleanStream->SetInput(streamLine->GetOutput()) ;
    streamTube->SetInput(cleanStream->GetOutput()) ;
    //streamTube->SetInput(streamLine->GetOutput()) ;

    // Mapper
    streamTubeMapper->SetInput(streamTube->GetOutput()) ;
    // Actor
    streamTubeActor->SetMapper(streamTubeMapper);
    // streamTubeActor->GetProperty()->BackfaceCullingOn();

    renv->AddActor(streamTubeActor);
    printf("finished streamtubing...\n");fflush(stdout) ;

  } else if(DO_RIBBON) {
    printf("\nStreamribboning...\n");

    streamRibbon->SetInput(streamLine->GetOutput());
    streamRibbon->SetWidth(Ribbon_Width);
    //streamRibbon->SetWidthFactor(1);
    streamLineMapper->SetInput(streamRibbon->GetOutput());
    streamLineMapper->SetScalarRange(vdata->GetScalarRange()); 
    streamLineActor->SetMapper(streamLineMapper);
    renv->AddActor(streamLineActor);

  } else {

    printf("\nStreamlining...\n");
    //streamLine->Print(cout) ;

    streamLineMapper->SetInput(streamLine->GetOutput());
    streamLineMapper->SetScalarRange(vdata->GetScalarRange()); 
    streamLineActor->SetMapper(streamLineMapper);
    streamLineActor->GetProperty()->SetLineWidth(Stream_LineWidth);
    renv->AddActor(streamLineActor);

  }

  if(DO_LINE_WIDGET) {

    fprintf(stderr,"Inside DO_LINE_WIDGET\n");

    //streamLine->SetSource(seeds) ;

    vtkLWCallback *myCallback = vtkLWCallback::New();
    myCallback->PolyData = seeds;

    if(DO_STREAM) myCallback->Actor = streamLineActor;
    if(DO_STUBE)  myCallback->Actor = streamTubeActor;

    lineWidget->SetInteractor(irenv);
    //lineWidget->SetInput(sgrid);
    lineWidget->SetInput(vdata);
    lineWidget->SetAlignToYAxis();
    lineWidget->PlaceWidget();
    lineWidget->GetPolyData(seeds);
    lineWidget->ClampToBoundsOn();
    lineWidget->AddObserver(vtkCommand::InteractionEvent,myCallback);
  }

  /* Render! */ 

  renv->ResetCamera() ;
  renvWin->Render();  

}

/* streamline callback */
void stream_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
  fprintf(stderr,"Inside stream_cb\n");
  DO_STREAM = 1 ;	/* Streamline */  
  DO_STUBE  = 0 ;	/* Streamtube */
  compute_streamlines( w, client_data, call_data) ;

}

/* streamtube callback */
void tube_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
  fprintf(stderr,"Inside tube_cb\n");
  DO_STREAM = 0 ;	/* Streamline */  
  DO_STUBE  = 1 ;	/* Streamtube */
  compute_streamlines( w, client_data, call_data) ;

}

Widget setup_vector_widgets(Widget sh,vtkXOpenGLRenderWindow *rw)
{
  Widget exit_button;
  Widget vfield_button;
  Widget stream_button;
  Widget tube_button;
  Widget button_rowcol;
  Widget toplevel;
  Widget form;
  Widget widg_ret ;
  XmString xstr ;

  /* X Window stuff begin */

  XtSetLanguageProc(NULL,NULL,NULL);

  // get the display connection and give it to the renderer

  rw->SetDisplayId(XtDisplay(sh));

  toplevel = XtVaCreateWidget("Diffusion Vector Visualizer",
			       topLevelShellWidgetClass,sh,
                               XmNdepth, rw->GetDesiredDepth(),
                               XmNvisual, rw->GetDesiredVisual(),
                               XmNcolormap, rw->GetDesiredColormap(),
                               NULL);

  form     = XtVaCreateWidget("form",xmFormWidgetClass, toplevel, NULL);
  widg_ret = XtVaCreateManagedWidget("vtk_vector_widg",
				     xmPrimitiveWidgetClass, form, 
                                     XmNwidth, 300, XmNheight, 300,
                                     XmNleftAttachment, XmATTACH_FORM,
                                     XmNrightAttachment, XmATTACH_FORM,
                                     XmNtopAttachment, XmATTACH_FORM,
                                     NULL);

  // rowcol for buttons

  button_rowcol =  XtVaCreateWidget("AFNI" , xmRowColumnWidgetClass , form ,
				    XmNorientation  , XmHORIZONTAL ,
				    XmNpacking      , XmPACK_TIGHT ,
				    XmNheight, 40,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNtopAttachment, XmATTACH_WIDGET,
				    XmNtopWidget, widg_ret,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_FORM,
				    XmNinitialResourcesPersistent , False ,
				    NULL); 

  // Vector field button

  xstr = XmStringCreateLtoR( "Vector field" , XmFONTLIST_DEFAULT_TAG ) ;
  vfield_button = XtVaCreateManagedWidget("AFNI" , xmPushButtonWidgetClass , 
					button_rowcol ,
					XmNlabelString , xstr ,
					XmNtraversalOn , False ,
					XmNinitialResourcesPersistent , False ,
					NULL ) ;

  XtAddCallback(vfield_button,XmNactivateCallback,vfield_cb,(XtPointer *)toplevel);

  // Streamline button

  xstr = XmStringCreateLtoR( "Streamline" , XmFONTLIST_DEFAULT_TAG ) ;
  stream_button = XtVaCreateManagedWidget("AFNI" , xmPushButtonWidgetClass , 
					button_rowcol ,
					XmNlabelString , xstr ,
					XmNtraversalOn , False ,
					XmNinitialResourcesPersistent , False ,
					NULL ) ;

  XtAddCallback(stream_button,XmNactivateCallback,stream_cb,(XtPointer *)toplevel);

  // Streamtube button

  xstr = XmStringCreateLtoR( "Streamtube" , XmFONTLIST_DEFAULT_TAG ) ;
  tube_button = XtVaCreateManagedWidget("AFNI" , xmPushButtonWidgetClass , 
					button_rowcol ,
					XmNlabelString , xstr ,
					XmNtraversalOn , False ,
					XmNinitialResourcesPersistent , False ,
					NULL ) ;

  XtAddCallback(tube_button,XmNactivateCallback,tube_cb,(XtPointer *)toplevel);

  // Exit button

  xstr = XmStringCreateLtoR( "Exit" , XmFONTLIST_DEFAULT_TAG ) ;
  exit_button = XtVaCreateManagedWidget("AFNI" , xmPushButtonWidgetClass , 
					button_rowcol ,
					XmNlabelString , xstr ,
					XmNtraversalOn , False ,
					XmNinitialResourcesPersistent , False ,
					NULL ) ;


  XtAddCallback(exit_button,XmNactivateCallback,quit_cb,(XtPointer *)toplevel);

  // Management

  XtManageChild(button_rowcol) ;
  XtManageChild(form) ;
  XtRealizeWidget(toplevel) ;
  XtMapWidget(toplevel);

  return (widg_ret) ;
}

Widget setup_widgets(Widget sh,vtkXOpenGLRenderWindow *rw)
{
  Widget button;
  Widget toplevel;
  Widget form;
  Widget widg_ret ;

  /* X Window stuff begin */

  XtSetLanguageProc(NULL,NULL,NULL);

  // get the display connection and give it to the renderer

  rw->SetDisplayId(XtDisplay(sh));

  toplevel = XtVaCreateWidget("Diffusion Visualizer",
			       topLevelShellWidgetClass,sh,
                               XmNdepth, rw->GetDesiredDepth(),
                               XmNvisual, rw->GetDesiredVisual(),
                               XmNcolormap, rw->GetDesiredColormap(),
                               NULL);

  form     = XtVaCreateWidget("form",xmFormWidgetClass, toplevel, NULL);
  widg_ret = XtVaCreateManagedWidget("vtk_widg",
				     xmPrimitiveWidgetClass, form, 
                                     XmNwidth, 300, XmNheight, 300,
                                     XmNleftAttachment, XmATTACH_FORM,
                                     XmNrightAttachment, XmATTACH_FORM,
                                     XmNtopAttachment, XmATTACH_FORM,
                                     NULL);
  button    = XtVaCreateManagedWidget("Exit",
				      xmPushButtonWidgetClass, form,
                                      XmNheight, 40,
                                      XmNbottomAttachment, XmATTACH_FORM,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, widg_ret,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNrightAttachment, XmATTACH_FORM,
				      XmNtraversalOn, False ,
				      XmNrecomputeSize, False ,
				      XmNinitialResourcesPersistent , False ,
                                      NULL);
  XtAddCallback(button,XmNactivateCallback,quit_cb,(XtPointer *)toplevel);
  XtManageChild(form);
  XtRealizeWidget(toplevel);
  XtMapWidget(toplevel);

  return (widg_ret) ;
}
