/*=======================================================================
  Program:   Launch VTK window
  Language:  C++
  Date:      $Date: 2004/02/19 16:45:06 $
  Version:   $Revision: 1.1.1.1 $
  Author:    Cyrus Jam
=========================================================================*/

//
// Work in Progress 
//

#include "vtk_window.h"

// First include the required header files for the VTK classes we are using.
#include "vtkConeSource.h"
#include "vtkPolyDataMapper.h"
#include "vtkRenderWindow.h"
#include "vtkCamera.h"
#include "vtkActor.h"
#include "vtkRenderer.h"

#include <math.h>

/* Includes for Motif */
#include "vtkXOpenGLRenderWindow.h"
#include "vtkXRenderWindowInteractor.h"

#include <Xm/PushB.h>
#include <Xm/Form.h>

/* Callbacks */
void quit_cb(Widget,XtPointer,XtPointer);

/* Widget setup */
void setup_widgets(Widget,vtkXOpenGLRenderWindow*);

/* X window stuff */
Widget vtk_widg;		/* Would be better to include as arg
				   to setup_widgets() but couldn't get
				   it to work. */

void Launch_vtk_window( Widget shell )
{

  vtkXOpenGLRenderWindow *renWin = vtkXOpenGLRenderWindow::New();
  vtkRenderer *ren1 = vtkRenderer::New();

  setup_widgets(shell,renWin);
  
  /* X Window stuff end */

  fprintf( stderr, ":) Launch vtk window\n" );	

  vtkConeSource *cone = vtkConeSource::New();
  cone->SetHeight( 3.0 );
  cone->SetRadius( 1.0 );
  cone->SetResolution( 10 );
  
  //
  // In this example we terminate the pipeline with a mapper process object.
  // (Intermediate filters such as vtkShrinkPolyData could be inserted in
  // between the source and the mapper.)  We create an instance of
  // vtkPolyDataMapper to map the polygonal data into graphics primitives. We
  // connect the output of the cone souece to the input of this mapper.
  //
  vtkPolyDataMapper *coneMapper = vtkPolyDataMapper::New();
  coneMapper->SetInput( cone->GetOutput() );

  //
  // Create an actor to represent the cone. The actor orchestrates rendering
  // of the mapper's graphics primitives. An actor also refers to properties
  // via a vtkProperty instance, and includes an internal transformation
  // matrix. We set this actor's mapper to be coneMapper which we created
  // above.
  //
  vtkActor *coneActor = vtkActor::New();
  coneActor->SetMapper( coneMapper );

  //
  // Create the Renderer and assign actors to it. A renderer is like a
  // viewport. It is part or all of a window on the screen and it is
  // responsible for drawing the actors it has.  We also set the background
  // color here.
  //
  ren1->AddActor( coneActor );
  ren1->SetBackground( 0.1, 0.2, 0.4 );

  //
  // Finally we create the render window which will show up on the screen.
  // We put our renderer into the render window using AddRenderer. We also
  // set the size to be 300 pixels by 300.
  //

  renWin->AddRenderer( ren1 );
  renWin->SetSize( 300, 300 );

  // we typecast to an X specific interactor
  // Since we have decided to make this an X program
  vtkXRenderWindowInteractor *iren = vtkXRenderWindowInteractor::New();
  iren->SetRenderWindow(renWin);
  iren->SetWidget(vtk_widg);

  //
  // Now we loop over 360 degreeees and render the cone each time.
  //
  int i;
  for (i = 0; i < 360; ++i)
    {
    // render the image
    renWin->Render();
    // rotate the active camera by one degree
    ren1->GetActiveCamera()->Azimuth( 1 );
    }

  //
  // Free up any objects we created. All instances in VTK are deleted by
  // using the Delete() method.
  //
  cone->Delete();
  coneMapper->Delete();
  coneActor->Delete();
  ren1->Delete();
  renWin->Delete();
  
  fprintf( stderr, ":( exiting vtk window\n" );	

}

/* quit callback */
void quit_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
   XtUnmapWidget( (Widget)client_data ) ; 
}

void setup_widgets(Widget sh,vtkXOpenGLRenderWindow *rw)
{
  Widget button;
  Widget toplevel2;
  Widget form;

  /* X Window stuff begin */

  XtSetLanguageProc(NULL,NULL,NULL);

  // get the display connection and give it to the renderer

  rw->SetDisplayId(XtDisplay(sh));

  toplevel2 = XtVaCreateWidget("top2",
			       topLevelShellWidgetClass,sh,
                               XmNdepth, rw->GetDesiredDepth(),
                               XmNvisual, rw->GetDesiredVisual(),
                               XmNcolormap, rw->GetDesiredColormap(),
                               NULL);

  form     = XtVaCreateWidget("form",xmFormWidgetClass, toplevel2, NULL);
  vtk_widg = XtVaCreateManagedWidget("vtk_widg",
				     xmPrimitiveWidgetClass, form, 
                                     XmNwidth, 300, XmNheight, 300,
                                     XmNleftAttachment, XmATTACH_FORM,
                                     XmNrightAttachment, XmATTACH_FORM,
                                     XmNtopAttachment, XmATTACH_FORM,
                                     NULL);
  button    = XtVaCreateManagedWidget("Exit",
				      xmPushButtonWidgetClass, form,
                                      XmNheight, 40,
                                      XmNbottomAttachment, XmATTACH_FORM,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, vtk_widg,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNrightAttachment, XmATTACH_FORM,
                                      NULL);
  XtAddCallback(button,XmNactivateCallback,quit_cb,(XtPointer *)toplevel2);
  XtManageChild(form);
  XtRealizeWidget(toplevel2);
  XtMapWidget(toplevel2);
}
