/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkPolyLineCurvature.h,v $
  Language:  C++
  Date:      $Date: 2003/05/13 00:00:00 $
  Version:   $Revision: 1.0 $

  Copyright (c) 1993-2002 Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
// .NAME vtkPolyLineCurvature - compute curvature and torsion of polylines...
// .SECTION Description


#ifndef __vtkPolyLineCurvature_h
#define __vtkPolyLineCurvature_h

#include "vtkPolyDataToPolyDataFilter.h"

#define VTK_CURVATURE 0
#define VTK_TORSION 1

//Note that there is no export handling, so this may not work with Win32
class vtkPolyLineCurvature : public vtkPolyDataToPolyDataFilter
{
public:
  vtkTypeRevisionMacro(vtkPolyLineCurvature,vtkPolyDataToPolyDataFilter);
  void PrintSelf(ostream& os, vtkIndent indent);

  // Description:
  // Constructor
  //  static vtkPolyLineCurvature *New();
  vtkPolyLineCurvature *New();
  
  // Description:
  // Set the scalar output type
  vtkSetClampMacro(ScalarOutput,int,VTK_CURVATURE,VTK_TORSION);
  vtkGetMacro(ScalarOutput,int);
  void SetScalarOutputToCurvature() {this->SetScalarOutput(VTK_CURVATURE);};
  void SetScalarOutputToTorsion() {this->SetScalarOutput(VTK_TORSION);};
  char *GetScalarOutputAsString();

protected:
  vtkPolyLineCurvature();
//  ~vtkPolyLineCurvature();
  
  // Usual data generation method
  void Execute();
  
  int ScalarOutput;


private:
  vtkPolyLineCurvature(const vtkPolyLineCurvature&);  // Not implemented.
  void operator=(const vtkPolyLineCurvature&);  // Not implemented.

};

#endif

inline char *vtkPolyLineCurvature::GetScalarOutputAsString(void)
{
  if ( this->ScalarOutput == VTK_CURVATURE )
    {
    return (char *)"Curvature";
    }
  else
    {
    return (char *)"Torsion";
    }
}

