/*===<vtkPlotting.cxx>===================================================*/

#include "vtk_Plotting.h"

//Handle old versions of VTK
#ifndef vtkFloatingPointType
#define vtkFloatingPointType vtkFloatingPointType
typedef float vtkFloatingPointType;
#endif

#define VTK_RWI_MESSAGE \
{printf("\nActivating rendering window interactor ...\n" \
	"(Hit 'q' in window to exit)\n");fflush(stdout);}

/* Used for lut creation */ 
/*static int      Ncolors = 256 ;*/
static int      Ncolors = 402 ;
//static vtkFloatingPointType HueRange[] = {0.15,1.0} ;
static vtkFloatingPointType HueRange[] = {0.6667,0.0} ;
static vtkFloatingPointType SatRange[] = {1.0,1.0} ;
static vtkFloatingPointType ValRange[] = {1.0,1.0} ;
static vtkFloatingPointType AlfRange[] = {1.0,1.0} ;

/*===<CreateStandardColorTable.c>=============================================

   Title: CREATE STANDARD COLOR lookup TABLE

 Purpose: Create vtk lut with standard values

  Author: LR Frank

 History: 12Mar04 lrf

*/
vtkLookupTable *CreateStandardColorTable( vtkFloatingPointType *range )
{
  return CreateColorTable( Ncolors, 
			   HueRange,
			   SatRange,
			   ValRange,
			   AlfRange,
			   range ) ;
}
/*===<CreateColorTable.c>======================================================

   Title: CREATE COLOR lookup TABLE

 Purpose: Create vtk lut

  Author: LR Frank

 History: 31Jul03 (Woods Hole) lrf

*/
vtkLookupTable *CreateColorTable( int ncolors, 
				  vtkFloatingPointType *hue_range,
				  vtkFloatingPointType *sat_range,
				  vtkFloatingPointType *val_range,
				  vtkFloatingPointType *alf_range,
				  vtkFloatingPointType *range )
{
  vtkLookupTable *lut = vtkLookupTable::New();

  lut->SetNumberOfColors( ncolors ) ;
  lut->SetHueRange( hue_range ) ;
  lut->SetSaturationRange( sat_range ) ;
  lut->SetValueRange( val_range ) ;
  lut->SetAlphaRange( alf_range ) ;
  lut->SetRange( range ) ;
  lut->Build( ) ;		// No build, no colors ...

  return lut ;
}

/* Classic pain-in-the-ass non-documented "simple" vtk operation
   that took me half a day to figure out! */

#define ClearCubeAxesLabels(cAxes) \
do{ \
     cubeAxes->GetXAxisActor2D()->SetTickVisibility(0); \
     cubeAxes->GetXAxisActor2D()->SetLabelVisibility(0); \
     cubeAxes->GetYAxisActor2D()->SetTickVisibility(0); \
     cubeAxes->GetYAxisActor2D()->SetLabelVisibility(0); \
     cubeAxes->GetZAxisActor2D()->SetTickVisibility(0); \
     cubeAxes->GetZAxisActor2D()->SetLabelVisibility(0); \
} while(0);

/*===<Draw_Cube.c>========================================================

*/
void Draw_Cube( vtkRenderer *the_renderer, vtkFloatingPointType *cube_bounds )
{

  vtkCubeSource *cube = vtkCubeSource::New();
    cube->SetBounds(cube_bounds) ;
  vtkOutlineFilter *cubeFilter = vtkOutlineFilter::New();
    cubeFilter->SetInput(cube->GetOutput());
  vtkPolyDataMapper *cubeMapper = vtkPolyDataMapper::New();
    cubeMapper->SetInput(cubeFilter->GetOutput()) ;

  vtkActor *cubeActor = vtkActor::New();
    cubeActor->SetMapper(cubeMapper) ;
    cubeActor->GetProperty()->SetColor( White ) ;

  the_renderer->AddActor(cubeActor);

  vtkFloatingPointType tcolor[] = {1.,0.,0.} ;
  vtkCubeAxesActor2D *cubeAxes = vtkCubeAxesActor2D::New();
     cubeAxes->SetCamera(the_renderer->GetActiveCamera()) ;
     cubeAxes->GetAxisTitleTextProperty()->SetColor(tcolor);
     ClearCubeAxesLabels(cubeAxes) ;

  the_renderer->AddProp(cubeAxes) ;

}

/*===<Draw_Sphere.c>========================================================

*/
void Draw_Sphere( vtkRenderer *the_renderer, 
		  vtkFloatingPointType radius, int theta_res, int phi_res )
{
  vtkFloatingPointType origin[] = { 0., 0., 0. };

  vtkSphereSource *sphere = vtkSphereSource::New();
    sphere->SetThetaResolution( theta_res );
    sphere->SetPhiResolution( phi_res );
    sphere->SetRadius( radius );
    sphere->SetCenter( origin );

  vtkPolyDataMapper *sphereMapper = vtkPolyDataMapper::New();
    sphereMapper->SetInput(sphere->GetOutput()) ;

  vtkActor *sphereActor = vtkActor::New();
    sphereActor->SetMapper(sphereMapper) ;
    sphereActor->GetProperty()->SetColor( Gray ) ;
    sphereActor->GetProperty()->SetOpacity(0.25);

  the_renderer->AddActor(sphereActor);

}

/*===<xyplot.c>========================================================

     Title: X-Y PLOT using vtk

   Purpose: Plot graph of line

      Call: int xyplot( float *val, float *pos, int numPts, 
                        char *xtitle, char *ytitle, char *ptitle )
      
     Input: val = value (y-value)
            pos = position (x-value)
	    numPts = length of val, pos
	    xtitle = Text of x-axis
	    ytitle = Text of y-axis
	    ptitle = Plot title

    Return: 

   History: 03_01_08 - lrf
*/
/* int xyplot( float *val, float *pos, int numPts, 
   char *xtitle, char *ytitle, char *ptitle ) */

int xyplot( vtkFloatArray *Data,
	    char *xtitle, char *ytitle, char *ptitle )

{
  vtkFloatingPointType xrange[2],yrange[2];
  int numPts = Data->GetNumberOfTuples();
  int Ncomp  = Data->GetNumberOfComponents();
  int Nplots = Ncomp/2;
  int debug = 0;

  vtkRenderer *ren = vtkRenderer::New();
  vtkRenderWindow *renWin = vtkRenderWindow::New();
  renWin->AddRenderer( ren );
  vtkRenderWindowInteractor *iren = vtkRenderWindowInteractor::New();
  iren->SetRenderWindow( renWin );

  vtkXYPlotActor* plotter = vtkXYPlotActor::New();
    plotter->GetPositionCoordinate()->SetValue( 0.05, 0.05, 0);
    plotter->GetPosition2Coordinate()->SetValue( 0.95, 0.95, 0);
    plotter->SetXValuesToValue(); // This is what makes x=x!
    plotter->SetNumberOfXLabels( 6 );
    plotter->SetTitle( ptitle );
    plotter->SetXTitle( xtitle );
    plotter->SetYTitle( ytitle );
    plotter->GetProperty()->SetLineWidth( 2 );
    plotter->SetLabelFormat("%g");
    plotter->GetProperty()->SetColor( Black ); /* Axes color */

  vtkPolyData *data ;
  vtkFloatingPointType xrmin=BIGVAL, xrmax=-BIGVAL ;
  vtkFloatingPointType yrmin=BIGVAL, yrmax=-BIGVAL ;
  vtkFloatingPointType *val = (vtkFloatingPointType *) 
     calloc(numPts, sizeof(vtkFloatingPointType));
  vtkFloatingPointType *pos = (vtkFloatingPointType *) 
     calloc(numPts, sizeof(vtkFloatingPointType));
  vtkFloatingPointType xydata[Ncomp];

  for (int i=0; i<Nplots; i++){
    for (int j=0; j<numPts; j++){
      Data->GetTuple(j,xydata) ;
      pos[j] = xydata[i*2];
      val[j] = xydata[i*2+1];
    }
    data = VectorsToVTKPolyData(pos,val,numPts,xrange,yrange) ;
    if( xrange[0] <  xrmin ) xrmin = xrange[0] ;
    if( yrange[0] <  yrmin ) yrmin = yrange[0] ;
    if( xrange[1] >= xrmax ) xrmax = xrange[1] ;
    if( yrange[1] >= yrmax ) yrmax = yrange[1] ;
    plotter->SetPlotColor( i, ColorTable[i] );         /* Plot color */
    plotter->AddInput(data);
  }

  free(val) ;
  free(pos) ;

  xrange[0] = xrmin ;  xrange[1] = xrmax ;
  yrange[0] = yrmin ;  yrange[1] = yrmax ;

  plotter->SetXRange( xrange[0], xrange[1] );
  plotter->SetYRange( yrange[0], yrange[1] );

  if(debug) {
    printf("\n");
    printf("xyplot:  xrange = (%f,%f), yrange = (%f,%f)\n",
	   xrange[0],xrange[1],yrange[0],yrange[1]);
  }

  /* Axis label */
  vtkTextProperty *tprop = plotter->GetTitleTextProperty();
    tprop->SetColor(Magenta);	               
    tprop->SetFontFamilyToArial();
    plotter->SetAxisTitleTextProperty(tprop);
    plotter->SetTitleTextProperty(tprop);

  /* Axis numbering */
  vtkTextProperty *aprop = plotter->GetAxisLabelTextProperty();
    aprop->SetColor(Black);
    plotter->SetAxisLabelTextProperty(aprop);

  ren->SetBackground( White );
  ren->SetViewport( 0, 0, 1, 1 );
  ren->AddActor2D( plotter );
  renWin->SetSize( 600, 300);

  // Position the actors

  renWin->Render();
  iren->SetEventPosition(200,200);
  iren->SetKeyCode('r');
  iren->InvokeEvent(vtkCommand::CharEvent,NULL);
  ren->ResetCameraClippingRange();
  renWin->Render();
  iren->SetKeyCode('t');
  iren->InvokeEvent(vtkCommand::CharEvent,NULL);
  iren->Start();

  renWin->Delete();
  ren->Delete();

  return 1;
}
/**<plot.c>*******************************************************

     Title: PLOT

   Purpose: Single x-y plot

      Call: void plot(float *tcoord, float *amp, int nn, 
                      char *xlabel, char *ylabel, char *title)

     Input: tcoord[nn] = x-axis
            amp[nn] = y-axis
	    nn = number of points
	    xlabel = label for x-axis
	    ylabel = label for y-axis
	    title = plot title

    Author: Lawrence R. Frank

   History: 23-Aug-03 
   
    Notes: This routine uses the more general "xyplot" that is
           capable of making multiple x-y plots on the same window.
*/

void plot(float *tcoord, float *amp, int nn, 
	  char *xlabel, char *ylabel, char *title)
{
  int Nplots = 1 ;		/* the plot */
  int Ncomp  = 2 * Nplots ;	/* (x,y) for Nplots */

  vtkFloatArray *xydata = vtkFloatArray::New(); 
  xydata->SetNumberOfComponents(Ncomp);
  xydata->SetNumberOfTuples(nn);

  vtkFloatingPointType xy[Ncomp];

  for(int ii=0; ii<nn; ii++) {

    xy[0] = tcoord[ii] ;
    xy[1] = amp[ii] ;

    xydata->SetTuple(ii,xy);
  }

  xyplot(xydata,xlabel,ylabel,title);

  xydata->Delete() ;
}

/*===<ParametricPlot3D.cxx>==================================================*/

int ParametricPlot3D( vtkFloatArray *xyz, float *time, int numPts, 
		      char *xtitle, char *ytitle, char *ztitle, char *ptitle,
		      int DO_SPHERE )
{
  int debug = 0;
  int i;

  vtkCamera             *camera = vtkCamera::New();
  vtkRenderer              *ren = vtkRenderer::New();
  vtkRenderWindow       *renWin = vtkRenderWindow::New();
  renWin->SetSize(300,300);
  vtkRenderWindowInteractor *iren = vtkRenderWindowInteractor::New();
  iren->SetRenderWindow(renWin);

  Boolean replay = YES ;

  /* Find input range */

  vtkFloatArray *inputRange = FloatArrayRange( xyz ) ;
  vtkFloatingPointType y[2];
  vtkFloatingPointType xmin,xmax,ymin,ymax,zmin,zmax ;
  for (int j=0; j<inputRange->GetNumberOfTuples(); j++) {
    inputRange->GetTuple(j,y) ; 
    if(j==0) { xmin = y[0] ; xmax = y[1] ; }
    if(j==1) { ymin = y[0] ; ymax = y[1] ; }
    if(j==2) { zmin = y[0] ; zmax = y[1] ; }
    printf("inputRange[%i] = (%f,%f)\n",j,y[0],y[1]);
  }
  vtkFloatingPointType xran = xmax - xmin ;
  vtkFloatingPointType yran = ymax - ymin ;
  vtkFloatingPointType zran = zmax - zmin ;

  vtkFloatingPointType x[3];

  vtkPoints   *points = vtkPoints::New();
  vtkPolyData *profileData = vtkPolyData::New();

  // Interpolate x, y and z by using the three spline filters and
  // create new points

  for ( i=0; i < numPts; i++ )
    {
      xyz->GetTuple(i,x);
      points->InsertPoint(i,x[0],x[1],x[2]) ;
    }

  // Create the polyline.
  vtkCellArray *lines = vtkCellArray::New();
  lines->InsertNextCell(numPts) ;

  for ( i=0; i < numPts; i++ ) lines->InsertCellPoint(i) ;

  vtkFloatingPointType range[2] ;
  profileData->SetPoints(points) ;
  profileData->SetLines(lines) ;

  // Add thickness to the resulting line.
  vtkTubeFilter *profileTubes = vtkTubeFilter::New();
  profileTubes->SetNumberOfSides(8) ;
  profileTubes->SetInput(profileData) ;
  profileTubes->SetRadius(.005) ;

  vtkPolyDataMapper *profileMapper = vtkPolyDataMapper::New();
  profileMapper->SetInput(profileTubes->GetOutput());

  /* Set up actor */
  static vtkFloatingPointType flims[2] = {.5,2.0} ;
  //double LineWidth = pinput("LineWidth",1.0,flims);
  double LineWidth = 1.0 ;
  vtkFloatingPointType  *LineColor = Black ;

  vtkFloatingPointType origin[] = {xran/2.,yran/2.,zran/2.};
  vtkActor *profile = vtkActor::New();
  profile->SetMapper(profileMapper);
  profile->GetProperty()->SetLineWidth( LineWidth );
  profile->SetOrigin(origin);
  profile->GetProperty()->SetRepresentationToWireframe(); 
  profile->GetProperty()->SetColor( LineColor ) ;

  // set up camera
  /* The camera is what determines the view you seen when the window
     comes up.  If it looks like you're in the "wrong" place, it's 
     the camera SetPosition that needs to be changed. */

  vtkFloatingPointType camscal = 2.0;	// this is approx the same as hitting keyboard "r"
  camera->SetPosition(camscal*xran,camscal*yran,camscal*zran);
  camera->Azimuth(-90.0);
  camera->SetViewUp(0,0,1);

  // set up renderer 

  ren->AddActor(profile);
  ren->SetBackground( White );
  ren->SetActiveCamera(camera) ;
  ren->ResetCameraClippingRange() ;

  // set up render window

  renWin->AddRenderer(ren);

  /* This next call really should be external to ParametricPlot3D
     since it is specific to bloch.c.  I just couldn't figure out
     how to get it to the renderer.  */

  vtkFloatingPointType cube_bounds[6] = {xmin,xmax,ymin,ymax,zmin,zmax} ;
  if(DO_SPHERE) {
    Draw_Sphere( ren , 1.0, 100, 50 ) ;
    //cube_bounds[6] = {-1.0,1.0,-1.0,1.0,-1.0,1.0};
    cube_bounds[0] = -1.0 ; cube_bounds[1] = 1.0 ;
    cube_bounds[2] = -1.0 ; cube_bounds[3] = 1.0 ;
    cube_bounds[4] = -1.0 ; cube_bounds[5] = 1.0 ;
  }

  Draw_Cube( ren , cube_bounds ) ;

  VTK_RWI_MESSAGE ;
  iren->Start();

  // Clean up

  points->Delete();
  profileData->Delete();
  lines->Delete();

  profileTubes->Delete();
  profileMapper->Delete();
  profile->Delete();

  ren->Delete();
  renWin->Delete();
  iren->Delete();
  
  return 1;
}
