/*=======================================================================
  Program:   Map signal to gradient directions using VTK 
  Language:  C++
  Date:      $Date: 2004/03/03 02:48:57 $
  Version:   $Revision: 1.3 $
  Author:    Lawrence Frank
=========================================================================*/

//
// Work in Progress 
//

#include "vtk_mapsignal.h"

// First include the required header files for the VTK classes we are using.

#include "vtkPolyDataMapper.h"
#include "vtkRenderWindow.h"
#include "vtkCamera.h"
#include "vtkActor.h"
#include "vtkRenderer.h"

//Handle old versions of VTK
#ifndef vtkFloatingPointType
#define vtkFloatingPointType vtkFloatingPointType
typedef float vtkFloatingPointType;
#endif

#include <math.h>

/*===<vtk_LoadSignal.c>=====================================================

  Title: LOAD SIGNAL into vtk struct

  Purpose: 

*/
vtkFloatArray *vtk_LoadSignal(float *data, int nvert)
{
  vtkFloatArray *VertexSignal = vtkFloatArray::New();

  VertexSignal->SetNumberOfComponents(nvert);
  VertexSignal->SetNumberOfTuples(1);
  Clear_FloatArray( VertexSignal ) ;

  VertexSignal->SetTuple(0,data) ;

  return(VertexSignal) ;
}

/*===<vtk_LoadGradients.c>=====================================================

  Title: LOAD GRADIENTS into vtk struct

  Purpose: 

*/
vtkPolyData *vtk_LoadGradients(float *gx, float *gy, float *gz,
			       int *tris, int ntris, int nvert) 
{
  int   debug = 0 ;

  vtkFloatingPointType verteces[nvert][3];

  /******** Load gradients *******/

  vtkPoints     *vertexStartPoints = vtkPoints::New();
  vtkFloatArray *Grad = vtkFloatArray::New();

  Grad->SetNumberOfComponents(3);
  Grad->SetNumberOfTuples(nvert);

  for (int i=0; i<nvert; i++) {
    if(debug) printf("g[%i]=(%f,%f,%f)\n",i,gx[i],gy[i],gz[i]) ;
    LOAD_VEC3(verteces[i],gx[i],gy[i],gz[i]) ;
    Grad->SetTuple(i,verteces[i]);
    vertexStartPoints->InsertNextPoint(verteces[i]);
  }

  if(debug) Print_FloatArray( Grad , "Grad",1) ;

  /******** Triangles *******/

  vtkPolyData *vertexGrid = vtkPolyData::New() ;
  vtkCellArray *triangles = vtkCellArray::New() ;

  for( int i = 0; i < ntris; i++ )
  {
    triangles->InsertNextCell(3);
    triangles->InsertCellPoint(tris[i*3+0]);
    triangles->InsertCellPoint(tris[i*3+1]);
    triangles->InsertCellPoint(tris[i*3+2]);
  }
  vertexGrid->SetPoints(vertexStartPoints);
  vertexGrid->SetPolys(triangles);

  /******** Initialize *******/

  vtkFloatArray *vScalars = vtkFloatArray::New();
  vScalars->SetNumberOfTuples(nvert);

  Fill_FloatArray( vScalars, 1.0, -1 ) ;
  vertexGrid->GetPointData()->SetScalars(vScalars) ;

  /* Clean up */

  Grad->Delete();
  triangles->Delete();
  vertexStartPoints->Delete();
  vScalars->Delete();

  return(vertexGrid) ;
}

/*===<MapSignal2VertexGrid.c>==================================================

  Title: MAP SIGNAL TO VERTEX GRID
  Purpose: map signal at verteces onto structure for viewing

*/
vtkPolyData *MapSignal2VertexGrid(vtkPolyData *vGrid , 
				  vtkFloatArray *vSignal) 
{
  int   nvert = vGrid->GetNumberOfPoints();
  vtkFloatingPointType vsig[nvert], verteces[nvert][3];
  vtkFloatingPointType vmax = -BIGVAL, vmin = BIGVAL;
  int   debug = 0 ;

  if(debug) vGrid->Print(cout);

  vSignal->GetTuple(0,vsig);
  for(int idx = 0; idx < nvert; ++idx) {
    if(vsig[idx]>vmax) vmax = vsig[idx];
    if(vsig[idx]<vmin) vmin = vsig[idx];
  }

  if(debug) {printf("nvert = %i\n",nvert); PRINT_FV(vsig,nvert,"vsig") ; }

  vtkFloatArray *vScalars = vtkFloatArray::New();
  vScalars->SetNumberOfTuples(nvert);

  vtkPoints *vNewPoints = vtkPoints::New();
  vNewPoints->SetNumberOfPoints(nvert) ;

  vtkFloatingPointType ftmp[1];

  vtkPolyData *vNewGrid = vtkPolyData::New() ;
  vNewGrid->CopyStructure(vGrid);

  for(int idx = 0; idx < nvert; ++idx) {

    vtkFloatingPointType mul = vsig[idx]/vmax;//((vsig[idx]-vmin)/(vmax-vmin));
    ftmp[0] = mul;//vsig[idx]/vmax ;

    vScalars->SetTuple(idx,ftmp);
    vGrid->GetPoint(idx,verteces[idx]);

    verteces[idx][0] *= mul;
    verteces[idx][1] *= mul;
    verteces[idx][2] *= mul;

    if(debug) {
      printf("verteces[%i]={%f,%f,%f}\n",idx,
	     verteces[idx][0],
	     verteces[idx][1],
	     verteces[idx][2]);}

    //vNewPoints->InsertNextPoint(verteces[idx]);
    vNewPoints->SetPoint(idx,verteces[idx]);

  }
  if(debug) printf("\n");
  vNewGrid->GetPointData()->SetScalars(vScalars) ;
  vNewGrid->SetPoints(vNewPoints);

  vNewPoints->Delete();
  vScalars->Delete();

  return(vNewGrid) ;
}
