#ifndef VTK_WINDOW_H
#define VTK_WINDOW_H

/***
*  Author: Cyrus Jam
* 
*  Generic VTK Wrapper to Display data from AFNI
*
*  WORK IN PROGRESS
*
*  Data: 9/11/03
*
*/

#ifndef VTK_include
#define VTK_include
#include "vtk_all.h"
#include "vtkParametricFunctionSource.h"
#endif

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <sys/types.h>
#include <string.h>

#include <X11/Intrinsic.h> /* for Boolean */

// First include the required header files for the VTK classes we are using.
#include "vtkDataSetMapper.h"
#include "vtkPolyDataMapper.h"
#include "vtkRenderWindow.h"
#include "vtkCamera.h"
#include "vtkActor.h"
#include "vtkRenderer.h"
#include "vtkAxesActor.h"
#include "vtkOrientationMarkerWidget.h"
#include "vtkParametricEllipsoid.h"
#include "vtkParametricFunctionSource.h"

/* Includes for Motif */
#include "vtkXOpenGLRenderWindow.h"
#include "vtkXRenderWindowInteractor.h"

#include <Xm/PushB.h>
#include <Xm/Form.h>

#include "afni.h"

#define VTK_RWI_MESSAGE \
{printf("\nActivating rendering window interactor ...\n" \
	"(Hit 'q' in window to exit)\n");fflush(stdout);}

#include "vtk_Utils.h"
#include "vtk_Plotting.h"

//Handle old versions of VTK
#ifndef vtkFloatingPointType
#define vtkFloatingPointType vtkFloatingPointType
typedef float vtkFloatingPointType;
#endif

#include "../utils/io.h"
#include "../diffuse/diffuse.h"
#include "vtk_mapsignal.h"

#include "../ftdst/dst_interface.h"
#include <string>
#include <iostream>
#include <fstream>
#include <cerrno>
#include <sstream>
#include <stack>
using namespace std;

const int diffusion_steps = 100; // maybe make user-selectable
const int animation_frames = 10; // maybe make user-selectable

void Launch_vtk_window( Widget shell) ;
void Send_xhair_to_vtk_window( int voxel_ijk, THD_3dim_dataset *dwi,
  THD_3dim_dataset *shd, THD_3dim_dataset *dt2, DIF_state *difState,
  vtkPolyData *grid );

void Launch_vtk_vector_window( Widget shell, THD_3dim_dataset *dset) ;
//void Send_Data_to_vtk_vector_window( vtkPolyData *data ) ;
void send_xhair_to_vtk_vector_window(float x, float y, float z);
void Send_VData_to_vtk_vector_window( vtkStructuredPoints *data ) ;
void Send_IData_to_vtk_vector_window( vtkStructuredPoints *data, 
  THD_3dim_dataset *anat ) ;
void DisplayVectorField(vtkPolyData *data); 

void send_3dview_to_vtk_vector_window( Three_D_View *threeDview, 
  THD_3dim_dataset *dwi, DIF_state *difState );

extern float DIFF_evaluate( MCW_arrowval * ) ;

struct UIState {
public:
  float xhair_x, xhair_y, xhair_z;
  bool drawSlice[3];
  int sliceNum[3];
  int maxSliceNum[3];

  Widget sliceEnable[3];
  //      Widget posWidget[3];
  Widget sliders[3];

  bool loaded;
  int frame;
  int specular;
  int ambient;
  int diffuse;
  int opacity;
  int num_contours;
  int min_contour;
  int max_contour;
  Widget frame_slider;
  Widget specular_slider;
  Widget ambient_slider;
  Widget diffuse_slider;
  Widget num_contours_slider;
  Widget min_contour_slider;
  Widget max_contour_slider;
  Widget opacity_slider;
  Widget left_animation_frame, right_animation_frame;
  char sliderText[3][10];
  Widget posText[3];

  bool vFieldEnabled;
               // for contour control panel
  bool contourEnabled;
  bool isosurfEnabled;
  Widget isoSurface_value_slider;
  Widget isoSurface_opacity_slider;
  char isoSurface_value_sliderText[20], isoSurface_opacity_sliderText[20];
  float isoSurface_value, isoSurface_opacity;
  float isosurf_value_max, isosurf_value_min;
  static const int ISOSURF_VALUE_DECIMAL_PTS = 2;
  static const int ISOSURF_OPACITY_DECIMAL_PTS = 2;

  int numContours;
  static const int MIN_NUM_CONTOURS = 1;
  static const int MAX_NUM_CONTOURS = 20;
  static const int DEFAULT_NUM_CONTOURS = 3;
  vtkFloatingPointType lowContourLevel;
  vtkFloatingPointType highContourLevel;
  int contourDecimalPoints;
  int contourIncrement;
  vtkFloatingPointType minContourLevel;
  vtkFloatingPointType maxContourLevel;

  Widget numContoursText;
  Widget minContourText;
  Widget maxContourText;

                   // for DST control panel
  bool diffsimFT_Enabled;
  float dstFA, dstDistRatio, dstCurv, dstCoher, dstIntegTime; 
  Widget dst_b0_slider[2]; 
  char dst_b0_sliderText[2][20];
  int dst_b0_bound[2];
               // for 4th-DT
  float dst_dt4_GFA, dst_dt4_distRatio, dst_dt4_transAngle, 
    dst_dt4_integTime;

  static const int DST_THRESHOLD_DECIMAL_POINTS = 2;
  static const int DST_THRESHOLD_MIN = 0;
  static const int DST_THRESHOLD_MAX = 100;
  static const int DST_THRESHOLD_MAX_ANGLE = 90;
  static const int DST_THRESHOLD_INCR = 5;
  static const int DST_THRESHOLD_INCR_ANGLE = 1;
  static const int DST_THRESHOLD_DEFAULT_FA = 40;
  static const int DST_THRESHOLD_DEFAULT = 90;
  static const int DST_THRESHOLD_DEFAULT_ANGLE = 30;
  static const int DST_THRESHOLD_DECIMAL_POINTS_TIME = 4;
  static const int DST_THRESHOLD_MIN_TIME = 1;
  static const int DST_THRESHOLD_MAX_TIME = 10;
  static const int DST_THRESHOLD_INCR_TIME = 1;
  static const int DST_THRESHOLD_DEFAULT_TIME = 10;
  static const int DST_VOXEL_SIZE_MIN = 1;
  static const int DST_VOXEL_SIZE_MAX = 25;
  static const int DST_VOXEL_SIZE_DEFAULT = 15;
  static const int DST_VOXEL_SIZE_DECIMAL_POINTS = 1;
  bool dst_drawStreamtube;
  bool dst_isotropicVoxel;
  float dst_voxelSize;
  vtkFloatingPointType dst_tubeRadius;

  bool streamEnabled;
  bool drawStreamtube;
  vtkFloatingPointType tubeRadius;
  static const int RADIUS_DECIMAL_POINTS = 1;
  static const int MIN_TUBE_RADIUS = 1;
  static const int MAX_TUBE_RADIUS = 10;
  static const int DEFAULT_TUBE_RADIUS = 2;
  bool colorByCurvature;

  enum probeType_t
    {
      POINT,
      LINE,
      PLANE
    };
  probeType_t probeType;

  enum trackType_t {
    CONTOUR,
    CLOUD
  };
  trackType_t trackType;

  static const int MAX_PROBE_RES = 512;
  static const int DEFAULT_PROBE_RES = 10;
  int probeResolution;

  static const vtkFloatingPointType STREAM_MAX_TIME;
  static const vtkFloatingPointType VIS_STEP_TIME;
  vtkFloatingPointType streamStepLength;
  vtkFloatingPointType streamFA;
  static const int STEPLENGTH_DECIMAL_POINTS = 2;
  static const int MIN_STEPLENGTH = 1;
  static const int MAX_STEPLENGTH = 100;
  static const int DEFAULT_STEPLENGTH = 5;

  UIState():
    xhair_x(0.0),
    xhair_y(0.0),
    xhair_z(0.0),
    vFieldEnabled(false),
    isosurfEnabled(false),
    contourEnabled(false),
    numContours(DEFAULT_NUM_CONTOURS),
    lowContourLevel(0.0),
    highContourLevel(0.0),
    contourDecimalPoints(0),
    contourIncrement(1),
    minContourLevel(0.0),
    maxContourLevel(0.0),
    numContoursText(NULL),
    minContourText(NULL),
    maxContourText(NULL),
    diffsimFT_Enabled(false),
    dstFA( DST_THRESHOLD_DEFAULT_FA / pow(10,DST_THRESHOLD_DECIMAL_POINTS) ),
    dstDistRatio( DST_THRESHOLD_DEFAULT / 
      pow(10,DST_THRESHOLD_DECIMAL_POINTS) ),
    dstCurv( DST_THRESHOLD_DEFAULT_ANGLE ),
    dstCoher( DST_THRESHOLD_DEFAULT_ANGLE ),
    dstIntegTime( DST_THRESHOLD_DEFAULT_TIME / 
      pow(10,DST_THRESHOLD_DECIMAL_POINTS_TIME) ),
    dst_drawStreamtube(false),
    dst_isotropicVoxel(false),
    dst_voxelSize( DST_VOXEL_SIZE_DEFAULT/
      pow(10,DST_VOXEL_SIZE_DECIMAL_POINTS) ),
    dst_tubeRadius(DEFAULT_TUBE_RADIUS/pow(10, RADIUS_DECIMAL_POINTS)),
    dst_dt4_GFA( DST_THRESHOLD_DEFAULT_FA / 
      pow(10,DST_THRESHOLD_DECIMAL_POINTS) ),
    dst_dt4_distRatio( DST_THRESHOLD_DEFAULT / 
      pow(10,DST_THRESHOLD_DECIMAL_POINTS) ),
    dst_dt4_transAngle( DST_THRESHOLD_DEFAULT_ANGLE ),  
    dst_dt4_integTime( DST_THRESHOLD_DEFAULT_TIME / 
      pow(10,DST_THRESHOLD_DECIMAL_POINTS_TIME) ),
    streamEnabled(false),
    drawStreamtube(false),
    tubeRadius(DEFAULT_TUBE_RADIUS/pow(10, RADIUS_DECIMAL_POINTS)),
    colorByCurvature(false),
    probeType(POINT),
    trackType(CONTOUR),
    loaded(false),
    frame(1),
    frame_slider(NULL),
    specular_slider(NULL),
    ambient_slider(NULL),
    diffuse_slider(NULL),
    num_contours_slider(NULL),
    min_contour_slider(NULL),
    max_contour_slider(NULL),
    opacity_slider(NULL),
    left_animation_frame(NULL),
    right_animation_frame(NULL),
    specular(20),
    ambient(80),
    diffuse(30),
    opacity(20),
    num_contours(3),
    min_contour(4),
    max_contour(128),
    probeResolution(DEFAULT_PROBE_RES),
    streamFA(DST_THRESHOLD_DEFAULT_FA/pow(10,DST_THRESHOLD_DECIMAL_POINTS)),
    streamStepLength(DEFAULT_STEPLENGTH/pow(10, STEPLENGTH_DECIMAL_POINTS))
  {
    for (int i = 0; i < 3; ++i)
      {
	drawSlice[i] = false;
	sliceNum[i] = 0;
	maxSliceNum[i] = 0;
	sliceEnable[i] = NULL;
	//	    posWidget[i] = NULL;
	sliders[i] = NULL;
	sprintf(sliderText[i], "0");
	posText[i] = NULL;
      }
  };

};

#endif
