/*=======================================================================
  Program:   Launch VTK window
  Language:  C++
  Date:      $Date: 2004/03/03 02:48:57 $
  Version:   $Revision: 1.5 $
  Author:    Cyrus Jam
=========================================================================*/

//
// Work in Progress 
//

#define VTK_RWI_MESSAGE \
{printf("\nActivating rendering window interactor ...\n" \
	"(Hit 'q' in window to exit)\n");fflush(stdout);}

#include "vtk_window.h"

/* Global ... */
vtkXRenderWindowInteractor *iren,*irenv ;
vtkXOpenGLRenderWindow     *renWin,*renvWin ;
vtkRenderer                *ren1,*renv ;
vtkPolyDataMapper          *dataMapper,*vgridMapper ;
vtkActor                   *dataActor,*vgridActor ;
vtkOutlineFilter           *outline ;

/* Global vector data for different modes ... */ 
vtkStructuredGrid *vgrid ;
vtkStructuredGrid *igrid ;

/* Processing switches */ 
Boolean DO_PROBES       = 0 ;	/* Probes */
Boolean DO_PLANES       = 0 ;	/* Planes */
Boolean DO_CONTOUR      = 0 ;	/* Contour */
Boolean DO_STREAM       = 0 ;	/* Streamline */
Boolean DO_STUBE        = 0 ;	/* Streamtube */
Boolean DO_VORTICITY    = 0 ;	/* Vorticity calc for streamline */ 
Boolean DO_RIBBON       = 0 ;	/* Ribbon filter */
Boolean DO_LINE_WIDGET  = 0;	/* Line widget   */ 
Boolean DO_PLANE_WIDGET = 0;	/* Plane widget   */ 

/* Widget setup */
Widget setup_widgets(Widget,vtkXOpenGLRenderWindow*);
Widget setup_vector_widgets(Widget,vtkXOpenGLRenderWindow*);

static MCW_arrowval *fai_thresh_av ;

static float flims[2] ;

/** Callbacks ************************************************************/

/* Line widget */

class vtkLWCallback : public vtkCommand
{
public:
  static vtkLWCallback *New() 
    { return new vtkLWCallback; }
  virtual void Execute(vtkObject *caller, unsigned long, void*)
    {
      vtkLineWidget *lineWidget = reinterpret_cast<vtkLineWidget*>(caller);
      lineWidget->GetPolyData(this->PolyData);
      this->Actor->VisibilityOn();
    }
  vtkLWCallback():PolyData(0),Actor(0) {}
  vtkPolyData *PolyData;
  vtkActor *Actor;
};

/* Plane widget */

class vtkPWCallback : public vtkCommand
{
public:
  static vtkPWCallback *New() 
    { return new vtkPWCallback; }
  virtual void Execute(vtkObject *caller, unsigned long, void*)
    {
      vtkPlaneWidget *planeWidget = reinterpret_cast<vtkPlaneWidget*>(caller);
      planeWidget->GetPolyData(this->PolyData);
      this->Actor->VisibilityOn();
    }
  vtkPWCallback():PolyData(0),Actor(0) {}
  vtkPolyData *PolyData;
  vtkActor *Actor;
};

/* Plane source */

class vtkPSCallback : public vtkCommand
{
public:
  static vtkPSCallback *New() 
    { return new vtkPSCallback; }
  virtual void Execute(vtkObject *caller, unsigned long, void*)
    {
      vtkPlaneSource *planeSource = reinterpret_cast<vtkPlaneSource*>(caller);
      //planeSource->GetPolyData(this->PolyData);
      this->Actor->VisibilityOn();
    }
  vtkPSCallback():PolyData(0),Actor(0) {}
  vtkPolyData *PolyData;
  vtkActor *Actor;
};

/** end callbacks ************************************************************/

/*===<DisplayVectorField.c>==================================================

  Title: DISPLAY VECTOR FIELD
  Purpose: 

*/
void DisplayVectorField(void)
{
  char *program_name = "DisplayVectorField";
  Boolean FIX_ARROW_LENGTHS = 0;
  double Arrow_Scale_Factor = 1.0 ; // Why???
  float range[2];

  int debug  = 1 ;
  int debug2 = 0 ;

  if(vgrid==NULL) {
    fprintf(stderr,"%s: vgrid is NULL!\n",program_name);
    return ;
  }

  if(debug2) {
    fprintf(stderr,"Inside %s\n",program_name);
    vgrid->Print(cout) ;
  }

  // determine scalar range

  vgrid->GetScalarRange(range);
  if(debug) fprintf(stderr,"%s: range = (%f,%f)\n",program_name,range[0],range[1]) ;

  /***************** Threshold data below fai threshold *****************/

  vtkThreshold *thresher = vtkThreshold::New();
  if(debug) fprintf(stderr,"Inside %s: Setting data clipping range to (%f,%f)\n",
		    program_name,fai_thresh_av->fval,range[1]);
  thresher->ThresholdByUpper(fai_thresh_av->fval);
  thresher->SetInput(vgrid);

  /* vtkDataSetMapper *clipMapper = vtkDataSetMapper::New();
     clipMapper->SetInput(clipper->GetOutput());
     clipMapper->ScalarVisibilityOff();  */

  /***************** Construct arrows *****************/

  /* source data = vtkGlyphSource2D */

  vtkGlyphSource2D *arrow = vtkGlyphSource2D::New();
  arrow->SetGlyphTypeToArrow();  
  arrow->SetScale(.5);
  arrow->FilledOff();

  /* actual arrows = vtkGlyph3D */

  vtkGlyph3D *glyph = vtkGlyph3D::New();
  glyph->SetSource(arrow->GetOutput());

  /* scaling mystery */

  glyph->SetScaleModeToScaleByVector();
  glyph->SetScaleMode(VTK_SCALE_BY_VECTOR);
  if(FIX_ARROW_LENGTHS) {
    glyph->ClampingOn();
    Arrow_Scale_Factor *= 200. ; // this is 1/.005
  }
  printf("Arrow_Scale_Factor = %f\n",Arrow_Scale_Factor);
  glyph->SetScaleFactor(Arrow_Scale_Factor);
  glyph->SetColorModeToColorByScalar();	/* use reflectivity scalars for color */
  glyph->SetVectorModeToUseVector();
  //glyph->SetInput(vgrid);
  glyph->SetInput(thresher->GetOutput());
  glyph->OrientOn();
  glyph->Modified() ;

  /* Done creating arrows, add them ... */
  vgridMapper->SetInput(glyph->GetOutput());

  /***************** end arrow construction *****************/

  /*** Colormap ***/ 

  range[0] = fai_thresh_av->fval;		// Careful you don't use range after this!
  vtkLookupTable *lut = CreateStandardColorTable( range ) ;

  vgridMapper->SetLookupTable(lut); /* Done creating lookup table, add it ... */

  /*** Outline data ***/ 

  if(outline==NULL) outline = vtkOutlineFilter::New();

  outline->SetInput(vgrid);
  vtkPolyDataMapper *outlineMapper = vtkPolyDataMapper::New();
  outlineMapper->SetInput(outline->GetOutput());
  vtkActor *outlineActor = vtkActor::New();
  outlineActor->SetMapper(outlineMapper);
  outlineActor->GetProperty()->SetColor(0,0,0);

  /* Done outlining, add it ... */

  renv->AddActor(outlineActor); 

  /*** end outline ***/ 

  /* actor, renderer ...*/ 
  vgridActor->SetMapper(vgridMapper) ;
  renv->AddActor(vgridActor) ;
  renv->ResetCamera() ;
  renvWin->Render();  
}

void Launch_vtk_vector_window(Widget shell)
{

  renvWin = vtkXOpenGLRenderWindow::New();
  renv = vtkRenderer::New();

  Widget vtk_vector_widg = setup_vector_widgets(shell,renvWin);
  
  /* X Window stuff end */

  fprintf( stderr, ":) Launch vtk vector window\n" );	

  vgridMapper = vtkPolyDataMapper::New(); // data mapper
  vgridActor = vtkActor::New();	// actor
  vgridActor->SetMapper(vgridMapper);

  renv->SetBackground( 0.1, 0.2, 0.4 );	// renderer
  renvWin->AddRenderer( renv );	// render win 
  renvWin->SetSize( 300, 300 );

  // x win
  //vtkXRenderWindowInteractor *irenv = vtkXRenderWindowInteractor::New();
  irenv = vtkXRenderWindowInteractor::New();
  irenv->SetRenderWindow(renvWin);
  irenv->SetWidget(vtk_vector_widg);

  renvWin->Render();
  irenv->Initialize(XtWidgetToApplicationContext(vtk_vector_widg)) ;
  
  fprintf( stderr, ":( exiting vtk window\n" );	

}

// send vector data to vector window

void Send_VData_to_vtk_vector_window( vtkStructuredGrid *data )
{
  char *program_name = "Send_VData_to_vtk_vector_window" ;
  int debug = 0 ;

  if(debug) {
    fprintf(stderr,"Inside %s\n",program_name);
    data->Print(cout) ;
  }

  vgrid = vtkStructuredGrid::New();
  vgrid->ShallowCopy(data) ;

}

// send image data to vector window

void Send_IData_to_vtk_vector_window( vtkStructuredGrid *data )
{
  char *program_name = "Send_IData_to_vtk_vector_window" ;
  int debug = 0 ;

  if(debug) {
    fprintf(stderr,"Inside %s\n",program_name);
    data->Print(cout) ;
  }

  igrid = vtkStructuredGrid::New();
  igrid->ShallowCopy(data) ;

}

void Send_Data_to_vtk_window( vtkPolyData *data )
{
  printf("Inside Send_Data_to_vtk_window:\n"); fflush(stdout);
  dataMapper->SetInput( data );
  dataActor->SetMapper( dataMapper );
  ren1->AddActor( dataActor );
  ren1->ResetCamera() ;
  renWin->Render();  
}

void Launch_vtk_window(Widget shell)
{

  //vtkXOpenGLRenderWindow *renWin = vtkXOpenGLRenderWindow::New();
  //vtkRenderer *ren1 = vtkRenderer::New();
  renWin = vtkXOpenGLRenderWindow::New();
  ren1 = vtkRenderer::New();

  Widget vtk_widg = setup_widgets(shell,renWin);
  
  /* X Window stuff end */

  fprintf( stderr, ":) Launch vtk window\n" );	
  
  //
  // In this example we terminate the pipeline with a mapper process object.
  // (Intermediate filters such as vtkShrinkPolyData could be inserted in
  // between the source and the mapper.)  We create an instance of
  // vtkPolyDataMapper to map the polygonal data into graphics primitives. We
  // connect the output of the cone souece to the input of this mapper.
  //
  //vtkPolyDataMapper *dataMapper = vtkPolyDataMapper::New();
  dataMapper = vtkPolyDataMapper::New();
  //dataMapper->SetInput( data );

  //
  // Create an actor to represent the data. The actor orchestrates rendering
  // of the mapper's graphics primitives. An actor also refers to properties
  // via a vtkProperty instance, and includes an internal transformation
  // matrix. We set this actor's mapper to be dataMapper which we created
  // above.
  //
  //vtkActor *dataActor = vtkActor::New();
  dataActor = vtkActor::New();
  //dataActor->SetMapper( dataMapper );

  //
  // Create the Renderer and assign actors to it. A renderer is like a
  // viewport. It is part or all of a window on the screen and it is
  // responsible for drawing the actors it has.  We also set the background
  // color here.
  //
  //ren1->AddActor( dataActor );
  ren1->SetBackground( 0.1, 0.2, 0.4 );

  //
  // Finally we create the render window which will show up on the screen.
  // We put our renderer into the render window using AddRenderer. We also
  // set the size to be 300 pixels by 300.
  //

  renWin->AddRenderer( ren1 );
  renWin->SetSize( 300, 300 );

  // we typecast to an X specific interactor
  // Since we have decided to make this an X program
  //vtkXRenderWindowInteractor *iren = vtkXRenderWindowInteractor::New();
  iren = vtkXRenderWindowInteractor::New();
  iren->SetRenderWindow(renWin);
  iren->SetWidget(vtk_widg);

  renWin->Render();
  iren->Initialize(XtWidgetToApplicationContext(vtk_widg)) ;

  //
  // Free up any objects we created. All instances in VTK are deleted by
  // using the Delete() method.
  //

  /*dataMapper->Delete();
    dataActor->Delete();
    ren1->Delete();
    renWin->Delete(); */
  
  fprintf( stderr, ":( exiting vtk window\n" );	

}

/* quit callback */

void quit_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
  if(vgrid!=NULL) vgrid->Delete() ;
  XtUnmapWidget( (Widget)client_data ) ; 
}

void fai_thresh_av_CB( MCW_arrowval * av , XtPointer cd )
{
  char *program_name = "fai_thresh_av_CB";
  int cc ;

  fprintf(stderr,"Inside %s: fval = %f\n",program_name,av->fval);

  return ;
}

/*-------------------------------------------------------------------------
   fai_thresh_cb
   When the user presses Enter in a textfield that might have an
   expression entered, evaluate the expression numerically and
   load that number back into the textfield.
---------------------------------------------------------------------------*/

void fai_thresh_cb( Widget wtex, XtPointer client_data, XtPointer call_data )
{
   MCW_arrowval * av         = (MCW_arrowval *) client_data ;
   XmAnyCallbackStruct * cbs = (XmAnyCallbackStruct *) call_data ;
   float sval ;
   int iv ;

   MCW_invert_widget(wtex) ;

   sval = DIFF_evaluate( av ) ;
   AV_assign_fval( av , sval ) ;

   MCW_invert_widget(wtex) ;
   return ;
}

/* vector field callback */

void vfield_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
  char *program_name = "vfield_cb" ;
  int debug = 0 ;

  if(debug) {
    fprintf(stderr,"Inside %s\n",program_name);
    vgrid->Print(cout);
  }

  DisplayVectorField() ;

}

/* compute contours */
void compute_contours(Widget w, XtPointer client_data, XtPointer call_data)
{
  char *program_name = "compute_contours" ;
  int debug = 0 ;

  if(debug) {
    fprintf(stderr,"Inside %s: DO_CONTOURS=%i\n",program_name,DO_STREAM);
    igrid->Print(cout);
  }

  float range[2];
  igrid->GetScalarRange(range);
  if(debug) fprintf(stderr,"%s: range = (%f,%f)\n",program_name,range[0],range[1]) ;
  range[0] = 0;			// Peg lower limit to zero

  int dims[3];
  igrid->GetDimensions(dims);
  if(1) printf("Grid dimensions = (%d,%d,%d)\n",dims[0],dims[1],dims[2]);

  SET_LIMS(flims,1,20) ;
  int numCont = (int) pinput("number of contour levels",3,flims);
  SET_LIMS(flims,range[0]+.1*range[1],range[1]) ;
  float minCont = pinput("mininum contour val",range[0]+.1*range[1],flims);
  SET_LIMS(flims,minCont,range[1]) ;
  float maxCont = pinput("maximum contour val",range[1],flims);

  /***************** Threshold data below fai threshold *****************/

  vtkThreshold *thresher = vtkThreshold::New();
  thresher->SetInput(igrid);
  if(debug) fprintf(stderr,"Inside %s: Setting data clipping range to (%f,%f)\n",
		    program_name,50*range[0],range[1]);
  thresher->ThresholdByUpper(50*range[0]);

  /*************************************************************************/

  //vtkContourFilter *iso = vtkContourFilter::New();

  /*
    iso->SetNumberOfContours(numCont);
    iso->SetValue(0,minCont);
    for( i = 1; i < numCont; i++ )
	    iso->SetValue(0,minCont + i*(maxCont - minCont)/(numCont-1));
    printf("%d contours\n", iso->GetNumberOfContours());
  */

  // Color table
  vtkLookupTable *hueLut = vtkLookupTable::New();
  hueLut->SetTableRange(range[0],range[1]);
  hueLut->SetHueRange(0, 1);
  hueLut->SetSaturationRange(1, 1);
  //hueLut->SetValueRange(!CLEAN_NODATA, 1);  /* Doesn't work */
  hueLut->Build();

  //iso->GenerateValues(numCont,minCont, maxCont);

  vtkContourFilter   *iso[numCont] ;
  vtkPolyDataNormals *normals[numCont] ;
  vtkPolyDataMapper  *isoMapper[numCont] ;
  vtkLODActor        *isoActor[numCont] ;

  int use_normals = 0;		// doesn't work if - 0!
  for( int icont = 0; icont < numCont; icont++ ) {

    iso[icont] = vtkContourFilter::New();
    //iso[icont]->SetInput(igrid);
    iso[icont]->SetInput(thresher->GetOutput());
    iso[icont]->SetValue(0,minCont + icont*(maxCont - minCont)/(numCont-1));

    normals[icont] = vtkPolyDataNormals::New();
    normals[icont]->SetInput(iso[icont]->GetOutput());
    normals[icont]->SetFeatureAngle(45);

    // Mapper
    isoMapper[icont] = vtkPolyDataMapper::New();
    if(use_normals) {
      isoMapper[icont]->SetInput(normals[icont]->GetOutput()) ;
    } else {
      isoMapper[icont]->SetInput(iso[icont]->GetOutput()) ;
    }
    isoMapper[icont]->ScalarVisibilityOn();
    isoMapper[icont]->SetScalarRange(range);
    //isoMapper[icont]->SetScalarRange(minCont,maxCont);
    //isoMapper[icont]->SetScalarModeToUsePointFieldData();
    //isoMapper[icont]->ColorByArrayComponent("VelocityMagnitude",0);
    isoMapper[icont]->SetLookupTable(hueLut);

    // Actor
    isoActor[icont] = vtkLODActor::New();
    isoActor[icont]->SetMapper(isoMapper[icont]);
    isoActor[icont]->SetNumberOfCloudPoints(1000);
    isoActor[icont]->GetProperty()->SetDiffuseColor(ColorTable[icont%NColors]);
    isoActor[icont]->GetProperty()->SetOpacity(0.25);

    // add to renderer
    renv->AddActor(isoActor[icont]); // outline actor

  }

  /* Render! */ 

  renv->ResetCamera() ;
  renvWin->Render();  

  fprintf(stderr,"Finished contouring.\n");

}

/* compute streamlines */
void compute_streamlines(Widget w, XtPointer client_data, XtPointer call_data)
{
  char *program_name = "compute_streamlines" ;
  int debug = 0 ;

  if(debug) {
    fprintf(stderr,"Inside %s: DO_STREAM=%i, DO_STUBE=%i\n",
	    program_name,DO_STREAM,DO_STUBE);
    vgrid->Print(cout);
  }

  int dims[3];
  vgrid->GetDimensions(dims);
  if(1) printf("Grid dimensions = (%d,%d,%d)\n",dims[0],dims[1],dims[2]);

  //if(DO_STREAM) DO_RIBBON = logqyn("Ribbon filter","y") ;
  DO_VORTICITY = logqyn("Compute vorticity","n") ;
  DO_PLANES    = logqyn("Plane sources","y") ;
  DO_PROBES    = !DO_PLANES ;

  if(DO_PROBES) {
    DO_PLANE_WIDGET = logqyn("Plane widget","y") ;
    if(!DO_PLANE_WIDGET) DO_LINE_WIDGET = logqyn("Line widget","y") ;
  }

  /*** Outline data ***/ 

  if(outline==NULL) outline = vtkOutlineFilter::New();

  outline->SetInput(vgrid);
  vtkPolyDataMapper *outlineMapper = vtkPolyDataMapper::New();
  outlineMapper->SetInput(outline->GetOutput());
  vtkActor *outlineActor = vtkActor::New();
  outlineActor->SetMapper(outlineMapper);
  outlineActor->GetProperty()->SetColor(0,0,0);

  renv->AddActor(outlineActor);   /* Done outlining, add it ... */

  /*** done outlining data ***/ 

  /* Set up streamlining players */ 

  vtkStreamLine     *streamLine       = vtkStreamLine::New();
  vtkPolyDataMapper *streamLineMapper = vtkPolyDataMapper::New();
  vtkActor          *streamLineActor  = vtkActor::New();

  vtkTubeFilter     *streamTube       = vtkTubeFilter::New();
  vtkPolyDataMapper *streamTubeMapper = vtkPolyDataMapper::New();
  vtkActor          *streamTubeActor  = vtkActor::New();

  vtkRibbonFilter   *streamRibbon     = vtkRibbonFilter::New();

  /******** Probing widgets ********/ 

  vtkLineWidget   *lineWidget = vtkLineWidget::New();
  vtkPlaneWidget *planeWidget = vtkPlaneWidget::New();
  vtkPlaneSource *planeSource = vtkPlaneSource::New();

  /******** Probing sources ********/ 

  vtkPolyData    *seeds = vtkPolyData::New();
  vtkProbeFilter *probe = vtkProbeFilter::New();

  /* Integration params */ 

  /*
  float maxTime    = 100.0 ;
  float IntStepLen = 0.2 ;
  float StepLen    = 0.0001 ;
  */
  float TermSpeed  = 0.002 ;

  SET_LIMS(flims,1.0,1000.0) ;
  float maxTime = pinput("Streamline max time",100.0,flims);
  SET_LIMS(flims,0.01,1.0) ; 
  float IntStepLen = pinput("Streamline integration step length",0.5,flims);
  SET_LIMS(flims,0.00001,0.1) ;
  float StepLen = pinput("Streamline step length",0.05,flims);

  /* Graphics params */ 

  float Stream_LineWidth = 1.0 ;
  float Ribbon_Width     = 1.0 ;
  float Tube_Radius      = 1.0 ;

  /* The StepLength parameter controls the time increment used to
     generate individual points along the streamline(s). Smaller
     values result in more line primitives but smoother
     streamlines. The StepLength instance variable is defined in
     terms of time (i.e., the distance that the particle travels in
     the specified time period). Thus, the line segments will be
     smaller in areas of low velocity and larger in regions of high
     velocity. 

     IntegrationStepLength, defined by the superclass vtkStreamer,
     is used to control integration step size and is expressed as a
     fraction of the cell length.) */

  /* Set up the integrator */ 

  vtkRungeKutta4 *integ = vtkRungeKutta4::New();
  streamLine->SetIntegrator(integ) ;
  streamLine->SetMaximumPropagationTime(maxTime) ;
  streamLine->SetNumberOfThreads(1) ; 
  streamLine->SetStepLength(StepLen) ; 
  streamLine->SetIntegrationStepLength(IntStepLen) ; 
  streamLine->SetIntegrationDirectionToIntegrateBothDirections() ; 
  streamLine->SetTerminalSpeed(TermSpeed) ;

  if(DO_VORTICITY) {
    streamLine->SpeedScalarsOff() ;
    streamLine->OrientationScalarsOn() ;
    streamLine->VorticityOn() ;
  } else {
    streamLine->SpeedScalarsOn() ;
    streamLine->OrientationScalarsOff() ;
    streamLine->VorticityOff() ;
  }

  /***************** Threshold data below fai threshold *****************/

  float range[2];
  vgrid->GetScalarRange(range);

  vtkThreshold *thresher = vtkThreshold::New();
  thresher->SetInput(vgrid);
  if(debug) fprintf(stderr,"Inside %s: Setting data threshold range to (%f,%f)\n",
		    program_name,fai_thresh_av->fval,range[1]);
  thresher->ThresholdByUpper(fai_thresh_av->fval);

  /*************************************************************************/

  streamLine->SetInput(thresher->GetOutput());	

  /* Set source for appropriate probing widget */ 

  if(DO_LINE_WIDGET) {
      streamLine->SetSource(seeds) ;
  }

  if(debug) streamLine->Print(cout) ;

  if(DO_STUBE) {
    printf("\nStreamtubing...\n");

    streamTube->SetRadius(Tube_Radius) ;
    streamTube->SetNumberOfSides(12) ;
    streamTube->UseDefaultNormalOn() ;
      
    // Source
    // I don't think this cleaning does anything!
    vtkCleanPolyData *cleanStream = vtkCleanPolyData::New();
    cleanStream->SetInput(streamLine->GetOutput()) ;
    streamTube->SetInput(cleanStream->GetOutput()) ;
    //streamTube->SetInput(streamLine->GetOutput()) ;

    // Mapper
    streamTubeMapper->SetInput(streamTube->GetOutput()) ;
    // Actor
    streamTubeActor->SetMapper(streamTubeMapper);
    // streamTubeActor->GetProperty()->BackfaceCullingOn();

    renv->AddActor(streamTubeActor);
    printf("finished streamtubing...\n");fflush(stdout) ;

  } else if(DO_RIBBON) {
    printf("\nStreamribboning...\n");

    streamRibbon->SetInput(streamLine->GetOutput());
    streamRibbon->SetWidth(Ribbon_Width);
    //streamRibbon->SetWidthFactor(1);
    streamLineMapper->SetInput(streamRibbon->GetOutput());
    streamLineMapper->SetScalarRange(range); 
    streamLineActor->SetMapper(streamLineMapper);
    renv->AddActor(streamLineActor);

  } else {

    printf("\nStreamlining...\n");
    //streamLine->Print(cout) ;

    streamLineMapper->SetInput(streamLine->GetOutput());
    streamLineMapper->SetScalarRange(range); 
    streamLineActor->SetMapper(streamLineMapper);
    streamLineActor->GetProperty()->SetLineWidth(Stream_LineWidth);
    renv->AddActor(streamLineActor);

  }

  if(DO_LINE_WIDGET) {

    fprintf(stderr,"Inside DO_LINE_WIDGET\n");

    vtkLWCallback *myCallback = vtkLWCallback::New();
    myCallback->PolyData = seeds;

    if(DO_STREAM) myCallback->Actor = streamLineActor;
    if(DO_STUBE)  myCallback->Actor = streamTubeActor;

    lineWidget->SetInteractor(irenv);
    //lineWidget->SetInput(vgrid);
    lineWidget->SetInput(thresher->GetOutput());
    lineWidget->SetAlignToYAxis();
    lineWidget->PlaceWidget();
    lineWidget->GetPolyData(seeds);
    lineWidget->ClampToBoundsOff();
    lineWidget->AddObserver(vtkCommand::InteractionEvent,myCallback);

  } else if (DO_PLANE_WIDGET) {

    fprintf(stderr,"Inside DO_PLANE_WIDGET\n");

    vtkPWCallback *myCallback = vtkPWCallback::New();
    vtkPolyData *plane = vtkPolyData::New();
    myCallback->PolyData = plane;

    if(DO_STREAM) myCallback->Actor = streamLineActor;
    if(DO_STUBE)  myCallback->Actor = streamTubeActor;

    planeWidget->SetInteractor(irenv);
    //planeWidget->SetInput(vgrid);
    planeWidget->SetInput(thresher->GetOutput());
    planeWidget->NormalToXAxisOn();
    SET_LIMS(flims,1,MAX(MAX(dims[0],dims[1]),dims[2])) ;
    int pwRes = (int) pinput("plane widget resolution",4,flims);
    planeWidget->SetResolution(pwRes);
    planeWidget->SetRepresentationToOutline();
    planeWidget->PlaceWidget();
    planeWidget->GetPolyData(plane);

    planeWidget->AddObserver(vtkCommand::InteractionEvent,myCallback);

    probe->SetInput(plane) ;
    probe->SetSource(thresher->GetOutput());
    streamLine->SetSource(probe->GetOutput()) ;

    vtkPolyDataMapper *probeMapper = vtkPolyDataMapper::New();
    probeMapper->SetInput((vtkPolyData *)probe->GetOutput());

    vtkActor *probeActor = vtkActor::New();
    probeActor->SetMapper(probeMapper);
    probeActor->GetProperty()->SetRepresentationToWireframe();
    renv->AddActor(probeActor); 

  } else if(DO_PLANES) {

    fprintf(stderr,"Inside DO_PLANE_SOURCE\n");

    /*
    vtkPSCallback *myCallback = vtkPSCallback::New();
    vtkPolyData *plane = vtkPolyData::New();
    myCallback->PolyData = plane;

    if(DO_STREAM) myCallback->Actor = streamLineActor;
    if(DO_STUBE)  myCallback->Actor = streamTubeActor;
    */

    SET_LIMS(flims,1,dims[0]) ;
    int xdim = (int) pinput("xdim",1,flims);
    SET_LIMS(flims,1,dims[1]) ;
    int ydim = (int) pinput("ydim",1,flims);
    planeSource->SetResolution(xdim,ydim);

    float orig[] = {0.,0.,0.};
    float cent[] = {dims[0]/2.,dims[1]/2,dims[2]/2.};
    planeSource->SetCenter(cent);
    float pt[3];
    pt[0] = dims[0]; pt[1] = orig[1]; pt[2] = dims[2]/2.;
    planeSource->SetPoint1(pt);
    pt[0] = orig[0]; pt[1] = dims[1]; pt[2] = dims[2]/2.;
    planeSource->SetPoint2(pt);

    vtkTransform *trans = vtkTransform::New();
    trans->Translate(-orig[0]/2.,-orig[1]/2.,0.) ;

    vtkTransformPolyDataFilter *filt = vtkTransformPolyDataFilter::New();
    filt->SetInput(planeSource->GetOutput());
    filt->SetTransform(trans);

    // Outline
    vtkOutlineFilter *outTrans = vtkOutlineFilter::New();
    outTrans->SetInput(filt->GetOutput());
    vtkPolyDataMapper *mapOut = vtkPolyDataMapper::New();
    mapOut->SetInput(outTrans->GetOutput());
    vtkActor *outActor = vtkActor::New();
    outActor->SetMapper(mapOut);
    outActor->GetProperty()->SetColor(1.,1.,.1);
    renv->AddActor(outActor); 

    //planeSource->AddObserver(vtkCommand::InteractionEvent,myCallback);

    probe->SetInput(planeSource->GetOutput()) ;
    probe->SetSource(thresher->GetOutput());
    streamLine->SetSource(planeSource->GetOutput()) ;

    vtkPolyDataMapper *probeMapper = vtkPolyDataMapper::New();
    probeMapper->SetInput((vtkPolyData *)probe->GetOutput());

    vtkActor *probeActor = vtkActor::New();
    probeActor->SetMapper(probeMapper);
    probeActor->GetProperty()->SetRepresentationToWireframe();

    renv->AddActor(probeActor); 

  }

  /* Render! */ 

  renv->ResetCamera() ;
  renvWin->Render();  

}

/* streamline callback */
void stream_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
  fprintf(stderr,"Inside stream_cb\n");
  DO_STREAM = 1 ;	/* Streamline */  
  DO_STUBE  = 0 ;	/* Streamtube */

  compute_streamlines( w, client_data, call_data) ;

}

/* streamtube callback */
void tube_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
  fprintf(stderr,"Inside tube_cb\n");
  DO_STREAM = 0 ;	/* Streamline */  
  DO_STUBE  = 1 ;	/* Streamtube */

  compute_streamlines( w, client_data, call_data) ;

}

/* contour callback */
void contour_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
  fprintf(stderr,"Inside contour_cb\n");
  DO_CONTOUR = 1 ;

  compute_contours( w, client_data, call_data) ;

}

  /* Current hardwired - haven't really figured out how this works yet! */ 
#define FRAC             0.77
#define VTK_WIDG_WIDTH   400
#define VTK_WIDG_HEIGHT  400
#define BUTTON_HEIGHT    40

Widget setup_vector_widgets(Widget sh,vtkXOpenGLRenderWindow *rw)
{
  Widget exit_button;
  Widget vfield_button;
  Widget stream_button;
  Widget tube_button;
  Widget contour_button;
  Widget button_rowcol, button_row1, button_row2;
  Widget toplevel;
  Widget form;
  Widget widg_ret ;
  XmString xstr ;

  /* X Window stuff begin */

  XtSetLanguageProc(NULL,NULL,NULL);

  // get the display connection and give it to the renderer

  rw->SetDisplayId(XtDisplay(sh));

  toplevel = XtVaCreateWidget("Diffusion Vector Visualizer",
			       topLevelShellWidgetClass,sh,
                               XmNdepth, rw->GetDesiredDepth(),
                               XmNvisual, rw->GetDesiredVisual(),
                               XmNcolormap, rw->GetDesiredColormap(),
                               NULL);

  form = XtVaCreateWidget("form",
			  xmFormWidgetClass, toplevel, 
			  XmNwidth, 600, XmNheight, 600,
			  NULL);

  widg_ret = XtVaCreateManagedWidget("vtk_vector_widg",
				     xmPrimitiveWidgetClass, form, 
                                     XmNwidth, VTK_WIDG_WIDTH, XmNheight, VTK_WIDG_HEIGHT,
                                     XmNleftAttachment, XmATTACH_FORM,
                                     XmNrightAttachment, XmATTACH_FORM,
                                     XmNtopAttachment, XmATTACH_FORM,
				     XmNbottomAttachment , XmATTACH_POSITION ,
				     XmNbottomPosition   , (int)((1.- FRAC) * VTK_WIDG_HEIGHT) ,
                                     NULL);

  // rowcol for buttons


  int Nrows = 2;			// number of button_rows

  button_rowcol =  XtVaCreateWidget("AFNI" , xmRowColumnWidgetClass , form ,
				    XmNorientation  , XmVERTICAL ,
				    XmNpacking      , XmPACK_TIGHT ,
				    XmNheight, Nrows * BUTTON_HEIGHT,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNtopAttachment, XmATTACH_WIDGET,
				    XmNtopWidget, widg_ret,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_FORM,
				    /*XmNrecomputeSize , False ,*/
				    XmNtraversalOn , False ,
				    XmNinitialResourcesPersistent , False ,
				    NULL); 

  button_row1 =  XtVaCreateWidget("AFNI" , xmRowColumnWidgetClass , button_rowcol ,
				  XmNorientation  , XmHORIZONTAL ,
				  XmNpacking      , XmPACK_TIGHT ,
				  XmNheight, BUTTON_HEIGHT,
				  XmNtraversalOn , False ,
				  XmNinitialResourcesPersistent , False ,
				  NULL); 

  button_row2 =  XtVaCreateWidget("AFNI" , xmRowColumnWidgetClass , button_rowcol ,
				  XmNorientation  , XmHORIZONTAL ,
				  XmNpacking      , XmPACK_TIGHT ,
				  XmNheight, BUTTON_HEIGHT,
				  XmNtraversalOn , False ,
				  XmNinitialResourcesPersistent , False ,
				  NULL); 

  // Vector field button

  xstr = XmStringCreateLtoR( "Vector field" , XmFONTLIST_DEFAULT_TAG ) ;
  vfield_button = XtVaCreateManagedWidget("AFNI" , xmPushButtonWidgetClass , 
					  button_row1 ,
					  XmNlabelString , xstr ,
					  XmNtraversalOn , False ,
					  XmNrecomputeSize , False ,
					  XmNinitialResourcesPersistent , False ,
					  NULL ) ;

  XtAddCallback(vfield_button,XmNactivateCallback,vfield_cb,(XtPointer *)toplevel);

  // Streamline button

  xstr = XmStringCreateLtoR( "Streamline" , XmFONTLIST_DEFAULT_TAG ) ;
  stream_button = XtVaCreateManagedWidget("AFNI" , xmPushButtonWidgetClass , 
					button_row1 ,
					XmNlabelString , xstr ,
					XmNtraversalOn , False ,
					XmNinitialResourcesPersistent , False ,
					NULL ) ;

  XtAddCallback(stream_button,XmNactivateCallback,stream_cb,(XtPointer *)toplevel);

  // Streamtube button

  xstr = XmStringCreateLtoR( "Streamtube" , XmFONTLIST_DEFAULT_TAG ) ;
  tube_button = XtVaCreateManagedWidget("AFNI" , xmPushButtonWidgetClass , 
					button_row1 ,
					XmNlabelString , xstr ,
					XmNtraversalOn , False ,
					XmNinitialResourcesPersistent , False ,
					NULL ) ;

  XtAddCallback(tube_button,XmNactivateCallback,tube_cb,(XtPointer *)toplevel);

  // Contour button

  xstr = XmStringCreateLtoR( "Contour" , XmFONTLIST_DEFAULT_TAG ) ;
  contour_button = XtVaCreateManagedWidget("AFNI" , xmPushButtonWidgetClass , 
					button_row1 ,
					XmNlabelString , xstr ,
					XmNtraversalOn , False ,
					XmNinitialResourcesPersistent , False ,
					NULL ) ;

  XtAddCallback(contour_button,XmNactivateCallback,contour_cb,(XtPointer *)toplevel);


  // FAI threshold

  static float fai_step = .1 ;
  static float fai_thresh = .5 ;
  static float fai_max = 1.0;

  fai_thresh_av =  new_MCW_arrowval( button_row2, "FAI thresh", 
				     MCW_AV_downup, 
				     0, 100*fai_max, (int)(100.*fai_thresh),
				     MCW_AV_edittext , 2,
				     fai_thresh_av_CB,NULL,NULL,NULL ) ;

  fai_thresh_av->fstep = fai_step;

  MCW_reghelp_children(fai_thresh_av->wrowcol,"FAI threshold") ;
  XtAddCallback( fai_thresh_av->wtext, XmNactivateCallback, fai_thresh_cb, fai_thresh_av) ;

  // Exit button

  xstr = XmStringCreateLtoR( "Exit" , XmFONTLIST_DEFAULT_TAG ) ;
  exit_button = XtVaCreateManagedWidget("AFNI" , xmPushButtonWidgetClass , 
					button_row2 ,
					XmNlabelString , xstr ,
					XmNtraversalOn , False ,
					XmNinitialResourcesPersistent , False ,
					NULL ) ;


  XtAddCallback(exit_button,XmNactivateCallback,quit_cb,(XtPointer *)toplevel);

  // Management

  XtManageChild(button_row1) ;
  XtManageChild(button_row2) ;
  XtManageChild(button_rowcol) ;
  XtManageChild(form) ;
  XtRealizeWidget(toplevel) ;
  XtMapWidget(toplevel);

  return (widg_ret) ;
}

Widget setup_widgets(Widget sh,vtkXOpenGLRenderWindow *rw)
{
  Widget button;
  Widget toplevel;
  Widget form;
  Widget widg_ret ;

  /* X Window stuff begin */

  XtSetLanguageProc(NULL,NULL,NULL);

  // get the display connection and give it to the renderer

  rw->SetDisplayId(XtDisplay(sh));

  toplevel = XtVaCreateWidget("Diffusion Visualizer",
			       topLevelShellWidgetClass,sh,
                               XmNdepth, rw->GetDesiredDepth(),
                               XmNvisual, rw->GetDesiredVisual(),
                               XmNcolormap, rw->GetDesiredColormap(),
                               NULL);

  form     = XtVaCreateWidget("form",xmFormWidgetClass, toplevel, NULL);
  widg_ret = XtVaCreateManagedWidget("vtk_widg",
				     xmPrimitiveWidgetClass, form, 
                                     XmNwidth, 300, XmNheight, 300,
                                     XmNleftAttachment, XmATTACH_FORM,
                                     XmNrightAttachment, XmATTACH_FORM,
                                     XmNtopAttachment, XmATTACH_FORM,
				     XmNbottomAttachment , XmATTACH_POSITION ,
				     XmNbottomPosition   , (int)((1.- FRAC) * VTK_WIDG_HEIGHT) ,
                                     NULL);

  button    = XtVaCreateManagedWidget("Exit",
				      xmPushButtonWidgetClass, form,
                                      XmNheight, 40,
                                      XmNbottomAttachment, XmATTACH_FORM,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, widg_ret,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNrightAttachment, XmATTACH_FORM,
				      XmNtraversalOn, False ,
				      XmNrecomputeSize, False ,
				      XmNinitialResourcesPersistent , False ,
                                      NULL);
  XtAddCallback(button,XmNactivateCallback,quit_cb,(XtPointer *)toplevel);
  XtManageChild(form);
  XtRealizeWidget(toplevel);
  XtMapWidget(toplevel);

  return (widg_ret) ;
}
